#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "fitscat.h"
#include "tabutil_globals.h"
#include "utils_globals.h"
#include "project_globals.h"
#include "coord_trans.h"

/* 
   19.09.2003:
   changes to make aplastrom work with flipped
   coordinate systems. In fuction xy2rd, the offdiagonal
   pc matrix elements have to change sign if the coordinates
   are flipped w.r.t. the east left, north up direction
   (simple rotations always worked fine).
   The applied correction seems to work but I do not understand
   why!!
*/

affine_params *init_coord_trans(char *name)
{
     FILE   *fd;
     char   line[132];
     float  A0 = 0.0, A1 = 1.0, A2 = 0.0, B0 = 0.0, B1 = 0.0, B2 = 1.0;
     affine_params *p;

     ED_CALLOC(p, "init_coord_trans", affine_params, 1);
     if (!ED_FCHECK("init_coord_trans", name)) {
        ED_FOPEN(fd, "init_coord_trans", name, "rt");
        for (; fgets(line, 132, fd) != NULL;) {
           if (line[0] != '#') {     
              sscanf(line,"%f %f %f %f %f %f",
                    &A0,&A1,&A2,&B0,&B1,&B2);
              break;
           }
        }
        ED_FCLOSE(fd, "init_coord_trans");
     }
     p->a0 = A0;
     p->a1 = A1;
     p->a2 = A2;
     p->b0 = B0;
     p->b1 = B1;
     p->b2 = B2;

     return p;
}

void update_coord_trans(affine_params *p,
                        float A0, float A1, float A2,
                        float B0, float B1, float B2) 
{
    p->a0 = A0;
    p->a1 = A1;
    p->a2 = A2;
    p->b0 = B0;
    p->b1 = B1;
    p->b2 = B2;
}

void get_coord_trans(affine_params *p,
                     float *A0, float *A1, float *A2, 
                     float *B0, float *B1, float *B2)
{
    *A0 = p->a0;
    *A1 = p->a1;
    *A2 = p->a2;
    *B0 = p->b0;
    *B1 = p->b1;
    *B2 = p->b2;
}

void apply_coord_trans(affine_params *p, float *x, float *y, int n)
{
    float	A0, A1, A2, B0, B1, B2;
    float	x_t, y_t;
    int 	i;

    get_coord_trans(p, &A0, &A1, &A2, &B0, &B1, &B2);

    for (i=0;i<n;i++) {
       x_t = x[i];
       y_t = y[i];
       x[i] = A0 + A1 * x_t + A2 * y_t;
       y[i] = B0 + B1 * x_t + B2 * y_t;
    }
}

tabstruct *create_affine_tab(affine_params **p, int n)
{
    tabstruct	*tab;
    float	*a0, *a1, *a2, *b0, *b1, *b2;
    int		i;

    ED_CALLOC(a0, "create_affine_tab", float, n);
    ED_CALLOC(a1, "create_affine_tab", float, n);
    ED_CALLOC(a2, "create_affine_tab", float, n);
    ED_CALLOC(b0, "create_affine_tab", float, n);
    ED_CALLOC(b1, "create_affine_tab", float, n);
    ED_CALLOC(b2, "create_affine_tab", float, n);

    for (i=0;i<n;i++) {
       a0[i] = p[i]->a0;
       a1[i] = p[i]->a1;
       a2[i] = p[i]->a2;
       b0[i] = p[i]->b0;
       b1[i] = p[i]->b1;
       b2[i] = p[i]->b2;
    }
    tab = new_tab("AFFINE");
    add_key_to_tab(tab, "A0", n, a0, T_FLOAT, 1, "pixels", 
                       "Affine X position offset");
    add_key_to_tab(tab, "A1", n, a1, T_FLOAT, 1, "", 
                       "Affine X position X scale");
    add_key_to_tab(tab, "A2", n, a2, T_FLOAT, 1, "", 
                       "Affine X position Y scale");
    add_key_to_tab(tab, "B0", n, b0, T_FLOAT, 1, "pixels", 
                       "Affine Y position offset");
    add_key_to_tab(tab, "B1", n, b1, T_FLOAT, 1, "", 
                       "Affine Y position X scale");
    add_key_to_tab(tab, "B2", n, b2, T_FLOAT, 1, "", 
                       "Affine Y position Y scale");
    return tab;
}

affine_params **load_affine_tab(tabstruct *tab, int *n)
{
    keystruct		**keys;
    char		**keynames;
    int			i, AFFKEYS = 6; 
    affine_params	**p;
    float		*a0, *a1, *a2, *b0, *b1, *b2;

    ED_CALLOC(keys,     "load_affine_tab", keystruct *, AFFKEYS);
    ED_CALLOC(keynames, "load_affine_tab", char *,      AFFKEYS);
    for (i=0;i<AFFKEYS;i++)
       ED_CALLOC(keynames[i], "load_affine_tab", char, MAXCHAR);
    strcpy(keynames[0], "A0");
    strcpy(keynames[1], "A1");
    strcpy(keynames[2], "A2");
    strcpy(keynames[3], "B0");
    strcpy(keynames[4], "B1");
    strcpy(keynames[5], "B2");
    read_keys(tab, keynames, keys, AFFKEYS, NULL);
    for (i=0;i<AFFKEYS;i++) 
       if (!keys[i])
         syserr(FATAL,"load_affine_tab", "No key %s in table\n", keynames[i]);
    *n = keys[0]->nobj;
    ED_COPY_PTR(a0, "load_affine_tab", keys[0], T_FLOAT, "float");
    ED_COPY_PTR(a1, "load_affine_tab", keys[1], T_FLOAT, "float");
    ED_COPY_PTR(a2, "load_affine_tab", keys[2], T_FLOAT, "float");
    ED_COPY_PTR(b0, "load_affine_tab", keys[3], T_FLOAT, "float");
    ED_COPY_PTR(b1, "load_affine_tab", keys[4], T_FLOAT, "float");
    ED_COPY_PTR(b2, "load_affine_tab", keys[5], T_FLOAT, "float");
    for (i=0;i<AFFKEYS;i++) 
       ED_FREE(keynames[i]);
    ED_FREE(keynames);
    ED_FREE(keys);
    ED_CALLOC(p, "load_affine_tab", affine_params *, (*n));
    for (i=0;i<*n;i++) {
       ED_CALLOC(p[i], "load_affine_tab", affine_params, 1);
       update_coord_trans(p[i], a0[i], a1[i], a2[i], b0[i], b1[i], b2[i]);
    }
    return p;
}

void rd2xy(double ra, double dec, 
           double dra, double ddec, double cra, double cdec,
           double pc11, double pc12, double pc21, double pc22,
           double *r, double *d, float *x, float *y, int n)
{
    double	*X, *Y, *R, *D, tmp, det;
    int		i;

    ra  *= DEG2RAD;
    dec *= DEG2RAD;
    ED_CALLOC(X, "rd2xy", double, n);
    ED_CALLOC(Y, "rd2xy", double, n);
    ED_CALLOC(R, "rd2xy", double, n);
    ED_CALLOC(D, "rd2xy", double, n);
    for (i=0;i<n;i++) {
       R[i] = r[i] * DEG2RAD;
       D[i] = d[i] * DEG2RAD;
    }
    abs_to_rel(ra, dec, 0.0, R, D, X, Y, n);
    det = -(pc21*pc12-pc11*pc22);
    for (i=0;i<n;i++) {
       x[i] = X[i] * RAD2DEG / dra;
       y[i] = Y[i] * RAD2DEG / ddec;
       tmp  = ( x[i] * pc22 - y[i] * pc12) / det;
       y[i] = ( - x[i] * pc21 + y[i] * pc11) / det;
       x[i] = tmp;
       x[i] += cra;
       y[i] += cdec;
    }
    ED_FREE(X); 
    ED_FREE(Y); 
    ED_FREE(R); 
    ED_FREE(D); 
}

void xy2rd(double ra, double dec, 
           double dra, double ddec, double cra, double cdec,
           double pc11, double pc12, double pc21, double pc22,
           float *x, float *y, double *r, double *d, int n)
{
    double	*X, *Y, *R, *D, tmp, det;
    int		i;

    ra  *= DEG2RAD;
    dec *= DEG2RAD;
    ED_CALLOC(X, "xy2rd", double, n);
    ED_CALLOC(Y, "xy2rd", double, n);
    ED_CALLOC(R, "xy2rd", double, n);
    ED_CALLOC(D, "xy2rd", double, n);
    /* get determinant; if positive (there is a flip in
       the coordinate system), we change the sign of pc21
       and pc12. It works but I do not really understand
       why !!
    */
    det = (pc11*pc22-pc12*pc21)*dra*ddec;
    if (det > 0)
    {
      pc21 *= -1.;
      pc12 *= -1.;
    }
    for (i=0;i<n;i++) {
       X[i] = (x[i] - cra) * dra * DEG2RAD;
       Y[i] = (y[i] - cdec) * ddec * DEG2RAD;
       tmp  = X[i] * pc11 + Y[i] * pc21;
       Y[i] = X[i] * pc12 + Y[i] * pc22;
       X[i] = tmp;
    }
    rel_to_abs(ra, dec, 0.0, X, Y, R, D, n);
    for (i=0;i<n;i++) {
       if ((r[i] = R[i] * RAD2DEG) < 0.0) r[i] += 360.0;
       if (r[i] > 360.0) r[i] -= 360.0;
       d[i] = D[i] * RAD2DEG;
    }
    ED_FREE(X); 
    ED_FREE(Y); 
    ED_FREE(R); 
    ED_FREE(D); 
}
