#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "fitscat.h"
#include "field_desc.h"
#include "dist_globals.h"
#include "options_globals.h"
#include "sort_globals.h"
#include "precess_defs.h"
#include "precess_globals.h"
#include "utils_globals.h"
#include "bin_search_globals.h"
#include "tabutil_globals.h"

static double *pixscale;

tabstruct *dist_fields_create(field_params **p, int n)
{
    tabstruct	*tab;
    long int	*xwid, *ywid;
    double	*ra, *dec, *crpix1, *crpix2, *cdelt1, *cdelt2;
    int		i;
    char	*chan_name, *filt_name;
    long int	*id, *chan_nr;

    ED_MALLOC(xwid,      "icat_fields_create", long int, n);
    ED_MALLOC(ywid,      "icat_fields_create", long int, n);
    ED_MALLOC(ra,        "icat_fields_create", double, n);
    ED_MALLOC(dec,       "icat_fields_create", double, n);
    ED_MALLOC(crpix1,    "icat_fields_create", double, n);
    ED_MALLOC(crpix2,    "icat_fields_create", double, n);
    ED_MALLOC(cdelt1,    "icat_fields_create", double, n);
    ED_MALLOC(cdelt2,    "icat_fields_create", double, n);
    ED_MALLOC(pixscale,  "icat_fields_create", double, n);
    ED_MALLOC(id,        "icat_fields_create", long int, n);
    ED_MALLOC(chan_nr,   "icat_fields_create", long int, n);
    ED_CALLOC(chan_name, "icat_fields_create", char , (n*16));
    ED_CALLOC(filt_name, "icat_fields_create", char , (n*16));
    for (i=0;i<n;i++) {
       id[i]       = i+1;
       xwid[i]     = p[i]->xwid;
       ywid[i]     = p[i]->ywid;
       ra[i]       = p[i]->crval1;
       dec[i]      = p[i]->crval2;
       crpix1[i]   = p[i]->crpix1;
       crpix2[i]   = p[i]->crpix2;
       cdelt1[i]   = p[i]->cdelt1;
       cdelt2[i]   = p[i]->cdelt2;
       pixscale[i] = p[i]->cdelt1;
       chan_nr[i]  = -p[i]->chan_nr;
       strcpy(&chan_name[i*16] ," ");
       strcpy(&filt_name[i*16] ," ");
    }
    tab = new_tab("FIELDS");
    add_key_to_tab(tab,"CHANNEL_NR", n, chan_nr, T_LONG, 1,
                       "","Channel number");
    add_key_to_tab(tab,"CHANNEL_NAME", n, chan_name, T_STRING, 16,
                       "","Channel name");
    add_key_to_tab(tab,"FILTER_NAME", n, filt_name, T_STRING, 16,
                       "","Filter name");
    add_key_to_tab(tab,"CRVAL1", n, ra, T_DOUBLE, 1,
                       "Deg","RA  at the reference pixel");
    add_key_to_tab(tab,"CRVAL2", n, dec, T_DOUBLE, 1,
                       "Deg","DEC at the reference pixel");
    add_key_to_tab(tab,"CRPIX1", n, crpix1, T_DOUBLE, 1,
                       "pixel","Reference X pixel");
    add_key_to_tab(tab,"CRPIX2", n, crpix2, T_DOUBLE, 1,
                       "pixel","Reference Y pixel");
    add_key_to_tab(tab,"CDELT1", n, cdelt1, T_DOUBLE, 1,
                       "deg/pixel","pixel size in projected degrees");
    add_key_to_tab(tab,"CDELT2", n, cdelt2, T_DOUBLE, 1,
                       "deg/pixel","pixel size in projected degrees");
    add_key_to_tab(tab,"MAPNAXS1", n, xwid, T_LONG, 1,
                       "","x-coordinate size of the map in pixels");
    add_key_to_tab(tab,"MAPNAXS2", n, ywid, T_LONG, 1,
                       "","y-coordinate size of the map in pixels");
    add_key_to_tab(tab,"2MUFIELD", n, id, T_LONG, 1,
                        "","position in the strip");
    return tab;
}

void dist_fields_write(catstruct *cat, tabstruct *tab, 
                       int *obj_pos, int *obj_nr, int n)
{
    add_key_to_tab(tab,"OBJECT_POS", n, obj_pos, T_LONG, 1,
                       "","Position of the first object from that field");
    add_key_to_tab(tab,"OBJECT_COUNT", n, obj_nr, T_LONG, 1,
                       "","Number of objects in that field");
    update_tab(tab);
    if (add_tab(tab, cat, 1) == RETURN_ERROR) {
       syserr(FATAL,"icat_fields_write","Error adding table to catalog\n");
    }
}

void dist_objects_create(catstruct *cat, int obj_nr, int field,
                                         float *xdist, float *ydist)
{
    tabstruct	*gtab;
    int		i;
    short int	*fldnr;
    float	*x, *y;

    ED_MALLOC(fldnr,"icat_objects_create", short int, obj_nr);
    ED_MALLOC(x,    "icat_objects_create", float    , obj_nr);
    ED_MALLOC(y,    "icat_objects_create", float    , obj_nr);

    for (i=0;i<obj_nr; i++) {
       fldnr[i] = field;
       x[i]     = xdist[i];
       y[i]     = ydist[i];
    }
    gtab = new_tab("OBJECTS");
    add_key_to_tab(gtab,"FIELD_POS", obj_nr, fldnr, T_SHORT, 1,
                        "","Reference number to field parameters");
    add_key_to_tab(gtab,"Xdist", obj_nr, x, T_FLOAT, 1,
                        "pixel","X positional difference of pair");
    add_key_to_tab(gtab,"Ydist", obj_nr, y, T_FLOAT, 1,
                        "pixel","Y positional difference of pair");
    update_tab(gtab);
    if (add_tab(gtab, cat, field) == RETURN_ERROR) {
       syserr(FATAL,"icat_objects_create","Error adding table to catalog\n");
    }
}

