/* History:
   10.01.2000 (TE):
   changes "extern control_struct control" to
           "control_struct control".
   It seems that with the former the structure was not
   initialiced correctly (AIX system).

   29.04.03:
   removed compiler warning for main not being of return
   type "int".

   10.01.2005:
   I removed compiler warnings under gcc with optimisation
   enabled (-O3).

   11.03.2005:
   function main:
   - I changed the parameters to save_distort_tab for reformating
     the HISTORY written to output catalogs.
   - changes to avoid compiler warnings under gcc with 
     '-Wall -Wpedantic -W'
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "make_distort.h"
#include "options_globals.h"
#include "utils_globals.h"
#include "model_globals.h"
#include "thresh_globals.h"
#include "field_desc.h"
#include "astrom_io_globals.h"
#include "astrom_globals.h"
#include "output_globals.h"

control_struct control;

char header[MAXCHAR];

option my_opts[] = {
            {"XPOS", TEXT, control.xpos, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "Xpos",
             {"", "", "", "", ""}, 1,
             "Name of PAIRS table column containing the x position"},
            {"YPOS", TEXT, control.ypos, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "Ypos",
             {"", "", "", "", ""}, 1,
             "Name of PAIRS table column containing the y position"},
            {"ERRPOS", TEXT, control.errpos, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "ERRA_IMAGE",
             {"", "", "", "", ""}, 1,
             "Name of PAIRS table column containing the postional error estimate"},
            {"CRVAL1", TEXT, control.crval1, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "CRVAL1",
             {"", "", "", "", ""}, 1,
             "Name of FIELDS table column containing the field center RA"},
            {"CRVAL2", TEXT, control.crval2, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "CRVAL2",
             {"", "", "", "", ""}, 1,
             "Name of FIELDS table column containing the field center DEC"},
            {"CDELT1", TEXT, control.cdelt1, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "CDELT1",
             {"", "", "", "", ""}, 1,
             "Name of FIELDS table column containing the x pixel size"},
            {"CDELT2", TEXT, control.cdelt2, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "CDELT2",
             {"", "", "", "", ""}, 1,
             "Name of FIELDS table column containing the y pixel size"},
            {"CRPIX1", TEXT, control.crpix1, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "CRPIX1",
             {"", "", "", "", ""}, 1,
             "Name of FIELDS table column containing the reference x pixel"},
            {"CRPIX2", TEXT, control.crpix2, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "CRPIX2",
             {"", "", "", "", ""}, 1,
             "Name of FIELDS table column containing the reference y pixel"},
            {"XWID", TEXT, control.xwid, 0, 0, 0.0, 0.0, 
              0, 0.0, NO, "MAPNAXS1",
             { "", "", "", "", ""}, 1,
              "Name of FIELDS table column containing the x field width"},
            {"YWID", TEXT, control.ywid, 0, 0, 0.0, 0.0, 
              0, 0.0, NO, "MAPNAXS2",
             { "", "", "", "", ""}, 1,
              "Name of FIELDS table column containing the y field width"},
            {"", BOOLEAN, NULL, 0, 0, 0.0, 0.0, 0, 0.0, NO, "", 
             {"YES", "NO", "", "", ""}, 0,
              "dummy key to end structure my_opts"}
            };

void usage(char *me)
{
    fprintf(stderr,"Usage: %s \n",me);
    fprintf(stderr,"		-i incat1 [[incat2] incat3] ... \n");
    fprintf(stderr,"		-o outcat1 [[outcat2] outcat3] ... \n");
    fprintf(stderr,"		-c conffile\n");
    fprintf(stderr,"		[-OPTION1 OPTVAL1 [-OPTION OPTVAL2] ...]\n");
 
    exit(1);
}
 
void init(int argc, char *argv[])
{
   int i, j, k;

   if (argc == 1) usage(argv[0]);
   init_options(0, my_opts);
   for (i=1; i<argc; i++) {
     if (argv[i][0] == '-') {
        switch(argv[i][1]) {
        case 'c': strcpy(control.conffile,argv[++i]);
                  break;
        case '@': print_options(argv[0]);
                  exit(0);
        case 'i': control.ilist = 0;
                  strcpy(control.listfile[control.ilist],argv[++i]);
                  while (i+1<argc && argv[i+1][0] != '-') {
                     strcpy(control.listfile[++control.ilist],argv[++i]);
                  }
                  ++control.ilist;
                  break;
        case 'o': j = 0;
                  strcpy(control.outfile[j],argv[++i]);
                  while (i+1<argc && argv[i+1][0] != '-') {
                     strcpy(control.outfile[++j],argv[++i]);
                  }
                  if (j+1 != control.ilist) {
                     syserr(FATAL, "init", "Input & Output lists have different length");
                  }
                  break;
        default:  parse_options(control.conffile);
                  j = i++; k = i;
                  set_command_option(argv[j], argv[k]);
                  break;
        }
     } else {
        syserr(WARNING,"init","Command line syntax error\n");
        usage(argv[0]);
     }
  }
  finish_options(control.conffile);
}

void make_distort(model_struct *model, int c)
{
    float	**fldpol;
    int		f, n;

    ED_CALLOC(fldpol,    "make_distort", float *, 2);
    ED_CALLOC(fldpol[0], "make_distort", float, model->np);
    ED_CALLOC(fldpol[1], "make_distort", float, model->np);
    for (f=0;f<model->nf;f++) {
      bi_cubicproject (f, c, fldpol, &n, model);
      add_distort_line(f, c, fldpol, n, model);
    }
}

int main(int argc, char *argv[])
{ 
    int                i, j, c = 0, nfields;
    catstruct          *incat;
    tabstruct          *astrom_tab, *fields_tab;
    field_params       **field_desc;
    model_struct       model;
    thresh_struct      *thresholds;

    init(argc, argv);

    sprintf(header,"%s Version: %s (%s)",PROGRAM,VERS,DATE);
    VPRINTF(NORMAL,"\n   %s\n\n", header);
    for (i=0;i<control.ilist;i++) {
      if (!(incat = read_cat(control.listfile[0]))) 
        syserr(FATAL, "aplastrom", "Could not open catalog %s\n", 
               control.listfile);
      if (!(fields_tab = name_to_tab(incat, "FIELDS", 0))) 
        syserr(FATAL,"aplastrom", "No FIELDS table in input catalog");
      field_desc = load_field_description(fields_tab, &nfields,
                             control.crval1, control.crval2,
                             control.cdelt1, control.cdelt2,
                             control.crpix1, control.crpix2,
                             control.xwid, control.ywid);
      if ((astrom_tab = name_to_tab(incat, "ASTROM", 0))) {
         VPRINTF(NORMAL, "Applying full Astrometric Calibration\n");
         fetch_model(astrom_tab, &model, &thresholds);
         print_param(&model);
         if (model.ncam == 1) {
           c = 0;
         } else {
           for (j=0;j<model.ncam;j++)
             if (model.bands[j] == field_desc[0]->chan_nr) {
                c = j;
                break;
             }
         }
         init_distort_tab(&model);
         make_distort(&model, c);
         save_distort_tab(incat, control.outfile[i], c,
                          "make_distort", argc, argv);
      }
    }
    return 0;
}
