/* included the line
   control_struct control;

   at the very beginning. This was omitted and led
   to a crash

   19.07.01:
   removed compiler warnings (DEC)
   lines in the input ASCII Catalog starting with a '#'
   are considered comments now.

   22.08.03:
   the keys ra and dec are now type double. The accuracy
   of float is not enough.

   05.12.2003:
   corrected a bug in the allocation of the nra array
   (main routine). One element more (ndec+2 instead of ndec+1) 
   has to be allocated.

   27.03.2006:
   I renamed sorting and indexing functions to the new naming scheme
   described in the sort_globals.h file
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#include "fitscat.h"
#include "tabutil_globals.h"
#include "lists_globals.h"
#include "utils_globals.h"
#include "utils_macros.h"
#include "precess_globals.h"
#include "options_globals.h"
#include "make_icat_globals.h"
#include "make_icat_vers.h"
#include "sort_globals.h"

#define BLK_SIZE 1000

control_struct control;

option my_opts[] = {
            {"ID_COLUMN", INT, &control.id_col, 0, 255, 0.0, 0.0,
              1, 0.0, NO, "",
             {"", "", "", "", ""}, 0,
             "Number of the input ascii files column containing the ID"},
            {"PLATE_COLUMN", INT, &control.plate_col, 0, 255, 0.0, 0.0,
              2, 0.0, NO, "",
             {"", "", "", "", ""}, 0,
             "Number of the input ascii files column containing the Plate Number"},
            {"RA_COLUMN", INT, &control.ra_col, 0, 255, 0.0, 0.0,
              3, 0.0, NO, "",
             {"", "", "", "", ""}, 0,
             "Number of the input ascii files column containing the Right Ascension"},
            {"DEC_COLUMN", INT, &control.dec_col, 0, 255, 0.0, 0.0,
              4, 0.0, NO, "",
             {"", "", "", "", ""}, 0,
             "Number of the input ascii files column containing the Declination"},
            {"MAG_COLUMN", INT, &control.mag_col, 0, 255, 0.0, 0.0,
              5, 0.0, NO, "",
             {"", "", "", "", ""}, 0,
             "Number of the input ascii files column containing the Magnitude"},
            {"TILE_WIDTH", FLOAT, &control.tile_width, 0, 0, 0.0, 1.0,
              0, 0.5, NO, "",
             {"", "", "", "", ""}, 0,
             "Size of the tiles in degrees"},
             {""},
            };

void usage(char *me)
{
    fprintf(stderr,"Usage: %s \n",me);
    fprintf(stderr,"                 -i incat\n");
    fprintf(stderr,"                 -o outcat\n");
    fprintf(stderr,"                 -c conffile\n");

    exit(1);
}
 
void init(int argc, char *argv[])
{
   int i, j, k;

   if (argc == 1) usage(argv[0]);
   init_options(0, my_opts);
   for (i=1; i<argc; i++) {
     if (argv[i][0] == '-') {
        switch(tolower((int)argv[i][1])) {
        case 'c': strcpy(control.conffile,argv[++i]);
                  break;
        case '@': print_options(argv[0]);
                  exit(0);
        case 'i': control.ilist = 0;
                  strcpy(control.listfile[control.ilist],argv[++i]);
                  while (i+1<argc && argv[i+1][0] != '-') {
                     strcpy(control.listfile[++control.ilist],argv[++i]);
                  }
                  ++control.ilist;
                  break;
        case 'o': strcpy(control.outfile[0],argv[++i]);
                  break;
        default:  parse_options(control.conffile);
                  j = i++; k = i;
                  set_command_option(argv[j], argv[k]);
                  break;
        }
     } else {
        syserr(WARNING,"init","Command line syntax error\n");
        usage(argv[0]);
     }
  }
  finish_options(control.conffile);
}

/*
 * Global parameters
 */
double	*crval1, *crval2, *cdelt1, *cdelt2, ra_width, *naxis1, *naxis2;
int	*obj_pos, *obj_count, *fpos;
double	*ra, *dec;
float	*mag;
int	*objid, *plate;
int	nobj, nblck = 1, middle; 

int create_icat(int nfields, int nobjs)
{
    catstruct	*outcat;
    tabstruct	*ftab, *otab;

    outcat = new_cat(1);
    init_cat(outcat);
    ftab = new_tab("FIELDS");
    add_key_to_tab(ftab, "OBJECT_POS", nfields, obj_pos, T_LONG, 1, 
                   "","Object position of frame");
    add_key_to_tab(ftab, "OBJECT_COUNT", nfields, obj_count, T_LONG, 1, 
                   "","Number of objects in frame");
    add_key_to_tab(ftab, "CRVAL1", nfields, crval1, T_DOUBLE, 1, 
                   "deg","Right Ascension of plate middle");
    add_key_to_tab(ftab, "CRVAL2", nfields, crval2, T_DOUBLE, 1, 
                   "deg","Declination of plate middle");
    add_key_to_tab(ftab, "CDELT1", nfields, cdelt1, T_DOUBLE, 1, 
                   "deg/pixel","Semi pixel scale in x coordinate");
    add_key_to_tab(ftab, "CDELT2", nfields, cdelt2, T_DOUBLE, 1, 
                   "deg/pixel","Semi pixel scale in y coordinate");
    add_key_to_tab(ftab, "NAXIS1", nfields, naxis1, T_DOUBLE, 1, 
                   "","Semi image width in x coordinate");
    add_key_to_tab(ftab, "NAXIS2", nfields, naxis2, T_DOUBLE, 1, 
                   "","Semi image width in y coordinate");
    update_tab(ftab);
    add_tab(ftab, outcat, 0);
    otab = new_tab("OBJECTS");
    add_key_to_tab(otab, "FIELD_POS", nobjs, fpos, T_LONG, 1, 
                   "", "Frame number");
    add_key_to_tab(otab, "REGNO", nobjs, plate, T_LONG, 1, 
                   "", "Running frame number");
    add_key_to_tab(otab, "ID", nobjs, objid, T_LONG, 1, 
                   "","Object ID number");
    add_key_to_tab(otab, "Ra", nobjs, ra, T_DOUBLE, 1, 
                   "deg","Right Ascension of object");
    add_key_to_tab(otab, "Dec", nobjs, dec, T_DOUBLE, 1, 
                   "deg","Declination of object");
    add_key_to_tab(otab, "Mag", nobjs, mag, T_FLOAT, 1, 
                   "Mag", "Magnitude of object");
    update_tab(otab);
    add_tab(otab, outcat, 0);
    VPRINTF(NORMAL,"Saving catalog %s\n", control.outfile[0]);
    save_cat(outcat, control.outfile[0]);
    return 0;
}

int get_fpos(double ra, double dec, int *nra, double width)
{
    int		i, j, k, fpos;
    double	ra_width;

    i = (int) fabs(dec / width);
    ra_width = 360.0 / (double) nra[i];
    j = (int) (ra / ra_width);
    fpos = 0;
    for (k=0;k<i;k++) fpos += nra[k];
    fpos += j;

    return (fpos);
}

int main(int argc, char *argv[])

/****** make_icat ***************************************************
 *
 *  NAME	
 *      make_icat - Program to create an astrometric input catalog
 *
 *  SYNOPSIS
 *      make_icat -i incat -o outcat 
 *            [-c configfile] [CONFOPTION]
 *
 *  FUNCTION
 *      This program creates an astrometric catalog in the format required
 *      for the preastrom program. The input catalog must be a list of
 *      stars with appropriate information. This is at minimum:
 *        Unique Objects ID  Integer
 *        Ra position        Double
 *        Dec position       Double
 *        Magnitude          Float
 *      The positions do not have to be in the J2000 system, but for preastrom
 *      they are converted to J2000 if necessary.
 *
 *  INPUTS
 *      -i incat       - The name of the input catalog 
 *      -o outcat      - The name of the output  catalog 
 *     [-c configfile] - The name of the configuration file. The default
 *                       name is /denisoft/conf/photom.conf
 *     [CONFOPTION]    - Pairs of configuration VARIABLE VALUE
 *                       sequences that allow dynamically
 *                       overwritting of configuration file values.
 *
 *  RESULTS
 *
 *  RETURNS
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *     E.R. Deul - dd 07-04-1997
 *
 ******************************************************************************/
{ 
    int		i, j;
    char	header[MAXCHAR], line[MAXCHAR], *t;
    int		ndec, *nra, ncount, nfields, nuniq;
    FILE	*fd;

    int		*intwrk, *idxfpos;
    float	*fltwrk;
    double	*dblwrk;

    init(argc, argv);

    sprintf(header,"\n   %s Version: %s (%s)\n\n",NAME,VERS,DATE);
    VPRINTF(NORMAL,header);

    ndec = 180.0 / control.tile_width;
    ED_CALLOC(nra, "make_icat", int, ndec+2);
    ncount = 0;
    for (i=0;i<ndec/2+1;i++) {
       nra[i] = nra[ndec/2+1+i] =
                360.0 * cos(control.tile_width *
                           (0.5 + (float)i) * 3.14159 / 180.0) /
                        control.tile_width;
       ncount+= nra[i];
    }
    ncount *= 2;
    nfields = ncount;
    ED_CALLOC(crval1,    "make_icat", double, ncount+1);
    ED_CALLOC(crval2,    "make_icat", double, ncount+1);
    ED_CALLOC(cdelt1,    "make_icat", double, ncount+1);
    ED_CALLOC(cdelt2,    "make_icat", double, ncount+1);
    ED_CALLOC(naxis1,    "make_icat", double, ncount+1);
    ED_CALLOC(naxis2,    "make_icat", double, ncount+1);
    ED_CALLOC(obj_pos,   "make_icat", int,    ncount+1);
    ED_CALLOC(obj_count, "make_icat", int,    ncount+1);
    middle = ncount/2+1;
    ncount = 0;
    for (i=0;i<ndec/2+1;i++) {
       ra_width = 360.0 / (double) nra[i];
       for (j=0;j<nra[i];j++) {
          crval2[j+ncount] = (0.5 + (double) i) * control.tile_width;
          crval2[j+ncount+middle] = -crval2[j+ncount];
          crval1[j+ncount] = crval1[j+middle+ncount] = 
                             ra_width * (0.5 + (double)j);
          naxis1[j+ncount] = naxis1[j+middle+ncount] = 
          naxis2[j+ncount] = naxis2[j+middle+ncount] = 
                             120.0;
          cdelt1[j+ncount] = cdelt1[j+middle+ncount] = 
          cdelt2[j+ncount] = cdelt2[j+middle+ncount] = 
                             control.tile_width / 120.0;
       }
       ncount += nra[i];
    }
    ED_FOPEN(fd, "make_icat", control.listfile[0], "rt");
    nobj = 0;
    ED_CALLOC(objid, "make_icat", int,    nblck*BLK_SIZE);
    ED_CALLOC(plate, "make_icat", int,    nblck*BLK_SIZE);
    ED_CALLOC(ra,    "make_icat", double,  nblck*BLK_SIZE);
    ED_CALLOC(dec,   "make_icat", double,  nblck*BLK_SIZE);
    ED_CALLOC(mag,   "make_icat", float,  nblck*BLK_SIZE);
    ED_CALLOC(fpos,  "make_icat", int,    nblck*BLK_SIZE);

    while (fgets(line, MAXCHAR, fd) != NULL) 
    {
       /* lines starting with '#' are considered comments */
       if(line[0] != '#')
       {
  	 ncount = 1;
  	 if (nobj == nblck*BLK_SIZE) {
  	    nblck++;
  	    ED_REALLOC(objid, "make_icat", int,    nblck*BLK_SIZE);
  	    ED_REALLOC(plate, "make_icat", int,    nblck*BLK_SIZE);
  	    ED_REALLOC(ra,    "make_icat", double,  nblck*BLK_SIZE);
  	    ED_REALLOC(dec,   "make_icat", double,  nblck*BLK_SIZE);
  	    ED_REALLOC(mag,   "make_icat", float,  nblck*BLK_SIZE);
  	    ED_REALLOC(fpos,  "make_icat", int,    nblck*BLK_SIZE);
  	 }
  	 t = strtok(line, " 	");
  	 while (t != NULL) {
  	    if (control.id_col >= 0) {
  	       if (control.id_col == ncount) 
  		  objid[nobj] = atoi(t);
  	    } else { 
  	       objid[nobj] = nobj;
  	    }
  	    if (control.plate_col >= 0) {
  	       if (control.plate_col == ncount) 
  		  plate[nobj] = atoi(t);
  	    } else { 
  	       plate[nobj] = -1;
  	    }
  	    if (control.ra_col >= 0) {
  	       if (control.ra_col == ncount) 
  		  ra[nobj] = atof(t);
  	    } else {
  	       syserr(FATAL, "make_icat", "No column specified for Ra\n");
  	    }
  	    if (control.dec_col >= 0) {
  	       if (control.dec_col == ncount) 
  		  dec[nobj] = atof(t);
  	    } else {
  	       syserr(FATAL, "make_icat", "No column specified for Dec\n");
  	    }
  	    if (control.mag_col >= 0) {
  	       if (control.mag_col == ncount) 
  		  mag[nobj] = (float) atof(t);
  	    } else {
  	       mag[nobj] = 15.0;
  	    }
  	    t = strtok(NULL, " 	");
  	    ncount++;
  	 }
  	 fpos[nobj] = get_fpos(ra[nobj], dec[nobj], nra, control.tile_width);
  	 if (dec[nobj] < 0.0) fpos[nobj] += middle;
  	 nobj++;
  	 if ((nobj % 100) == 0) {
  	    VPRINTF(NORMAL,"Reading object: %d\n", nobj);
  	 }
       }
    }
    if (nobj == 0)
       syserr(FATAL,"make_icat", "No objects in input catalog!\n");
    VPRINTF(NORMAL,"Sorting objects into tiles\n");
    ED_CALLOC(idxfpos, "make_icat", int, nobj);
    iindex(nobj, fpos, idxfpos);
/*
 *  Now sort the arrays
 */
    ED_CALLOC(dblwrk, "make_icat", double, nobj);
    for (i=0;i<nobj;i++) dblwrk[i] = ra[i];
    for (i=0;i<nobj;i++) ra[i] = dblwrk[idxfpos[i]];
    for (i=0;i<nobj;i++) dblwrk[i] = dec[i];
    for (i=0;i<nobj;i++) dec[i] = dblwrk[idxfpos[i]];
    ED_FREE(dblwrk);
    ED_CALLOC(fltwrk, "make_icat", float, nobj);
    for (i=0;i<nobj;i++) fltwrk[i] = mag[i];
    for (i=0;i<nobj;i++) mag[i] = fltwrk[idxfpos[i]];
    ED_FREE(fltwrk);
    ED_CALLOC(intwrk, "make_icat", int, nobj);
    for (i=0;i<nobj;i++) intwrk[i] = objid[i];
    for (i=0;i<nobj;i++) objid[i] = intwrk[idxfpos[i]];
    for (i=0;i<nobj;i++) intwrk[i] = fpos[i];
    for (i=0;i<nobj;i++) fpos[i] = intwrk[idxfpos[i]];
    if (plate[0] == -1) {
       for (i=0;i<nobj;i++) plate[i] = fpos[i];
    } else {
       for (i=0;i<nobj;i++) intwrk[i] = plate[i];
       for (i=0;i<nobj;i++) plate[i] = intwrk[idxfpos[i]];
    }
    ED_FREE(intwrk);
    for (i=0;i<nobj;i++) obj_count[fpos[i]]++;
    obj_pos[0] = 1;
    for (i=1;i<nfields;i++) {
       obj_pos[i] = obj_pos[i-1] + obj_count[i-1];
    }
    
    ED_FREE(idxfpos);
/*
 *  Select only the frames with objects in them
 */
    nuniq = nobj;
    intwrk = uniq_int(fpos, &nuniq);

    for (i=0;i<nuniq;i++) {
       obj_pos[i]   = obj_pos[intwrk[i]];
       obj_count[i] = obj_count[intwrk[i]];
       crval1[i]    = crval1[intwrk[i]];
       crval2[i]    = crval2[intwrk[i]];
       cdelt1[i]    = cdelt1[intwrk[i]];
       cdelt2[i]    = cdelt2[intwrk[i]];
       naxis1[i]    = naxis1[intwrk[i]];
       naxis2[i]    = naxis2[intwrk[i]];
    }
/*
 *  Update the object frame position pointers 
 *  (plates are sorted, make use of that)
 */
    j = 0;
    for (i=0;i<nobj;i++) {
       if (fpos[i] > intwrk[j]) j++;
       fpos[i] = j+1;
    }
    ED_FREE(intwrk);
    create_icat(nuniq, nobj);
    ED_FREE(nra);
    
    return 0;
}




