#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "utils_globals.h"
#include "thresh_globals.h"
#include "model_globals.h"
#include "astrom_globals.h"

#define STRLEN 9

/* 03.07.01:
   function bicubicproject:
   initialised mfdeg to zero
   (to avoid potential problems if compilers initialise
   variables to random values)
*/


int init_model(int ppdeg, int *ffdeg, int ncam, int nf, model_struct *model)
/*-----------------------------------------------------------------------------
 * DENIS: astrometric reduction
 * Setup the maximum degrees for the polynomial plate coefficient, and compute
 * the resulting number of parameters for each colour channel.
 * Parameters:
 *     PPDEG    INTEGER       Maximum polynomial degree in X and Y
 *     FFDEG(3) INTEGER       Maximum polynomial degree in field number
 ----------------------------------------------------------------------------*/
{
    int		n, nn, i, ip, i1, i2, i3, ic, mfdeg;
    char	**xname,**yname,**fname;
    char	ch1[STRLEN], ch2[STRLEN], ch3[STRLEN];
/*
 *  Initialize the text strings
 */
    model->ncam = ncam;
    model->nf   = nf;
    model->pdeg = ppdeg;
    for (i=0,mfdeg=0;i<3;i++) {
      model->ffdeg[i] = ffdeg[i];
      mfdeg = MAX(ffdeg[i], mfdeg);
    }
    ED_CALLOC(xname, "init_model", char *, (ppdeg+1));
    ED_CALLOC(yname, "init_model", char *, (ppdeg+1));
    ED_CALLOC(fname, "init_model", char *, (mfdeg+1));
    for (i=0;i<=ppdeg;i++) {
      ED_CALLOC(xname[i], "init_model", char, STRLEN);
      ED_CALLOC(yname[i], "init_model", char, STRLEN);
    }
    for (i=0;i<=mfdeg;i++) 
       ED_CALLOC(fname[i], "init_model", char, STRLEN);
/*
 *  Initialize arrays
 */
    ED_CALLOC(model->bands,  "init_model", int, ncam);
    ED_CALLOC(model->parnum, "init_model", int, ncam);
    ED_CALLOC(model->paroff, "init_model", int, ncam);
/*
 *  Build names for monomial factors
 */
    strcpy(xname[0], "1");
    strcpy(xname[1], "X");
    strcpy(yname[0], "1");
    strcpy(yname[1], "Y");
    for (i=2;i<=ppdeg;i++) {
       sprintf(xname[i],"X**%d",i);
       sprintf(yname[i],"Y**%d",i);
    }
    for (i=0;i<=mfdeg;i++) 
      sprintf(fname[i], "Cheb%d(F)",i);
    for (ic=0,nn=0;ic<model->ncam;ic++) {
      for (ip=0,n=0;ip<=ppdeg;ip++) 
        for (i1=ip,i2=0;i2<=ip;i1--,i2++) 
          for (i3=0;i3<=ffdeg[MIN((i1+i2),2)];i3++) 
            if (i1+i2 <= ppdeg) 
              if (ic > 0 || (i1 > 0 || i2 > 0)) {
                n++;
                nn++;
              }
      model->parnum[ic] = n;
    }
    ED_CALLOC(model->names, "init_model", char *, nn);
    for (i=0;i<nn;i++) 
      ED_CALLOC((model->names)[i], "init_model", char, MAXCHAR);
    for (ic=0,nn=0;ic<model->ncam;ic++) 
      for (ip=0;ip<=ppdeg;ip++) 
        for (i1=ip,i2=0;i2<=ip;i1--,i2++) 
          for (i3=0;i3<=ffdeg[MIN((i1+i2),2)];i3++) 
            if (i1+i2 <= ppdeg) 
              if (ic > 0 || (i1 > 0 || i2 > 0)) {
                strcpy(ch3, fname[i3]);
                strcpy(ch2, yname[i2]);
                strcpy(ch1, xname[i1]);
                sprintf(model->names[nn],"%s*%s*%s",ch1,ch2,ch3);
                nn++;
              }
/*
 *
 */
    for (ic=0,model->np=0;ic<model->ncam;ic++) {
      model->paroff[ic] = model->np;
      model->np += model->parnum[ic];
    }
    for (i=0;i<=ppdeg;i++) {
      ED_FREE(xname[i]);
      ED_FREE(yname[i]);
    }
    for (i=0;i<=mfdeg;i++)
      ED_FREE(fname[i]);
    ED_FREE(xname);
    ED_FREE(yname);
    ED_FREE(fname);
/*
 *  Allocate space for the data arrays
 */
    ED_CALLOC(model->alpha,   "init_model", float *,   (model->np + model->nf));
    ED_CALLOC(model->beta,    "init_model", float *,   2);
    ED_CALLOC(model->params,  "init_model", float *,   2);
    ED_CALLOC(model->centers, "init_model", double **, 2);
    ED_CALLOC(model->crpix,   "init_model", float **,  2);
    for (i=0;i<(model->np + model->nf);i++) 
      ED_CALLOC(model->alpha[i], "init_model", float, (model->np + model->nf));
    for (i=0;i<2;i++) {
      ED_CALLOC(model->beta[i],   "init_model", float, (model->np + model->nf));
      ED_CALLOC(model->params[i], "init_model", float, (model->np + model->nf));
    }
    for (i=0;i<2;i++) {
      ED_CALLOC(model->centers[i], "init_model", double *, (model->ncam));
      ED_CALLOC(model->crpix[i],   "init_model", float *,  (model->ncam));
      for (ic=0;ic<model->ncam;ic++) {
        ED_CALLOC(model->centers[i][ic], "init_model", double, (model->nf));
        ED_CALLOC(model->crpix[i][ic],   "init_model", float,  (model->nf));
      }
    }
    return TRUE;
}
/**/
void bi_cubic(int f, int c, float x, float y, model_struct *model, float *p)
/*-----------------------------------------------------------------------------
 * DENIS: astrometric reduction
 * Compute the value of all components of the plate model at one (X,Y,F) 
 * position for a given colour band.
 * Parameters:
 *     F      Integer       Field number
 *     IB     Integer       Colour band number
 *     X      Real*4        First pixel coordinate
 *     Y      Real*4        Second pixel coordinate       
 *     P      Real*4(*)     Values of all functions in plate model
 *---------------------------------------------------------------------------*/
{
    float	*xx, *yy, *ff;
    float	xscaled, yscaled, fscaled;
    int		i, ip, i1, i2, i3, n, mfdeg;
/*
 * Compute polynomial factors once
 * For a better numerical conditioning, pixel coordinates and field number 
 * are rescaled from [0,NPIX] and [1,NFIELD] to [-1,1].
 * Make use of the true reference center positions from the header
 */
    for (i=0,mfdeg=0;i<3;i++) 
      mfdeg = MAX(model->ffdeg[i], mfdeg);
    ED_CALLOC(xx, "bi_cubic", float, (model->pdeg + 1));
    ED_CALLOC(yy, "bi_cubic", float, (model->pdeg + 1));
    ED_CALLOC(ff, "bi_cubic", float, (mfdeg + 1));
    xscaled = ((x - model->crpix[0][c][f])/((float) model->mpix)) * 2.0;
    for (xx[0] = 1.0, i=0;i<model->pdeg;i++) 
      xx[i+1] = xx[i] * xscaled;
    yscaled = ((y - model->crpix[1][c][f])/((float) model->mpix)) * 2.0;
    for (yy[0] = 1.0, i=0;i<model->pdeg;i++) 
      yy[i+1] = yy[i] * yscaled;
/*     
 * Rescale field number to [-1,+1] for better conditioning
 */
    if (model->mf > 1) {
      fscaled = (f - 1.0) / (model->mf - 1.0) * 2.0 - 1.0;
    } else {
/*
 *    Don't divide by zero...
 */
      fscaled = 0;
    }
/*
 *  High degrees are sometimes needed for field number. Use Chebychev 
 *  polynomials for better stability
 */
    ff[0] = 1.0;
    if (mfdeg > 0) {
      ff[1] = fscaled;
      if (mfdeg > 1) 
        for (i=2;i<mfdeg+1;i++)
          ff[i] = 2.0 * ff[i-1] * fscaled - ff[i-2];
    }
/*
 *  Set a limit on degree of the homogeneous XY polynomial, and suppress 
 *  constant for the first colour band.
 *  Due to the coarse encoder step of the 1m telescope, the translation term is 
 *  truly random at the target accuracy. It must be determined for each field
 *  Caution, this produces a non intuitive ordering of the parameters (F, Y,
 *  then X). It doesn't really matter as long as this is done consistently, 
 *  but is potentially surprising when looking at them.
 */
    for (ip=0,n=0;ip<=model->pdeg;ip++) 
      for (i1=ip,i2=0;i2<=ip;i1--,i2++) 
        for (i3=0;i3<=model->ffdeg[MIN((i1+i2),2)];i3++) 
          if (i1+i2 <= model->pdeg) 
            if (c > 0 || (i1 > 0 || i2 > 0)) {
              p[n] = xx[i1] * yy[i2] * ff[i3];
              n++;
            }
    ED_FREE(ff);
    ED_FREE(yy);
    ED_FREE(xx);
}
/**/
void bi_cubicgrad(int f, int c, float x, float y, model_struct *model, 
                  float **p)
/*-----------------------------------------------------------------------------
 * DENIS: astrometric reduction
 * Compute the gradient (relative to pixel positions) of all components of the 
 * plate model at one (X,Y,F) 
 * position for a given colour band.
 * Parameters:
 *     F      Integer       Field number
 *     IB     Integer       Colour band number
 *     X      Real*4        First pixel coordinate
 *     Y      Real*4        Second pixel coordinate       
 *     P      Real*4(2,*)   Values of all gradients in plate model
 *---------------------------------------------------------------------------*/
{
    float	*xx, *yy, *gx, *gy, *ff;
    float	xscaled, yscaled, fscaled;
    int		i, ip, i1, i2, i3, n, mfdeg;

    for (i=0,mfdeg=0;i<3;i++) 
      mfdeg = MAX(model->ffdeg[i], mfdeg);
    ED_CALLOC(xx, "bi_cubicgrad", float, (model->pdeg + 1));
    ED_CALLOC(yy, "bi_cubicgrad", float, (model->pdeg + 1));
    ED_CALLOC(gx, "bi_cubicgrad", float, (model->pdeg + 1));
    ED_CALLOC(gy, "bi_cubicgrad", float, (model->pdeg + 1));
    ED_CALLOC(ff, "bi_cubicgrad", float, (mfdeg + 1));
/*
 * Compute polynomial factors once
 */
    xscaled = ((x - model->crpix[0][c][f])/((float) model->mpix)) * 2.0;
    for (i=0,xx[0] = 1.0, gx[0] = 0.0; i<model->pdeg; i++) {
      gx[i+1] = xx[i] * (float) (i+1);
      xx[i+1] = xx[i] * xscaled;
    }
    yscaled = ((y - model->crpix[1][c][f])/((float) model->mpix)) * 2.0;
    for (i=0,yy[0] = 1.0, gy[0] = 0.0; i<model->pdeg; i++) {
      gy[i+1] = yy[i] * (float) (i+1);
      yy[i+1] = yy[i] * yscaled;
    }
/*
 * Field number polynomial. No need for gradient (no error on field number...).
 * A Chebychev polynomial is used for better stability.
 * Actual number of fields would be better. Add this to argument list?
 */
    if (model->mf> 1) {
      fscaled = (f - 1.0) / (model->mf - 1.0) * 2.0 - 1.0;
    } else {
/*
 *    Don't divide by zero...
 */
      fscaled = 0;
    }
    ff[0] = 1.0;
    if (mfdeg > 0) 
    {
      ff[1] = fscaled;
      if (mfdeg > 1) 
      {
        for (i=2;i<mfdeg+1;i++)
	{
          ff[i] = 2.0 * ff[i-1] * fscaled - ff[i-2];
	}
      }
    }
/*
 *  Set a limit on degree of the homogeneous XY polynomial, and suppress 
 *  constant for the first colour band.
 *  Due to the coarse encoder step of the 1m telescope, the translation term is 
 *  truly random at the target accuracy. It must be determined for each field
 */
    for (ip=0,n=0;ip<=model->pdeg;ip++) 
    {
      for (i1=ip,i2=0;i2<=ip;i1--,i2++) 
      {
        for (i3=0;i3<=model->ffdeg[MIN((i1+i2),2)];i3++) 
	{
          if (i1+i2 <= model->pdeg) 
	  {
            if (i1 > 0 || i2 > 0) 
	    {
              if (i1+i2 <= model->pdeg) 
              {
                p[0][n] = gx[i1] * yy[i2] * ff[i3] / model->mpix * 2.0 ;
                p[1][n] = xx[i1] * gy[i2] * ff[i3] / model->mpix * 2.0 ;
                n++;
              } 
              else 
              {
                if (c > 0) 
                {
                  p[0][n] = p[1][n] = 0.0;
                  n++;
                }
              }
	    }
	  }
	}
      }
    }
   ED_FREE(ff);
   ED_FREE(yy);
   ED_FREE(xx);
   ED_FREE(gy);
   ED_FREE(gx);
}
/**/
void bi_cubicproject(int f, int c, float **fldpol, int *num, 
                     model_struct *model)
/*--------------------------------------------------------------------------
 * DENIS: astrometric reduction
 * Compute a polynomial astrometric model for one specific field one which
 * is valid for the full strip
 * Parameters:
 *     F      Integer       Field number
 *     C      Integer       Color band number
 *     FLDPOL Real*4(2,*)   Coefficients of the field astrometric solution
 *--------------------------------------------------------------------------*/
{
    char	**xname,**yname, ch1[STRLEN], ch2[STRLEN];
    int		i, j, k, l, ip, i1, i2, i3, ic, n, mfdeg=0;
    float	fscaled, *ff;

    for (i=0,mfdeg=0;i<3;i++) 
      mfdeg = MAX(model->ffdeg[i], mfdeg);
    ED_CALLOC(ff, "bi_cubicproject", float, (mfdeg + 1));
    ED_CALLOC(xname, "bi_cubicproject", char *, (model->pdeg+1));
    ED_CALLOC(yname, "bi_cubicproject", char *, (model->pdeg+1));
    for (i=0;i<=model->pdeg;i++) {
       ED_CALLOC(xname[i], "bi_cubicproject", char, STRLEN);
       ED_CALLOC(yname[i], "bi_cubicproject", char, STRLEN);
    }
/*
 *  Build names for the nominal factor (on a single frame basis)
 */
    strcpy(xname[0], "1");
    strcpy(xname[1], "X");
    strcpy(yname[0], "1");
    strcpy(yname[1], "Y");
    for (i=2;i<=model->pdeg;i++) {
       sprintf(xname[i],"X**%d",i);
       sprintf(yname[i],"Y**%d",i);
    }
    ic = c;
    for (ip=0,n=0;ip<=model->pdeg;ip++) 
      for (i1=ip,i2=0;i2<=ip;i1--,i2++) 
          if (i1+i2 <= model->pdeg) 
            if (ic > 0 || (i1 > 0 || i2 > 0)) {
              strcpy(ch2, yname[i2]);
              strcpy(ch1, xname[i1]);
              sprintf(model->names[n],"%s*%s",ch1,ch2);
              n++;
            }
   *num = n;
/*     
 * Rescale field number to [-1,+1] for better conditioning
 */
    if (model->mf > 1) {
      fscaled = (f - 1.0) / (model->mf - 1.0) * 2.0 - 1.0;
    } else {
/*
 *    Don't divide by zero...
 */
      fscaled = 0;
    }
/*
 *  High degrees are sometimes needed for field number. Use Chebychev 
 *  polynomials for better stability
 */
    ff[0] = 1.0;
    if (mfdeg > 0) {
      ff[1] = fscaled;
      if (mfdeg > 1) 
        for (i=2;i<mfdeg+1;i++)
          ff[i] = 2.0 * ff[i-1] * fscaled - ff[i-2];
    }
/*
 * Compress the strip-wide coefficient along the field index. Slightly tricky
 * because of the not totally intuitive coefficient ordering.
 */
    for (ip=0,j=0,k=0;ip<=model->pdeg;ip++) 
      for (i1=ip,i2=0;i2<=ip;i1--,i2++) 
          if (i1+i2 <= model->pdeg) 
            if (c > 0 || (i1 > 0 || i2 > 0)) {
              fldpol[0][k] = 0.0;
              fldpol[1][k] = 0.0;
              for (i3=0;i3<=model->ffdeg[MIN((i1+i2),2)];i3++) {
                fldpol[0][k] += model->params[0][j+model->paroff[c]] * ff[i3];
                fldpol[1][k] += model->params[1][j+model->paroff[c]] * ff[i3];
                j++;
              }
/*
 *            Scale the parameters back to true polynomials
 */
              for (l=0;l<ip;l++) {
                fldpol[0][k] *= (2.0 / model->mpix);
                fldpol[1][k] *= (2.0 / model->mpix);
              }
              k++;
            }
    if (k > model->np)
       syserr(FATAL, "bi_cubicproject", "Number of paramaters too large");
    for (i=0;i<=model->pdeg;i++) {
      ED_FREE(xname[i]);
      ED_FREE(yname[i]);
    }
    ED_FREE(xname);
    ED_FREE(yname);
    ED_FREE(ff);
}
