#include <stdio.h>
#include <stdlib.h>
#include "utils_macros.h"
#include "thresh_globals.h"
#include "model_types.h"
#include "options_globals.h"
#include "astrom_io_globals.h"
#include "stat_globals.h"

/**/
void make_statistics(model_struct *model, thresh_struct *thresholds, 
                     int reject)
{
/*--------------------------------------------------------------------------
 * DENIS astrometric package:
 * Use the fitted plate coefficients and their covariance matrix to compute
 * positions and standard errors of detected stars, and produce usefull
 * statistics.
 *--------------------------------------------------------------------------
 */
    reflist_struct      *refpairs;
    overlist_struct     *overpairs;
/*
 *  There are two types of data to be processed:
 *  pairing information between extracted objects and reference stars and
 *  pairing information between extracted objects in the catalog
 *  Let's run through them both and build the design matrices on the way
 *
 *  First get the reference pairs
 */
    init_thresh_stat(thresholds);
    refpairs = new_refpairs();
    while (get_refpairs(refpairs, thresholds, model, reject, TRUE, FALSE));
    free_refpairs(refpairs);
/*
 *  Then get all the internal pairing (inter catalog and cross catalog)
 */       
    overpairs = new_overpairs();
    while (get_overpairs(overpairs, thresholds, model, reject, TRUE, FALSE));
    free_overpairs(overpairs);
    calc_thresholds(thresholds);
    finish_thresh_stat(thresholds);
}

void make_residuals(char *outfile, option *my_opts, char *header,
                    model_struct *model, thresh_struct *thresholds)
{
    reflist_struct      *refpairs;
    overlist_struct     *overpairs;
    int			nref, nover;

    init_residuals(outfile, my_opts, header);
/*
 *  First get the reference pairs
 */
    refpairs = new_refpairs();
    nref = 0;
    while (get_refpairs(refpairs, thresholds, model, TRUE, FALSE, TRUE))
      nref++;
    free_refpairs(refpairs);
    VPRINTF(VERBOSE, "Number of reference stars: %d\n", nref);
/*
 *  Then get all the internal pairing (inter catalog and cross catalog)
 */ 
    overpairs = new_overpairs();
    nover = 0;
    while (get_overpairs(overpairs, thresholds, model, TRUE, FALSE, TRUE))
      nover++;
    free_overpairs(overpairs);
    VPRINTF(VERBOSE, "Number of overlap stars: %d\n", nover);
    finish_residuals();
}
