/* History:
   13.01.2000:
   nr_free_vector -> free_vector

   09.03.2006:
   I removed the last argument from the NR free_... functions.
   That argument is not used in the functions.
*/

#include <stdio.h>
#include <math.h>
#include "svd_solve.h"
#include "utils_globals.h"

void svdcmp(float **, int, int, float *, float **);
void svbksb(float **, float *, float **, int, int, float *, float *);

void solve(float **A, int nparms, int nobs, float **B, float **X, int nb)
{
    int		i,j;
    float	*b,*x;
    float	**U,*W,**V,wmin,wmax;
/*
 *  Prepare decomposition
 */
    U      = matrix(1,nobs,1,nparms);
    W      = vector(1,nparms);
    V      = matrix(1,nparms,1,nparms);
    for (i=1;i<nobs+1;i++) {
       for (j=1;j<nparms+1;j++) {
          U[i][j] = A[j-1][i-1];
       }
    }
/*
 *  Single Value Decompose the A matrix
 */
    svdcmp(U,nobs,nparms,W,V);
/*
 *  Check for representable dynamic range
 */
    wmax=0.0;
    for (i=1;i<nparms+1;i++) {
       if (W[i]>wmax) wmax=W[i];
    }
    wmin=wmax*1.0e-6;
    for (i=1;i<nparms+1;i++) {
       if (W[i]<wmin) W[i]=0.0;
    }
/* 
 *  Solve the b = Ax problems
 */
    b = vector(1,nobs);
    x = vector(1,nobs);
    for (i=0;i<nb;i++) {
       for (j=1;j<nobs+1;j++)
          b[j] = B[i][j-1];
       svbksb(U,W,V,nobs,nparms,b,x);
       for (j=1;j<nobs+1;j++)
          X[i][j-1] = x[j];
    }
/*
 *  finish up
 */
    free_vector(b,1);
    free_vector(x,1);
    free_matrix(U,1,nobs,1);
    free_vector(W,1);
    free_matrix(V,1,nparms,1);
    return;
}
