#include <string.h>
#include <stdlib.h>
#include "flips.h"

/* 21.11.00:
   added the #include <string.h> statement

   10.03.01:
   removed all compiler warnings under DEC

   25.04.03:
   when writing int keywords to FITS they images,
   they are now correctly aligned to the right
   margin of the text field.

   29.01.2005:
   the write_FITS_header_line routine now deals with
   COMMENT and HISTORY keywords corerectly.

   22.05.2005:
   function write_FITS_header_float:
   The keyword is now right justified and occupies 20
   characters.
*/
/*----------------------------------------------------------------------------*/
int read_FITS_header_int(line)
char line[];
{
int value;
int i;

   for(i=0;i<71;i++)
      line[i]=line[i+9];
   sscanf(line,"%d",&value);

   return(value);
}
/*----------------------------------------------------------------------------*/
float read_FITS_header_float(line)
char line[];
{
float value;
int i;

   for(i=0;i<71;i++)
      line[i]=line[i+9];
   sscanf(line,"%f",&value);

   return(value);
}
/*----------------------------------------------------------------------------*/
void write_FITS_header_line(OUT,name,text)
FILE *OUT;
char name[];
char text[];
{
   char line[81];
   int i,j;
   int posoff=10; /* starting point for values if the
                  keyword is not COMMENT or HISTORY */

   memset(line, ' ', 80);
   line[80]='\0';

   /* write parameter name */
   i=0;
   while(name[i]!='\0' && i<8) 
   {
      line[i]=name[i];
      i++;
   }
   while(i<8) line[i++]=' ';
   /* deal with HISTORY and COMMENT lines */
   if(strncmp(name, "HISTORY",7)==0 || strncmp(name, "COMMENT",7)==0)
   {
      posoff=8;
   }
   else
   {
     line[8]='=';
     line[9]=' ';
   }

   /* write value */
   i=strlen(text);
   j=0;
   while(j<i) 
   {
      line[posoff+j]=text[j];
      j++;
   }
   fprintf(OUT,line);
}

/*----------------------------------------------------------------------------*/
void write_FITS_header_END(OUT)
FILE *OUT;
{
char line[81];
int i;

      strcpy(line,"END");
      i=3;
      while(i<80) line[i++]=' ';
      line[80]='\0';
      fprintf(OUT,line);
}
/*----------------------------------------------------------------------------*/
void write_FITS_header_int(OUT,name,param)
FILE *OUT;
char name[];
int param;
{
char line[81];
char word[81];
int i,j;
   
   /* write parameter name */
   i=0;
   while(name[i]!='\0') {
      line[i]=name[i];
      i++;
   }
   while(i<8) line[i++]=' ';
   line[8]='=';
   line[9]=' ';

   /* write value */
   sprintf(word,"%20d\n",param);
   j=0;
   while(word[j]!='\0') {
      line[10+j]=word[j];
      j++;
   }

   i=j+9;
   while(i<80) line[i++]=' ';
   line[80]='\0';
   fprintf(OUT,line);
}
/*----------------------------------------------------------------------------*/
void write_FITS_header_float(OUT,name,param)
FILE *OUT;
char name[];
float param;
{
char line[81];
char word[81];
int i,j;

   /* write parameter name */
   i=0;
   while(name[i]!='\0') 
   {
      line[i]=name[i];
      i++;
   }
   while(i<8) line[i++]=' ';
   line[8]='=';
   line[9]=' ';

   /* write value */
   sprintf(word,"%20f\n",param);
   j=0;
   while(word[j]!='\0') 
   {
      line[10+j]=word[j];
      j++;
   }

   i=j+9;
   while(i<80) line[i++]=' ';
   line[80]='\0';
   fprintf(OUT,line);
}
/*----------------------------------------------------------------------------*/
struct IMAGE read_FITS_header(input)
struct IMAGE input;
{
FILE *IN,*open_file();
int cmpt;
char line[81];

int read_FITS_header_int();
float read_FITS_header_float();

   IN=open_file(input.name,"r");
   cmpt=0;

   input.bitsign=1; /* default */
   input.scale=1.0;
   input.offset=0.0;

   do { /* scan the whole FITS header */
      fread_80(IN,line);

      if(!strcpr("BITPIX",line))
      {
	 input.bitpix=read_FITS_header_int(line);
	 input.bytepix=(int)(fabs(input.bitpix)/8.);
      }
      else if(!strcpr("NAXIS1",line))
	 input.nx=read_FITS_header_int(line);

      else if(!strcpr("NAXIS2",line))
	 input.ny=read_FITS_header_int(line);

      else if(!strcpr("BSCALE",line))
      {
	 input.scale=read_FITS_header_float(line);
      }

      else if(!strcpr("BZERO",line))
      {
	 input.offset=read_FITS_header_float(line);
      }

      else if(!strcpr("BITSIGN",line))
	 input.bitsign=read_FITS_header_int(line);

      cmpt++;
   }
   while(strcpr("END",line));

   /* number of blocks in the FITS header (2880 bytes = 80*36) */
   input.nblk=cmpt/36;
   if(cmpt%36)
	input.nblk++;

   input.startdata=blocksize*input.nblk; /* position of the first data */

   fclose(IN);

   return(input);
}

/*----------------------------------------------------------------------------*/
struct IMAGE write_FITS_header(input,output)
struct IMAGE input;
struct IMAGE output;
{
FILE *IN,*OUT,*open_file();
char line[81];
int cmpt,i;

   IN =open_file(input.name, "r");
   OUT=open_file(output.name,"w");

   cmpt=0;

   do 
   {
      fread_80(IN,line);

      if(!strcpr("BITPIX",line)) 
      {
         write_FITS_header_int(OUT,"BITPIX",output.bitpix);
      }

      else if(!strcpr("NAXIS1",line))
	 write_FITS_header_int(OUT,"NAXIS1",output.nx);

      else if(!strcpr("NAXIS2",line))
      {
	 write_FITS_header_int(OUT,"NAXIS2",output.ny);
	 /* as NAXIS2 must be in the header we can write
            BSCALE and BZERO after it. If they were in the 
            original image they are ignored later
	 */
	 write_FITS_header_float(OUT,"BSCALE",output.scale);
	 write_FITS_header_float(OUT,"BZERO",output.offset);
	 cmpt+=2;
      }

      else if(!strcpr("BSCALE",line))
	{cmpt--;}	

      else if(!strcpr("BZERO",line))
	{cmpt--;}

      else
	 fprintf(OUT,"%s",line);

      cmpt++;
   }
   while(strcpr("END",line));

   fclose(IN);

   /* number of blocks in the FITS header (2880 bytes = 80*36) */
   /* the following is necessary as with a possible insertion
      of BZERO, BSCALE header keywords, the nblk may be shifted
   */
   output.nblk=cmpt/36;
   if(cmpt%36)
   {
	output.nblk++;
   }

   output.startdata=blocksize*output.nblk; /* position of the first data */

   cmpt%=36;
   if(cmpt) 
   {
      for(;cmpt<36;cmpt++) 
      {
         for(i=0;i<80;i++) 
            fprintf(OUT," ");
      }
   }
   fclose(OUT);

   return(output);
}

/*----------------------------------------------------------------------------*/
struct IMAGE create_FITS_header(output)
struct IMAGE output;
{
FILE *OUT,*open_file();
int cmpt,i;
float buf;

   OUT=open_file(output.name,"w");
    
   cmpt=0;

   write_FITS_header_line(OUT,"SIMPLE","T");
   cmpt++;

   write_FITS_header_int(OUT,"BITPIX",16);
   cmpt++;

   write_FITS_header_int(OUT,"NAXIS",2);
   cmpt++;

   write_FITS_header_int(OUT,"NAXIS1",output.nx);
   cmpt++;

   write_FITS_header_int(OUT,"NAXIS2",output.ny);
   cmpt++;

   buf=1;
   write_FITS_header_float(OUT,"BSCALE",buf);
   cmpt++;

   buf=0;
   write_FITS_header_float(OUT,"BZERO",buf);
   cmpt++;

   write_FITS_header_END(OUT);
   cmpt++;

   output.nblk=cmpt/36;
   cmpt%=36;
   if(cmpt) {
      for(;cmpt<36;cmpt++) {
         for(i=0;i<80;i++)
            fprintf(OUT," ");
      }
      output.nblk++;
   }
   output.startdata=blocksize*output.nblk;

   fclose(OUT);

   return(output);
}

/*----------------------------------------------------------------------------*/
struct IMAGE add_FITS_header_text(name,text,output)
char name[];
char text[];
struct IMAGE output;
{
FILE *OUT,*open_file();
char line[81];
int cmpt,i;
  
   if(text[0]!='\0') { /* must not be an empty string */

      OUT=open_file(output.name,"r+w");

      cmpt=0;
      do {
         fread_80(OUT,line);
         cmpt++;
      }
      while(strcpr("END",line));

      fseek(OUT,-80,1);
      write_FITS_header_line(OUT,name,text);
      write_FITS_header_END(OUT);

      /* check the header size, add one block if necessary */
      cmpt++; /* we have one more now */
      output.nblk=cmpt/36;
      cmpt%=36;
      if(cmpt) {
         for(;cmpt<36;cmpt++) {
            for(i=0;i<80;i++)
               fprintf(OUT," ");
         }
         output.nblk++;
      }

      /* update position of first data */
      output.startdata=blocksize*output.nblk;

      fclose(OUT);
   }

   return(output);
}
