/* History:
  16.07.00:
  added imagefile pointer to the image structure

  17.07.00:
  added bitsign field to IMAGE structure.

  01.08.00:
  added stuff for fringing

  10.03.01:
  removed all compiler warnings under DEC

  25.07.03:
  changed in OVERSCAN struct the member 'mod2d'
  to 'modline' and added the parameter 'minreject'
  and 'maxreject'.

  23.01.05:
  I removed the not needed CPU struct.

  05.02.05:
  I introduced the MAXCHAR macro giving the
  maximum length of character strings used in
  FLIPS.

  09.09.2005:
  I included code for getting the basename of a file (including
  the path). The standard C library function did not compile under
  IRIX64.
*/

#include <stdio.h>
#include <math.h>
#include <time.h>

#define blocksize 2880
#define null 0
#define dynamic 65535
#define hdynamic 32767

/* The maximum number of characters allowed in strings */
#ifndef MAXCHAR
#define MAXCHAR 255
#endif

#define DARKTAB_NMAX	20
#define PIXTYPE         float   /* we work internally in this pixel type */
#define DATA_BUFSIZE    262144

#ifdef INT64
typedef int             LONG;
typedef unsigned int    ULONG;
#else
typedef long            LONG;
typedef unsigned long   ULONG;
#endif

struct IMAGE 
{
	char  name[MAXCHAR];
	char  comment[MAXCHAR];
	int   bitpix;	 
	int   bytepix;
        int   bitsign;   /* 1 for signed, 0 for unsigned values (8 and 16 bit images) */
	int   nx;
	int   ny;
	int   nblk;
        int   domodescale; /* flag for imcombine */
        int   doover;    /* flag for imred */
        int   dodark;    /* flag for imred */
        int   doflat;    /* flag for imred */
	float flatsubst; /* values in a flat whose modulus is smaller than this will
                            be substituted by 1 */
        int   domask;    /* flag for imred */
	int   dofringe;  /* flag for imred */
        int   startdata; /* position of first pixel in FITS file */
        int   rescale;   /* rescale input images in imcombine */
	int   fulldyn;   /* change FITS header OFFSET for unsigned 16 bits */
	int   x;	 /* coordinates */
	int   y;	 /* coordinates */
	float dyn_min;	 /* CCD min dynamic in ADUs */
	float dyn_max;	 /* CCD max dynamic in ADUs */
	int   flip_x;	 /* flipping along X axis */
	int   flip_y;	 /* flipping along Y axis */
        float mode;      /* mode of output image in imcombine */
        float fact;      /* multiplicative factor for output images */
	float fringefact;
        float scale;
        float offset;
	FILE *imagefile; /* File pointer through that the image is accessed */
};

struct LIST {
        int  nfile;
        char name[MAXCHAR];
};

struct RASTER {
	int x;
	int y;
	int xo;
	int yo;
	int xc;
	int yc;
	int sizex;
	int sizey;
	int xmin;
	int ymin;
	int xmax;
	int ymax;
        int cut;
        int center;
	int full;
};

struct BIN {
        int x;
        int y;
};

struct OVERSCAN {
	int modline;
        int xmin;
        int xmax;
        int minreject;
        int maxreject;
	float median;
};

struct STAT {
	int percent;
};

struct CLIPPING {
        int   ccdclip;
        int   median;
        float noise;
        float gain;
        float e_rqe;
	float threshold;
        float lcut;
        float hcut;
	int hrej;
        float lsigmas;
        float hsigmas;
	float rejectsigma; /* combination: pixels higher/lower than the mode of the
                              image +/- rejectsigma*sigma are not used in combinations
                           */
};

struct VERBOSE {
        int verbose;
	int x;
	int y;
};

struct BUFFER {
       int height;
};

struct OPERATION {
       char operateur;
       int image;
       float constant;
       float constantsave;
};

struct DARKTAB {
       int   file;                  /* file (list) or a table of number(s) */
       char  darkadu[MAXCHAR];      /* the string for table of values */
       float darkval[DARKTAB_NMAX]; /* table for values form darkadu string */
       int   nelt;		    /* number of values in table */
};

double random_double();

/* function declarations */
int strcpr(char a[], char b[]);
void fread_80(FILE *file, char line[]);
void give_date(char date[]);
void get_filename(char path[], char name[]);
void get_notab(char in[], char out[]);
void write_file (struct IMAGE *file, PIXTYPE *buf, unsigned int size);
void read_file (struct IMAGE *file, PIXTYPE *buf, unsigned int size);
int fread_header(FILE *file, char header[]);
void sort(unsigned long n, PIXTYPE arr[]);
void in_line(FILE *IN, char param[], char name[]);
void in_string(FILE *IN, char param[], char name[]);
char * get_basename(const char *filename);
