#include <string.h>
#include <stdlib.h>
#include "imred.h"
#include "global_vers.h"

/* 
   28.08.00:
   included cutting of input images after reduction

   24.04.03:
   included the stdlib library to avoid warnings due to the
   undefined 'exit' function.

   25.04.03:
   fixed bugs concerning padding bytes (to fill up blocks
   of 2880 bytes) to the final output image.

   25.07.03:
   The old member mod2d in the ovscan region was substituted
   by the new modline.

   29.01.2005:
   I added Global Version number information to output
   FITS headers

   10.02.2005:
   I added sanity checks on the final size of the images
   if 'cutting' is activated.
*/
/*----------------------------------------------------------------------------*/
void fitsopen()
{
extern struct IMAGE input,output,dark,flat,mask;
extern struct RASTER raster;
FILE *open_file();

struct IMAGE read_FITS_header(),write_FITS_header(),add_FITS_header_text();

char text[200],buf[200];

   /* load header from input file */
   input=read_FITS_header(input);

   /* keep the same size */
   if(raster.cut==0)
   {
     output.nx=input.nx;
     output.ny=input.ny;
     raster.xmin=0;
     raster.ymin=0;
   }
   else
   {
     /* sanity checks for the raster size */
     if(raster.sizex + raster.x > input.nx)
     {
       raster.sizex = input.nx - raster.x;
       printf("\nWarning: Reset 'sizex' to maximum value of %d.\n\n", 
              raster.sizex);
     }

     if(raster.sizey + raster.y > input.ny)
     {
       raster.sizey = input.ny - raster.y;
       printf("\nWarning: Reset 'sizey' to maximum value of %d.\n\n", 
              raster.sizey);
     }

     output.nx=raster.sizex;
     output.ny=raster.sizey;
   }

   /* copy header from input to output and add comment */
   if(output.bitpix==0)
   {
     output.offset=input.offset;
     output.scale=input.scale;
     output.bitpix=input.bitpix;
     output.bytepix=input.bytepix;
   }
   if(output.bitpix==16)
   {
     output.offset=32768.0;
     output.scale=1.0;
   }
   if(output.bitpix==-32)
   {
     output.offset=0.0;
     output.scale=1.0;
   }

   output=write_FITS_header(input,output);

   /* put HISTORY information in FITS header */
   output=add_FITS_header_text("HISTORY"," ",output);
   sprintf(text,"imred: %s", __PIPEVERS__);
   output=add_FITS_header_text("HISTORY",text,output);
   give_date(buf);
   sprintf(text, "imred: called at %s", buf); 
   output=add_FITS_header_text("HISTORY",text,output);

   get_filename(input.name,text);
   sprintf(buf,"input image:  %s",text);
   strcpy(text,"imred: ");
   strcat(text,buf);
   output=add_FITS_header_text("HISTORY",text,output);

   get_filename(output.name,text);
   sprintf(buf,"output image: %s",text);
   strcpy(text,"imred: ");
   strcat(text,buf);
   output=add_FITS_header_text("HISTORY",text,output);

   if(input.doover) 
   {
      if(ovscan.modline)
         sprintf(buf,"overscan correction per line: %d %d %d %d 1 %d",
                 ovscan.xmin,ovscan.xmax,ovscan.minreject,
                 ovscan.maxreject,input.ny);
      else
         sprintf(buf,"Cte overscan correction (median): %d %d 1 %d",
                 ovscan.xmin,ovscan.xmax,input.ny);
      strcpy(text,"imred: ");
      strcat(text,buf);
      output=add_FITS_header_text("HISTORY",text,output);
   }
   if(dark.dodark && darktab.file) 
   {
      get_filename(dark.name,text);
      sprintf(buf,"dark/bias correction: - (%s * %.2f)",text,dark.fact);
      strcpy(text,"imred: ");
      strcat(text,buf);
      output=add_FITS_header_text("HISTORY",text,output);
   }
   else if(dark.dodark && !darktab.file) 
   {
      sprintf(buf,"Cte dark/bias correction: subtract %.2f ADU",current_dark);
      strcpy(text,"imred: ");
      strcat(text,buf);
      output=add_FITS_header_text("HISTORY",text,output);
   }
   if(flat.doflat) 
   {
      get_filename(flat.name,text);
      sprintf(buf,"flat correction: * %.2f / %s",flat.fact,text);
      strcpy(text,"imred: ");
      strcat(text,buf);
      output=add_FITS_header_text("HISTORY",text,output);
   }
   if(mask.domask) 
   {
      get_filename(mask.name,text);
      sprintf(buf,"mask correction: * %s",text);
      strcpy(text,"imred: ");
      strcat(text,buf);
      output=add_FITS_header_text("HISTORY",text,output);
   }
   if(fringe.dofringe) 
   {
      get_filename(fringe.name,text);
      sprintf(buf,"fringe correction: * %.2f / %s",fringe.fact, text);
      strcpy(text,"imred: ");
      strcat(text,buf);
      output=add_FITS_header_text("HISTORY",text,output);
   }
   if(raster.cut)
   {
      sprintf(buf,"CUT: Xo=%d Yo=%d Xs=%d Ys=%d",
      raster.x, raster.y, raster.sizex,
      raster.sizey);
      strcpy(text,"imred: ");
      strcat(text,buf);
      output=add_FITS_header_text("HISTORY",text,output);
   }

   get_notab(output.comment,buf);
   if(strlen(buf)!=0) 
   {
      strcpy(text,"imred: ");
      strcat(text,buf);
      output=add_FITS_header_text("COMMENT",text,output);
   }

   /* load header from dark file */
   if(dark.dodark && darktab.file)
   {
	dark=read_FITS_header(dark);
   }

   /* load header from flat file */
   if(flat.doflat)
   {
        flat=read_FITS_header(flat);
   }

   /* load header from mask file */
   if(mask.domask)
   {
        mask=read_FITS_header(mask);
   }

   /* load header from fringe file */
   if(fringe.dofringe)
   {
        fringe=read_FITS_header(fringe);
   }

   if(dark.dodark && darktab.file) 
   {
      if((output.nx != dark.nx)||(output.ny != dark.ny)) 
      {
        printf("\nDifferent size for input image and dark image, exiting.\n\n");
        fitsclose(0);
        exit(0);
      }
   }
   if(flat.doflat) 
   {
      if((output.nx != flat.nx)||(output.ny != flat.ny)) 
      {
        printf("\nDifferent size for input image and flat image, exiting.\n\n");
        fitsclose(0);
        exit(0);
      }
   }
   if(mask.domask) 
   {
      if((output.nx != mask.nx)||(output.ny != mask.ny)) 
      {
        printf("\nDifferent size for input image and mask image, exiting.\n\n");
        fitsclose(0);
        exit(0);
      }
   }
   if(fringe.dofringe) 
   {
      if((output.nx != fringe.nx)||(output.ny != fringe.ny)) 
      {
        printf("\nDifferent size for input image and fringe image, exiting.\n\n");
        fitsclose(0);
        exit(0);
      }
   }

   /* All files ok, we can open and jump to image data area */
   input.imagefile=open_file(input.name,"rb");
   fseek(input.imagefile,input.startdata,0);

   if(dark.dodark && darktab.file) 
   {
      dark.imagefile=open_file(dark.name,"rb");
      fseek(dark.imagefile,dark.startdata,0);
   }

   if(flat.doflat) 
   {
      flat.imagefile=open_file(flat.name,"rb");
      fseek(flat.imagefile,flat.startdata,0);
   }

   if(mask.domask) 
   {
      mask.imagefile=open_file(mask.name,"rb");
      fseek(mask.imagefile,mask.startdata,0);
   }

   if(fringe.dofringe) 
   {
      fringe.imagefile=open_file(fringe.name,"rb");
      fseek(fringe.imagefile,fringe.startdata,0);
   }

   output.imagefile=open_file(output.name,"ab");
   fseek(output.imagefile,output.startdata,0);
}

/*----------------------------------------------------------------------------*/
void fitsalloc()
{
extern struct IMAGE input,dark,flat,mask;
extern PIXTYPE *filein,*filedk,*filefl,*filemk;
extern PIXTYPE **overscan;

int i;
PIXTYPE **alloc_square_float();

   i=output.nx;
   if(i<blocksize)
   {
	i=blocksize;
   }

   filein=(PIXTYPE *) malloc(i*sizeof(PIXTYPE));
   if(dark.dodark && darktab.file)
   {
      filedk=(PIXTYPE *) malloc(i*sizeof(PIXTYPE));
   }
   if(flat.doflat)
   {
      filefl=(PIXTYPE *) malloc(i*sizeof(PIXTYPE));
   }
   if(mask.domask)
   {
      filemk=(PIXTYPE *) malloc(i*sizeof(PIXTYPE));
   }
   if(fringe.dofringe)
   {
      filefr=(PIXTYPE *) malloc(i*sizeof(PIXTYPE));
   }

   if(input.doover)
   {
      overscan=alloc_square_float((ovscan.xmax-ovscan.xmin+1),input.ny);
   }
}

/*----------------------------------------------------------------------------*/
void fitsclose(fill_file)
int fill_file; /* FITS standard complete block up to 2880 bytes */
{
   extern struct IMAGE input,output,dark,flat,mask;
   extern PIXTYPE *filein,*filedk,*filefl,*filemk;

   int i,z;
   char blockfill[blocksize];

   if(fill_file) 
   { /* no problem during initialization */
      /* fill the output file remaining block with 0s */
      z = blocksize-(output.bytepix*output.nx*output.ny)%blocksize;
      if(z) 
      {
         for(i=0; i<=z-1; i++)
	 {
            blockfill[i]=0;
	 }
         if((fwrite((char *)blockfill, z, 1, output.imagefile)) !=1 ) 
         {
           printf("\nCannot pad %d bytes to %s, exiting\n\n",z,output.name);
           exit(1);
         }
      }
   }

   /* close files + free memory */
   fclose(input.imagefile);
   fclose(output.imagefile);
   free(filein);
   if(dark.dodark && darktab.file) 
   {
       fclose(dark.imagefile);
       free(filedk);
   }
   if(flat.doflat) 
   {
       fclose(flat.imagefile);
       free(filefl);
   }
   if(mask.domask) 
   {
       fclose(mask.imagefile);
       free(filemk);
   }
   if(fringe.dofringe) 
   {
       fclose(fringe.imagefile);
       free(filefr);
   }
   if(input.doover)
   {
       free(overscan);
   }
}
