/*----------------------------------------------------------------------------
Code to combine 8 CCDs of the SSO WFI in a mosaic 

24.04.03:
included the stdlib library to avoid warnings due to the
undefined 'exit' function.

13.07.2004:
corrected a bug in the padding (the number of bytes
to be padded is output.nx*output.ny*output.bytepix
instead of output.nx*output.ny).
----------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "immosaic_SSO.h"

#define	MOSWBUF 2000000 /* 4Mb */

/* 12K raw data: 16 bits BSCALE=1 BZERO=32K */

extern struct IMAGE input[8],output;
FILE *open_file();
struct IMAGE read_FITS_header(),write_FITS_header();

/*----------------------------------------------------------------------------*/
int main(argc,argv)
int	argc;
char	*argv[];
{
int	nmospix;
int	i,j=0,k,l,z;
int     bitpix;
int	nxo,nyo,nxi,nyi,bin;
int     xdim, ydim;

float   sum;


PIXTYPE *pixtab;
PIXTYPE *im_out;
PIXTYPE *px;

	if(argc < 10) 
        {
	   printf("Syntax: immosaic X.*.fits binsize outbitpix [filename]\n");
	   printf("X = root of the filename (443165o)\n");
	   printf("with binsize >= 1\n");
	   printf("For bitpix only 16 and -32 are supported\n");
	   printf("Example: immosaic 443165o*.fits 8 16\n");
	   printf("         immosaic 443165o*MODF.fits 8 -32 443165o-B8.fits\n");
	   exit(1);
	}

        /* Get input images filename */
	for (i=0; i<8; i++) 
        {
            if     (i==0) j=4;
            else if(i==1) j=3;
            else if(i==2) j=2;
            else if(i==3) j=1;
            else if(i==4) j=5;
            else if(i==5) j=6;
            else if(i==6) j=7;
            else if(i==7) j=8;
	    strcpy(input[i].name, argv[j]);
	    input[i] = read_FITS_header(input[i]);
	}
        printf("Combining files:\n");
        printf("  %s  %s  %s  %s  \n",argv[1],argv[2],argv[3],argv[4]);
        printf("  %s  %s  %s  %s  \n\n",argv[8],argv[7],argv[6],argv[5]);

        /* Get binning factor */
	sscanf(argv[9],"%d",&bin);
        if (bin==0) 
        {
           printf("binsize must be >=1\n");
	   exit(1);
	}

	if (argc > 10)
	{
	  sscanf(argv[10],"%d",&bitpix);	  
	}

        /* Get output filename */
        if (argc == 12)
	{
           strcpy(output.name, argv[11]);
	}
        else
	{
           strcpy(output.name, "sso.fits");
	}
	    
        /* Prepare output file */
	if(bitpix == 16)
	{
 	  output.bitpix=16;
	  output.bytepix=2;
	  output.offset=32768.0;
	}
	else
	{
 	  output.bitpix=-32;
	  output.bytepix=4;
	  output.offset=0.0;	  
	}

	output.scale=1.0;

	xdim=(int)(input[0].nx/bin)*bin;
	ydim=(int)(input[0].ny/bin)*bin;
	printf("using the dimension %d x %d of input images\n", xdim, ydim);

	nxo = output.nx = (4*xdim)/bin;
	nyo = output.ny = (2*ydim)/bin;
	output=write_FITS_header(input[0],output);

        /* input images size */
	nxi = input[0].nx; 
        nyi = input[0].ny;

        printf("Binning=%d => creating `%s' - %d x %d pixels\n",
               bin,output.name,nxo,nyo);


        /* The real images */
	nmospix = nxi * 4; /* 4 CCDs per half-bank */

        /* open files and go to data */
	for(i=0; i< 8; i++)
	{
	  input[i].imagefile=open_file(input[i].name,"rb");
	  fseek(input[i].imagefile, input[i].startdata, 0);
	}
        output.imagefile=open_file(output.name,"ab");
        fseek(output.imagefile, output.startdata, 0);
	
	pixtab = (PIXTYPE *)malloc(bin*nmospix*sizeof(PIXTYPE));
	im_out = (PIXTYPE *)malloc(nxo*sizeof(PIXTYPE));

			/*===================*/
			/* Lower half mosaic */
			/*===================*/

	for (i=0; i<nyo/2; i++) 
        {
	    px=pixtab;
            /* process "bin" lines */
	    for(k=0; k < bin; k++)
	    {
	      for(l=0; l<4; l++)
	      {
		read_file(&(input[l]), pixtab, nxi);
		pixtab+=nxi;
	      }
	    }

	    pixtab=px;

	    for (j=0; j<nxo; j++) 
            {
	        px = pixtab + j*bin;
	        sum = 0.0;
	        for (k=0; k<bin; k++) 
		{
	            for (l=0; l<bin; l++)
		    {
	                sum += (float)(*px++);
		    }
	            px += nmospix - bin;
		}
	        sum/=(float)(bin*bin);

	        im_out[j] = sum;
            }
		
            /* Write the line to the file */
     	    write_file(&(output), im_out, nxo);

	    if (i%2==0)
	    {
	       printf("Lower half: %d to %d\r",i,nyo/2);
	    }
	}
        printf("                                                \r");

                        /*===================*/
                        /* Upper half mosaic */
                        /*===================*/


	for (i=0; i<nyo/2; i++) 
	{
	    /* load buffers */
	    px=pixtab;
	    for(k=0; k < bin; k++)
	    {
	      for(l=4; l<8; l++)
	      {
		read_file(&(input[l]), pixtab, nxi);
		pixtab+=nxi;
	      }
	    }

	    pixtab=px;
            for (j=0; j<nxo; j++) 
	    {
                px = pixtab + j*bin;
                sum = 0;
                for (k=0; k<bin; k++) 
                {
                    for (l=0; l<bin; l++)
		    {
                        sum += (float)(*px++);
		    }
                    px += nmospix - bin;
                }
                sum/=(bin*bin);
                im_out[j] = (PIXTYPE)sum;
            }

            /* Write the line to the file */
     	    write_file(&(output), im_out, output.nx);
	    if (i%2==0)
	       printf("Upper half: %d to %d\r",i,nyo/2);
	}

        /* Fill in FITS file */
        z = blocksize-(output.nx*output.ny*output.bytepix)%blocksize;
        if(z) 
        {
           for(i=0; i<=z-1; i++)
	   {
              im_out[i]=0.0;
	   }
           write_file(&(output),im_out,z);
        }

	/* close files and bye */
	for(i=0; i<8; i++)
	{
	  fclose(input[i].imagefile);
	}
	fclose(output.imagefile);
	
	free(pixtab);
	free(im_out);

	return(0);
}
