#include <stdlib.h>
#include "imred.h"

/* 21.12.:
   - substituted the fquicksort routine by the sort from NRC
     (I forgot this so far in this file)

  25.07.2003:
  I changed the overscan correction in the following way:
  - Instead of estimating the overscan of each line with the
    median of the pixels in the overscan region, I now consider
    the mean. From its calculation the 'minreject' lowest and
    'maxreject' highest values are excluded.
  - The smoothing over several lines is no longer done.

  09.09.2005:
  I removed superfluous arguments (xmin, xmax) from smooth_overscan
*/
/*----------------------------------------------------------------------------*/
void find_overscan()
{
extern struct IMAGE input,output;
extern struct OVERSCAN ovscan;

struct IMAGE add_FITS_header_text();
FILE *open_file();

int sizey;
char text[80];

   sizey=input.ny;

   /* load Y overscan in memory */
   load_overscan(overscan,ovscan.xmin,ovscan.xmax,1,sizey);

   /* compute the median along Y axis */
   median_overscan(overscan,ovscan.xmin,ovscan.xmax,1,sizey,
                   ovscan.minreject,ovscan.maxreject);

   /* smooth the overscans along the Y axis */
   /*smooth_overscan(overscan,ovscan.xmin,ovscan.xmax,1,sizey);*/

   /* print statistics */
   printf(" median overscan: %.1f ADU\n",ovscan.median);

   /* Put it in FITS file */
   fclose(output.imagefile);
   sprintf(text,"imred: median overscan = %.1f ADU",ovscan.median);
   output=add_FITS_header_text("HISTORY",text,output);

   output.imagefile=open_file(output.name,"ab");
   fseek(output.imagefile,output.startdata,0);
   
}

/*----------------------------------------------------------------------------*/
void load_overscan(PIXTYPE **overscan,
                   int xmin, int xmax,
                   int ymin, int ymax)
{
extern struct IMAGE input;

int i,j;

   fseek(input.imagefile, ((ymin-1)*input.nx+xmin-1)*(input.bytepix),1);

   for(j=0;j<(ymax-ymin+1);j++) 
   {
        read_file(&(input), filein, (xmax-xmin+1));
	fseek(input.imagefile, (input.nx-(xmax-xmin+1))*(input.bytepix),1);
	for(i=0;i<(xmax-xmin+1);i++) 
        {
	  overscan[i][j]=filein[i];
	}
   }

   /* go back to the first data */
   fseek(input.imagefile,input.startdata,0);
}

/*----------------------------------------------------------------------------*/
void median_overscan(PIXTYPE **overscan,
                     int xmin, int xmax,
                     int ymin, int ymax,
                     int minreject, int maxreject)
{
   int i,j,k;
   PIXTYPE *tab;
   PIXTYPE *buf;

   /* sanity check if enough pixels for mean calculation are left */
   if((xmax-xmin+1)-(minreject+maxreject) < 5)
   {
     printf("Not enough pixels (at least 5) for useful mean calculation left in overscan\n");
     exit(1);
   } 

   /* look for the median value in the whole overscan */
   tab=(PIXTYPE *)malloc(((xmax-xmin+1)*(ymax-ymin+1))*sizeof(PIXTYPE));
   buf=(PIXTYPE *)malloc((ymax-ymin+1)*sizeof(PIXTYPE));

   /* Compute the median over the whole 2D area */
   k=0;
   for(j=0;j<(xmax-xmin+1);j++) 
   {
       for(i=0;i<(ymax-ymin+1);i++)
       {
          tab[k++]=overscan[j][i];
       }
   }

   sort((unsigned long)k,tab-1);

   ovscan.median=tab[k/2];

   /* Build the vector along Y axis by taking the mean per line 
      rejecting the 'minreject' lowest and 'maxreject' highest
      values
   */
   for(j=0;j<(ymax-ymin+1);j++) 
   {
       for(i=0;i<(xmax-xmin+1);i++)
       {
	 tab[i]=overscan[i][j];
       }
 
       sort((unsigned long)(xmax-xmin+1), tab-1);

       overscan[0][j] = 0.0;
       for(k=minreject; k<((xmax-xmin+1)-maxreject); k++)
       {
	   overscan[0][j]+=tab[k];
       }
       overscan[0][j]/=((xmax-xmin+1)-(minreject+maxreject));
   }
 
   free(buf);
   free(tab);
}

/*----------------------------------------------------------------------------*/
void smooth_overscan(overscan,ymin,ymax)
PIXTYPE **overscan;
int ymin,ymax;
{
   int i,j;
   PIXTYPE *buf;
   PIXTYPE *tab;
   int filt=41; /* median filter size */

   buf=(PIXTYPE *)malloc((ymax-ymin+1)*sizeof(PIXTYPE));
   tab=(PIXTYPE *)malloc(filt*sizeof(PIXTYPE));

   for(i=filt/2;i<=(ymax-ymin-filt/2-1);i++) 
   {
       for(j=0;j<filt;j++)
       {
           tab[j]=overscan[0][i+j-filt/2];
       }
       sort((unsigned long)filt, tab-1);
       /*fquicksort(tab,0,filt-1);*/
       buf[i]=tab[filt/2];
   }

   for(i=filt/2;i<=(ymax-ymin-filt/2-1);i++)
   {
       overscan[0][i]=buf[i];
   }

   free(tab);
   free(buf);
}



