/* 19.08: substituted fquicksort by the NRC quicksort routine 
   16.10: changed estimate of sigma:
          it is now estimated by 33% of data points left to the
          mode
   18.10: the mode is now estimated within 50% of all points surrounding
          the median
   16.11: corrected an error in the determination of the tab bin
          where the mode is located
          (replaced i=tab[maxindex] by i=tabindex[maxindex])

   10.03.01:
   removed all compiler warnings under DEC

   17.09.01:
   introduced an extra treatment for images consisting
   only of one value. In this case the mode calculation
   led to a core dump.

   10.10.01:
   - the constraint of an image having only "one" value and hence
     sigma=0 was relaxed to deltah<1.0e-03 that is treated as zero
     now. The old treatment led to a core dump under Sun (probably
     as deltah/(nbins-1) then led to a number smaller than represented
     by floats).
   - The mean and median is now calculated with ALL the values
     in the sample.

   24.04.03:
   included the stdlib library to avoid warnings due to the
   undefined 'exit' function.

   13.10.03:
   corrected an error going through pixels in y coordinate.
   We always went through ALL lines in y and not only those
   according to the percentage asked for.
*/

#include <stdlib.h>
#include <stdlib.h>
#include "immode.h"

/*----------------------------------------------------------------------------*/
void stat_mediane()
{
float median,x,mean,sigma,mode=0.0,p,pmax;
float deltah;
PIXTYPE *tab,*filein;
int   deltax, deltay;
int   i,j,k,count;
int   nbr_px;
long *lhisto;
long *tabindex;
int nbins=100; /* the number of bins for the calculation of the mode */
int maxindex=0; /* in which bin is the mode located (for quickly
                   estimating sigma 
                */
static int gauss[5]={18,21,22,21,18};
int ngauss=5;

   /* alloc memory to read the input file */
   filein=(PIXTYPE *)malloc(output.nx*sizeof(PIXTYPE));

   /* Calculate offsets between pixels in X */
   nbr_px=(output.nx*stati.percent)/100;
   deltax=(int)(((float)output.nx)/((float)nbr_px)+0.5);
   nbr_px=(output.ny*stati.percent)/100;
   deltay=(int)(((float)output.ny)/((float)nbr_px)+0.5);

   lhisto = (long *) calloc (nbins,sizeof(long));
   tabindex = (long *) calloc (nbins,sizeof(long));

   /* Total number of pixels read (just for malloc) */
   nbr_px = (output.nx*output.ny);

   tab=(PIXTYPE *) calloc(nbr_px,sizeof(PIXTYPE));
   if(tab == 0) 
   {
      printf("Cannot allocate memory for tab in stat_mediane\n");
      exit(1);
   }

   nbr_px=mean=0;
   fseek(input.imagefile,(raster.ymin*input.nx+raster.xmin)*(input.bytepix),1);

   for(j=raster.ymin;j<raster.ymax;j+=deltay) 
   {
       read_file(&(input), filein, output.nx);
       fseek(input.imagefile,(((deltay-1)*input.nx)+(input.nx-output.nx))*(input.bytepix),1);
       i=0;
       while(i<output.nx) 
       {
	  x=filein[i];

          if((x>=input.dyn_min)&&(x<=input.dyn_max))
	  {
             tab[nbr_px++]=x;
	  }

          i+=deltax;
       }
   }

   /* Sort the whole table */
   sort((unsigned long)nbr_px, tab-1);
   /*fquickSort(tab,0,nbr_px-1);*/

   /* Statistics for the mean and median on the 0% to 95% range */
   j=(nbr_px*00)/100;
   k=(nbr_px*100)/100;
   nbr_px = k - j;
   mean=0.0;

   /* calculate mean and median */
   for(i=j;i<k;i++)
   {
       tab[i-j]=tab[i];
       mean+=tab[i];
   }

   mean/=(float)nbr_px;
   median=tab[nbr_px/2];

   /* now build histogram for the mode */
   deltah=(tab[(int)(0.75*nbr_px)]-tab[(int)(0.25*nbr_px)])/(nbins-1);
   /*deltah=(tab[nbr_px-1]-tab[0])/(nbins-1);*/

   /* deal with the case of an image having only one value 
      correctly
   */
   if(deltah > 1.0e-03)
   {
     count=1;
     tabindex[0]=0;
  
  
     for(i=(int)(0.25*nbr_px); i< (int)(0.75*nbr_px); i++)
     {
       if(tab[i] < tab[(int)(0.25*nbr_px)]+count*deltah)
       {
  	 lhisto[count-1]++;
       }
       else
       {
  	 lhisto[count-1]++;
  	 tabindex[count]=i;
  	 count++;
       }
     }
  
     /* smooth histogram */
     pmax=-50000;
  
     for(k=2; k<nbins-3; k++)
     {
  	 p=0;
  	 for(i=0;i<ngauss;i++)
  	 {
  	     p+=gauss[i]*lhisto[k+i-2];
  	 }
  
  	 if(p>pmax)
  	 {
  	   pmax=p;
  	   maxindex=k;
  	   mode=tab[(int)(0.25*nbr_px)]+(k)*deltah;
  	 }
     }     
  
     /* now estimate sigma; it is estimated by 33% of the values
  	left to the mode
     */
  
     /* first determine the position of the mode within the array */
     i=tabindex[maxindex];
  
     while( tab[i] < mode && i < nbr_px)
     {
       i++;
     }
  
     sigma = tab[i]-tab[(int)(0.17*nbr_px)];
   }
   else
   {
     mode=tab[0];
     sigma=0.0;
   }

   fprintf(STATI,"%s\t%5.0f\n",input.name,mode);
   fprintf(STATA,"%s\t%6.2f\t%6.2f\t%6.2f\t%6.2f\n",input.name,mode,median,mean,sigma);
   printf("%s   mode = %5.0f | %5.1f/%5.1f\n",
          input.name,mode,mean,sigma);

   free(lhisto);
   free(tabindex);
   free(tab);
   free(filein);
}
