#!/bin/bash

# testscript for the 'imhead' program

if [ $# -ne 1 ]; then
  echo "$0 <path to imhead command>"
  exit 1;
fi

IMHEAD=$1

echo "Testing program ${IMHEAD}"

FAIL=0

echo "Testing split of header and data sections from FITS images"

# create two FITS images and replace the header of one
# with that of the other:

ic -c 256 256 '1' > test0_$$.fits
ic -c 256 256 '0' > test1_$$.fits

# get a VALUE keyword into the first header:
replacekey test0_$$.fits "VALUE   =                    1" HISTORY

# now isolate header and data sections of the two images with
# imhead and give the header of the first to the data of the 
# second image:
imhead -b < test0_$$.fits > head0_$$.txt # header of the first image
imhead -D < test0_$$.fits > data0_$$.dat # data of the first image
imhead -D < test1_$$.fits > data1_$$.dat # data of the second image

# just create a new image with the header of the first and the data
# of the second; the 'cat' command does the trick:
cat head0_$$.txt data1_$$.dat > test1_mod_$$.fits

VALUEKEY="`dfits test1_mod_$$.fits | fitsort -d VALUE | awk '{print $2}'`"

if [ "${VALUEKEY}" != "1" ]; then
  echo "Header key VALUE in test1_mod_$$.fits is ${VALUEKEY}"
  echo "It should be '1'."
  FAIL=$(( ${FAIL} + 1 ))
fi

# if fitsverify is present we also check validity of the
# created FITS file:
if [ -x `which fitsverify` ]; then
  fitsverify -q test1_mod_$$.fits

  if [ $? -ne 0 ]; then
    echo "WARNING: The created FITS file does not seem to comply with the"
    echo "FITS standard!!"
  fi
fi

if [ ${FAIL} -eq 0 ]; then
  rm head0_$$.txt
  rm data0_$$.dat data1_$$.dat
  rm test1_mod_$$.fits
fi

echo "${FAIL} tests failed for program ${IMHEAD}"

