/*
  20.04.03:
  removed compiler warnings under Linux

  05.05.03:
  included the tsize.h file

  10.01.2005:
  I removed compiler warnings under gcc with optimisation
  enabled (-O3).

  11.03.2005:
  Changes to avoid compiler warnings with '-Wall -Wpedantic -W'.
*/

#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <string.h>
#include "lists_globals.h"
#include "fitscat.h"
#include "tsize.h"
#include "utils_globals.h"
#include "options_globals.h"
#include "aplastrom_globals.h"
#include "coord_trans.h"
#include "field_desc.h"
#include "tabutil_globals.h"
#include "obj_obj_util.h"
#include "thresh_globals.h"
#include "model_globals.h"
#include "output_globals.h"
#include "astrom_io_globals.h"
#include "project_defs.h"
#include "compute_globals.h"

#define A_WCS 0 
#define B_WCS 1 
#define E_WCS 2 
#define T_WCS 3 
#define POSERR 4
#define RA 5
#define DEC 6
#define RMS_RND 7
#define RMS_SYS 8
#define VEC_WCS 9
#define MAT_WCS 10

#define XPOS 0
#define YPOS 1
#define A 2 
#define B 3 
#define T 4 
#define FIELD_POS 5
#define XX 6
#define XY 7
#define YY 8
#define VEC 9
#define MAT 10

#define AFFINE 0
#define ASTROM 1
/*
 *    Global variables do not take as much memory
 */

char header[30000];


option my_opts[] = {
            {"CRVAL1", TEXT, control.crval1, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "CRVAL1",
              {"", "", "", "", ""}, 0,
              "Name of FIELDS table column containing the field center RA"},
            {"CRVAL2", TEXT, control.crval2, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "CRVAL2",
              {"", "", "", "", ""}, 0,
              "Name of FIELDS table column containing the field center DEC"},
            {"CDELT1", TEXT, control.cdelt1, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "CDELT1",
              {"", "", "", "", ""}, 0,
              "Name of FIELDS table column containing the x pixel size"},
            {"CDELT2", TEXT, control.cdelt2, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "CDELT2",
              {"", "", "", "", ""}, 0,
              "Name of FIELDS table column containing the y pixel size"},
            {"CRPIX1", TEXT, control.crpix1, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "CRPIX1",
              {"", "", "", "", ""}, 0,
              "Name of FIELDS table column containing the reference x pixel"},
            {"CRPIX2", TEXT, control.crpix2, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "CRPIX2",
              {"", "", "", "", ""}, 0,
              "Name of FIELDS table column containing the reference y pixel"},
            {"NAXIS1", TEXT, control.xwid, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "MAPNAXS1",
              {"", "", "", "", ""}, 0,
              "Name of FIELDS table column containing the x field width"},
            {"NAXIS2", TEXT, control.ywid, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "MAPNAXS2",
              {"", "", "", "", ""}, 0,
              "Name of FIELDS table column containing the y field width"},
            {"XPOS", TEXT, control.xpos, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "Xpos",
              {"", "", "", "", ""}, 0,
              "Name of OBJECTS table column containing the x position"},
            {"YPOS", TEXT, control.ypos, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "Ypos",
              {"", "", "", "", ""}, 0,
              "Name of OBJECTS table column containing the y position"},
            {"POSERR", TEXT, control.poserr, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "PosErr",
              {"", "", "", "", ""}, 0,
              "Name of OBJECTS table column containing the postional error estimate"},
            {"RA", TEXT, control.ra, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "Ra",
              {"", "", "", "", ""}, 0,
              "Name of OBJECTS table column containing the Right Ascension"},
            {"DEC", TEXT, control.dec, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "Dec",
              {"", "", "", "", ""}, 0,
              "Name of OBJECTS table column containing the Declination"},
            {"A_WCS", TEXT, control.a_wcs, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "A_WCS",
              {"", "", "", "", ""}, 0,
              "Name of OBJECTS table column containing the major axis in WCS"},
            {"B_WCS", TEXT, control.b_wcs, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "B_WCS",
              {"", "", "", "", ""}, 0,
              "Name of OBJECTS table column containing the minor axis in WCS"},
            {"E_WCS", TEXT, control.e_wcs, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "E_WCS",
              {"", "", "", "", ""}, 0,
              "Name of OBJECTS table column containing the ellipticity in WCS"},
            {"THETAWCS", TEXT, control.t_wcs, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "THETAWCS",
              {"", "", "", "", ""}, 0,
              "Name of OBJECTS table column containing the position angle in WCS"},
            {"A", TEXT, control.a, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "A",
              {"", "", "", "", ""}, 0,
              "Name of OBJECTS table column containing the major axis in pixels"},
            {"B", TEXT, control.b, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "B",
              {"", "", "", "", ""}, 0,
              "Name of OBJECTS table column containing the minor axis in pixels"},
            {"THETA", TEXT, control.t, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "Theta",
              {"", "", "", "", ""}, 0,
              "Name of OBJECTS table column containing the position angle in pixels"},
            {"VECTOR", TEXT, control.vec, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "",
              {"", "", "", "", ""}, 0,
              "Name of OBJECTS table column containing the vector in pixels"},
            {"MATRIX", TEXT, control.mat, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "",
              {"", "", "", "", ""}, 0,
              "Name of OBJECTS table column containing the matrix in pixels"},
            {"XM2", TEXT, control.xm2, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "XM2",
              {"", "", "", "", ""}, 0,
              "Name of OBJECTS table column containing the x variance"},
            {"YM2", TEXT, control.ym2, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "YM2",
              {"", "", "", "", ""}, 0,
              "Name of OBJECTS table column containing the y variance"},
            {"CORR", TEXT, control.corr, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "Corr",
              {"", "", "", "", ""}, 0,
              "Name of OBJECTS table column containing the covariance"},
            {"RMS_RND", TEXT, control.rms_rnd, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "RMS_RND",
              {"", "", "", "", ""}, 0,
              "Name of OBJECTS table column containing random position error astrometric solution"},
            {"RMS_SYS", TEXT, control.rms_sys, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "RMS_SYS",
              {"", "", "", "", ""}, 0,
              "Name of OBJECTS table column containing systematic position error astrometric solution"},
            {"XPIXSIZE", FLOAT, &control.xpixsize, 0, 0, -100.0, 100.0,
              0, 1.0, NO, "",
              {"", "", "", "", ""}, 0,
              "The scaling of pixels with which CDELT1 is multiplied"},
            {"YPIXSIZE", FLOAT, &control.ypixsize, 0, 0, -100.0, 100.0,
              0, 1.0, NO, "",
              {"", "", "", "", ""}, 0,
              "The scaling of pixels with which CDELT2 is multiplied"},
            {"DATA_TYPE", STRING, control.data_type, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "OTHER",
              {"DENIS", "OOPS", "OTHER", "", ""}, 0,
              "The type of data used"},
            {"", BOOLEAN, NULL, 0, 0, 0.0, 0.0, 0, 0.0, NO, "", 
             {"YES", "NO", "", "", ""}, 0,
              "dummy key to end structure my_opts"}
            };

void usage(char *me, char *head)
{
    fprintf(stderr,"%s\n\n", head);
    fprintf(stderr,"Usage: %s \n", me);
    fprintf(stderr,"		-i incat1		(input catalog name)\n");
    fprintf(stderr,"		-o outcat1		(output catalog name)\n");
    fprintf(stderr,"		[-a]			(force using AFFINE table)\n");
    fprintf(stderr,"		[-c conf_file]\n");
    fprintf(stderr,"		[-OPTION1 OPTVAL1 [-OPTION OPTVAL2] ...]\n\n");

    exit(1);
}

void init(int argc, char *argv[], char *head, control_struct *c)
{
/*
 *  Default file name is calling name with .conf extension
 */
    int i, j, k;

    strcpy(c->table_name,"OBJECTS");
    c->do_affine = NO;
    if (argc == 1) usage(argv[0], head);
    init_options(0, my_opts);
    for (i=1; i<argc; i++) {
      if (argv[i][0] == '-') {
         switch(argv[i][1]) {
         case 'c': strcpy(c->conffile,argv[++i]);
                   break;
         case 'i': strcpy(c->listfile,argv[++i]);
                   break;
         case 'o': strcpy(c->outfile,argv[++i]);
                   break;
         case 't': strcpy(c->table_name,argv[++i]);
                   break;
         case 'a': c->do_affine = YES;
                   break;
         case '@': print_options(argv[0]);
                   exit(0);
         default:  parse_options(c->conffile);
                   j = i++; k = i;
                   if (argv[j]&&argv[k])
                     set_command_option(argv[j], argv[k]);
                   else
                     usage(argv[0], head);
                   break;
         }
      } else {
         syserr(WARNING,"init","Command line syntax error\n");
         usage(argv[0], head);
      }
   }
   finish_options(c->conffile);

   VPRINTF(NORMAL, "%s\n\n", head);
}

keystruct **aplastrom_out_keys(int type, int *n)
{
    keystruct	**keys;
    int		nkeys = 0, nadd = 0;
    char	newname[MAXCHAR];

    switch (type) {
    case AFFINE:
        nkeys = 7;
        break;
    case ASTROM:
        nkeys = 9;
        if (control.vec[0] != '\0') nkeys++;
        if (control.mat[0] != '\0') nkeys++;
        break;
    default:
        syserr(FATAL, "aplastrom_out_keys", "Unknow output type\n");
    }
    ED_CALLOC(keys, "aplastrom_out_keys", keystruct *, nkeys);
    keys[0] = new_key(control.a_wcs);
    keys[0]->ttype = T_FLOAT;
    keys[0]->nbytes = t_size[T_FLOAT];
    sprintf(keys[0]->unit,"Deg");
    sprintf(keys[0]->comment,"Major axis object in world coordinates");
    keys[1] = new_key(control.b_wcs);
    keys[1]->ttype = T_FLOAT;
    keys[1]->nbytes = t_size[T_FLOAT];
    sprintf(keys[1]->unit,"Deg");
    sprintf(keys[1]->comment,"Minor axis object in world coordinates");
    keys[2] = new_key(control.e_wcs);
    keys[2]->ttype = T_FLOAT;
    keys[2]->nbytes = t_size[T_FLOAT];
    sprintf(keys[2]->unit," ");
    sprintf(keys[2]->comment,"Ellipticity of object in world coordinates");
    keys[3] = new_key(control.t_wcs);
    keys[3]->ttype = T_FLOAT;
    keys[3]->nbytes = t_size[T_FLOAT];
    sprintf(keys[3]->unit,"Deg");
    sprintf(keys[3]->comment,"Position angle object ellipse in world coordinates");
    keys[4] = new_key(control.poserr);
    keys[4]->ttype = T_FLOAT;
    keys[4]->nbytes = t_size[T_FLOAT];
    sprintf(keys[4]->unit,"Deg");
    sprintf(keys[4]->comment,"Position error of object");
    keys[5] = new_key(control.ra);
    keys[5]->ttype = T_DOUBLE;
    keys[5]->nbytes = t_size[T_DOUBLE];
    sprintf(keys[5]->unit,"Deg");
    sprintf(keys[5]->comment,"Right ascension object in world coordinates");
    keys[6] = new_key(control.dec);
    keys[6]->ttype = T_DOUBLE;
    keys[6]->nbytes = t_size[T_DOUBLE];
    sprintf(keys[6]->unit,"Deg");
    sprintf(keys[6]->comment,"Declination object in world coordinates");
    if (nkeys > 7) {
      keys[7] = new_key(control.rms_rnd);
      keys[7]->ttype = T_FLOAT;
      keys[7]->nbytes = t_size[T_FLOAT];
      sprintf(keys[7]->unit,"Deg");
      sprintf(keys[7]->comment,"Random positional error astrometric solution");
      keys[8] = new_key(control.rms_sys);
      keys[8]->ttype = T_FLOAT;
      keys[8]->nbytes = t_size[T_FLOAT];
      sprintf(keys[8]->unit,"Deg");
      sprintf(keys[8]->comment,
                     "Systematic positional error astrometric solution");
      if (control.vec[0] != '\0') {
        nadd++;;
        sprintf(newname, "%s_WCS", control.vec);
        keys[8+nadd] = new_key(newname);
        keys[8+nadd]->ttype = T_FLOAT;
        keys[8+nadd]->naxis  = 1;
        ED_MALLOC(keys[8+nadd]->naxisn, "", int, keys[8+nadd]->naxis);
        keys[8+nadd]->naxisn[0] = 2;
        keys[8+nadd]->nbytes = t_size[T_FLOAT] * 2;
        sprintf(keys[8+nadd]->unit,"Deg");
        sprintf(keys[8+nadd]->comment,
                     "Vector transformed with astrometric solution");
      }
      if (control.mat[0] != '\0') {
        nadd++;;
        sprintf(newname, "%s_WCS", control.mat);
        keys[8+nadd] = new_key(newname);
        keys[8+nadd]->ttype = T_FLOAT;
        keys[8+nadd]->naxis  = 1;
        ED_MALLOC(keys[8+nadd]->naxisn, "", int, keys[8+nadd]->naxis);
        keys[8+nadd]->naxisn[0] = 4;
        keys[8+nadd]->nbytes = t_size[T_FLOAT] * 4;
        sprintf(keys[8+nadd]->unit,"Deg");
        sprintf(keys[8+nadd]->comment,
                     "Matrix transformed with astrometric solution");
      }
    }
    *n = nkeys;
    return keys;
}

void make_affine_apply(tabstruct *in_objtab, tabstruct *tab,
                       keystruct **inkeys, tabstruct *out_objtab, 
                       keystruct **outkeys, affine_params **affine_p,
                       field_params **field_p)
{
    short int	*curr_field;
    int		i, j, remaining, count = 0;
    float	*xpos, *ypos, *a, *b, *t;
    float	*a_wcs, *b_wcs, *e_wcs, *t_wcs, *poserr;
    double	*ra, *dec, lcdelt1, lcdelt2;
    keystruct	*key, *okey;

    do {
/*
 *     Because the order of the output table's keys is the same as that 
 *     for the input (see above) we can walk both key chains in parallel
 */
       remaining = read_obj(in_objtab, tab);
       key  = in_objtab->key;
       okey = out_objtab->key;
       for (j=0;j<in_objtab->nkey;j++) {
          memcpy(okey->ptr, key->ptr, key->nbytes);
          key = key->nextkey;
          okey = okey->nextkey;
       }
       xpos       = inkeys[XPOS]->ptr;
       ypos       = inkeys[YPOS]->ptr;
       a          = inkeys[A]->ptr;
       b          = inkeys[B]->ptr;
       t          = inkeys[T]->ptr;
       curr_field = inkeys[FIELD_POS]->ptr;
       a_wcs      = outkeys[A_WCS]->ptr;
       b_wcs      = outkeys[B_WCS]->ptr;
       e_wcs      = outkeys[E_WCS]->ptr;
       t_wcs      = outkeys[T_WCS]->ptr;
       poserr     = outkeys[POSERR]->ptr;
       ra         = outkeys[RA]->ptr;
       dec        = outkeys[DEC]->ptr;
       i = *curr_field - 1;
       apply_coord_trans(affine_p[i], xpos, ypos, 1);
       if (strcmp(control.data_type,"DENIS") == 0) {
          lcdelt1 = fabs(field_p[i]->cdelt1) * control.xpixsize;
          lcdelt2 = fabs(field_p[i]->cdelt2) * control.ypixsize;
       } else {
          lcdelt1 = field_p[i]->cdelt1 * control.xpixsize;
          lcdelt2 = field_p[i]->cdelt2 * control.ypixsize;
       }
       xy2rd(field_p[i]->crval1, field_p[i]->crval2, 
             lcdelt1, lcdelt2, 
             field_p[i]->crpix1, field_p[i]->crpix2,
             field_p[i]->pc11,
             field_p[i]->pc12,
             field_p[i]->pc21,
             field_p[i]->pc22, xpos, ypos, ra, dec, 1);
       *poserr = *a_wcs = *a * fabs(field_p[i]->cdelt1);
       *b_wcs = *b * fabs(field_p[i]->cdelt1);
       if (*a_wcs>0.0) {
          *e_wcs = 1-(*b_wcs)/(*a_wcs);
       } else {
          *e_wcs = 0.0; 
       }
       *t_wcs = *t;
/* 
 *     Add the additional output columns
 */
       count++;
       if ((count) % 2048 == 0)  
          VPRINTF(NORMAL,"Objects processed: %d \n",count);
       write_obj(out_objtab);
    } while (remaining);
    end_writeobj(out_objtab->cat, out_objtab);
    VPRINTF(NORMAL,"Objects processed: %d \n\n",count);
}

static void ellipse(double xx, double xy, double yy, float *a, float *b,
                    float *theta)
/*------------------------------------------------------------------------------
 * DENIS astrometric reduction:
 * Parameters:
 *     XX      Real     2nd order moment in x                           Input
 *     YY      Real     2nd order moment in y                           Input
 *     XY      Real     2nd order x, y correlation                      Input
 *     A       Real     Major axis ellips approximation                 Output
 *     B       Real     Minor axis ellips approximation                 Output
 *     E       Real     Ellips ellipticity approximation                Output
 *     THETA   Real     Ellips positiona angle approximation            Output
 * dd 27-01-98 New derivation method
 *------------------------------------------------------------------------*/
{
    double	tmp;

    if ((tmp = fabs(xx-yy)) > 0.0) {
      *theta = atan2(2.0*xy,xx-yy)/2.0;
    } else {
      *theta = DPI/4.0;
    }
    tmp = xx-yy;
    tmp = sqrt(0.25*tmp*tmp + xy*xy);
    *a  = sqrt(0.5*(xx+yy)+tmp);
    *b  = sqrt(0.5*(xx+yy)-tmp);
}

void make_astrom_apply(tabstruct *in_objtab, tabstruct *tab,
                       keystruct **inkeys, tabstruct *out_objtab, 
                       keystruct **outkeys, model_struct *model,
                       field_params **field_p)
{
    short int	*curr_field;
    int		i, j, c, remaining, count = 0;
    float	*xpos, *ypos, *epos;
    float	*a_wcs, *b_wcs, *e_wcs, *t_wcs, *rnd, *sys;
    double	*vec, *vec_wcs, *mat, *mat_wcs;
    double	*ra, *dec, *xx, *xy, *yy, xx_wcs, xy_wcs, yy_wcs; 
    keystruct	*key, *okey;

    do {
       remaining = read_obj(in_objtab, tab);
/*
 *     Because the order of the output table's keys is the same as that 
 *     for the input (see above) we can walk both key chains in parallel
 */
       key  = in_objtab->key;
       okey = out_objtab->key;
       for (j=0;j<in_objtab->nkey;j++) {
          memcpy(okey->ptr, key->ptr, key->nbytes);
          key = key->nextkey;
          okey = okey->nextkey;
       }
       curr_field = inkeys[FIELD_POS]->ptr;
       i = *curr_field - 1;
       if (model->ncam == 1) {
         c = 0;
       } else {
         c = field_p[i]->chan_nr - 1;
       }
       xpos       = inkeys[XPOS]->ptr;
       ypos       = inkeys[YPOS]->ptr;
       ra         = outkeys[RA]->ptr;
       dec        = outkeys[DEC]->ptr;
       compute_position(xpos, ypos, &i, &c, ra, dec, 1, model);
       *ra    /= DEG2RAD;
       *dec   /= DEG2RAD;
       epos       = inkeys[POSERR]->ptr;
       rnd        = outkeys[RMS_RND]->ptr;
       sys        = outkeys[RMS_SYS]->ptr;
       compute_errors(xpos, ypos, epos, &i, &c, rnd, sys, 1, model);
       xx         = inkeys[XX]->ptr;
       xy         = inkeys[XY]->ptr;
       yy         = inkeys[YY]->ptr;
       compute_mom(xpos, ypos, &i, &c, xx, yy, xy, 
                   &xx_wcs, &yy_wcs, &xy_wcs, 1, model);
       a_wcs      = outkeys[A_WCS]->ptr;
       b_wcs      = outkeys[B_WCS]->ptr;
       e_wcs      = outkeys[E_WCS]->ptr;
       t_wcs      = outkeys[T_WCS]->ptr;
       ellipse(xx_wcs, xy_wcs, yy_wcs, a_wcs, b_wcs, t_wcs);
       *a_wcs /= DEG2RAD;
       *b_wcs /= DEG2RAD;
       *t_wcs /= DEG2RAD;
       if (*a_wcs>0.0) {
          *e_wcs = 1-(*b_wcs)/(*a_wcs);
       } else {
          *e_wcs = 0.0; 
       }
       if (control.vec[0] != '\0') {
         vec     = inkeys[VEC]->ptr;
         vec_wcs = outkeys[VEC_WCS]->ptr;
         compute_vec(xpos, ypos, &i, &c, &vec[0], &vec[1],
                     &vec_wcs[0], &vec_wcs[1], 1, model);
       }
       if (control.mat[0] != '\0') {
         mat     = inkeys[MAT]->ptr;
         mat_wcs = outkeys[MAT_WCS]->ptr;
         compute_mat(xpos, ypos, &i, &c, &mat[0], &mat[1], &mat[2], &mat[3],
                  &mat_wcs[0], &mat_wcs[1], &mat_wcs[2], &mat_wcs[3],1, model);
       }
/* 
 *     Add the additional output columns
 */
       count++;
       if ((count) % 2048 == 0)  
          VPRINTF(NORMAL,"Objects processed: %d \n",count);
       write_obj(out_objtab);
    } while (remaining);
    end_writeobj(out_objtab->cat, out_objtab);
}

int main(int argc, char *argv[])

/****** aplastrom ***************************************************
 *
 *  NAME	
 *      aplastrom - Apply the astrometric calibration
 *
 *  SYNOPSIS
 *      aplastrom -i incat1 -o outcat1
 *                [-c configfile] [CONFOPTION]
 *
 *  FUNCTION
 *
 *  INPUTS
 *      -i incat1      - Name of input catalog
 *      -o outcat1     - Name of output catalogs
 *     [-c configfile] - Name of the configuration file. 
 *     [CONFOPTION]    - Pairs of configuration VARIABLE VALUE
 *                       sequences that allow dynamically
 *                       overwritting of configuration file values.
 *
 *  RESULTS
 *
 *  RETURNS
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 28-05-1998
 *
 ******************************************************************************/
{
     int		i, j, nfields;
     catstruct		*incat, *outcat;
     tabstruct		*astrom_tab, *affine_tab, *fields_tab;
     tabstruct		*in_objtab, *out_objtab;
     keystruct		**outkeys, **inkeys;
     int		noutkeys, ninkeys = 9;
     char		**keynames;
     affine_params	**affine_p;
     field_params	**field_desc;
     model_struct	model;
     thresh_struct	*thresholds;

     sprintf(header,"%s Version: %s (%s)",NAME,VERS,DATE);
     sprintf(header,"%-70.70s",header);
     init(argc, argv, header, &control);

     for (i=1;i<argc;i++)
       sprintf(header, "%s %s", header, argv[i]);
/*
 *   Open the catalogs
 */
     ED_CALLOC(keynames, "aplastrom", char *, ninkeys);
     for (i=0;i<ninkeys;i++)
        ED_CALLOC(keynames[i], "aplastrom", char, MAXCHAR);
     strcpy(keynames[XPOS], control.xpos);
     strcpy(keynames[YPOS], control.ypos);
     strcpy(keynames[A], control.a);
     strcpy(keynames[B], control.b);
     strcpy(keynames[T], control.t);
     strcpy(keynames[FIELD_POS], "FIELD_POS");
     strcpy(keynames[XX], control.xm2);
     strcpy(keynames[XY], control.corr);
     strcpy(keynames[YY], control.ym2);

     ED_CALLOC(inkeys, "aplastrom", keystruct *, ninkeys);
     if (!(incat = read_cat(control.listfile))) 
       syserr(FATAL, "aplastrom", "Could not open catalog %s\n", 
             control.listfile);
     if (!(fields_tab = name_to_tab(incat, "FIELDS", 0))) 
       syserr(FATAL,"aplastrom", "No FIELDS table in input catalog");
     field_desc = load_field_description(fields_tab, &nfields,
                            control.crval1, control.crval2,
                            control.cdelt1, control.cdelt2,
                            control.crpix1, control.crpix2,
                            control.xwid, control.ywid);
     if ((astrom_tab = name_to_tab(incat, "ASTROM", 0)) && 
         (control.do_affine == NO)) {
        VPRINTF(NORMAL, "Applying full Astrometric Calibration\n\n");
        fetch_model(astrom_tab, &model, &thresholds);
        print_param(&model);
        model.xscale = field_desc[0]->cdelt1*DEG2RAD*control.xpixsize;
        model.yscale = field_desc[0]->cdelt2*DEG2RAD*control.ypixsize;
        if (model.ncam == 1) {
           i = 0;
        } else {
           i = field_desc[0]->chan_nr-1;
        }
        for (j=0;j<nfields;j++) {
          model.crpix[0][i][j]   = field_desc[j]->crpix1;
          model.crpix[1][i][j]   = field_desc[j]->crpix2;
        }
        outkeys = aplastrom_out_keys(ASTROM, &noutkeys);
        init_catalogs(incat, &outcat, control.outfile,
                      control.table_name, inkeys, keynames, ninkeys,
                      outkeys, noutkeys, &in_objtab, &out_objtab,
                      "aplastrom", argc, argv);
        make_astrom_apply(in_objtab, name_to_tab(incat,control.table_name,0),
                          inkeys, out_objtab, outkeys,
                          &model, field_desc);
     } else {
        if ((affine_tab = name_to_tab(incat, "AFFINE", 0))) {
           VPRINTF(NORMAL, "Applying Preastrometric AFFINE transformation\n\n");
           affine_p = load_affine_tab(affine_tab, &nfields);
           outkeys = aplastrom_out_keys(AFFINE, &noutkeys);
           init_catalogs(incat, &outcat, control.outfile,
                         control.table_name, inkeys, keynames, ninkeys,
                         outkeys, noutkeys, &in_objtab, &out_objtab,
                         "aplastrom", argc, argv);
           make_affine_apply(in_objtab, name_to_tab(incat,control.table_name,0),
                             inkeys, out_objtab, outkeys,
                             affine_p, field_desc);
        } else {
           syserr(FATAL, "aplastrom",
            "No astrometric information in catalog, please run preastrom");
        }
     }
     close_cat(incat);
     free_cat(incat,1);
     close_cat(outcat);
     free_cat(outcat,1);
     VPRINTF(NORMAL,"Done\n\n");

     return 0;
}
