#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#include "fitscat.h"
#include "options_globals.h"
#include "utils_globals.h"
#include "model_globals.h"
#include "thresh_globals.h"
#include "timing_globals.h"
#include "stat_globals.h"
#include "output_globals.h"
#include "build_globals.h"
#include "solve_globals.h"
#include "astrom_globals.h"
#include "astrom_io_globals.h"
#include "astrom_vers.h"

/* 
   03.06.2004:
   I removed the '-x' option from the usage description.
   This option actually did nothing.

   30.01.2005:
   I adapted the calling sequence from store_model
   (necessary to add HISTORY information to the output
    catalog)

   11.03.2005:
   Changes to avoid compiler warnings with '-W -Wall -pedantic'
*/

char	header[30000];

control_struct control;

option my_opts[] = {
            {"RA", TEXT, control.ra, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "Ra",
             {"", "", "", "", ""}, 1,
             "Name of PAIRS table column containing the Right Ascension"},
            {"DEC", TEXT, control.dec, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "Dec",
             {"", "", "", "", ""}, 1,
             "Name of PAIRS table column containing the Declination"},
            {"ERR", TEXT, control.err, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "PosErr",
             {"", "", "", "", ""}, 1,
             "Name of PAIRS table column containing the postional error estimate"},
            {"XPOS", TEXT, control.xpos, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "Xpos",
             {"", "", "", "", ""}, 1,
             "Name of PAIRS table column containing the x position"},
            {"YPOS", TEXT, control.ypos, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "Ypos",
             {"", "", "", "", ""}, 1,
             "Name of PAIRS table column containing the y position"},
            {"FIELD", TEXT, control.field, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "Field",
             {"", "", "", "", ""}, 1,
             "Name of PAIRS table columns contianing the Field number"},
            {"CAMERA", TEXT, control.camera, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "CHANNEL_NR",
             {"", "", "", "", ""}, 1,
             "Name of PAIRS table columns contianing the Camera number"},
            {"ERRPOS", TEXT, control.errpos, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "ERRA_IMAGE",
             {"", "", "", "", ""}, 1,
             "Name of PAIRS table column containing the postional error estimate"},
            {"XM2", TEXT, control.xm2, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "XM2",
             {"", "", "", "", ""}, 1,
             "Name of OBJECTS table column containing the x variance"},
            {"YM2", TEXT, control.ym2, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "YM2",
             {"", "", "", "", ""}, 1,
             "Name of OBJECTS table column containing the y variance"},
            {"CORR", TEXT, control.corr, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "Corr",
             {"", "", "", "", ""}, 1,
             "Name of OBJECTS table column containing the covariance"},
            {"CRVAL1", TEXT, control.crval1, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "CRVAL1",
             {"", "", "", "", ""}, 1,
             "Name of FIELDS table column containing the field center RA"},
            {"CRVAL2", TEXT, control.crval2, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "CRVAL2",
             {"", "", "", "", ""}, 1,
             "Name of FIELDS table column containing the field center DEC"},
            {"CDELT1", TEXT, control.cdelt1, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "CDELT1",
             {"", "", "", "", ""}, 1,
             "Name of FIELDS table column containing the x pixel size"},
            {"CDELT2", TEXT, control.cdelt2, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "CDELT2",
             {"", "", "", "", ""}, 1,
             "Name of FIELDS table column containing the y pixel size"},
            {"CRPIX1", TEXT, control.crpix1, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "CRPIX1",
             {"", "", "", "", ""}, 1,
             "Name of FIELDS table column containing the reference x pixel"},
            {"CRPIX2", TEXT, control.crpix2, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "CRPIX2",
             {"", "", "", "", ""}, 1,
             "Name of FIELDS table column containing the reference y pixel"},
            {"XWID", TEXT, control.xwid, 0, 0, 0.0, 0.0, 
              0, 0.0, NO, "MAPNAXS1",
             { "", "", "", "", ""}, 1,
              "Name of FIELDS table column containing the x field width"},
            {"YWID", TEXT, control.ywid, 0, 0, 0.0, 0.0, 
              0, 0.0, NO, "MAPNAXS2",
             { "", "", "", "", ""}, 1,
              "Name of FIELDS table column containing the y field width"},
            {"EPOCH", FLOAT, &control.epoch, 0, 0, 1900.0, 2050.0,
              0, 2000.0, NO, "",
             {"", "", "", "", ""}, 1,
             "The value of the epoch for which reference data is given"},
            {"NITER", INT, &control.niter, 1, 20, 0.0, 0.0,
              3, 0.0, NO, "",
             {"", "", "", "", ""}, 1,
             "The number of iteration during solving"},
            {"XMIN", INT, &control.xmin, -32768, 32767, 0.0, 0.0,
              15, 0.0, NO, "",
             {"", "", "", "", ""}, 1,
             "Minimum x-coordinate of useful pixel plane"},
            {"YMIN", INT, &control.ymin, -32768, 32767, 0.0, 0.0,
              10, 0.0, NO, "",
             {"", "", "", "", ""}, 1,
             "Maximum x-coordinate of useful pixel plane"},
            {"XMAX", INT, &control.xmax, 0, 32767, 0.0, 0.0,
              753, 0.0, NO, "",
             {"", "", "", "", ""}, 1, 
             "Minimum y-coordinate of useful pixel plane"},
            {"YMAX", INT, &control.ymax, 0, 32767, 0.0, 0.0,
              758, 0.0, NO, "",
             {"", "", "", "", ""}, 1,
             "Maximum y-coordinate of useful pixel plane"},
            {"PDEG", INT, &control.pdeg, 0, 5, 0.0, 0.0,
              2, 0.0, NO, "",
             {"", "", "", "", ""}, 1,
             "Degrees of freedom for plate polynomials"},
            {"FDEG", FARRAY, control.fdeg, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "",
             {"", "", "", "", ""}, 1,
             "Degrees of freedom of Chebychev polynomials"},
            {"XPIXSIZE", FLOAT, &control.xpixsize, 0, 0, -100.0, 100.0,
              0, 1.0, NO, "",
              {"", "", "", "", ""}, 0,
              "The scaling of pixels with which CDELT1 is multiplied"},
            {"YPIXSIZE", FLOAT, &control.ypixsize, 0, 0, -100.0, 100.0,
              0, 1.0, NO, "",
              {"", "", "", "", ""}, 0,
              "The scaling of pixels with which CDELT2 is multiplied"},
            {"DATA_TYPE", TEXT, control.data_type, 0, 0, 0.0, 0.0, 
              0, 0.0, NO, "OTHER",
             { "DENIS", "OTHER", "", "", ""}, 1,
              "The type of data to be processed"},
            {"", BOOLEAN, NULL, 0, 0, 0.0, 0.0, 0, 0.0, NO, "", 
             {"YES", "NO", "", "", ""}, 0,
              "dummy key to end structure my_opts"}
           };

void usage(char *me, char *head)
{
    fprintf(stderr,"%s \n\n",head);
    fprintf(stderr,"Usage: %s \n",me);
    fprintf(stderr,"		-i incat1 [[incat2] incat3] ...		(input catalogs)\n");
    fprintf(stderr,"		-o outcat1 [[outcat2] outcat3] ...	(output catalogs)\n");
    fprintf(stderr,"		-p icatfile				(pairs catalog)\n");
    fprintf(stderr,"		[-c conffile]\n");
    fprintf(stderr,"		[-r stat_file_path]\n");
    fprintf(stderr,"		[-OPTION1 OPTVAL1 [-OPTION OPTVAL2] ...]\n");
 
    exit(1);
}
 
void init(int argc, char *argv[], char *head)
{
   int i, j, k;

   if (argc == 1) usage(argv[0], head);
   init_options(0, my_opts);
   sprintf(control.statfile,"residuals");
   for (i=1; i<argc; i++) {
     if (argv[i][0] == '-') {
        switch(argv[i][1]) {
        case 'c': strcpy(control.conffile,argv[++i]);
                  break;
        case 'p': strcpy(control.pairfile,argv[++i]);
                  break;
        case 'r': strcpy(control.statfile,argv[++i]);
                  break;
        case 'i': control.ilist = 0;
                  strcpy(control.listfile[control.ilist],argv[++i]);
                  while (i+1<argc && argv[i+1][0] != '-') 
                  {
                     strcpy(control.listfile[++control.ilist],argv[++i]);
                  }
                  ++control.ilist;
                  break;
        case 'o': j = 0;
                  strcpy(control.outfile[j],argv[++i]);
                  while (i+1<argc && argv[i+1][0] != '-') 
                  {
                     strcpy(control.outfile[++j],argv[++i]);
                  }
                  if (j+1 != control.ilist) {
                     syserr(FATAL, "init", "Input & Output lists have different length");
                  }
                  break;
        case '@': print_options(argv[0]);
                  exit(0);
        default:  parse_options(control.conffile);
                  j = i++; k = i;
                  if (argv[j]&&argv[k]) 
                    set_command_option(argv[j], argv[k]);
                  else 
                    usage(argv[0], head);
                  break;
        }
     } else {
        syserr(WARNING,"init","Command line syntax error\n");
        usage(argv[0], head);
     }
  }
  finish_options(control.conffile);

  VPRINTF(NORMAL,"\n   %s\n\n",head);
}

void update_par(model_struct *m)
/*--------------------------------------------------------------------------
 * DENIS astrometric package:
 * Tries to determine reasonable outlier rejection thresholds from crude
 * residual statistics.
 * Arguments:
 *   NP         Integer     Number of parameters                      Input
 *   PARAMS     Real        Current parameter estimate                Updated
 *   DELTA      Real        Parameter correction                      Input
 *--------------------------------------------------------------------------
 */
{
    int		i;
    float	*ptr_p, *ptr_b;

    ptr_p = m->params[0];
    ptr_b = m->beta[0];
    for (i=m->np+m->nf;i--;) *(ptr_p++) += *(ptr_b++);
    ptr_p = m->params[1];
    ptr_b = m->beta[1];
    for (i=m->np+m->nf;i--;) *(ptr_p++) += *(ptr_b++);
}
/**/

int main(int argc, char *argv[])
{
/*
 *  Main program for the astrometric reduction of the DENIS survey using the
 *  information in the overlap between fields, together with some reference
 *  stars
 */
    int			i, iter, reject;
    float		cpu0, cpu1, cpu2;
    thresh_struct	*thresholds;
    model_struct	model;
    char		*header;

    ED_CALLOC(header, "main", char, argc*MAXCHAR);

     sprintf(header,"%s Version: %s (%s)",NAME,VERS,DATE);
     sprintf(header,"%-70.70s",header);

     init(argc, argv, header);

     for (i=1;i<argc;i++)
     {
       sprintf(header, "%s %s", header, argv[i]);
     }

/*     
 *  Reads control parameters and catalogs
 */
    VPRINTF(NORMAL,"Opening catalogs\n");
    open_catalogs(&model);
    init_pairs(control.pairfile, &model);
/*
 *  Don't reject outliers in first iteration 
 */
    reject = FALSE;
    thresholds = init_thresholds(model.ncam);
/*
 *  Start the main solution loop
 */
    cpu2 = cpu_so_far();
    for (iter=0;iter<control.niter;iter++) 
    {
/* 
 *     Build the equations 
 */
       cpu1 = cpu0 = cpu_so_far();
       VPRINTF(NORMAL,"Starting iteration %d\n", iter);
       print_thresholds(thresholds, reject);
       build_equations(&model, thresholds, reject);
       VPRINTF(VERBOSE, "End of equation construction %f s iteration %d\n", 
                        cpu_so_far()-cpu0, iter);
/*       
 *     Solve the equations and update parameter estimate with last delta's
 */
       cpu0  = cpu_so_far();
       solve_equations(&model);
       update_par(&model);
       VPRINTF(VERBOSE, "End of equation solution %f s iteration %d\n", 
                        cpu_so_far()-cpu0, iter);
/*
 *     Stop iterating when none of the rejection threshold decreases by 
 *     more than 5%
 */
       reject = TRUE;
       make_statistics(&model, thresholds, reject);
       if (threshold_reduction(thresholds) < 1.05) {
          iter = control.niter;
          VPRINTF(VERBOSE, "Exiting iterations\n");
       }
       VPRINTF(VERBOSE, "Total cpu time for iteration %d: %f s\n", 
               iter, cpu_so_far()-cpu1);
    }
/*
 *  Print fitted parameters
 */
    print_param(&model);
/* 
 *  Exit processing: print statistics, and store the solution as a 
 *  FITS binary table
 */
    make_residuals(control.statfile, my_opts, header, &model, thresholds);
    store_model(&model, thresholds, my_opts, argc, argv);
    free_thresholds(thresholds);
    VPRINTF(VERBOSE, "Total cpu time: %f s\n", cpu_so_far()-cpu2);

    ED_FREE(header);

    return 0;
}
