/* History:
   10.01.2000 (TE):
   routine: overlist_struct
   memory is only allocated if nover > 0

   02.08.00:
   changed (function init_pairs)   
   if ((c_t != c_f) && (f_f == f_t)) {
   to
   if ((c_t == -c_f) && (f_f == f_t)) {
   
   It seems that this caused the very high outliers I observed
   all the time. 

   23.10.00:
   initialised the variables a_max and a_min in exam_covar.
   Omitting this led to core dumps under alpha.

   20.03.00:
   substituted
        if (f_t != f_f || f_t != c_f) {
   by
        if (f_t != f_f) {

   First this was an error (second condition!!) and then
   selfmatches obviously should not be included
   in the astrometric solution.

   26.06.01:
   resolved int/long int/LONG issues for variables lptr and
   maxpix.

   21.04.03:
   included freeing of fptr (store_model)   

   20.07.2004:
   I removed compiler warnings due to possibly
   undefined variables (compilation with optimisation
   turned on)

   15.10.2004:
   routine: open_catalogs:
   I included a sanity check for the number of
   fields within the input catalogs. The program stops
   if they differ in the various input files.

   10.01.2005:
   I removed compiler warnings under gcc with optimisation
   enabled (-O3).

   30.01.2005:
   I adapted calls to addhistoryto_cat to the new argument
   list of this function.

   I changed store_model to include HISTORY information to
   the out catalogs of astrom.

   11.03.2005:
   function save_distort_tab:
   I changed the call from addhistoryto_cat to historyto_cat and
   changed the argument list accordingly. 

   30.12.2012:
   I removed compiler warnings by replacing sprintf
   statements without a format string with strncpy commands. 
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "fitscat.h"
#include "options_globals.h"
#include "utils_globals.h"
#include "tabutil_globals.h"
#include "precess_defs.h"
#include "project_defs.h"
#include "thresh_globals.h"
#include "model_globals.h"
#include "field_desc.h"
#include "compute_globals.h"
#include "astrom_io_globals.h"
#include "astrom_globals.h"

extern control_struct control;

static int *chan_nr;
static refpair_struct  *ref;
static overpair_struct *over;
static int nref, nover;
static int nrefb, noverb, nblock = 256;

static catstruct	*res_cat;
static tabstruct	*res_tab;
static keystruct	**res_keys;

#define DRA  0
#define DDEC 1
#define RA   2
#define DEC  3
#define X1   4 
#define X2   5
#define Y1   6
#define Y2   7
#define F1   8
#define F2   9
#define C1  10
#define C2  11
/**/
void finish_residuals()
{
    end_writeobj(res_tab->cat, res_tab);
    close_cat(res_tab->cat);
}
/**/
void save_resi_pair(float dra, float ddec, double ra, double dec,
                    float x1, float y1, float x2, float y2,
                    int f1, int f2, int c1, int c2)
{
    double	*dptr;
    float	*fptr;
    LONG	*lptr;

    fptr  = res_keys[DRA]->ptr;
    *fptr = dra;
    fptr  = res_keys[DDEC]->ptr;
    *fptr = ddec;
    dptr  = res_keys[RA]->ptr;
    *dptr = ra;
    dptr  = res_keys[DEC]->ptr;
    *dptr = dec;
    fptr  = res_keys[X1]->ptr;
    *fptr = x1;
    fptr  = res_keys[Y1]->ptr;
    *fptr = y1;
    fptr  = res_keys[X2]->ptr;
    *fptr = x2;
    fptr  = res_keys[Y2]->ptr;
    *fptr = y2;
    lptr  = res_keys[F1]->ptr;
    *lptr = f1;
    lptr  = res_keys[F2]->ptr;
    *lptr = f2;
    lptr  = res_keys[C1]->ptr;
    *lptr = c1;
    lptr  = res_keys[C2]->ptr;
    *lptr = c2;
    write_obj(res_tab);
}
/**/
void save_refpair(refpair_struct *p, double ra, double dec)
{
    save_resi_pair(((float)(ra - (double)p->ra)*cos(dec)/DEG2RAD), 
                   ((float)(dec - (double) p->dec)/DEG2RAD),
                   p->ra/DEG2RAD, p->dec/DEG2RAD, -1.0, -1.0, p->xobj, p->yobj, 
                   -1, p->f, -1, p->c);
}
/**/
void save_overpair(overpair_struct *p, 
                   double ra1, double dec1, double ra2, double dec2)
{
    save_resi_pair(((float)(ra1-ra2)*cos(dec1)/DEG2RAD), 
                   ((float)(dec1-dec2)/DEG2RAD),
                   ra1/DEG2RAD, dec1/DEG2RAD, 
                   p->xobj1, p->yobj1, p->xobj2, p->yobj2, 
                   p->f1, p->f2, p->c1, p->c2);
}
/**/
#define NKEYS 7

void init_pairs(char *name, model_struct *model)
/****** init_pairs ***************************************************
 *
 *  NAME	
 *      init_pairs - Initilaize the pairing information reading
 *
 *  SYNOPSIS
 *      void init_pairs()
 *
 *  FUNCTION
 *      This function reads the pairing information from the designaed file
 *      and stores it into the local structures. The information read
 *      form each pairing is: Xpos, Ypos, Ra, Dec, ErrPos. For the
 *      internal pairing information only the Xpos. Ypos, ErrPos are
 *      retained, while for reference star information the Ra, Dec are
 *      also kept.
 *
 *  INPUTS
 *      name  - The name of the pairing catalog file
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      TRUE on success, FALSE on error
 *
 *  MODIFIED GLOBALS
 *      The static variables are filled with the appropriate information
 *      chan_nr
 *
 *  NOTES
 *      The routine open_catalogs must have been run to provide the
 *      chan_nr with useful information
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 16-07-1998
 *
 *****************************************************************************/
{
    catstruct	*cat;
    tabstruct	*tab, *intab;
    keystruct	*key;
    char	*keynames[NKEYS];
    int		i, j, remaining;
    double	ra_t=0.0, ra_f=0.0;
    double	dec_t=0.0, dec_f=0.0;
    float       xpos_t=0.0, xpos_f=0.0;
    float       ypos_t=0.0, ypos_f=0.0;
    float       err_t=0.0, err_f=0.0;
    int		f_t=0, f_f=0;
    int		c_t=0, c_f=0;

    if (!(cat = read_cat(name))) {
       syserr(FATAL, "init_pairs","Could not read catalog: %s\n", name);
    }
    if (!(tab = name_to_tab(cat, "PAIRS", 0))) {
       syserr(FATAL, "init_pairs","No PAIRS table in catalog %s", name);
    }
    VPRINTF(NORMAL, "Reading pairs\n");
    nrefb = noverb = 1;
    ED_CALLOC(ref,  "init_pairs", refpair_struct,  nrefb*nblock);
    ED_CALLOC(over, "init_pairs", overpair_struct, noverb*nblock);
    intab = init_readobj(tab);
    for (i=0;i<NKEYS;i++)
    { 
      ED_CALLOC(keynames[i], "init_pairs", char, MAXCHAR);
    }
    strcpy(keynames[1], control.dec);
    strcpy(keynames[2], control.xpos);
    strcpy(keynames[3], control.ypos);
    strcpy(keynames[4], control.err); 
    strcpy(keynames[5], control.field);
    strcpy(keynames[6], control.camera);
    do 
    {
      remaining = read_obj(intab, tab);
      if ((key = name_to_key(intab, control.ra)) == NULL) 
        syserr(FATAL,"init_pairs", "No key %s in table PAIRS\n", control.ra);
      else 
      {
        ra_t = ((double *)key->ptr)[0];
        ra_f = ((double *)key->ptr)[1];
      }
      if ((key = name_to_key(intab, control.dec)) == NULL) 
        syserr(FATAL,"init_pairs", "No key %s in table PAIRS\n", control.dec);
      else 
      {
        dec_t = ((double *)key->ptr)[0];
        dec_f = ((double *)key->ptr)[1];
      }
      if ((key = name_to_key(intab, control.xpos)) == NULL) 
        syserr(FATAL,"init_pairs", "No key %s in table PAIRS\n", control.xpos);
      else {
        xpos_t = ((float *)key->ptr)[0];
        xpos_f = ((float *)key->ptr)[1];
      }
      if ((key = name_to_key(intab, control.ypos)) == NULL) 
        syserr(FATAL,"init_pairs", "No key %s in table PAIRS\n", control.ypos);
      else {
        ypos_t = ((float *)key->ptr)[0];
        ypos_f = ((float *)key->ptr)[1];
      }
      if ((key = name_to_key(intab, control.err)) == NULL) 
        syserr(FATAL,"init_pairs", "No key %s in table PAIRS\n", control.err);
      else {
        err_t = ((float *)key->ptr)[0];
        err_f = ((float *)key->ptr)[1];
      }
      if ((key = name_to_key(intab, control.field)) == NULL) 
        syserr(FATAL,"init_pairs", "No key %s in table PAIRS\n", control.field);
      else {
        f_t = (int)((short int *)key->ptr)[0];
        f_f = (int)((short int *)key->ptr)[1];
      }
      if ((key = name_to_key(intab, control.camera)) == NULL) 
        syserr(FATAL,"init_pairs","No key %s in table PAIRS\n", control.camera);
      else 
      {
        c_t = (int)((LONG *)key->ptr)[0];
        c_f = (int)((LONG *)key->ptr)[1];
      }

      if (c_t == 0 || c_f == 0) 
         syserr(FATAL, "init_pairs", 
                       "Found a catalog with camera number = 0 !!\n");
/*
 *    The camera number is negative for the reference catalog entries
 */
      if (c_t < 0 || c_f < 0) 
      {
        /* if ((c_t != c_f) && (f_f == f_t)) { */

        if ((c_t == -c_f) && (f_f == f_t)) {
/*
 *        This is a good pairing with a reference object
 */
          if (nref == (nrefb*nblock - 1)) 
	  {
            nrefb++;
            ED_REALLOC(ref,  "init_pairs", refpair_struct, nrefb*nblock);
          }
	  if (c_t < 0) 
          {
            ref[nref].ra   = ra_t * DPI/180.0;
            ref[nref].dec  = dec_t * DPI/180.0;
            ref[nref].err  = err_t * DPI/180.0/12.;
            ref[nref].xobj = xpos_f;
            ref[nref].yobj = ypos_f;
            ref[nref].eobj = err_f;
            ref[nref].f    = f_f - 1;

            for (j=0;j<control.ilist;j++)
	    {
              if (chan_nr[j] == c_f)
	      { 
                ref[nref].c = j;
	      }
	    }
          } else {
            ref[nref].ra   = ra_f * DPI/180.0;
            ref[nref].dec  = dec_f * DPI/180.0;
            ref[nref].err  = err_f * DPI/180.0/12.;
            ref[nref].xobj = xpos_t;
            ref[nref].yobj = ypos_t;
            ref[nref].eobj = err_t;
            ref[nref].f    = f_t - 1;

            for (j=0;j<control.ilist;j++)
	    {
              if (chan_nr[j] == c_t) 
	      {
                ref[nref].c = j;
	      }
	    }
          }
          nref++;
        }
	/* printf("%d\n", ref[nref-1].c); */
      } else {
        if (f_t != f_f) {
/*
 *        This is a good pairing among extracted objects
 */
          if (nover == (noverb*nblock - 1)) {
            noverb++;
            ED_REALLOC(over,  "init_pairs", overpair_struct, noverb*nblock);
          }
          over[nover].xobj1 = xpos_f;
          over[nover].yobj1 = ypos_f;
          over[nover].eobj1 = err_f / (model->xscale / DEG2RAD);
          over[nover].f1    = f_f - 1;
          for (j=0;j<control.ilist;j++)
            if (chan_nr[j] == c_f)
              over[nover].c1 = j;
          over[nover].xobj2 = xpos_t;
          over[nover].yobj2 = ypos_t;
          over[nover].eobj2 = err_t / (model->xscale / DEG2RAD);
          over[nover].f2    = f_t - 1;
          for (j=0;j<control.ilist;j++)
            if (chan_nr[j] == c_t) 
              over[nover].c2 = j;
          nover++;
        }
      }
    } while (remaining);
    if (close_cat(cat) != RETURN_OK)
      syserr(FATAL, "init_pairs","Could not close catalog: %s\n", name);
}
/**/
reflist_struct *new_refpairs()
/****** new_refpairs ***************************************************
 *
 *  NAME	
 *      new_refpairs - Start the reference pair reading
 *
 *  SYNOPSIS
 *      void init_refpairs()
 *
 *  FUNCTION
 *      Initialize the reference pair reading routines
 *
 *  INPUTS
 *      None
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *      curr_nref
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 16-07-1008
 *
 ******************************************************************************/
{
    reflist_struct	*pair_ptr;

    ED_CALLOC(pair_ptr,    "init_refpairs", reflist_struct, 1);
    ED_CALLOC(pair_ptr->p, "init_refpairs", refpair_struct *, nref);

    return pair_ptr;
}
/**/
int get_refpairs(reflist_struct *pair_ptr, thresh_struct *thresholds, 
                 model_struct *model, int check, int calc_err, int store)
/****** get_refpairs ***************************************************
 *
 *  NAME	
 *      get_refpairs - Get a refernce pair member (check for thresh reject)
 *
 *  SYNOPSIS
 *       get_refpairs(refpair_struct **refpair, int check, int calc_err,
 *                    float *thresholds, model_struct *model)
 *
 *  FUNCTION
 *       This routine gets the next refefnce pair object from the input
 *       stream. It optionally checks for the rejection criteria and
 *       continues to get a reference pair until one falls within the
 *       criteria or until the end of the list is reached.
 *
 *  INPUTS
 *       check      - Do threshold rejection Yes or NO
 *       calc_err   - Calculate error statistics
 *       thresholds - The list of rejection thresholds
 *       model      - The full model structure
 *
 *  RESULTS
 *       n          - The number of reference pairs
 *
 *  RETURNS
 *       refpair    - Array of pointers to the current reference pair(s) 
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 16-07-1998
 *
 ******************************************************************************/
{
    double		ra, dec, d2;
    int			ncur, n, done = FALSE, something = FALSE;
    float		lthresh;
    refpair_struct	**p;

    n    = 0;
    p    = pair_ptr->p;
    ncur = pair_ptr->ncur;
    while (!done) {
      if (ncur < nref) 
      {
/*
 *      Just obtained a genuine star, lets process it
 */
        p[n] = &ref[ncur++];
        done = TRUE;
        if (check) 
        {
/*
 *        Optionally compute position and compare against reference 
 *        to reject outliers
 */
          compute_position(&p[n]->xobj, &p[n]->yobj, &p[n]->f,    &p[n]->c, 
                           &ra, &dec, 1, model);
          d2 = (ra - p[n]->ra)  * (ra - p[n]->ra) * cos(dec) * cos(dec) + 
               (dec - p[n]->dec) * (dec - p[n]->dec);
          lthresh = get_threshold(thresholds, p[n]->c, -1, -1, -1);
          if (d2 > lthresh * lthresh) {
            done = FALSE;
          } else {
            if (calc_err) 
              save_thresh_stats(thresholds, d2, p[n]->c, -1, -1, -1);
            if (store)
              save_refpair(p[n], ra, dec);
            n++;
            something = TRUE;
          }
        } else {
          n++;
          something = TRUE;
        }
      } else {
/*
 *      Bail out if we have reached the end of input
 */
        break;
      }
    }
    pair_ptr->npair = n;
    pair_ptr->ncur  = ncur;

    return something;
}
void free_refpairs(reflist_struct *p)
{
    ED_FREE(p->p);
    ED_FREE(p);
}
/**/
overlist_struct *new_overpairs()
/****** new_overpairs ***************************************************
 *
 *  NAME	
 *      new_overpairs - Start the overlap pair reading
 *
 *  SYNOPSIS
 *      void new_overpairs()
 *
 *  FUNCTION
 *      Initialize the overlap pair reading routines
 *
 *  INPUTS
 *      None
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *      curr_npair
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 16-07-1008
 *
 ******************************************************************************/
{
    overlist_struct	*pair_ptr;

    ED_CALLOC(pair_ptr,    "init_overpairs", overlist_struct, 1);
    if(nover > 0)
    {
      ED_CALLOC(pair_ptr->p, "init_overpairs", overpair_struct *, nover);
    }

    return pair_ptr;
}
/**/
int get_overpairs(overlist_struct *pair_ptr, thresh_struct *thresholds, 
                  model_struct *model, int check, int calc_err, int store)
/****** get_overpairs ***************************************************
 *
 *  NAME	
 *      get_overpairs - Get the next overlap pair from the list
 *
 *  SYNOPSIS
 *      int get_overpairs(overpair_struct *overpair, int check,
 *                        thresh_struct *thresholds, model_struct *model)
 *
 *  FUNCTION
 *       This routine gets the next overlap pair object from the input
 *       stream. It optionally checks for the rejection criteria and
 *       continues to get a ooverlap pairs until one falls within the
 *       criteria or until the end of the list is reached.
 *
 *  INPUTS
 *      check      - Check distance against threshold
 *      calc_err   - Calculate error statistics
 *      thresholds - Maximum distance to catalog position
 *      model      - The full model structure
 *
 *  RESULTS
 *      overpair   - Pointer to the current overlap pair
 *
 *  RETURNS
 *       True if the output contains a full set of information
 *       False if the end of the list is reached
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 17-07-1998
 *
 ******************************************************************************/
{     
    double		ra1, ra2, dec1, dec2, d2;
    float		lthresh;
    int			n, ncur, done = FALSE, something = FALSE;
    overpair_struct	**p;
    
    n    = 0;
    p    = pair_ptr->p;
    ncur = pair_ptr->ncur;
    while (!done) {
      if (ncur < nover) {
        p[n] = &over[ncur++];
        done = TRUE;
/*
 *      Just read a genuine objects pair, lets process it
 */
        if (check) {
/*
 *        Optionally compute position and compare against reference 
 *        to reject outliers
 */
          compute_position(&p[n]->xobj1, &p[n]->yobj1, &p[n]->f1, &p[n]->c1, 
                           &ra1, &dec1, 1, model);
          compute_position(&p[n]->xobj2, &p[n]->yobj2, &p[n]->f2, &p[n]->c2, 
                           &ra2, &dec2, 1, model);
          d2 = (ra1 - ra2) * (ra1 - ra2) * cos(dec1) * cos(dec1) +
               (dec1 - dec2) * (dec1 - dec2);
          lthresh = get_threshold(thresholds, 
                                   p[n]->c1, p[n]->c2, p[n]->f1, p[n]->f2);
          if (d2 > lthresh * lthresh) {
            done = FALSE;
          }  else {
            if (calc_err) 
              save_thresh_stats(thresholds, d2, 
                                p[n]->c1, p[n]->c2, p[n]->f1, p[n]->f2);
            if (store) {
              save_overpair(p[n], ra1, dec1, ra2, dec2);
            }
            n++;
            something = TRUE;
          }
        } else {
          n++;
          something = TRUE;
        }
      } else {
/*
 *      Bail out if we have reached the end of input
 */
        break;
      }
    }
    pair_ptr->npair = n;
    pair_ptr->ncur  = ncur;

    return something;
}
/**/
void free_overpairs(overlist_struct *p)
{
    ED_FREE(p->p);
    ED_FREE(p);
}
/**/
#define FKEYS 9

void open_catalogs(model_struct *model)
{
    int		i, j, nf, mpix;
    catstruct	**cat;
    tabstruct	**tab;
    field_params       ***field_desc;
    int		nfields, nfieldsfirst=0, fdeg[3];

    mpix = 0; nf = 0;
    ED_CALLOC(field_desc, "open_catalogs", field_params **, control.ilist);
    ED_CALLOC(chan_nr,    "open_catalogs", int,             control.ilist);
    ED_CALLOC(cat,        "open_catalogs", catstruct *,     control.ilist);
    ED_CALLOC(tab,        "open_catalogs", tabstruct *,     control.ilist);
    for (i=0;i<control.ilist;i++) {
      if (!(cat[i] = read_cat(control.listfile[i]))) {
        syserr(FATAL, "open_catalogs","Could not read catalog: %s\n", 
               control.listfile[i]);
      }
      if (!(tab[i] = name_to_tab(cat[i], "FIELDS", 0))) {
        syserr(FATAL, "open_catalogs","No FIELDS table in catalog %s",
               control.listfile[i]);
      }
      field_desc[i] = load_field_description(tab[i], &nfields,
                            control.crval1, control.crval2,
                            control.cdelt1, control.cdelt2,
                            control.crpix1, control.crpix2,
                            control.xwid, control.ywid);

      /* test whether all input catalogs have the same
         number of input fields
      */
      if(i==0)
      {
	nfieldsfirst = nfields;
      }
      else
      {
	if(nfields != nfieldsfirst)
	{
	  syserr(FATAL, "open_catalogs", 
                 "catalog %s has %d fields, while it should have %d as %s\n", 
                 control.listfile[i], nfields, nfieldsfirst, 
                 control.listfile[0]);
	}
      }
      /* the following is unnecessary as all catalogs have to have
         the same number of elements for successful processing.
      */
      nf = MAX(nf, nfields);
    }
    for (j=0;j<3;j++) fdeg[j] = (int) control.fdeg[j];
    init_model(control.pdeg, fdeg, control.ilist, nf, model);
    for (i=0;i<control.ilist;i++) 
    {
      chan_nr[i] = field_desc[i][0]->chan_nr;
      model->bands[i] = field_desc[i][0]->chan_nr;
      if (i == 0) 
      {
        model->xscale = field_desc[i][0]->cdelt1*DEG2RAD;
        model->yscale = field_desc[i][0]->cdelt2*DEG2RAD;
      }
      for (j=0;j<nfields;j++) {
        model->crpix[0][i][j]   = field_desc[i][j]->crpix1;
        model->crpix[1][i][j]   = field_desc[i][j]->crpix2;
        model->centers[0][i][j] = field_desc[i][j]->crval1*DEG2RAD;
        model->centers[1][i][j] = field_desc[i][j]->crval2*DEG2RAD;
        mpix = MAX(mpix, field_desc[i][j]->xwid);
        mpix = MAX(mpix, field_desc[i][j]->ywid);
        field_desc[i][j]->cdelt1 *= control.xpixsize;
        field_desc[i][j]->cdelt2 *= control.ypixsize;
      }
      close_cat(cat[i]);
    }
    model->mpix = 100 * ((mpix / 100) + 1);
    model->mf   = model->nf;
}
/**/
#define EPSR4 1E-7

void prepare_correl(model_struct *model, float *stderrs, short int *correl,
                    float scaling)
/*-------------------------------------------------------------------------
 * Prepare for storage from the covariance matrix a standard error and a 
 * correlation.
 * Parameters:
 *     COVAR   Real*4   Covariance matrix                         Input
 *     N       Integer  Dimension of covaraince matrix            Input
 *     STDERR  Real*4   Standard error vector                     Output
 *     CORREL  Int*2    Correlation matrix as scaled I*2          Output
 *     SCALING Real*8   Scaling factor for FITS correl data       Input
 *     ERROR   Logical  Error status                              Output
 *-------------------------------------------------------------------------*/
{
    int		i, j, k;
    float	corr;

    for (i=0;i<model->np+model->nf;i++) {
      if (model->alpha[i][i] >= 0) {
        stderrs[i] = sqrt(model->alpha[i][i]);
      } else {
         syserr(FATAL, "prepare_correl", 
       "Serious problem in covariance matrix: Diagonal element %d is negative",
        i);
      }
    }
    for (j=0,k=0;j<model->np+model->nf;j++) {
      for (i=j;i<model->np+model->nf;i++) {
        if (stderrs[i] > 0.0 && stderrs[j] > 0.0) {
          corr = model->alpha[i][j] / stderrs[i] / stderrs[j];
          if (fabs(corr) > (1 + 5.0 * EPSR4)) {
            syserr(FATAL, "prepare_correl",
        "Covariance matrix is non semi-definite positive!! ABS(%f) >1!!! for %d,%d\n", corr, i, j);
          }
          correl[k++] = (short int)(corr*scaling+0.5);
        } else {
          if (model->alpha[i][j] == 0.0) {
            correl[k++] = 0;
          } else {
            syserr(FATAL, "prepare_correl", 
        "Covariance matrix is non semi-definite positive!! %d %d %f %f %f\n",
          i,j, model->alpha[i][j], stderrs[i], stderrs[j]);
          }
        }
      }
    }
}
/**/
void assemble_correl(model_struct *model, float *stderrs, short int *correl,
                     float zero, float scaling)
/*----------------------------------------------------------------------
 * Astrom
 *      Rebuild a covariance matrix from standard error vector and
 *       correlation matrix
 * Arguments :
 *   N          Integer     Size of covariance matrix                 Input
 *   STDERRS    Real()      Standard errors                           Input
 *   CORRELAT   I*2(,)      Correlation matrix                        Input
 *   COVAR      Real(,)     Covariance matrix                         Output
 *   ZERO       Real(,)     Offset of correlation matrix              Input
 *   ZERO       Real(,)     Offset of correlation matrix              Input
 *   SCALING    Real(,)     Scale factor of correlation matrix        Input
 *--------------------------------------------------------------------------*/
{
    int		i, j, k;

    for (j=0,k=0;j<model->np+model->nf;j++) {
      for (i=j;i<model->np+model->nf;i++) {
        model->alpha[i][j] = (((float) correl[k]) / scaling + zero) *
                             stderrs[i] * stderrs[j];
        model->alpha[j][i] = model->alpha[i][j];
        k++;
      }
    }
}
/**/
void exam_covar(model_struct *model)
{
    int		i, j, count, work;
    double	a_max=-10000.0, a_min=10000.0, fraction, absolute;

    for (i=0;i<model->np+model->nf;i++) 
    {
       for (j=0;j<model->np+model->nf;j++) 
       {
         a_max = MAX(a_max,model->alpha[i][j]);
         a_min = MIN(a_min,model->alpha[i][j]);
       }
    }
    VPRINTF(VERBOSE, "Min and Max of covariance matrix:\n");
    VPRINTF(VERBOSE, "  (radians)**2           (\")**2\n");
    VPRINTF(VERBOSE, "%f %f %f %f\n",a_min,a_max,
              a_min*(180*3600/3.14159)*(180*3600/3.14159),
              a_max*(180*3600/3.14159)*(180*3600/3.14159));
    fraction = 1e-2;
    absolute = 3e-2*3.14159/180./3600.;
    count = 0;
    for (i=0;i<model->np+model->nf;i++) {
       for (j=0;j<model->np+model->nf;j++) {
          if ((model->alpha[i][i] > 0.0) &&
              (model->alpha[j][j] > 0.0)) {
             if (model->alpha[i][j] > absolute*absolute) {
                work = (int) (model->alpha[i][j]/sqrt(model->alpha[i][i]*
                         model->alpha[j][j])*1000.0);
                if (work > fraction) count += 1;
             } else work = 0;
          } else { 
             if (model->alpha[i][j] == 0)
                work = 0;
             else {
                VPRINTF(VERBOSE, 
                    "Covariance matrix is non semi-definite positive!!");
                 work = 0;
             }
          }
          VPRINTF(VERBOSE,"%5d ",work);
       }
       VPRINTF(VERBOSE,"\n");
    }
    VPRINTF(VERBOSE, "Correlation matrix has %d elements out of %d ",
                       count, (model->np+model->nf)*(model->np+model->nf));
    VPRINTF(VERBOSE, "above both %lf percent of diagonal and  %lf**2\n",
                       fraction*100, absolute);
}
/**/
void store_model(model_struct *model, thresh_struct *thresholds, 
                 option *my_opts, int argc, char **argv)
{
    int		i, j, k, n;

    int		*n_fields, *n_colors, *n_params, *n_fitdims, *n_mpix;
    int		*n_bands;
    double	*n_centers;
    float	*n_fitparms, *n_thresh, *fptr, *n_stderrs;
    short int	*n_correl;
    tabstruct	*astrom_tab;
    catstruct	*incat, *outcat;

    float	bscale;

    exam_covar(model);

    for (i=0;i<model->ncam;i++) {
      if (!(incat = read_cat(control.listfile[i]))) {
        syserr(FATAL, "open_catalogs","Could not read catalog: %s\n", 
               control.listfile[i]);
      }
      astrom_tab = new_tab("ASTROM");

      save_options(astrom_tab, my_opts, 0);

      ED_MALLOC(n_fields, "store_model", int , 1);
      n_fields[0] = model->nf;
      add_key_to_tab(astrom_tab, "NFIELDS", 1, n_fields, T_LONG, 1,
                             "", "Number of fields in model");
      ED_MALLOC(n_mpix, "store_model", int , 1);
      n_mpix[0] = model->mpix;
      add_key_to_tab(astrom_tab, "MPIX", 1, n_mpix, T_LONG, 1,
                             "pixels","Scaling factor in the model");
      ED_MALLOC(n_colors, "store_model", int , 1);
      n_colors[0] = model->ncam;
      add_key_to_tab(astrom_tab, "NCOLORS", 1, n_colors, T_LONG, 1,
                             "", "Number of bands in model");
      ED_MALLOC(n_bands, "store_model", int , model->ncam);
      for (j=0;j<model->ncam;j++)
        n_bands[j] = model->bands[j];
      add_key_to_tab(astrom_tab, "NBANDS", 1, n_bands, T_LONG, model->ncam,
                             "", "Band numbers in model");
      ED_MALLOC(n_params, "store_model", int , 4);
      n_params[0] = model->pdeg;
      n_params[1] = model->ffdeg[0];
      n_params[2] = model->ffdeg[1];
      n_params[3] = model->ffdeg[2];
      add_key_to_tab(astrom_tab, "CTRLPARM", 1, n_params, T_LONG, 4,
                             "", "Number of control paramaters in model");
      ED_MALLOC(n_fitdims, "store_model", int , 1);
      n_fitdims[0] = model->np+model->nf;
      add_key_to_tab(astrom_tab, "NFITPARM", 1, n_fitdims, T_LONG, 1,
                             "","Number of free paramaters in model");
      ED_MALLOC(n_centers, "store_model", double, (model->nf*2*model->ncam));
      n = 2*(model->nf*model->ncam);
      for (k=0;k<model->ncam;k++) {
        for (j=0;j<model->nf;j++) {
          n_centers[j*2  +k*model->nf*2] = model->centers[0][k][j]/DEG2RAD;
          n_centers[j*2+1+k*model->nf*2] = model->centers[1][k][j]/DEG2RAD;
        }
      }
      add_key_to_tab(astrom_tab, "CENTERS", 1, n_centers, T_DOUBLE, n,
                              "deg","Nominal field centers");
      ED_MALLOC(n_fitparms, "store_model", float, ((model->np+model->nf) * 2));
      n = 2*(model->nf+model->np);
      for (j=0;j<model->np+model->nf;j++) {
        n_fitparms[j*2]   = model->params[0][j];
        n_fitparms[j*2+1] = model->params[1][j];
      }
      add_key_to_tab(astrom_tab, "FITPARMS", 1, n_fitparms, T_FLOAT, n,
                              "","Fitted parameters");
      ED_CALLOC(n_thresh, "store_model", float, thresholds->numthresh);
      fptr = get_thresholds(thresholds);
      for (j=0;j<thresholds->numthresh;j++)
      {
        n_thresh[j] = fptr[j];
      }
      ED_FREE(fptr);
      add_key_to_tab(astrom_tab, "THRESHOLDS", 1, n_thresh, T_FLOAT,
                             thresholds->numthresh, "arcsec",
                             "List of thresholds");
      ED_MALLOC(n_stderrs, "store_model", float, (model->np+model->nf));
      n = ((model->np+model->nf) * (model->np+model->nf+1)) / 2;
      ED_MALLOC(n_correl, "store_model", short int, n);
      bscale = 32767.0;
      prepare_correl(model, n_stderrs, n_correl, bscale);
      add_key_to_tab(astrom_tab, "STDERRS", 1, n_stderrs, T_FLOAT, 
                  model->np+model->nf, "rad",
                  "Standard errors on fitted parameters");
      add_key_to_tab(astrom_tab, "CORRELAT", 1, n_correl, T_SHORT, n,
                             "","Correlation matrix");
      outcat = new_cat(1);
      inherit_cat(incat, outcat);
      if (add_tab(astrom_tab, outcat, 1) == RETURN_ERROR) {
        syserr(FATAL,"main","Error adding table to catalog\n");
      }
      copy_tabs(incat, outcat);
      historyto_cat(outcat, "astrom", argc, argv);
      VPRINTF(NORMAL,"Saving catalog %s\n", control.outfile[i]);
      save_cat(outcat, control.outfile[i]);
    }
}
/**/
void fetch_model(tabstruct *intab, model_struct *model, 
                 thresh_struct **thresholds)
/*----------------------------------------------------------------------
 * Astrom
 *	Store the results of the plate model adjustment as a FITS binary table
 * Arguments :
 *   STRIP      Integer     Strip number                              Input
 *   NF         Integer     Number of fields in strip                 Input
 *   NBAND      Integer     Number of colour bands                    Input
 *   IBAND      Integer     Colour numbers                            Input
 *   PDEG       Integer     X and Y degree of plate model             Input
 *   FDEG       Integer     Field number degree of plate model        Input
 *   BETA       Real        Plate model coefficients                  Input
 *   NPAR       Integer     Number of coefficients in plate model     Input
 *   CENTERS    Real*8      Nominal field centers                     Input
 *   COVAR      Real        Covariance matrix of coefficients         Input
 *   COVAR_EXIST            Covariance matrix available?              Input
 *   THRESHOLD  Real        Outlier rejection threshold               Input
 *   MODEL_TYPE Ch(*)       Name of plate model solution              Input
 *   ERROR      Logical                                               Output
 *--------------------------------------------------------------------------*/
{
    keystruct	*key;
    int		j, k, n, *n_fields, *n_colors, *n_param, *n_fitdim, *n_mpix;
    double	*n_centers;
    float	*n_fitparm, *n_thresh, *n_stderr, bzero, bscale;
    short int	*n_correl;
    int		pdeg, fdeg[3], ncam, nf, np, *n_bands;
/*
 *  First get the primary information to build the model structure
 */
    ED_GETKEY(intab, key, "NFIELDS", n_fields, n, "fetch_model");
    nf = n_fields[0];
    model->mf = nf;
/*    if (n > 1) ED_FREE(n_fields); */
    ED_GETKEY(intab, key, "NCOLORS", n_colors, n, "fetch_model");
    ncam = n_colors[0];
/*   if (n > 1) ED_FREE(n_colors); */
    ED_GETKEY(intab, key, "NBANDS", n_bands, n, "fetch_model");
    ED_GETKEY(intab, key, "CTRLPARM", n_param, n, "fetch_model");
    pdeg = n_param[0];
    fdeg[0] = n_param[1];
    fdeg[1] = n_param[2];
    fdeg[2] = n_param[3];
/*  if (n > 1) ED_FREE(n_param); */
    ED_GETKEY(intab, key, "NFITPARM", n_fitdim, n, "fetch_model");
    np = n_fitdim[0];
    init_model(pdeg, fdeg, ncam, nf, model);
/*  if (n > 1) ED_FREE(n_fitdim); */
    if (np != (model->np+model->nf))
      syserr(FATAL, "fetch_model", "Discrepancy in model description\n");
/*
 *  Fill out the rest of the model structure
 */
    ED_GETKEY(intab, key, "MPIX", n_mpix, n, "fetch_model");
    model->mpix= n_mpix[0];
/*  if (n > 1) ED_FREE(n_mpix); */
    ED_GETKEY(intab, key, "CENTERS", n_centers, n, "fetch_model");
    for (k=0;k<model->ncam;k++) {
      model->bands[k] = n_bands[k];
      for (j=0;j<model->nf;j++) {
        model->centers[0][k][j] = n_centers[j*2  +k*model->nf*2]*DEG2RAD;
        model->centers[1][k][j] = n_centers[j*2+1+k*model->nf*2]*DEG2RAD;
      }
    }
/*  if (n > 1) ED_FREE(n_centers); */
    ED_GETKEY(intab, key, "FITPARMS", n_fitparm, n, "fetch_model");
    for (j=0;j<model->np+model->nf;j++) {
      model->params[0][j] = n_fitparm[j*2];
      model->params[1][j] = n_fitparm[j*2+1];
    }
/*  if (n > 1) ED_FREE(n_fitparm); */
    ED_GETKEY(intab, key, "THRESHOLDS", n_thresh, n, "fetch_model");
    *thresholds = init_thresholds(model->ncam);
    put_thresholds(*thresholds, n_thresh);
/*  if (n > 1) ED_FREE(n_thresh); */
    ED_GETKEY(intab, key, "STDERRS", n_stderr, n, "fetch_model");
    ED_GETKEY(intab, key, "CORRELAT", n_correl, n, "fetch_model");
    n = ((model->np+model->nf) * ((model->np+model->nf)+1)) / 2;
    bzero = 0.0; bscale = 32767.0;
    assemble_correl(model, n_stderr, n_correl, bzero, bscale);
/*  if (n > 1) ED_FREE(n_stderr); */
/*  if (n > 1) ED_FREE(n_correl); */
}
/**/
void init_residuals(char *outfile, option *my_opts, char *header)
{
    float	*fptr;
    double	*dptr;
    LONG	*lptr;

    ED_CALLOC(res_keys, "init_residuals", keystruct *, 12);
    res_cat = new_cat(1);
    init_cat(res_cat);
    addhistoryto_cat(res_cat, header, 1);

    strcpy(res_cat->filename, outfile);

    if (open_cat(res_cat, WRITE_ONLY) != RETURN_OK)
    {
      syserr(FATAL, "make_residuals","Cannot open output catalog %s\n",
             outfile);
    }

    res_tab = new_tab("RESIDUALS");

    save_options(res_tab, my_opts, 0);

    ED_CALLOC(fptr, "make_residuals", float , 1);
    res_keys[DRA] = add_key_to_tab(res_tab, "DRa", 1, fptr, T_FLOAT, 1,
                             "Deg", "Right Ascension residual");
    ED_CALLOC(fptr, "make_residuals", float , 1);
    res_keys[DDEC] = add_key_to_tab(res_tab, "DDec", 1, fptr, T_FLOAT, 1,
                             "Deg", "Declination residual");
    ED_CALLOC(dptr, "make_residuals", double, 1);
    res_keys[RA] = add_key_to_tab(res_tab, "Ra", 1, dptr, T_DOUBLE, 1,
                             "Deg", "Right Ascension object");
    ED_CALLOC(dptr, "make_residuals", double, 1);
    res_keys[DEC] = add_key_to_tab(res_tab, "Dec", 1, dptr, T_DOUBLE, 1,
                             "Deg", "Declination object");
    ED_CALLOC(fptr, "make_residuals", float , 1);
    res_keys[X1] = add_key_to_tab(res_tab, "Xpos1", 1, fptr, T_FLOAT, 1,
                             "Deg", "X position first object");
    ED_CALLOC(fptr, "make_residuals", float , 1);
    res_keys[Y1] = add_key_to_tab(res_tab, "Ypos1", 1, fptr, T_FLOAT, 1,
                             "Deg", "X position second object");
    ED_CALLOC(fptr, "make_residuals", float , 1);
    res_keys[X2] = add_key_to_tab(res_tab, "Xpos2", 1, fptr, T_FLOAT, 1,
                             "Deg", "Y position first object");
    ED_CALLOC(fptr, "make_residuals", float , 1);
    res_keys[Y2] = add_key_to_tab(res_tab, "Ypos2", 1, fptr, T_FLOAT, 1,
                             "Deg", "Y position second object");
    ED_CALLOC(lptr, "make_residuals", LONG, 1);
    res_keys[F1] = add_key_to_tab(res_tab, "F1", 1, lptr, T_LONG, 1,
                             "Deg", "Field number first object");
    ED_CALLOC(lptr, "make_residuals", LONG, 1);
    res_keys[F2] = add_key_to_tab(res_tab, "F2", 1, lptr, T_LONG, 1,
                             "Deg", "Field number second object");
    ED_CALLOC(lptr, "make_residuals", LONG, 1);
    res_keys[C1] = add_key_to_tab(res_tab, "C1", 1, lptr, T_LONG, 1,
                             "Deg", "Camera number first object");
    ED_CALLOC(lptr, "make_residuals", LONG, 1);
    res_keys[C2] = add_key_to_tab(res_tab, "C2", 1, lptr, T_LONG, 1,
                             "Deg", "Camera number second object");


    res_tab->cat = res_cat;
    init_writeobj(res_cat, res_tab);
}
#define NAMELEN 16
/*
 *   Global parameters for the plate distortion table
 */
static float	**x_distortions, **y_distortions, *xoffs, *yoffs;
static char	**names;
static int	poldim, num_np, num_flds, num_cam;
static LONG     *maxpix;

void init_distort_tab(model_struct *model)
{
/****** init_distort_tab ***************************************************
 *
 *  NAME	
 *      init_distort_tab - Initialize the plate distortion table
 *
 *  SYNOPSIS
 *      void init_distort_tab(int *nfields, int *ndim, int *nbands, int *mpix)
 *
 *  FUNCTION
 *      This routine initializes the structures required to store the 
 *      plate distortion parameters. It should be called once as the first
 *      routine that has to do with the creation of the plate distortion
 *      tables. 
 *
 *  INPUTS
 *      nfields   - The number of fields in the catalog
 *      ndim      - The polynomial dimension of the plate distortions
 *      nbands    - The number of separated catalog channels
 *      mpix      - The maximum pixel number (a scaling unit)
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *      The global structures for storing the plate distortion parameters
 *      are set up.
 *
 *  NOTES
 *      It is assumed that the plate distortion plynomial dimension of
 *      all fields is the same.
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 14-03-1997
 *
 ******************************************************************************/
    int		i, j;

    ED_CALLOC(names, "init_distort_tab", char *, model->np);
    for (i=0;i<model->np;i++) 
      ED_CALLOC(names[i], "init_distort_tab", char, NAMELEN);
    ED_CALLOC(x_distortions,  "init_distort_tab", float *, model->np);
    ED_CALLOC(y_distortions,  "init_distort_tab", float *, model->np);
    for (j=0;j<model->np;j++) {
      ED_CALLOC(x_distortions[j],  "init_distort_tab", float, model->nf);
      ED_CALLOC(y_distortions[j],  "init_distort_tab", float, model->nf);
    }
    ED_CALLOC(maxpix,   "init_distort_tab", LONG  , model->nf);   
    ED_CALLOC(xoffs,    "init_distort_tab", float , model->nf);   
    ED_CALLOC(yoffs,    "init_distort_tab", float , model->nf);   
    poldim = model->np;
    num_flds = model->nf;
    num_cam  = model->ncam;
    for (j=0;j<model->nf;j++) {
      maxpix[j] = model->mpix;
    }
}
void add_distort_line(int fieldnr, int nband, 
                      float **distort, int num, model_struct *model)
{
/****** add_distort_line ***************************************************
 *
 *  NAME	
 *      add_distort_line - Add a plate distortion parameter set
 *
 *  SYNOPSIS
 *      void add_distort_line(int *fieldnr, int *nband, float *distort)
 *
 *  FUNCTION
 *      Add the plate distortion parameters for a given field number to the
 *      global plate distortion parameter structure so that is can later be
 *      saved into the catalog as a table.
 *
 *  INPUTS
 *      fieldnr    - The field number of the plate distortion parameters
 *      nband      - The band number of the plate distortion parameters
 *      distort    - The array of plate distortion parameters
 *      offs       - The image center offset in radians (if nband != 1)
 *      centers    - The true center coordinates 
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *      The plate distortion parameters of the given plate are inserted
 *      at the correct position in the global structure
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *
 ******************************************************************************/
    int		i;

    if (fieldnr < 0 || fieldnr > num_flds) 
       syserr(FATAL, "add_distort_line", "Field number out of bounds");
    if (nband < 0 || nband > num_cam) 
       syserr(FATAL, "add_distort_line", "Camera number out of bounds");
    num_np = num;
    VPRINTF(NORMAL,"Saving plate distortions for field %d\n", fieldnr);
    if (nband == 0) {
      for (i = 0; i < num_np; i++) {
         x_distortions[i][fieldnr] = distort[0][i];
         y_distortions[i][fieldnr] = distort[1][i];
         strncpy(names[i], model->names[i], NAMELEN - 1);
      }
    } else {
      for (i = 0; i < num_np - 1; i++) {
         x_distortions[i][fieldnr] = distort[0][i+1];
         y_distortions[i][fieldnr] = distort[1][i+1];
         strncpy(names[i], model->names[i + 1], NAMELEN - 1);
      }
    }
    if (nband == 0) {
      xoffs[fieldnr] = model->params[0][fieldnr+model->np];
      yoffs[fieldnr] = model->params[1][fieldnr+model->np];
    } else {
      xoffs[fieldnr] = model->params[0][fieldnr+model->np] + distort[0][0];
      yoffs[fieldnr] = model->params[1][fieldnr+model->np] + distort[1][0];
    }
}


void save_distort_tab(catstruct *incat, char *outname, int c, 
                      char *programname, int argc, char **argv)
{
/****** save_distort_tab ***************************************************
 *
 *  NAME	
 *      save_distort_tab - Save the plate distortion global structure
 *
 *  SYNOPSIS
 *      void save_distort_tab()
 *
 *  FUNCTION
 *      This function writes the global plate distortion structure that was 
 *      filled by previous calls to the other routines in this module to
 *      the output catalogs. Each color has its own set of distortion
 *      polynomials.
 *
 *  INPUTS
 *      None
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R Deul - dd 14-03-1997
 *
 ******************************************************************************/
/*
 *  Function called from ASTROM to copy the original catalogs and paste the
 *  distort_tab tables into them
 */
    int		j, n;
    short int	*fp;
    catstruct   *outcat;
    tabstruct	*tab;
    char	name[MAXCHAR];

    ED_CALLOC(fp, "save_distort_tab", short int, num_flds);
    for (j=0;j<num_flds;j++) 
       fp[j] = j+1;
    tab = new_tab("DISTORTIONS");
    add_key_to_tab(tab, "FIELD_POS", num_flds, fp, T_SHORT,
                   1, "", "Field position");
    add_key_to_tab(tab, "MPIX", num_flds,  maxpix, T_LONG,
                   1, "pixel", "Maximum pixel range (Scaling parameter)");
    add_key_to_tab(tab, "PV1_1", num_flds, xoffs, T_FLOAT,
                   1, "rad", "Right Ascension offset from reference pixel");
    add_key_to_tab(tab, "PV1_2",num_flds, yoffs, T_FLOAT,
                   1, "rad", "Declination offset from reference pixel");
    n = num_np;
    if (c != 0) n--;
    for (j=0;j<n;j++) {
        sprintf(name, "PV%d_1", j+2);
        add_key_to_tab(tab, name, num_flds, 
                       x_distortions[j], T_FLOAT, 1,
                       "", names[j]);
        sprintf(name, "PV%d_2", j+2);
        add_key_to_tab(tab, name, num_flds, 
                       y_distortions[j], T_FLOAT, 1,
                       "", names[j]);
    }
    update_tab(tab);
    outcat = new_cat(1);
    inherit_cat(incat, outcat);
    if (add_tab(tab,outcat,1) == RETURN_ERROR) {
       syserr(FATAL,"main","Error adding table to catalog\n");
    }
    copy_tabs(incat, outcat);
    historyto_cat(outcat, programname, argc, argv);
    VPRINTF(NORMAL,"Saving output catalog %s\n", outname);
    save_cat(outcat, outname);
}

