/* 
   11.08.00:
   fixes a bug for the DATA_TYPE BOTH case:
   when calibrating several fields, the offset
   determinations of the previous field
   were used as starting point for the next field.
   The parameters a0_old and b0_old have to be
   reinitialised in this case.

   19.07.01:
   function make_pairing:
   In the calculation of the alapha and delta range to cover
   with reference objects, absolute values of the PC-matrix have
   to be taken to make the code work with rotated images.

   29.04.03:
   included 'string.h' to avoid compiler warnings

   20.07.2004:
   I removed compiler warnings due to possibly
   undefined variables (compilation with optimisation
   turned on)

   16.01.2005:
   I removed the definition from the constrolstruct
   for the preastrom program from preastrom_globals.h
   and added it to preastrom.c. Im make_pairing.c,
   an extern definition was added.

   11.03.2005:
   Changes to avoid compiler warnings with '-Wall -W -pedantic'

   27.03.2006:
   I renamed sorting and indexing functions to the new naming scheme
   described in the sort_globals.h file

   30.12.2012:
   I removed compiler warnings (set but not used variables)
*/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include "fitscat.h"
#include "utils_globals.h"
#include "tabutil_globals.h"
#include "options_globals.h"
#include "precess_defs.h"
#include "preastrom_globals.h"
#include "refs_io_defs.h"
#include "refs_io_types.h"
#include "refs_io_globals.h"
#include "field_desc.h"
#include "icat_globals.h"
#include "dist_globals.h"
#include "load_field_globals.h"
#include "trans_globals.h"
#include "coord_trans.h"
#include "offsets_globals.h"
#include "sort_globals.h"

/* the control_struct is defined in preastrom.c */
extern control_struct control;

static int select_brightest(float *x, float *y, float *p, int *id,
                            int n, double minp)
{
     int i, id_temp;
     int *x_id;

/*
 *   First sort on phot par (minp > 0) or magnitude (minp < 0)
 */
     ED_CALLOC(x_id, "select_brightest", int, n);
     if (n > 1) {
        if (minp < 0.0) {
           findex(n, p, x_id);
           fffisort(n, p, x, y, id);
        } else {
           findex(n, p, x_id);
           for (i=0;i<n/2;i++) {
              id_temp = x_id[i];
              x_id[i] = x_id[n-1-i];
              x_id[n-1-i] = id_temp;
           }
           fffirsort(n, p, x, y, id);
        }
     }
     for (i=0;i<n;i++) {
        if (minp < 0.0) {
           if (p[i] > -minp)
/*
              if (i > control.sel_min) break;
 */
           break;
        } else {
           if (p[i] < minp)
/*
              if (i > control.sel_min*4) break;
 */
           break;
        }
     }
     ED_FREE(x_id);
     return ((i<1)?0:i-1);
}


void make_pairing(catstruct *incat, catstruct *outcat, catstruct *poscat,
                  catstruct *distcat, char *directory)
{
    tabstruct	*tab, *ftab, *postab, *disttab;
    int		i, nfields;
    double	rac, decc;
    float	*gx, *gy;
    int	        *g_i;
    int		obj_prev, *obj_pos_nr, *obj_pos, 
                obj_prev_dist=0, *obj_dist_nr=NULL, 
                *obj_dist=NULL;
    float	*x = NULL, *y = NULL, *p = NULL;
    int         *id = NULL;
    float	a0, a1, a2, b0, b1, b2;
    float	t_a0 = 0.0, t_a1 = 1.0, t_a2 = 0.0, 
                t_b0 = 0.0, t_b1 = 0.0, t_b2 = 1.0;
    float	a0_old = control.aff_p[0], a1_old = control.aff_p[1], 
                a2_old = control.aff_p[2], b0_old = control.aff_p[3],
                b1_old = control.aff_p[4], b2_old = control.aff_p[5], 
                ea0, eb0;
    int		nobj, sobj, robj, iter, piter, ok;

    ref_objs    *refs;
    float       dra,ddec;
    double	poserr_corr, maxoff;
    affine_params **affine_p, *l_affine_p;
    field_params **field_p, *tfp;
    int		nobj_tot;
 
    if (!(ftab = name_to_tab(incat, "FIELDS", 0)))
       syserr(FATAL, "make_pairing", "No table FIELDS in catalog %s\n",
          incat->filename);
    field_p = load_field_description(ftab, &nfields, control.crval1,
                  control.crval2, control.cdelt1, control.cdelt2,
                  control.crpix1, control.crpix2, control.xwid,
                  control.ywid);
    if (!(tab = name_to_tab(incat, "OBJECTS", 0)))
       syserr(FATAL, "make_pairing", "No table OBJECTS in catalog %s\n",
          incat->filename);
    ED_CALLOC(obj_pos_nr,   "make_pairing", int, nfields);
    ED_CALLOC(obj_pos,      "make_pairing", int, nfields);
    obj_prev = 0;

    postab = icat_fields_create(field_p, nfields);
    if (strcmp(control.data_type, "DENIS")==0 ||
        strcmp(control.data_type, "OOPS")==0 ||
        strcmp(control.data_type, "BOTH")==0) {
       disttab = dist_fields_create(field_p, nfields);
       ED_CALLOC(obj_dist_nr,  "make_pairing", int, nfields);
       ED_CALLOC(obj_dist,     "make_pairing", int, nfields);
       obj_prev_dist = 0;
    } else {
       disttab = NULL;
       distcat = NULL;
    }
    ED_CALLOC(affine_p, "make_pairing", affine_params *, nfields);
    ED_CALLOC(l_affine_p, "make_pairing", affine_params, nfields);
    for (i = 0, nobj_tot = 0; i < nfields; i++) {
       affine_p[i] = init_coord_trans("");
       tfp = field_p[i];
       nobj = load_field(tab, ftab, &x, control.xpos, &y, control.ypos,
                                    &p, control.phot, &id, control.seqnr, 
                                    i+1, (short int) control.flag_mask);
       nobj_tot += nobj;
       VPRINTF(VERBOSE,"#%d %f %f %f %f %f %f %d %d %f %f %f %f\n",  i,
                       tfp->crval1, tfp->crval2, tfp->cdelt1, tfp->cdelt2,
                       tfp->crpix1, tfp->crpix2, tfp->xwid,   tfp->ywid,
                       tfp->pc11,   tfp->pc12,   tfp->pc21,   tfp->pc22);
       dra  = (fabs(tfp->cdelt1 * 
               (fabs(tfp->pc11) * tfp->xwid + fabs(tfp->pc12) * tfp->ywid))) /
              cos(tfp->crval2 * DEG2RAD) * 
              sqrt(fabs(tfp->pc22) + fabs(tfp->pc12));
       ddec = (fabs(tfp->cdelt2 * 
               (fabs(tfp->pc21) * tfp->xwid + fabs(tfp->pc22) * tfp->ywid))) *
              sqrt(fabs(tfp->pc22) + fabs(tfp->pc12));
       rac  = tfp->crval1  + 
              (tfp->pc11 * (tfp->xwid/2.0 - tfp->crpix1) +
               tfp->pc12 * (tfp->ywid/2.0 - tfp->crpix2)) * 
              tfp->cdelt1/ cos(tfp->crval2 * DEG2RAD);
       decc = tfp->crval2 + 
              (tfp->pc21 * (tfp->xwid/2.0 - tfp->crpix1) +
               tfp->pc22 * (tfp->ywid/2.0 - tfp->crpix2)) * tfp->cdelt2;
       if (strcmp(control.data_type, "DENIS")==0 || 
           strcmp(control.data_type, "OOPS") ==0) 
       {
         tfp->cdelt1 = fabs(tfp->cdelt1) * control.xpixsize;
         tfp->cdelt2 = fabs(tfp->cdelt2) * control.ypixsize;
       } else {
         tfp->cdelt1 *= control.xpixsize;
         tfp->cdelt2 *= control.ypixsize;
       }
       VPRINTF(VERBOSE,"#%s %f %f %f %f\n", control.data_type, 
              rac, decc, dra, ddec);
       VPRINTF(VERBOSE,"#%s %f %f\n", control.data_type, 
              tfp->cdelt1, tfp->cdelt2);
       refs = get_refs(directory, rac, decc, dra, ddec);
       if (!refs->nobj)
          syserr(FATAL, "make_pairing", "No reference objects found!\n");
       VPRINTF(NORMAL,"REF: %d objects %f %f %f %f\n",
            refs->nobj, rac, decc, dra, ddec);
       ED_CALLOC(gx,      "make_pairing", float,    refs->nobj);
       ED_CALLOC(gy,      "make_pairing", float,    refs->nobj);
       ED_CALLOC(g_i,     "make_pairing", int,      refs->nobj);
       rd2xy(tfp->crval1, tfp->crval2, tfp->cdelt1, tfp->cdelt2,
             tfp->crpix1, tfp->crpix2, tfp->pc11,   tfp->pc12,
             tfp->pc21,   tfp->pc22,
             refs->ra, refs->dec, gx, gy, refs->nobj);
       obj_pos_nr[i]  = refs->nobj;
       obj_pos[i]     = obj_prev;
       obj_prev       = obj_pos[i] + obj_pos_nr[i];
       icat_objects_create(poscat, refs->nobj, i+1,
                           refs->ra, refs->dec, control.ref_err, 
                           gx, gy, refs->bmag, refs->rmag, obj_pos[i]+1, g_i);
       robj = select_brightest(gx, gy, refs->bmag, g_i, refs->nobj, 
                               control.minp[1]);
       sobj = select_brightest(x, y, p, id, nobj, control.minp[0]);
       ok= FALSE; iter = 0;
       poserr_corr = 4.0/(1.0+cos(decc*DEG2RAD));
       if (!(strcmp(control.data_type, "DENIS")==0) &&
           !(strcmp(control.data_type, "OOPS") ==0) &&
           !(strcmp(control.data_type, "BOTH") ==0)) {
         while (!ok && iter < 1) {
         int smin;
            iter++;
            piter = 0;
            while (!ok && piter < 3) {
               piter++;
               smin=MIN(control.sel_min,nobj);
               smin=MIN(smin,robj);
               ok = transform(sobj,  x,  y, p,
                              robj, gx, gy, refs->bmag, 
                              (control.maxoff * iter), (float)control.rms_tol, 
                              (float)control.triangle_tol, 
                              control.xmin, control.xmax,
                              control.ymin, control.ymax,0.0,0.0,
                              (smin * piter),
                              &a0, &a1, &a2, &b0, &b1, &b2);
               VPRINTF(NORMAL,"Trans(%d): %f %f %f %f %f %f (%d,%d)\n", 
                         i+1, a0, a1, a2, b0, b1, b2, iter, piter);
            }
         } 
         a0_old = a0;
         a1_old = a1;
         a2_old = a2;
         b0_old = b0;
         b1_old = b1;
         b2_old = b2;
         update_coord_trans(affine_p[i], a0, a1, a2, b0, b1, b2);
         apply_coord_trans(affine_p[i], x, y, nobj);
       } 
       else 
       {
          a0_old = control.aff_p[0];
          b0_old = control.aff_p[3];

	  update_coord_trans(affine_p[i], 0.0, a1_old, a2_old, 
                                          0.0, b1_old, b2_old);
          apply_coord_trans(affine_p[i], x, y, nobj);
          sobj = select_brightest(x, y, p, id, nobj, control.minp[0]);
          find_offset(x,y,MIN(MAX(control.sel_min*4,MIN(sobj*4,nobj)),
                      nobj), 0.0, 0.0, gx,gy,
                      MIN(MAX(control.sel_min*4,MIN(robj*4,refs->nobj)),
                      refs->nobj), 0.0, 0.0,
                      control.maxoff, control.poserr, control.maxdist, 
                      (int) field_p[i]->ywid, (int) field_p[i]->ywid, 
                      &a0, &b0, &ea0, &eb0, 
                      i+1, &obj_dist_nr[i], distcat);
          obj_dist[i]   = obj_prev_dist;
          obj_prev_dist = obj_dist[i] + obj_dist_nr[i];
          if (strcmp(control.data_type, "DENIS")==0 || 
              strcmp(control.data_type, "OOPS")==0 ||
              strcmp(control.data_type, "BOTH")==0) {
	      VPRINTF(NORMAL,
                      "Checking (%.5f+-%.5f,%.5f+-%.5f) against (%.5f,%.5f)\n",
                      a0,ea0,b0,eb0,a0_old,b0_old);
             if ((a0_old==control.aff_p[0]) && (b0_old==control.aff_p[3])) {
/*
 *             This case is when no previous offset has/could be determined
 */
               if (fabs(ea0-eb0) < 1.e-5 && fabs(ea0-control.maxdist) < 1.e5) {
/*
 *               The offset has not been determined correctly
 */
                 update_coord_trans(affine_p[i], a0_old, a1_old, a2_old, 
                                                 b0_old, b1_old, b2_old);
                 update_coord_trans(l_affine_p,  a0_old, 1.0, 0.0, 
                                                 b0_old, 0.0, 1.0);
               } else {
/*
 *               The offset is most probably ok
 */
                 update_coord_trans(affine_p[i], a0, a1_old, a2_old, 
                                                 b0, b1_old, b2_old);
                 update_coord_trans(l_affine_p,  a0, 1.0, 0.0, 
                                                 b0, 0.0, 1.0);
                 a0_old = a0;
                 b0_old = b0;
               }
             } else {
/*
 *             We have a previous determination of offset
 */
               if (fabs(ea0-eb0) < 1.e-5 && fabs(ea0-control.maxdist) < 1.e5) {
/*
 *               The offset has not been determined correctly
 */
                 update_coord_trans(affine_p[i], a0_old, a1_old, a2_old, 
                                                 b0_old, b1_old, b2_old);
                 update_coord_trans(l_affine_p,  a0_old, 1.0, 0.0, 
                                                 b0_old, 0.0, 1.0);
               } else {
                 if (((fabs(a0-a0_old) < ea0*control.fudge) && 
                      (fabs(b0-b0_old) < eb0*control.fudge)
                    && i > 1) ||
                     (((fabs(a0-a0_old) > ea0*control.fudge) || 
                       (fabs(b0-b0_old) > eb0*control.fudge))
                    && i == 1)) {
/*
 *                 See if the new determination is within bounds
 *                 Either within bound for 3rd or higher frame#
 *                 or     out of bounds for 2nd frame
 */
                   update_coord_trans(affine_p[i], a0, a1_old, a2_old, 
                                                   b0, b1_old, b2_old);
                   update_coord_trans(l_affine_p,  a0, 1.0, 0.0, 
                                                   b0, 0.0, 1.0);
                   a0_old = a0;
                   b0_old = b0;
                 } else {
/*
 *                 Stay with old info, new determination out of bounds
 */
                   update_coord_trans(affine_p[i], a0_old, a1_old, a2_old, 
                                                   b0_old, b1_old, b2_old);
                   update_coord_trans(l_affine_p,  a0_old, 1.0, 0.0, 
                                                   b0_old, 0.0, 1.0);
                 }
               }
             }
          }
          if ((strcmp(control.data_type, "OOPS") == 0) ||
              (strcmp(control.data_type, "BOTH") == 0)) {
            apply_coord_trans(l_affine_p, x, y, nobj);
            maxoff=MAX(ea0,eb0)*3.0;
            ok = 0;
            if (sobj && robj) {
              ok = transform(nobj,  x,  y, p,
                             robj, gx, gy, refs->bmag, 
                             maxoff, (float)control.rms_tol, 
                             (float)control.triangle_tol, 
                             control.xmin, control.xmax,
                             control.ymin, control.ymax,0.0,0.0,
                             MIN(nobj,MIN(robj,control.sel_min)),
                             &a0, &a1, &a2, &b0, &b1, &b2);
            }
            if (ok && (fabs(a1-1.0) < 0.02) && (fabs(a2) < 0.02) 
                   && (fabs(b1) < 0.02) && (fabs(b2-1.0) < 0.02)
                   && ((fabs(a0-a0_old) < control.poserr*poserr_corr) || 
                       (fabs(b0-b0_old) < control.poserr*poserr_corr))) {
              t_a0 = a0; t_a1 = a1; t_a2 = a2; t_b0 = b0; t_b1 = b1; t_b2 = b2;
              VPRINTF(NORMAL,"Trans(%d,curr): %f %f %f %f %f %f (%d)\n", 
                      i+1, a0, a1, a2, b0, b1, b2, ok);
            }
            VPRINTF(NORMAL,"Trans(%d,prev): %f %f %f %f %f %f \n", 
                    i+1,a0_old,a1_old,a2_old,b0_old,b1_old,b2_old);
            update_coord_trans(affine_p[i], 
                   a0_old + a1_old*t_a0 + a2_old*t_b0,
                   a1_old*t_a1+a2_old*t_b1, a1_old*t_a2+a2_old*t_b2, 
                   b0_old+b1_old*t_a1 + b2_old*t_b0, 
                   b1_old*t_a1+b2_old*t_b1, b1_old*t_a2+b2_old*t_b2);
          }
       } 
       free_ref_obj(refs);
       ED_FREE(x);
       ED_FREE(y);
       ED_FREE(p);
       ED_FREE(id);
       ED_FREE(gx);
       ED_FREE(gy);
       ED_FREE(g_i);
    }
    icat_fields_write(poscat, postab, obj_pos, obj_pos_nr, nfields);
    if (strcmp(control.data_type, "DENIS")==0 || 
        strcmp(control.data_type, "OOPS")==0 ||
        strcmp(control.data_type, "BOTH")==0) {
       dist_fields_write(distcat, disttab, obj_dist, obj_dist_nr, nfields);
    }
    tab = create_affine_tab(affine_p, nfields);
    if (name_to_tab(outcat, tab->extname, 0)) {
       remove_tab(outcat, tab->extname, 0);
    }
    if (add_tab(tab, outcat, 0) == RETURN_ERROR) 
       syserr(FATAL, "make_pairing", 
           "Error adding table %s to output catalog", tab->extname);
    tab = name_to_tab(outcat, "OBJECTS", 0);
    free_field_desc(field_p, nfields);
    for (i=0;i<nfields;i++) {
       ED_FREE(affine_p[i]);
    }
    ED_FREE(affine_p);
    ED_FREE(l_affine_p);
}    
