#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "fitscat.h"
#include "sort_globals.h"
#include "options_globals.h"
#include "utils_globals.h"

#define MAXNUM 200

/*
   20.07.2004:
   I removed compiler warnings due to possibly
   undefined variables (compilation with optimisation
   turned on)

   27.03.2006:
   I renamed sorting and indexing functions to the new naming scheme
   described in the sort_globals.h file
*/

int pairing(float *xa, float *ya, int na, 
            float *xb, float *yb, int nb, 
            double err, int *pairs, int p_off, int *b_pairs, int b_off)
{
      int	lastj, i, j, inum, count = 0;
      double	xdif, ydif;
      float     diff[MAXNUM], min_diff;
      int	jiff[MAXNUM], min_jiff=0, old_jiff, 
                min_j=0, *idx = NULL;
      int	*idxa=NULL, *idxb=NULL;

      ED_CALLOC(idxa, "pairing", int, na);
      ED_CALLOC(idxb, "pairing", int, nb);
      ED_CALLOC(idx, "pairing", int, na);
      findex(na, xa, idxa);
      findex(nb, xb, idxb);
      lastj = 0;
      for (i=0;i<na;i++) {
         inum = 0; 
         for (j=lastj;j<nb;j++) {
            xdif = xa[idxa[i]] - xb[idxb[j]];
            ydif = ya[idxa[i]] - yb[idxb[j]];
            if (xdif < -err) {
               break;
            }
            if (err < xdif) {
               lastj = j;
            }
            if (fabs(xdif) <= err && fabs(ydif) <= err &&
               (diff[inum] = sqrt(xdif * xdif + ydif * ydif)) <= err) {
               jiff[inum] = j;
               if (++inum >= MAXNUM)
/*
 *                This is too crazy for words. It is impossible to say
 *                something sensible for a pairing in such circumstances.
 *                Either the errorbox is too large, or there are zilions
 *                of tiny spects on a very small area. Just stop acquiring
 *                pairing information here and hope for the best.
 */
                  break;
            }
         }
         if (inum > 0) {
            min_diff = err; min_jiff = 0;
            for (j=0;j<inum;j++) {
               if (min_diff >= diff[j]) {
                  min_diff = diff[j];
                  min_jiff = jiff[j];
               }
            }
            pairs[idxa[i]] = idxb[min_jiff];
            count++;
         } else {
            pairs[idxa[i]] = -1;
         }
      }
/*
 *    Now search through the paired list and find all na objects that were
 *    paired with the same nb object. For that list search the nearest
 */
      iindex(na, pairs, idx);
/*
 *    Sort index on nb array index number
 */
      old_jiff = -1;
      for (i=na-count;i<na;i++) {
         if (pairs[idx[i]] != -1) {
/*
 *          So this na object has been pairs with an nb object
 *          Make sure we correctly finish our job at i == na - 1
 */
            if (old_jiff != pairs[idx[i]] || i == (na - 1)) {
/*
 *             This starts a new sequence
 */
               if (old_jiff != -1) {
/*
 *                This is not the very first time we hit the doubles
 */
                  if ((i - min_jiff) > 1) {
                     min_diff = err;
/*
 *                   The min_jiff variable has been set to the first index of
 *                   the na list with similar nb objects
 */
                     if (i == (na - 1)) i++;
                     for (j=min_jiff;j<i;j++) {
                        xdif    = xa[idx[j]] - xb[old_jiff];
                        ydif    = ya[idx[j]] - yb[old_jiff];
                        diff[0] = sqrt(xdif * xdif + ydif * ydif);
/*
 *                      Save the index at minimum distance
 */
                        if (diff[0] <= min_diff) {
                           min_j = j;
                           min_diff = diff[0];
                        }
                     }
/*
 *                   Remove all pairs from the pairing list and add the single
 *                   nearest objects entry
 */
                     for (j=min_jiff;j<i;j++) {
                         pairs[idx[j]] = -1;
                         count--;
                     }
                     pairs[idx[min_j]] = old_jiff;
                     count++;
                     if (i == na) break;
                  }
               }
/*
 *             Save the first index of the list of similar nb paired objects
 *             Save the index to which they are all paired
 */
               min_jiff = i;
               old_jiff = pairs[idx[i]];
            }
         }
      }
      VPRINTF(NORMAL,"Number of pairs found: %d\n",count);
      ED_FREE(idx);
      ED_FREE(idxb);
      ED_FREE(idxa);
/*
 *    Fill the pair columns with the correct pointer values (1 based)
 */
      for (i=0;i<na;i++) {
         if (pairs[i] != -1) {
            b_pairs[pairs[i]] = i + 1 + p_off;
            pairs[i] += b_off + 1;;
         } else {
            pairs[i]++;
         }
      }
      return count;
}
