#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "fitscat.h"
#include "utils_globals.h"
#include "options_globals.h"
#include "preastrom_globals.h"

/* 16.01.2005:
   I removed the definition from the constrolstruct
   for the preastrom program from preastrom_globals.h
   and added it to preastrom.c. Im make_pairing.c,
   an extern definition was added.

  29.01.2005:
  I added history information to the output catalog

  11.03.2005:
  Changes to avoid compiler warnings with '-Wall -Wpedantic -W'.
*/

control_struct control;

option opts[] = {
   {"MAX_OFFSET", FLOAT, &control.maxoff, 0,0, 0.0, 2000.0, 
     0, 15.0, NO, "", 
     {"", "", "", "", ""}, 0,
     "Maximum allowed offset between extracted and reference objects [pix]"},
   {"POS_ERROR", FLOAT, &control.poserr, 0, 0, 0.0001, 1000.0,
     0, 5.0, NO, "", 
     {"", "", "", "", ""}, 0,
     "Positional error between extracted and reference objects [pix]"},
   {"FUDGE_FACTOR", FLOAT, &control.fudge, 0, 0, 0.0001, 1000.0,
     0, 2.0, NO, "", 
     {"", "", "", "", ""}, 0,
     "Multiplication factor for offset RMS difference checking"},
   {"MAX_DISTANCE", FLOAT, &control.maxdist, 0, 0, 0.0001, 100.0,
     0, 2.4, NO, "", 
     {"", "", "", "", ""}, 0,
     "Maximum distance between extracted and reference objects pairs [pix]"},
   {"MIN_PHOTS", FARRAY, control.minp, 0, 0, -1000000.0, 1000000.0,
     30, 0.0, NO, "", 
     {"", "", "", "", ""}, 0,
     "List of photometric parameter limits for pairing with reference objects"},
   {"AFFINE_PARS", FARRAY, control.aff_p, 0, 0, -1000000.0, 1000000.0,
     30, 0.0, NO, "", 
     {"", "", "", "", ""}, 0,
     "List of photometric parameter limits for pairing with reference objects"},
   {"CRVAL1", TEXT, control.crval1, 0, 0, 0.0, 0.0,
     0, 0.0, NO, "CRVAL1",
     {"", "", "", "", ""}, 0,
     "Name of the table column containing Right Ascension"},
   {"CRVAL2", TEXT, control.crval2, 0, 0, 0.0, 0.0,
     0, 0.0, NO, "CRVAL2",
     {"", "", "", "", ""}, 0,
     "Name of the table column containing Declination"},
   {"CRPIX1", TEXT, control.crpix1, 0, 0, 0.0, 0.0,
     0, 0.0, NO, "CRPIX1",
     {"", "", "", "", ""}, 0,
     "Name of the table column containing reference x coordinate pixel"},
   {"CRPIX2", TEXT, control.crpix2, 0, 0, 0.0, 0.0,
     0, 0.0, NO, "CRPIX2",
     {"", "", "", "", ""}, 0,
     "Name of the table column containing reference y coordinate pixel"},
   {"CDELT1", TEXT, control.cdelt1, 0, 0, 0.0, 0.0,
     0, 0.0, NO, "CDELT1",
     {"", "", "", "", ""}, 0, 
     "Name of the table column containing x coordinate pixel size"},
   {"CDELT2", TEXT, control.cdelt2, 0, 0, 0.0, 0.0,
     0, 0.0, NO, "CDELT2",
     {"", "", "", "", ""}, 0, 
     "Name of the table column containing y coordinate pixel size"},
   {"XWID", TEXT, control.xwid, 0, 0, 0.0, 0.0,
     0, 0.0, NO, "MAPNAXS1",
     {"", "", "", "", ""}, 0, 
     "Name of the table column containing x coordinate pixel width"},
   {"YWID", TEXT, control.ywid, 0, 0, 0.0, 0.0,
     0, 0.0, NO, "MAPNAXS2",
     {"", "", "", "", ""}, 0, 
     "Name of the table column containing y coordinate pixel width"},
   {"SEQNR", TEXT, control.seqnr, 0, 0, 0.0, 0.0,
     0, 0.0, NO, "SeqNr",
     {"", "", "", "", ""}, 0, 
     "Name of the table column containing object sequence number"},
   {"XPOS", TEXT, control.xpos, 0, 0, 0.0, 0.0,
     0, 0.0, NO, "Xpos",
     {"", "", "", "", ""}, 1,
     "Name of the table column containing object x coordinate position"},
   {"YPOS", TEXT, control.ypos, 0, 0, 0.0, 0.0,
     0, 0.0, NO, "Ypos",
     {"", "", "", "", ""}, 1,
     "Name of the table column containing object y coordinate position"},
   {"PHOT", TEXT, control.phot, 0, 0, 0.0, 0.0,
     0, 0.0, NO, "FLUX_APER",
     {"", "", "", "", ""}, 1,
     "Name of the table column containing object flux measures"},
   {"RMS_TOL", FLOAT, &control.rms_tol, 0, 0, 0.0, 100.0,
     0, 0.07, NO, "",
     {"", "", "", "", ""}, 0,
     "Rms tolerenace for triangulation matching"},
   {"TRIANGLE_TOL", FLOAT, &control.triangle_tol, 0, 0, 0.0, 100.0,
     0, 0.005, NO, "",
     {"", "", "", "", ""}, 0,
     "Rms tolerenace for triangulation matching"},
   {"XPIXSIZE", FLOAT, &control.xpixsize, 0, 0, -100.0, 100.0,
     0, 1.0, NO, "",
     {"", "", "", "", ""}, 0,
     "The scaling of pixels with which CDELT1 is multiplied"},
   {"YPIXSIZE", FLOAT, &control.ypixsize, 0, 0, -100.0, 100.0,
     0, 1.0, NO, "",
     {"", "", "", "", ""}, 0,
     "The scaling of pixels with which CDELT2 is multiplied"},
   {"XMIN", INT, &control.xmin, -32768, 32768, 0.0, 0.0,
     0, 0.0, NO, "",
     {"", "", "", "", ""}, 0,
     "Minimum x coordinate allowed in pairing"},
   {"XMAX", INT , &control.xmax, -32768, 32768, 0.0, 0.0,
     32768, 0.0, NO, "",
     {"", "", "", "", ""}, 0,
     "Maximum x coordinate allowed in pairing"},
   {"YMIN", INT, &control.ymin, -32768, 36768, 0.0, 0.0,
     0, 0.0, NO, "",
     {"", "", "", "", ""}, 0,
     "Minimum y coordinate allowed in pairing"},
   {"YMAX", INT, &control.ymax, -32768, 32768, 0.0, 0.0,
     32768, 0.0, NO, "",
     {"", "", "", "", ""}, 0,
     "Maximum y coordinate allowed in pairing"},
   {"SEL_MIN", INT, &control.sel_min, 0, 32768, 0.0, 0.0,
     50, 0.0, NO, "",
     {"", "", "", "", ""}, 0,
     "Minumim number of objects used in triangulation method"},
   {"REF_ERR", FLOAT, &control.ref_err, 0, 0, 0.0, 10.0,
     0, 0.00027777, NO, "",
     {"", "", "", "", ""}, 0,
     "The position error of objects in the reference catalog"},
   {"DATA_TYPE", TEXT, control.data_type, 0, 0, 0.0, 0.0,
     0, 0.0, NO, "OTHER",
     {"DENIS", "OTHER", "", "", ""}, 0,
     "The type of data to be reduced"},
   {"REF_RA", TEXT, control.ref_ra, 0, 0, 0.0, 0.0,
     0, 0.0, NO, "Ra",
     {"", "", "", "", ""}, 0,
     "The OBJECTS table column name of the reference catalog store Right Ascension"},
   {"REF_DEC", TEXT, control.ref_dec, 0, 0, 0.0, 0.0,
     0, 0.0, NO, "Dec",
     {"", "", "", "", ""}, 0,
     "The OBJECTS table column name of the reference catalog store Declination"},
   {"REF_MAG", TEXT, control.ref_mag, 0, 0, 0.0, 0.0,
     0, 0.0, NO, "Mag",
     {"", "", "", "", ""}, 0,
     "The OBJECTS table column name of the reference catalog store Magnitude"},
   {"REF_XWID", TEXT, control.ref_xwid, 0, 0, 0.0, 0.0,
     0, 0.0, NO, "NAXIS1",
     {"", "", "", "", ""}, 0,
     "The FIELDS table column name of the reference catalog store NAXIS1"},
   {"REF_YWID", TEXT, control.ref_ywid, 0, 0, 0.0, 0.0,
     0, 0.0, NO, "NAXIS2",
     {"", "", "", "", ""}, 0,
     "The FIELDS table column name of the reference catalog store NAXIS2"},
   {"FLAG_MASK", INT, &control.flag_mask, 0, 255, 0.0, 0.0,
     0, 0.0, NO, "",
     {"", "", "", "", ""}, 0,
     "The mask to select objects for astrometric pairing. (Flag & flag_mask)"},
   {"", BOOLEAN, NULL, 0, 0, 0.0, 0.0, 0, 0.0, NO, "", 
     {"YES", "NO", "", "", ""}, 0,
     "dummy key to end structure my_opts"} 
};

void usage(char *me, char *head)
{
    fprintf(stderr,"%s\n\n",head);
    fprintf(stderr,"Usage: %s -i incat		(input catalog name)\n",me);
    fprintf(stderr,"                 -o outcat		(output catalog name)\n");
    fprintf(stderr,"                 -a astrom_ref_cat	(input astrometric reference catalog)\n");
    fprintf(stderr,"                 -p refcat		(output astrometric reference catalog)\n");
    fprintf(stderr,"                 [-d distances catalog filename]\n");
    fprintf(stderr,"                 [-c configuration file]\n");
    fprintf(stderr,"                 [-OPTION1 OPTVAL1 [-OPTION2 OPTVAL2] ...]\n");
    exit(1);
}

void init(int argc, char *argv[], char *head)
{
/*
 *  Default file name is calling name with .conf extension
 */
    int		i, j, k;

    if (argc == 1) usage(argv[0], head);
    sprintf(control.posfile,"reference.cat");
    sprintf(control.distfile,"distances.cat");
    init_options(0, opts);
    for (i=1; i<argc; i++) {
      if (argv[i][0] == '-') {
         switch(argv[i][1]) {
         case 'i': control.ilist = 1;
                   strcpy(control.listfile,argv[++i]);
                   break;
         case 'o': strcpy(control.outfile,argv[++i]);
                   break;
         case 'p': strcpy(control.posfile,argv[++i]);
                   break;
         case 'd': strcpy(control.distfile,argv[++i]);
                   break;
         case 'c': strcpy(control.conffile,argv[++i]);
                   break;
         case 'a': strcpy(control.astromfile,argv[++i]);
                   break;
         case '@': print_options(argv[0]);
                   exit(0);
         default:  parse_options(control.conffile);
                   j = i++; k = i;
                   if (argv[j] && argv[k])
                     set_command_option(argv[j], argv[k]);
                   else
                     usage(argv[0], head);
                   break;
         }
      } else {
         syserr(WARNING,"init","Command line syntax error\n");
         usage(argv[0], head);
      }
   }
   finish_options(control.conffile);
   if (control.posfile[0] == '\0')
      syserr(FATAL, "init", "No output reference catalog %d\n", i);
   if (control.outfile[0] == '\0')
      syserr(FATAL, "init", "No output reference catalog %d\n", i);
   VPRINTF(NORMAL,"\n   %s\n\n",head);
}


int main(int argc, char *argv[])
/****** preastrom ***************************************************
 *
 *  NAME	
 *      preastrom - Perform preastrometric pairing
 *
 *  SYNOPSIS
 *      preastrom -i incat1 incat2 ... -o outcat1 outcat2 ..
 *               [-c configfile] [-t coord_transfile] 
 *                -p posref_file  
 *               [-a input_catfile] [CONFOPTION]
 *
 *  FUNCTION
 *      This program determines the preastrometric pairing. It tries to
 *      pair objects on the same frames of different input catalogs
 *      together with extracted objects from the input catalog. It writes
 *      the pairing information into the output catalogs, where outcat1 is
 *      the output catalog of incat1, outcat2 for incat2, etc.
 *      
 *  INPUTS
 *      -i incat1 incat2 ...  - The list of input catalog files
 *      -o outcat1 outca2 ... - The list of output catalog files
 *     [-c configfile]        - The name of the cofiguration file. The
 *                              default name is /denisoft/conf/preastrom.conf
 *      -p posref_file        - The name of the position reference catalog
 *                              produced alongside the outcat catalogs.
 *     [-a input_catfile]     - The name of the input catalog reference file.
 *                              The default name is /denisoft/conf/icat.cat
 *     [CONFOPTION]           - Pairs of configuration VARIABLE VALUE
 *                              sequences that allow dynamically
 *                              overwritting of configuration file values.
 *
 *  RESULTS
 *
 *  RETURNS
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *      The list of catalog files is a consecutive list of passband files.
 *      It is not allowed to mix strips!
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 02-07-1996
 *
 ******************************************************************************/
{ 
     char	header[30000];
     catstruct	*incat, *outcat, *poscat, *distcat;
     int	i;

     sprintf(header,"%s Version: %s (%s)",NAME,VERS,DATE);
     sprintf(header,"%-70.70s",header);
     init(argc, argv, header);

     for (i=1;i<argc;i++)
       sprintf(header, "%s %s", header, argv[i]);

     if ((incat = read_cat(control.listfile)) == NULL) {
       syserr(FATAL, "main", "Could not read catalog: %s\n", 
          control.listfile);
     }
     outcat = new_cat(1);
     inherit_cat(incat, outcat);
     copy_tabs(incat, outcat);
     poscat = new_cat(1);
     inherit_cat(incat, poscat);
     distcat = new_cat(1);
     inherit_cat(incat, distcat);
     make_pairing(incat, outcat, poscat, distcat, control.astromfile);
     if (distcat) {
       historyto_cat(distcat,"preastrom", argc, argv);
       save_cat(distcat, control.distfile);
       free_cat(distcat,1);
     }
     historyto_cat(poscat,"preastrom", argc, argv);
     save_cat(poscat, control.posfile);
     free_cat(poscat,1);
     historyto_cat(outcat,"preastrom", argc, argv);
     save_cat(outcat, control.outfile);
     free_cat(outcat,1);
     close_cat(incat);
     free_cat(incat,1);  

     return 0;
}
