#include <stdio.h>
#include <math.h>
#include "trans_defs.h"
#include "trans_types.h"
#include "options_globals.h"
#include "utils_globals.h"
#include "sort_globals.h"
#include "svd_solve.h"

/*
   27.03.2006:
   I renamed sorting and indexing functions to the new naming scheme
   described in the sort_globals.h file
*/

/* 
 *   Function Definitions 
 */

static void	Transform_Select (int *, OBJECT *, float *, float *, float *,
		                  int *, OBJECT *, float *, float *, float *, 
		                  int, float, int, int, int, int, int, int),
		Tol_Check (int, float, float, TRIANG, TRIANG *, 
		           int, int, int *),
		Save_Result(float [2][3], float *, float *, float *,
		            float *, float *, float *, int),
		Screen_Result(float [2][3]),
		Quicksort (TRIANG *, int, int),
		Bin_Search (float, TRIANG *, int *, int *, float, int);
static int	Generate_List_Triang (int , OBJECT *, TRIANG *),
		Found_Match (int, int, int, int, float, 
		             TRIANG *, TRIANG *, OBJECT *, OBJECT *, 
		             float [2][3], float ),
		Find_Transform (short [3][MAX_MATCH], int, OBJECT *, OBJECT *, 
		                float [2][3], float),
		Sides_Pos (int, int, int);

/*
 *   Global local variables
 */

static float  max_dis;

int transform(int next, float *xe, float *ye, float *pe,
              int nref, float *xr, float *yr, float *pr,
              float max_tol, float rms_tol, float triangle_tol, 
              int xmin, int xmax, int ymin, int ymax, int xoff, int yoff,
              int max_objs, 
              float *a0, float *a1, float *a2, 
              float *b0, float *b1, float *b2)
{
   int     Nobjs, ok;
   float   tolerance, max_rms;
   float   xfrm[2][3];
   int     ntriang;
   int     ntriangA, ntriangB;
   TRIANG  *List_triangA, *List_triangB;
   OBJECT  *List1, *List2;
/* 
 * Initialize 
 */
   Nobjs = max_objs;
   tolerance = triangle_tol;
   max_rms = rms_tol;
   max_dis = max_tol;
/* 
 * Setup triangle lists 
 */
   ntriang = (Nobjs - 2) * (Nobjs - 1) * Nobjs / 6;
   VPRINTF(DEBUG," nref = %d, next = %d, ntriang = %d, max_obj = %d\n",
          nref,next,ntriang,max_objs);
   ED_MALLOC(List1, "transform", OBJECT, nref);
   ED_MALLOC(List2, "transform", OBJECT, next);
   ED_MALLOC(List_triangA, "transform", TRIANG, ntriang);
   ED_MALLOC(List_triangB, "transform", TRIANG, ntriang);

/* 
 * Read the catalog for the brightest objects 
 */
   Transform_Select (&nref, List1, xr, yr, pr, 
                     &next, List2, xe, ye, pe, Nobjs, max_dis, 
                     xmin,xmax, ymin, ymax, xoff, yoff);
/* 
 *    Make the triangle lists 
 */
   if (next> 0) {
      ntriangA = Generate_List_Triang (next, List2, List_triangA);
      ntriangB = Generate_List_Triang (nref, List1, List_triangB);
      VPRINTF(DEBUG," ntriangA = %d, ntriangB = %d\n",ntriangA, ntriangB);

/*
 *    Match the triangles and compute the transformation 
 */
      ok = Found_Match (next, ntriangA, nref, ntriangB, tolerance,
                List_triangA, List_triangB, List2, List1, xfrm, max_rms);

   } else {
      ok = 0;
   }
/* 
 *    Save the transformation information and print output 
 */
   Save_Result(xfrm, a0, a1, a2, b0, b1, b2, ok);
   ED_FREE(List1);
   ED_FREE(List2);
   ED_FREE(List_triangA);
   ED_FREE(List_triangB);
   return ok;
}

/*
 *  All local functions
 */

/* 
 *  Transform_Select: Read the catalogs and Select on GSC position 
 */

static void Transform_Select (int *nobjs1, OBJECT *List1, 
                              float *xr, float *yr, float *pr, 
                              int *nobjs2, OBJECT *List2, 
                              float *xe, float *ye, float *pe,
                       int max_objs, float max_dis,
                       int xmin, int xmax, int ymin, int ymax,
                       int xoff, int yoff)
{
   int i, j, k, l, m, OK, *indx;
/*
 * Read as many ICAT stars as possible
 */
   i = 0; j = 0;
   ED_CALLOC(indx, "Transform_Select", int, (*nobjs1));
   findex(*nobjs1, pr, indx);
   for (;;) {
      List1[j].x = xr[indx[i]]-xoff;
      List1[j].y = yr[indx[i]]-yoff;
      List1[j].m = pr[indx[i]];
      i++; j++;
/*
 *    Make sure we do not get too many GSC stars in the List1 (declared to
 *    NOBJS or user defined
      if (i == max_objs/2 || i == *nobjs1) break;
 */
      if (i == max_objs || i == *nobjs1) break;
   }
   *nobjs1 = j;
   ED_FREE(indx);
/* 
 * Read the extracted objects catalogs. 
 */
/*
 * Make sure the phot data is set to zilch
 */
   for (i=0;i<max_objs;i++) List2[i].m = 0.0;
   j = 0; m = 0;
   for (;;) {
      OK = 0;
/* 
 *    Find out if this object is within the area bounds of a GSC star
 */
      if (xe[m] > xmin && xe[m] < xmax &&
          ye[m] > ymin && ye[m] < ymax) {
         for (i=0;i< *nobjs1; i++) {
            if (fabs(List1[i].x - xe[m]) < max_dis && 
                fabs(List1[i].y - ye[m]) < max_dis) {
               OK = 1;
               VPRINTF(DEBUG,"%f %f %d %f %f %d\n", List1[i].x, List1[i].y, i, xe[m], ye[m], m);
            }
         }
/*
 *       Insert the object in the phot sorted list
 */
         if (OK == 1) {
            OK = 0;
            if (j == 0) {
               List2[j].x = xe[m];
               List2[j].y = ye[m];
               List2[j].m = pe[m];
               if (j < (max_objs-1)) j++; 
            } else {
               for (k=0;k<j;k++) {
                  if (List2[k].m < pe[m]) {
                     if (j < (max_objs-1)) j++; 
                     for (l=j;l>k;l--) {
                        List2[l].x = List2[l-1].x;
                           List2[l].y = List2[l-1].y;
                        List2[l].m = List2[l-1].m;
                     } 
                     List2[k].x = xe[m];
                     List2[k].y = ye[m];
                     List2[k].m = pe[m];
                     OK = 1;
                     break;
                  }
               }
               if (OK == 0) {
                  if (j < (max_objs-1)) {
                     List2[j].x = xe[m];
                     List2[j].y = ye[m];
                     List2[j].m = pe[m];
                     j++; 
                  }
               }
            }
         }
      } else {
         VPRINTF(VERBOSE,"Skipping edge object %f %f %f\n", xe[m],ye[m],pe[m]);
      }
      m++;
      if (m == *nobjs2) {
         *nobjs2 = j;
         break;
      }
   }
   for (i=0;i<*nobjs1;i++) VPRINTF(VERBOSE,"%f %f %f\n",
                List1[i].x,  List1[i].y,  List1[i].m);
   VPRINTF(VERBOSE,"---------\n");
   for (i=0;i<j;i++) VPRINTF(VERBOSE,"%f %f %f\n",
                List2[i].x,  List2[i].y,  List2[i].m);
   VPRINTF(VERBOSE,"---------\n");
}


/* 
 * Routine Generate_list_triang : Create a list of triangles for candidate
 * objects from the List 
 */

static int Generate_List_Triang (int nobjs, OBJECT *List, TRIANG *List_triang)
{
   int     i, j, k, p=0;
   float   di, dj, dk, a, b, c, h1, h2; 
   float   *sides;
/* 
 * Minimize the number of triangle sides to compute 
 */
   ED_CALLOC(sides, "Generate_List_Triang", float, (nobjs * (nobjs - 1)/2 + 1));
   for (i = 0; i < nobjs; i++) {
      for (j = i + 1; j < nobjs; j++) {
         h1 = (List[i].x - List[j].x);
         h2 = (List[i].y - List[j].y);
         k = Sides_Pos (i, j, nobjs);
         sides[k] = sqrt(h1*h1 + h2*h2);
      }
   }
/* 
 * Order the triangle sides and compute the triangle space coords 
 */
   for (i = 0; i < (nobjs-2); i++) {
      for (j = i+1; j < (nobjs-1); j++) {
         for (k = j+1; k < nobjs; k++) {
            di = sides [Sides_Pos(i, j, nobjs)] ;
            dj = sides [Sides_Pos(j, k, nobjs)] ;
            dk = sides [Sides_Pos(k, i, nobjs)] ;

            if (dk > dj) {
               if (dk > di) {
                  if (dj > di) {   /* kji */
                     a = dk; b = dj; c = di;
                     List_triang[p].s1 = k;
                     List_triang[p].s2 = i;
                     List_triang[p].s3 = j;
                  } else {      /* kij */
                     a = dk; b = di; c = dj;
                     List_triang[p].s1 = i;
                     List_triang[p].s2 = k;
                     List_triang[p].s3 = j;
                  }
               } else {      /* ikj */
                  a = di; b = dk; c = dj;
                  List_triang[p].s1 = i;
                  List_triang[p].s2 = j;
                  List_triang[p].s3 = k;
               }
            } else if (dj > di) {
               if (di > dk) {      /* jik */
                  a = dj; b = di; c = dk;
                  List_triang[p].s1 = j;
                  List_triang[p].s2 = k;
                  List_triang[p].s3 = i;
               } else {      /* jki */
                  a = dj; b = dk; c = di;
                  List_triang[p].s1 = k;
                  List_triang[p].s2 = j;
                  List_triang[p].s3 = i;
               }
            } else {         /* ijk */
               a = di; b = dj; c = dk;
               List_triang[p].s1 = j;
               List_triang[p].s2 = i;
               List_triang[p].s3 = k;
            }
/* 
 *          Include only triangles with b/a < 0.9 
 */
            if ( b/a < 0.9 ) {
               List_triang[p].x = b/a;
               List_triang[p].y = c/a;
               p++;
            }
         }
      }
   }
   ED_FREE(sides);
   return (p);
}


/* 
 *  Found_Match: Find the objects with the best match between two catalogs
 *  Compute the transformation matrix and set the catalog header and     
 *  reference coordinates.  Output the results.                         
 */

static int Found_Match (int nobjsA, int ntriangA, int nobjsB, int ntriangB, 
                 float tolerance, 
                 TRIANG *List_triangA, TRIANG *List_triangB, 
                 OBJECT *List1, OBJECT *List2, 
                 float xfrm[2][3], float max_rms)
{
   short   matches[3][MAX_MATCH];
   int     i, j, k, l, n, first, last, *Table_match, nobjs, retcode;
   float   tolerance2 = tolerance * tolerance;
/* 
 * Allocate memory for the match table. 
 */
   nobjs = (nobjsA > nobjsB) ? nobjsA : nobjsB;
   ED_CALLOC(Table_match, "Found_Match", int, nobjsA * nobjsB);
/* 
 * Sort List_triangB by x coordinate.
 */
   Quicksort (List_triangB, 0, ntriangB - 1);
/* 
 * Find objects within tolerance distance in triangle space. 
 */
   for (i = 0; i < ntriangA; i++) {
      Bin_Search (List_triangA[i].x, List_triangB, &first, &last, 
                  tolerance, ntriangB);
      Tol_Check (nobjsB, tolerance, tolerance2, List_triangA[i],
      List_triangB, first, last, Table_match);
   }

   if (verbosity(DEBUG)) {
       n = 0;
       for (i=0; i<nobjsA*nobjsB; i++)
          n += Table_match[i];
       VPRINTF(DEBUG,"Number of trianges matched = %d\n", n/3);
   }
/* 
 * Find the nobjs points with the most matches. 
 */
   k = 0;
   for (i = 0; i < nobjsA; i++) {
      for (j = 0; j < nobjsB; j++) {
         n = Table_match[i*nobjsB+j];
         if (n > 0) {
            if (k < nobjs) {
               for (l=k; l>0 && n>matches[2][l-1]; l--) {
                  matches[0][l] = matches[0][l-1];
                  matches[1][l] = matches[1][l-1];
                  matches[2][l] = matches[2][l-1];
               }
               matches[0][l] = i;
               matches[1][l] = j;
               matches[2][l] = n;
               k++;
            } else if (n >= matches[2][nobjs-1]) {
               for (l=k; l>0 && n>matches[2][l-1]; l--) {
                  matches[0][l] = matches[0][l-1];
                  matches[1][l] = matches[1][l-1];
                  matches[2][l] = matches[2][l-1];
               }
               matches[0][l] = i;
               matches[1][l] = j;
               matches[2][l] = n;
               l = k < MAX_MATCH-1 ? k + 1: k;
               n = matches[2][nobjs-1];
               for (k=nobjs; k<l && n==matches[2][k]; k++);
            }
         }
         VPRINTF(DEBUG,"%2d ", Table_match[i*nobjsB+j]);
      }
      VPRINTF(DEBUG,"\n");
   }

   if (verbosity(DEBUG)) {
      for (i = 0; i < k; i++)
         VPRINTF(DEBUG,"%3d %3d %4d %f %f %f %f\n", matches[1][i]+1, matches[0][i]+1,
                matches[2][i], List1[matches[0][i]].x, List1[matches[0][i]].y,
                List2[matches[1][i]].x, List2[matches[1][i]].y);
   }
   ED_FREE(Table_match);
/* 
 * Compute transformation matrix from the nobjs best matches. 
 */
   retcode = Find_Transform (matches, k, List1, List2, xfrm, max_rms);
   return retcode;
}


/*
 * Find triangles within specified tolerance distance.
 */

static void Tol_Check (int nobjsB, float tolerance, float tolerance2, 
                       TRIANG List_triangA,  TRIANG *List_triangB, 
                       int first, int last, int *Table_match)
{
   float   temp1, temp2, distance;
   int     i, h1, h2;

   for (i = first ; i <= last; i++) {
      temp2 = (List_triangA.y - List_triangB[i].y);
      if (temp2 < tolerance) {
         temp1 = (List_triangA.x - List_triangB[i].x);
         distance = temp1 * temp1 + temp2 * temp2;
         if (distance < tolerance2) {
            h1 = List_triangA.s1;
            h2 = List_triangB[i].s1;
            Table_match[h1*nobjsB+h2]++;

            h1 = List_triangA.s2;
            h2 = List_triangB[i].s2;
            Table_match[h1*nobjsB+h2]++;

            h1 = List_triangA.s3;
            h2 = List_triangB[i].s3;
            Table_match[h1*nobjsB+h2]++;
         }
      }
   }
}


/*
 * Compute transformation matrix with iterative rejection 
 */

static int Find_Transform (short matches[3][MAX_MATCH], int m, 
                           OBJECT *List1, OBJECT *List2, 
                           float xfrm[2][3], float max_rms)
{
   int   i, j, i1, i2, ii, k;
   int   n = 3, nb = 2;
   float **a, **b, **c;
   float x, y, r2, sum, rms;

   ED_CALLOC(a, "Find_Transform", float *, n);
   for (i=0;i<n;i++)
      ED_CALLOC(a[i], "Find_Transform", float, MAX_MATCH);
   ED_CALLOC(b, "Find_Transform", float *, nb);
   ED_CALLOC(c, "Find_Transform", float *, nb);
   for (i=0;i<nb;i++) {
      ED_CALLOC(b[i], "Find_Transform", float, MAX_MATCH);
      ED_CALLOC(c[i], "Find_Transform", float, MAX_MATCH);
   }
/*
 * Remove all large distances from the matches
 */
   for (i=0;i<m;i++) {
      while((fabs(List1[matches[0][i]].x - List2[matches[1][i]].x) > max_dis ||
             fabs(List1[matches[0][i]].y - List2[matches[1][i]].y) > max_dis)&&
             i < m) {
         for (k=i;k<m-1;k++) {
            matches[0][k] = matches[0][k+1];
            matches[1][k] = matches[1][k+1];
         }
         m--;
      }
   }
   if (verbosity(DEBUG)) {
      VPRINTF(DEBUG,"After large distances removal\n");
      for (i=0;i<m;i++) {
         VPRINTF(DEBUG,"%d %f %f %f %f\n",i,List1[matches[0][i]].x,
              List1[matches[0][i]].y,List2[matches[1][i]].x,
              List2[matches[1][i]].y);
      }
   }
/* 
 * Require a minimum number of points. 
 */
   if (m < MIN_MATCH) {
      VPRINTF(VERBOSE,"Match not found: use more objects or larger tolerance\n");
            for(i=0;i<n;i++)
               ED_FREE(a[i]);
            ED_FREE(a);
            for(i=0;i<nb;i++) {
               ED_FREE(b[i]);
               ED_FREE(c[i]);
            }
            ED_FREE(b);
            ED_FREE(c);

      return 0;
   }
/*
 * Remove all double entries from the matches
 */
   for (i=0;i<m;i++) {
      for (j=i+1;j<m;j++) {
         while ((List2[matches[1][i]].x == List2[matches[1][j]].x ||
                 List1[matches[0][i]].x == List1[matches[0][j]].x) &&
                (j < m)) {
            for (k=j;k<m-1;k++) {
               matches[0][k] = matches[0][k+1];
               matches[1][k] = matches[1][k+1];
            }
            m--;
         }
      }
   }
   if (verbosity(DEBUG)) {
      VPRINTF(DEBUG,"After double matches \n");
      for (i=0;i<m;i++) {
         VPRINTF(DEBUG,"%d %f %f %f %f\n",i,List1[matches[0][i]].x,
              List1[matches[0][i]].y,List2[matches[1][i]].x,
              List2[matches[1][i]].y);
      }
   }
/* 
 * Compute the initial transformation with the 12 best matches. 
 */
   ii = 0;
   for (i=0; i<m && ii < 12; i++) {
      if (fabs(List1[matches[0][i]].x - List2[matches[1][i]].x) < max_dis &&
          fabs(List1[matches[0][i]].y - List2[matches[1][i]].y) < max_dis) {
         a[0][ii] = List1[matches[0][i]].x;
         a[1][ii] = List1[matches[0][i]].y;
         a[2][ii] = 1.0;
         b[0][ii] = List2[matches[1][i]].x;
         b[1][ii] = List2[matches[1][i]].y;
         VPRINTF(DEBUG, 
             "%d %f %f %f %f\n",ii,a[0][ii],a[1][ii],b[0][ii],b[1][ii]);
         ii++;
      }
   }
   if (ii < MIN_MATCH) {
      VPRINTF(VERBOSE,"Match not found: use more objects or larger tolerance\n");
            for(i=0;i<n;i++)
               ED_FREE(a[i]);
            ED_FREE(a);
            for(i=0;i<nb;i++) {
               ED_FREE(b[i]);
               ED_FREE(c[i]);
            }
            ED_FREE(b);
            ED_FREE(c);

      return 0;
   }
   VPRINTF(DEBUG,"Solving transformation\n");
   solve(a, n, ii, b, c, nb);
   for (i=0;i<n;i++) 
      for (j=0;j<nb;j++)
         xfrm[j][i] = c[j][i];
   if (verbosity(DEBUG)) Screen_Result(xfrm);
/* 
 * Start with all matches compute RMS and reject outliers.        
 * The outliers are found from the 60% point in the sorted residuals. 
 */

   for (;;) {
      sum = 0.;
      for (i=0; i<m; i++) {
         i1 = matches[0][i];
         i2 = matches[1][i];
         r2 = List1[i1].x;
         y = List1[i1].y;
         x = xfrm[0][0] * r2 + xfrm[0][1] * y + xfrm[0][2];
         y = xfrm[1][0] * r2 + xfrm[1][1] * y + xfrm[1][2];
         x -= List2[i2].x;
         y -= List2[i2].y;
         r2 = x * x + y * y;
         for (j=i; j>0 && r2<a[1][j-1]; j--)
            a[1][j] = a[1][j-1];
            a[0][i] = r2;
            a[1][j] = r2;
            sum += r2;
         }
/* 
 *       Set clipping limit and quit when no points are clipped. 
 */
         i = 0.6 * m;
         r2 = CLIP * a[1][i];
         if (r2 >= a[1][m-1]) {
            rms = sqrt (sum / m);
            break;
         }
/* 
 *       Clip outliers and redo the fit. 
 */
         j = 0;
         VPRINTF(DEBUG,"Clipping outliers\n");
         for (i=0; i<m; i++) {
            if (a[0][i] < r2) {
               i1 = matches[0][i];
               i2 = matches[1][i];
               matches[0][j] = i1;
               matches[1][j] = i2;
               a[0][j] = List1[i1].x;
               a[1][j] = List1[i1].y;
               a[2][j] = 1.;
               b[0][j] = List2[i2].x;
               b[1][j] = List2[i2].y;
               VPRINTF(DEBUG,
                  "%d %f %f %f %f\n",j,a[0][j],a[1][j],b[0][j],b[1][j]);
               j++;
            }
         }
         m = j;

         if (m < MIN_MATCH) {
            VPRINTF(VERBOSE,"Match not found: use more objects or larger tolerance\n");
            for(i=0;i<n;i++)
               ED_FREE(a[i]);
            ED_FREE(a);
            for(i=0;i<nb;i++) {
               ED_FREE(b[i]);
               ED_FREE(c[i]);
            }
            ED_FREE(b);
            ED_FREE(c);
   
            return 0;
         }
   solve(a, n, m, b, c, nb);
   for (i=0;i<n;i++)
      for (j=0;j<nb;j++)
         xfrm[j][i] = c[j][i];
         if (verbosity(DEBUG)) Screen_Result(xfrm);
      }
      VPRINTF(NORMAL,"Number of matches = %d, RMS of fit = %8.2f\n", m, rms);
      if (m >= 0) {
/*
 *       There is no such thing as a perfect match!
 */
         if (rms <1.e-3) return 0;
/*
 *       Some limits on our believe of the fit
 */
         VPRINTF(VERBOSE,"Accept if %f less then %f\n", rms/m, max_rms);
         if (rms/m > max_rms || m <= 3) return 0;
      }
      for(i=0;i<n;i++)
         ED_FREE(a[i]);
      ED_FREE(a);
      for(i=0;i<nb;i++) {
         ED_FREE(b[i]);
         ED_FREE(c[i]);
      }
      ED_FREE(b);
      ED_FREE(c);
      return -1;
}

static void Screen_Result( float xfrm[2][3])
{
   float a_0, a_1, a_2, b_0, b_1, b_2;

   a_1 = xfrm[0][0];
   a_2 = xfrm[0][1];
   a_0 = xfrm[0][2];
   b_1 = xfrm[1][0];
   b_2 = xfrm[1][1];
   b_0 = xfrm[1][2];

   VPRINTF(NORMAL,"Transformation coefficients: \n");
   VPRINTF(NORMAL," %10f %10f %10f", a_0, a_1, a_2);
   VPRINTF(NORMAL," %10f %10f %10f\n", b_0, b_1, b_2);
}

/* 
 * Save_Result: store the result of the transformation 
 */

static void Save_Result(float xfrm[2][3], 
                  float *a0, float *a1, float *a2, 
                  float *b0, float *b1, float *b2, 
                  int ok)
{
/* 
 *  Output results 
 */
   *a1 = xfrm[0][0];
   *a2 = xfrm[0][1];
   *a0 = xfrm[0][2];
   *b1 = xfrm[1][0];
   *b2 = xfrm[1][1];
   *b0 = xfrm[1][2];
/*
   if (!ok) {
      *a0 = *a1 = *a2 = *b0 = *b1 = *b2 =0.0; 
   }
*/
}


/*
 * Routine to select index in array of side lengths 
 */

static int Sides_Pos (int i, int j, int n)
{
   if (i < j)
      return(i*(2*n-i-3)/2 + j);
   else
      return(j*(2*n-j-3)/2 + i);
}


/* 
 *  Bin_Search : Binary Search in a array order by x coordinate 
 */

static void Bin_Search (float key, TRIANG *List_triang, 
                 int *first, int *last, float tolerance, int ntriang)
{
   int   min = 0, max = ntriang - 1, middle ;
   int   found = 0, i;  

   while ((!found) && (max - min > 1)) {
      middle  = (min + max ) / 2;
      if (fabs(List_triang[middle].x - key) < tolerance)
         found = 1;
      else if ( key < ( List_triang[middle ].x - tolerance))
         max = middle ;
      else if ( key > (List_triang[middle ].x + tolerance))
         min = middle;
   }
/* 
 * Not found 
 */
   if (!found) {
      *first = 2;
      *last  = 1;
      return;
   }

   for (i = middle; i > 0; i--) {
      if (fabs (List_triang[i].x - key) > tolerance)
         break;
   } 
   *first = i;

   for (i = middle; i < ntriang-1; i++) {
      if (fabs (List_triang[i].x - key) > tolerance)
         break;
   } 
   *last = i;
}


/* 
 *  Quicksort : Sort by x coordinate   
 */

static void Quicksort (TRIANG *List_Triang, int l, int r)
{
   TRIANG   v, t;
   int      i, j;

   if( r > l ) {
      v.x = List_Triang[r].x;
      i = l-1; 
      j = r;

      for(;;) {
         while(List_Triang[++i].x < v.x );
         while((j > 1) && (List_Triang[--j].x > v.x));
         if(i >= j)
            break;

         t.x = List_Triang[i].x; 
         List_Triang[i].x = List_Triang[j].x;
         List_Triang[j].x = t.x;

         t.y = List_Triang[i].y; 
         List_Triang[i].y = List_Triang[j].y;
         List_Triang[j].y = t.y;

         t.s1 = List_Triang[i].s1; 
         List_Triang[i].s1 = List_Triang[j].s1;
         List_Triang[j].s1 = t.s1;

         t.s2 = List_Triang[i].s2; 
         List_Triang[i].s2 = List_Triang[j].s2;
         List_Triang[j].s2 = t.s2;

         t.s3 = List_Triang[i].s3; 
         List_Triang[i].s3 = List_Triang[j].s3;
         List_Triang[j].s3 = t.s3;
      }

      t.x = List_Triang[i].x; 
      List_Triang[i].x = List_Triang[r].x;
      List_Triang[r].x = t.x;

      t.y = List_Triang[i].y; 
      List_Triang[i].y = List_Triang[r].y;
      List_Triang[r].y = t.y;

      t.s1= List_Triang[i].s1; 
      List_Triang[i].s1 = List_Triang[r].s1;
      List_Triang[r].s1 = t.s1;   

      t.s2 = List_Triang[i].s2; 
      List_Triang[i].s2 = List_Triang[r].s2;
      List_Triang[r].s2 = t.s2;

      t.s3 = List_Triang[i].s3; 
      List_Triang[i].s3 = List_Triang[r].s3;
      List_Triang[r].s3 = t.s3;

      Quicksort (List_Triang, l, i-1);
      Quicksort (List_Triang, i+1, r);
   }
}
