/* 17.08.00:
   
   I deleted a check for the keyword
   HIERARCH ESO DET CHIP INDEX
   in the subroutine merge_ext_img. This seems not
   to be a standard keyword (included for EIS only?)
   It should give back the chip number of the extension presumably.
   (Probably this is the reason for the inconsistency between
   IRAF numbering and my own one of chips 5 and 8.)

   10.01.05:
   I removed compiler warnings under gcc with optimisation
   enabled (-O3).

   28.01.05:
   I added HISTORY information about the splitting process
   in the image headers

   06.03.2005:
   I corrected the HISTORY information. Instead of the new
   filename I now give the original one for the file from
   that chips are splitted.
*/

/*
 				main.c

*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*
*	Part of:	LDACTest
*
*	Author:		E.Deul, DeNIS/LDAC
*
*	Contents:	parsing and main loop.
*
*	Last modify:	15/01/97
*
*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"fitscat_defs.h"
#include	"fitscat.h"
#include        "t_iso8601.h"
#include        "global_vers.h"

#define		SYNTAX \
"ldactest -i Catalog1 [-x EXTENSION_NUMBER][options]\n\n\
Options are:	-q (Quiet flag: defaulted to verbose!)\n"

#define		MAXCHAR 256
catstruct	*incat, *outcat;
int		qflag;

int clear_fits_doubles(char *buf, int size)
{
    char	key[9], line[81];
    int		i, j, nblocks = 0;

/* Remove the first END */
    if ((i = fitsfind(buf, "END     ")) == RETURN_ERROR)
       error(EXIT_FAILURE,"Error ","interpreting FITS header\n");
    memset(&buf[i*80], ' ', 80);
/* Remove first occurrence of each multiple FITS KEYWORD */
    for (i=0;i<size/80-1;i++) {
      strncpy(key, &buf[i*80], 8);
      key[8] = 0;
      if (!strcmp(key, "END     ")) break;
      if (!strcmp(key, "COMMENT ") || !strcmp(key, "HIERARCH")) continue;
      if ((j = fitsfind(&buf[(i+1)*80], key)) != RETURN_ERROR) {
        memcpy(&buf[i*80], &buf[(i+1+j)*80], 80);
        memset(&buf[(i+1+j)*80], ' ', 80);
      }
    }
/* Remove extension type of keys */
    if ((j = fitsfind(buf, "EXTNAME")) != RETURN_ERROR) 
        memset(&buf[j*80], ' ', 80);
    if ((j = fitsfind(buf, "XTENSION")) != RETURN_ERROR) 
        memset(&buf[j*80], ' ', 80);
    if ((j = fitsfind(buf, "EXTEND  ")) != RETURN_ERROR) 
        memset(&buf[j*80], ' ', 80);
/* Swap NAXIS1 and NAXIS2 to just after NAXIS */
    i = fitsfind(buf, "NAXIS ");
    if ((j = fitsfind(buf, "NAXIS1")) != RETURN_ERROR) {
       memcpy(line, &buf[j*80], 80);
       memcpy(&buf[j*80], &buf[(i+1)*80], 80);
       memcpy(&buf[(i+1)*80], line, 80);
    }
    if ((j = fitsfind(buf, "NAXIS2")) != RETURN_ERROR) {
       memcpy(line, &buf[j*80], 80);
       memcpy(&buf[j*80], &buf[(i+2)*80], 80);
       memcpy(&buf[(i+2)*80], line, 80);
    }
/* Compact header */
    for (j=0; j<size/80 || !strncmp(&buf[j*80],"END     ",8); j++) {
      if (buf[j*80] == ' ') {
        i = 1;
        while (buf[(i+j)*80] == ' '&&strncmp(&buf[(i+j)*80],"END     ",8)) i++;
        memcpy(&buf[j*80], &buf[(i+j)*80], 80);
        if (!strncmp(&buf[(i+j)*80],"END     ",8)) {
           nblocks = (j*80)/2880 + 1;
           break;
        }
        memset(&buf[(i+j)*80], ' ', 80);
        i++;
      }
    }
    return nblocks;
}

int merge_ext_img(catstruct *catin, int itab , catstruct *catout)
{
   tabstruct    *outtab, *prevtab, *nexttab, *tabin, *origtab;
   char		*orig_headbuf;
   int          i, orig_headnblock;
   /* int       j; */
   /* char      *line; */

/*Convert the table number to a pointer*/
  if (itab > catin->ntab-1 || itab < 0) 
    return RETURN_ERROR;
  for (i=1,tabin = catin->tab->nexttab; i<=itab && tabin ;i++) 
    tabin = tabin->nexttab;
  if (!tabin) 
    return RETURN_ERROR;
/*Check if a similar table doesn't already exist in the dest. cat */
  if ((outtab = name_to_tab(catout, tabin->extname, 0))) {
      return RETURN_ERROR;
  } else {
    prevtab = nexttab = NULL;
  }

    origtab = outcat->tab;

/*Now copy each segment of the original table*/
/*---First, allocate memory and copy data */
     orig_headbuf    = origtab->headbuf;
     orig_headnblock = origtab->headnblock;
     *origtab = *tabin;
     if (tabin->naxis) {
       QREALLOC(origtab->naxisn, int, tabin->naxis);
       memcpy(origtab->naxisn, tabin->naxisn, tabin->naxis*sizeof(int));
     }
     if (orig_headbuf)
       origtab->headbuf = orig_headbuf;
     if (tabin->headbuf) {
       QREALLOC(origtab->headbuf, char, 
               (orig_headnblock+tabin->headnblock)*FBSIZE);
       memcpy((&origtab->headbuf[orig_headnblock*FBSIZE]), tabin->headbuf,
              tabin->headnblock*FBSIZE);
       origtab->headnblock += orig_headnblock;
       origtab->headnblock = clear_fits_doubles(origtab->headbuf, 
                                                origtab->headnblock*FBSIZE);
     }
     if (tabin->bodybuf)
       QMEMCPY(tabin->bodybuf, origtab->bodybuf, char, tabin->tabsize);
     /*if ((j = fitsfind(origtab->headbuf, "HIERARCH ESO DET CHIP INDEX")) != RETURN_ERROR) {
        QMALLOC(line, char, 16);
        memccpy(line, &origtab->headbuf[j*80+30], '/', 15);
	j = atoi(line);
        QFREE(line);
     } else {
        error(EXIT_FAILURE,"Could not find"," HIERARCH ESO DET CHIP INDEX");
     }*/
     /*return j;*/

     return itab+1;
}
/********************************** main ************************************/
int main(int argc, char *argv[])

/****** ldacdesc ***************************************************
 *
 *  NAME	
 *      ldacdesc - Give a catalog description
 *
 *  SYNOPSIS
 *      ldacdesc -i incat [-t tabname][-q]
 *
 *  FUNCTION
 *      This program gives a description of the content of a catalog file.
 *      It provides information on which tables exist in the file, and
 *      for each table produces a list of charactersistics, such as
 *      the number of column and rows, the total number of bytes, etc.
 *
 *  INPUTS 
 *      -i incat   - The name of the catalog to be described
 *      -t tabname - The name of the table to be described
 *     [-q]        - Make the program verbose
 *
 *  RESULTS
 *
 *  RETURNS
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R Deul - dd 26-06-1996
 *      E. Bertin - 15-01-97 added handling of errors
 *
 ******************************************************************************/
  {
   char	 infilename[MAXCHAR], line[MAXCHAR], filename[MAXCHAR], *cptr;
   char newfilename[MAXCHAR];
   char tmpstring[MAXCHAR];
   int	i, a,ni,do_tab=0, itab = 0;

  if (argc<2)
    error(EXIT_FAILURE,"SYNTAX: ", SYNTAX);

/*default parameters */
  qflag = 1;
  ni = 0;
  for (a=1; a<argc; a++)
    switch((int)tolower((int)argv[a][1]))
      {
      case 'i':	snprintf(infilename, MAXCHAR-1, "%s", argv[++a]);
                ni++;
	        break;
      case 'x':	itab = atoi(argv[++a]);
                ni++;
                do_tab=1;
	        break;
      case 'q': qflag = 0;
	        break;
      default : error(EXIT_FAILURE,"SYNTAX: ", SYNTAX);
      }

  if (!ni)
    error(EXIT_SUCCESS,"SYNTAX: ", SYNTAX);

/*Load the input catalogs*/
  QFPRINTF(OUTPUT, "Reading catalog(s)");
  if (!(incat = read_cat(infilename)))
    error(EXIT_FAILURE, "*Error*: catalog not found: ", infilename);
  if (do_tab) {
      outcat = new_cat(1);
      inherit_cat(incat, outcat);
      if ((i = merge_ext_img(incat, itab-1, outcat)) == RETURN_ERROR)
         error(EXIT_FAILURE, "*Error*: could not find extension", "");
      strcpy(filename, infilename);
      if ((cptr = strrchr(filename, '/')))
        strcpy(filename, ++cptr);
      if (!(cptr = strstr(filename, ".fits")))
         error(EXIT_FAILURE, "*Error*: No .fits extension in file", infilename);
      *cptr = '\0';
      snprintf(newfilename, MAXCHAR-1, "%s_%d.fits", filename, i);
      snprintf(line, MAXCHAR-1, "Saving extension %d as %s", itab, newfilename);
      QFPRINTF(OUTPUT, line);
      /* add history information */
      addhistoryto_cat(outcat, " ", 0);
      snprintf(tmpstring, MAXCHAR-1, "fitssplit: %s", __PIPEVERS__);
      addhistoryto_cat(outcat, tmpstring, 0);
      snprintf(tmpstring, MAXCHAR-1, "fitssplit: called at %s", get_datetime_iso8601());
      addhistoryto_cat(outcat, tmpstring, 0);
      snprintf(tmpstring, MAXCHAR-1, "fitssplit: splitted chip %d from %s.fits", i, filename);
      addhistoryto_cat(outcat, tmpstring, 0);
      save_cat(outcat, newfilename);
  } else {
     about_cat(incat, stdout);
  }
  QFPRINTF(OUTPUT, "All done");
  QPRINTF(OUTPUT,"\n");

  return EXIT_SUCCESS;
  }
