#include <time.h>
#include <stdlib.h>
#include <string.h>
#include "flips.h"

/* 21.11.00: 
   bug fix in write_file:
   the spoonfull variable in the loops getting the pixel values and
   converting them to the output type were passed one time too much

   substitued in give_date() long definitions by time_t to avoid compiler
   warnings.

   03.10.01:
   removed all compiler warnings under DEC

   03.05.02:
   added a function "free_square_trish" in correspondence
   to "alloc_square_trish"

   10.01.03:
   removed compiler warnings under Linux (including malloc.h and
   stdlib.h)

   25.07.03:
   function in_int_m:
   the buf variable in that the parameters are raed
   is initialised with 0 now.

   17.01.2005:
   function fread_80:
   I checked whether values are actually read when calling
   getc. This should avoid a program freeze with FITS images of
   size 0.

   29.01.2005:
   function give_date():
   the time is now given in ISO8601 format.

   05.02.2005:
   I made the code more stable against overwriting fixed length
   characters at some places.

   09.09.2005:
   I included code for getting the basename of a file (including
   the path). The standard C library function did not compile under
   IRIX64.
*/

/*----------------------------------------------------------------------------*/
void fread_80(file,line)
FILE *file;
char line[];
{
int i;

   i=0;
   for(i=0;i<80;i++)
   {
     if((line[i]=getc(file))==EOF)
     {
       printf("error reading file in fread_80 (image header); exiting\n");
       exit(1);
     }
   }
   line[80]='\0';
}
/*----------------------------------------------------------------------------*/
void fread_line(file,line)
FILE *file;
char line[];
{
int i,c;

   i=0;
   /* while((line[i++]=getc(file))!='\n'); */
   /* while((c=getc(file))!='\n' && c!=' ' && c!='\t' && c!='#') */
   while((c=getc(file))!='\n' && c!='#' && i < (MAXCHAR - 1))
   {
      line[i++]=c;
   }
   line[i]='\0';
}
/*----------------------------------------------------------------------------*/
int fread_header(file,header)
FILE *file;
char header[];
{
int i,c;

   i=0;
   while((c=getc(file))=='\n' || c==' ' || c=='\t' || c=='#');

   header[i++]=c; /* found first character */
   while((c=getc(file))!='\n' && c!=' ' && c!='\t' && c!='#' && i < (MAXCHAR - 1))
   {
      header[i++]=c;
   }
   header[i]='\0';

   return(strcpr("END",header));
}
/*----------------------------------------------------------------------------*/
int fread_param(file,param)
FILE *file;
char param[];
{
int i,c;

   i=0;

   while((c=getc(file))=='\n' || c==' ' || c=='\t' || c=='#');

   param[i++]=c; /* found first character */
   while((c=getc(file))!='\n' && c!=' ' && c!='\t' && c!='#')
   {
      param[i++]=c;
   }
   param[i]='\0';

   return(strcpr("end",param));
}

/*----------------------------------------------------------------------------*/
void in_line(IN,param,name)
FILE *IN;
char param[];
char name[];
{
char word[MAXCHAR];
int i;

   i=ftell(IN);

   while(fread_param(IN,word)) {
      if(strcpr(word,name)==0)
         fread_line(IN,param);
   }

   fseek(IN,i,0);
}
/*----------------------------------------------------------------------------*/
void in_string(IN,param,name)
FILE *IN;
char param[];
char name[];
{
char word[MAXCHAR];
int i;

   i=ftell(IN);

   while(fread_param(IN,word)) 
   {
      if(strcpr(word,name)==0)
      {
         fread_param(IN,param);
      }
   }

   fseek(IN,i,0);
}
/*----------------------------------------------------------------------------*/
char in_char(IN,name)
FILE *IN;
char name[];
{
char param;
char word[MAXCHAR];
int i;

   param=0;
   i=ftell(IN);

   while(fread_param(IN,word)) {
      if(strcpr(word,name)==0)
         fscanf(IN,"%c",&param);
   }

   fseek(IN,i,0);

   return(param);
}

/*----------------------------------------------------------------------------*/
int in_int(IN,name)
FILE *IN;
char name[];
{
int param;
char word[MAXCHAR];
int i;

   param=0;
   i=ftell(IN);

   while(fread_param(IN,word)) {
      if(strcpr(word,name)==0)
	 fscanf(IN,"%d",&param);
   }

   fseek(IN,i,0);
   
   return(param);
}
/*----------------------------------------------------------------------------*/
int in_int_m(IN,name,rank)
FILE *IN;
char name[];
int rank;
{
char word[MAXCHAR];
int i;
int buf[4];

   buf[0]=0; 
   buf[1]=0; 
   buf[2]=0; 
   buf[3]=0;
 
   i=ftell(IN);

   while(fread_param(IN,word)) {
      if(strcpr(word,name)==0)
	 fscanf(IN,"%d %d %d %d",&buf[0],&buf[1],&buf[2],&buf[3]);
   }

   fseek(IN,i,0);

   return(buf[rank]);
}
/*----------------------------------------------------------------------------*/
float in_float(IN,name)
FILE *IN;
char name[];
{
float param;
char word[MAXCHAR];
int i;

   param=0;
   i=ftell(IN);

   while(fread_param(IN,word)) {
      if(strcpr(word,name)==0)
	 fscanf(IN,"%f",&param);
   }

   fseek(IN,i,0);

   return(param);
}
/*----------------------------------------------------------------------------*/
void ftab(file,n)
FILE *file;
int n;
{
int i;

   for(i=0;i<n*8;i++) 
      getc(file);
}
/*----------------------------------------------------------------------------*/
int strcpr(a,b)
char a[],b[];
{
int i;

   i=0;
   while(a[i]==b[i]) {
      if(a[++i]=='\0')
         return(0);
   }
   return(a[i]-b[i]);
}
/*----------------------------------------------------------------------------*/
void get_filename(path,name)
char path[];
char name[];
{
int i,j;
int last;
   
   last=0;
   i=0;
   j=0;
   while(last == 0) {
      if(path[i]=='/') {
         j=0;
	 i++;
      }
      else if(path[i]=='\0')
         last=1;
      else
         name[j++]=path[i++];
   }
   name[j]='\0';
}
/*----------------------------------------------------------------------------*/
void get_notab(in,out)
char in[];
char out[];
{
int i,j;

   i=j=0;
   while((in[i]=='	')||(in[i]==' '))
      i++;
   while((in[j+i]!='\0')&&(in[j+i]!='	')) {
      out[j]=in[j+i];
      j++;
   }
   out[j]='\0';
}

/*----------------------------------------------------------------------------*/
short **alloc_square_short(x,y)
int x,y;
{
auto short **square;
register int i;

   square = (short **) malloc((unsigned) x*sizeof(short *));
   if(square != 0) {
      for(i=0; i<x; i++) {
         square[i] = (short *)malloc((unsigned) y*sizeof(short));
         if (square[i] == 0) square = 0;
      }
   }
   else {
      printf("Error allocating memory\n");
      exit(0);
   }
   return(square);
}
/*----------------------------------------------------------------------------*/
int **alloc_square_int(x,y)
int x,y;
{
auto int **square;
register int i;

   square = (int **) malloc((unsigned) x*sizeof(int *));
   if(square != 0) {
      for(i=0; i<x; i++) {
          square[i] = (int *)malloc((unsigned) y*sizeof(int));
          if (square[i] == 0) square = 0;
      }
   }
   else{
      printf("Error allocating memory\n");
      exit(0);
   }
   return(square);
}
/*----------------------------------------------------------------------------*/
float **alloc_square_float(x,y)
int x,y;
{
auto float **square;
register int i;

   square = (float **) malloc((unsigned) x*sizeof(float *));
   if(square != 0) {
      for(i=0; i<x; i++) {
         square[i] = (float *)malloc((unsigned) y*sizeof(float));
         if (square[i] == 0) square = 0;
      }
   }
   else {
      printf("Error allocating memory\n");
      exit(0);
   }
   return(square);
}
/*----------------------------------------------------------------------------*/
PIXTYPE ***alloc_square_trish(n,x,y)
int n,x,y;
{
auto PIXTYPE ***square;
register int i,j;

   square = (PIXTYPE ***) malloc((unsigned) n*sizeof(PIXTYPE **));
   if(square != 0) {
      for(i=0; i<n; i++) {
         square[i] = (PIXTYPE **)malloc((unsigned) x*sizeof(PIXTYPE *));
         if(square[i] == 0) 
	    square = 0;
	 else {
            for(j=0; j<x; j++) {
               square[i][j] = (PIXTYPE *)malloc((unsigned) y*sizeof(PIXTYPE));
               if(square[i][j] == 0) 
	          square = 0;
	    }
         }
      }
   }
   if(square == 0) {
      printf("Error allocating memory in alloc_square_trish\n");
      exit(0);
   }
   return(square);
}
/*----------------------------------------------------------------------------*/
void free_square_trish(field,n,x,y)
int n,x,y;
PIXTYPE ***field;
{
register int i,j;

   for(i=0; i<n; i++) 
   {
       for(j=0; j<x; j++) 
       {
         if(field[i][j] != NULL)
         {
	   free(field[i][j]);
	   field[i][j] = NULL;	
	 }
       }

       if(field[i] != NULL)
       {
	 free(field[i]);
	 field[i] = NULL;
       }
   }

   if(field != NULL) 
   {
     free(field);
     field = NULL;
   }
}
/******************************* swapbytes **********************************/
/*
Swap bytes for doubles, longs and shorts (for DEC machines or PC for inst.).
*/
void	swapbytes(void *ptr, int nb, int n)
  {
   char	*cp;
   int	j;

  cp = (char *)ptr;

  if (nb&4)
    {
    for (j=n; j--; cp+=4)
      {
      cp[0] ^= (cp[3]^=(cp[0]^=cp[3]));
      cp[1] ^= (cp[2]^=(cp[1]^=cp[2]));
      }
    return;
    }

  if (nb&2)
    {
    for (j=n; j--; cp+=2)
      cp[0] ^= (cp[1]^=(cp[0]^=cp[1]));
    return;
    }

  if (nb&1)
    return;

  if (nb&8)
    {
    for (j=n; j--; cp+=8)
      {
      cp[0] ^= (cp[7]^=(cp[0]^=cp[7]));
      cp[1] ^= (cp[6]^=(cp[1]^=cp[6]));
      cp[2] ^= (cp[5]^=(cp[2]^=cp[5]));
      cp[3] ^= (cp[4]^=(cp[3]^=cp[4]));
      }
    return;
    }

  printf("*Internal Error*: Unknown size in swapbytes()");
  exit(1);

  return;
  }
/*----------------------------------------------------------------------------*/
void give_date(date)
char date[];
{
   time_t thetime;
   struct tm *tm;

   thetime=time(NULL); 
   tm = gmtime(&thetime);

   sprintf(date,"%04d-%02d-%02dT%02d:%02d:%02d",
          tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday,
          tm->tm_hour, tm->tm_min, tm->tm_sec);
}

/*----------------------------------------------------------------------------*/
FILE *open_file(filename,mode)
char filename[];
char mode[];
{
FILE *IN,*fopen();

   IN=fopen(filename,mode);
   if(IN==0) 
   {
      printf("Cannot access to the file %s\nwith mode '%s', exiting.\n\n",
	     filename,mode);
      exit(1);
   }
   return(IN);
}

/*----------------------------------------------------------------------------*/
int open_fileint(filename,mode)
char filename[];
int mode; /* mode=0 for read, 1 for write, 2 for read-write */
{
int ident,open();

   if((ident=open(filename,mode,0)) == -1) { /* open */
      printf("\nCannot access to the file %s\nwith mode %d', exiting.\n\n",
             filename,mode);
      exit(0);
   }
   return(ident);
}

/*----------------------------------------------------------------------------*/
void seek_file(struct IMAGE *file, int size) 
{
int lseek();

   if(lseek(file->imagefile, size, 0) <0) 
   {
      printf("\nCannot lseek %d bytes in file %s, exiting\n\n",size,file->name);
      exit(0);
   }
}

/*----------------------------------------------------------------------------*/
void seek_cfile(file,size)
int file;
int size;
{
int lseek();

   if(lseek(file, size, 1) <0) {
      printf("\nCannot lseek %d bytes in file ident %d, exiting\n\n",size,file);
      exit(0);
   }
}

/*----------------------------------------------------------------------------*/
/* 17.07.:
  changed the routine to take an image structure and
  to read any kind of image into a float array
*/
void read_file (struct IMAGE *file, PIXTYPE *buf, unsigned int size)
{
   int i; 
   unsigned int spoonful, bowl;
   static char	bufdata0[DATA_BUFSIZE];
   char *bufdata;
   float bs,bz;

   bs=file->scale;
   bz=file->offset;

   bowl = DATA_BUFSIZE/file->bytepix;
   spoonful = size<bowl?size:bowl;

   
   for(; size>0; size -= spoonful)
   {
     if (spoonful>size)
     {
       spoonful = size;
     }
     bufdata = bufdata0;

     if((fread((char *)bufdata, spoonful*(file->bytepix), (size_t) 1, file->imagefile)) !=1 ) 
     {
       printf("\nCannot read %d bytes in %s, exiting\n\n",spoonful*(file->bytepix),file->name);
       exit(1);
     }

     switch(file->bitpix)
       {
       case 8:
         if (file->bitsign)
           for (i=spoonful; i--;)
             *(buf++) = *(bufdata++)*bs + bz;
         else
           for (i=spoonful; i--;)
             *(buf++) = *((unsigned char *)bufdata++)*bs + bz;
         break;

       case 16:
#       ifdef BSWAP
         swapbytes(bufdata, 2, spoonful);
#       endif
         if (file->bitsign)
           for (i=spoonful; i--; bufdata += sizeof(short))
             *(buf++) = *((short *)bufdata)*bs + bz;
         else
           for (i=spoonful; i--; bufdata += sizeof(unsigned short))
             *(buf++) = *((unsigned short *)bufdata)*bs + bz;
         break;

       case 32:
#       ifdef BSWAP
         swapbytes(bufdata, 4, spoonful);
#       endif
         if (file->bitsign)
           for (i=spoonful; i--; bufdata += sizeof(LONG))
             *(buf++) = *((LONG *)bufdata)*bs + bz;
         else
           for (i=spoonful; i--; bufdata += sizeof(ULONG))
             *(buf++) = *((ULONG *)bufdata)*bs + bz;
           break;

       case -32:
#       ifdef BSWAP
         swapbytes(bufdata, 4, spoonful);
#       endif
         for (i=spoonful; i--; bufdata += sizeof(float))
	 {
           *(buf++) = *((float *)bufdata)*bs + bz;
	 }
         break;

       case -64:
#       ifdef BSWAP
         swapbytes(bufdata, 8, spoonful);
#       endif
         for (i=spoonful; i--; bufdata += sizeof(double))
           *(buf++) = *((double *)bufdata)*bs + bz;
         break;

       default:
         printf("*FATAL ERROR*: unknown BITPIX type in read_file");
	    exit(1);
         break;
       }
     }   
}

/*----------------------------------------------------------------------------*/
/* 18.07.: changed the routine so that it can write any type
           of output image (in the moment only float)
   02.08: corrected a probable error:
          the write statement has to stay inside the size loop,
          not outside.
   21.11.:
          the spoonfull variable in the loops getting the pixel values and
          converting them to the output type were passed one time too much
*/
void write_file (struct IMAGE *file, PIXTYPE *buf, unsigned int size)
{
   int i; 
   unsigned int bowl, spoonful;
   static char bufdata1[DATA_BUFSIZE];
   char *bufdata;
   float *fbufdata;
   short *sbufdata; /* we only support the signed version of 16 bit
                       data for output */
   float bs,bz;
   float x;

   bs=file->scale;
   bz=file->offset;

   bowl = DATA_BUFSIZE/file->bytepix;
   spoonful = size<bowl?size:bowl;

   for(; size>0; size -= spoonful)
   {
     if (spoonful>size)
     {
       spoonful = size;
     }

     switch(file->bitpix)
     {
       case 16:
         sbufdata=(short *)bufdata1;
         for (i=spoonful; i>0; i--)
	 {
	   x=*((float *)(buf++));
	   if(x<0)
	   {
	     x=0.0;
	   }
	   if(x>hdynamic)
	   {
	     x=hdynamic;
	   }
	   
	   x=x+(int)(x-(int)x+0.5);
	   *(sbufdata++) = (short)(((int)x-bz)/bs);
	 }
#       ifdef BSWAP
         swapbytes(bufdata1, 2, spoonful);
#       endif
	 bufdata=(char *)bufdata1; 

         break;
       case -32:
         fbufdata=(float *)bufdata1;
         for (i=spoonful; i>0; i--)
	 {
	   x=*((float *)(buf++));
	   *(fbufdata++) = (x-bz)/bs;
	 }
#       ifdef BSWAP
         swapbytes(bufdata1, 4, spoonful);
#       endif
	 bufdata=(char *)bufdata1; 

         break;

       default:
         printf("*FATAL ERROR*: unsupported BITPIX type in write_file");
	    exit(1);
         break;
     }

     if((fwrite((char *)bufdata, spoonful*(file->bytepix), 1, file->imagefile)) !=1 ) 
     {
       printf("\nCannot write %d bytes to %s, exiting\n\n",spoonful*(file->bytepix),file->name);
       exit(1);
     }

   }   
}


/*----------------------------------------------------------------------------*/
double random_gauss(sigma)
double sigma;
{
double x,y;
double z=1;
double r=-1;
double res=0;

  while(z>0.25) {
     x=random_double()-0.5;
     y=random_double()-0.5;
     z = x*x + y*y;
  }
  while(r<=0.0)
     r=random_double();

  res=sigma*sqrt(-2.0*log(r)/z)*x;

  return(res);
}

/*----------------------------------------------------------------------------*/
double random_double()
{
  /* Generate a random number between 0.0 and 1.0 */
  return (double)random() / (0x7fffffffL+1.0);
}

/*----------------------------------------------------------------------------*/
/*
 * Find out the base name of a file (i.e. without prefix path)
 */

char * get_basename(const char *filename)
{
  char *p ;
  p = strrchr (filename, '/');
  return p ? p + 1 : (char *) filename;
}

