#include <string.h>
#include <stdlib.h>
#include "imcombflat.h"
#include "global_vers.h"

/* 03.05.02:
   function fitsclose():
   the freeing of bufim has to be done by
   "free_sqaure_trish" instead of a simple free

   24.04.03:
   function fitsclose():
   I use a new variable "blockfill" to fill up
   the FITS file to complete blocks of 2880 bytes.
   It happened that the previously used buffout 
   had not enough memory allocated leading to
   crashes occasionally.

   I removed the global variables buffout and fltout
   that are never used.

   I included stdlib.h to avoid compiler warnings due
   to the undefined "exit" function

   25.04.03:
   fixed bugs concerning padding bytes (to fill up blocks
   of 2880 bytes) to the final output image.

   18.01.2005:
   I removed two fscanf commands trying to read elements from
   the tab array that were out of range.

   29.01.2005:
   I added Global Version number information to output
   FITS headers

   03.02.2005:
   I corrected several occurences of imcomlat to imcombflat
*/
/*----------------------------------------------------------------------------*/
void fitsopen()
{
extern struct IMAGE *infile,output;
extern struct CLIPPING clipping;

int i;
float m_mode;
float tab[2];
char text[200],buf[200];

PIXTYPE ***alloc_square_trish();

FILE *in, *open_file();
struct IMAGE read_FITS_header(),write_FITS_header(),add_FITS_header_text();

   /* load the parameters */
   m_mode=0;
   in=open_file(list.name,"r"); /* open list */
   for(i=0;i<list.nfile;i++) 
   {
       fscanf(in,"%s",infile[i].name);
       tab[0]=tab[1]=-1;
       fscanf(in,"%f",&tab[0]);
       fscanf(in,"%f",&tab[1]);

       if(tab[0]==-1) 
       { /* no values, basic combination */
	  rescale_sky=0;
	  mode[i]=0.0;
	  sigma[i]=1000000.0;
	  highrej[i]=sigma[i]*clipping.rejectsigma;
	  lowrej[i]=-sigma[i]*clipping.rejectsigma;
       }
       else 
       { 
         if(tab[1]==-1) 
  	 { /* no values, basic combination */
  	    rescale_sky=1;
  	    mode[i]=tab[0];
  	    sigma[i]=100000.0;
	    highrej[i]=mode[i]+sigma[i]*clipping.rejectsigma;
	    lowrej[i]=mode[i]-sigma[i]*clipping.rejectsigma;
  	 }
	 else
	 {
	   rescale_sky=1;
	   mode[i]=tab[0];
           sigma[i]=tab[1];
	   highrej[i]=mode[i]+sigma[i]*clipping.rejectsigma;
	   lowrej[i]=mode[i]-sigma[i]*clipping.rejectsigma;
	 }
       }
       m_mode+=mode[i];

       infile[i]=read_FITS_header(infile[i]);
       if(i==0) 
       { /* basic copy of header from infile[0] to output */
	  output.nx=infile[0].nx;
	  output.ny=infile[0].ny;
	  if(output.bitpix==0)
	  {
            output.offset=infile[0].offset;
            output.scale=infile[0].scale;
            output.bitpix=infile[0].bitpix;
	    output.bytepix=infile[0].bytepix;
	  }
	  if(output.bitpix==16)
	  {
	    output.offset=32768.0;
	    output.scale=1.0;
	  }
	  if(output.bitpix==-32)
	  {
	    output.offset=0.0;
	    output.scale=1.0;
	  }
	  output=write_FITS_header(infile[0],output);
       }
   }
   fclose(in);

   /* put comments and info in output file */
   /* put date in FITS header */
   output=add_FITS_header_text("HISTORY"," ",output);
   sprintf(text,"imcombflat: %s", __PIPEVERS__);
   output=add_FITS_header_text("HISTORY",text,output);
   give_date(buf);
   sprintf(text, "imcombflat: called at %s", buf); 
   output=add_FITS_header_text("HISTORY",text,output);

   get_filename(output.name,text);
   sprintf(buf,"output image: %s",text);
   strcpy(text,"imcombflat: ");
   strcat(text,buf);
   output=add_FITS_header_text("HISTORY",text,output);

   if(clipping.median == 0)
      strcpy(text,"imcombflat: mean ");
   else if(clipping.median == 1)
      strcpy(text,"imcombflat: median ");
   else
      strcpy(text,"imcombflat: minimum ");
   sprintf(buf,"of %d images",list.nfile);
   strcat(text,buf);
   output=add_FITS_header_text("HISTORY",text,output);

   if(!clipping.ccdclip) 
   {
      sprintf(buf,"standard sigma clipping [-%2.1f;+%2.1f]",
              clipping.lsigmas, clipping.hsigmas);
      strcpy(text,"imcombflat: ");
      strcat(text,buf);
      output=add_FITS_header_text("HISTORY",text,output);
   } 
   else 
   {
      sprintf(buf,"CCD clipping [-%2.1f;+%2.1f] n=%2.1f g=%2.2f e_rqe=%2.4f",
	      clipping.lsigmas, clipping.hsigmas,
              clipping.noise, clipping.gain, clipping.e_rqe);
      strcpy(text,"imcombflat: ");
      strcat(text,buf);
      output=add_FITS_header_text("HISTORY",text,output);
   }

   sprintf(buf,"thresholding from %5.2f to %5.2f ADUs",  
              clipping.lcut, clipping.hcut);
   strcpy(text,"imcombflat: ");
   strcat(text,buf);

   sprintf(buf,"using a rejection sigma of %5.2f",  
              clipping.rejectsigma);
   strcpy(text,"imcombflat: ");
   strcat(text,buf);

   output=add_FITS_header_text("HISTORY",text,output);

   if(output.rescale == 0)
      sprintf(buf,"no rescaling/offsetting");
   else if(output.rescale == 1) 
   {
      if(rescale_sky == 0)
      {
         sprintf(buf,"no rescaling/offsetting");
      }
      else if(rescale_sky == 1)
      {
         sprintf(buf,"mode rescaling");
      }
      /*else if(rescale_sky == 2)
      {
         sprintf(buf,"mode offsetting and flux rescaling");
      }*/
   }
   else if(output.rescale == 2)
      sprintf(buf,"mean modes rescaling");
   else if(output.rescale == 3)
      sprintf(buf,"mode offsetting");

   printf("-> Combine algorithm : %s\n",buf);
   strcpy(text,"imcombflat: ");
   strcat(text,buf);
   output=add_FITS_header_text("HISTORY",text,output); 

   /* check if all images size are consistent */
   for(i=1;i<list.nfile;i++) 
   {
       if((infile[i].nx != infile[0].nx)||(infile[i].ny != infile[0].ny)) 
       {
          printf("\nDifferent size between input images, exiting.\n\n");
          fitsclose();
          exit(0);
       }
   }

   m_mode/=list.nfile;
   if(output.rescale==2) 
   {
      printf("-> output image mode = %5.0f\n",m_mode);
      output.mode=m_mode;
   }

   for(i=0;i<list.nfile;i++) 
   {
       get_filename(infile[i].name,text);
       printf("%s %.0f %.3f\n",
              text,mode[i],sigma[i]);
   }

   strcpy(text,"imcombflat: ");
   if(output.rescale==1) {
     sprintf(buf,"output mode = %5.0f",output.mode);
     strcat(text,buf);
     output=add_FITS_header_text("HISTORY",text,output);
   }
   else if(output.rescale==2) {
     sprintf(buf,"output mode = %5.0f = mean(modes in)",output.mode);
     strcat(text,buf);
     output=add_FITS_header_text("HISTORY",text,output);
   }

   /* add some infos in the FITS header */
   for(i=0;i<list.nfile;i++) {
          get_filename(infile[i].name,text);
          sprintf(buf,"%s %.0f %2.3f",
                  text,mode[i],sigma[i]);

          strcpy(text,"imcombflat: ");
          strcat(text,buf);
          output=add_FITS_header_text("HISTORY",text,output);
   }

   get_notab(output.comment,buf);
   if(strlen(buf)!=0) 
   {
      strcpy(text,"imcombflat: ");
      strcat(text,buf);
      output=add_FITS_header_text("COMMENT",text,output);
   }

   /* All files ok, we can open and jump to image data area */
   for(i=0;i<list.nfile;i++) 
   {
       infile[i].imagefile=open_file(infile[i].name,"rb");
       fseek(infile[i].imagefile,infile[i].startdata,0);
   }
   output.imagefile=open_file(output.name,"ab");
   fseek(output.imagefile,output.startdata,0);
}

/*----------------------------------------------------------------------------*/

void fitsalloc()
{
extern struct LIST list;
extern struct IMAGE *infile;

extern PIXTYPE ***bufim;
extern PIXTYPE *lineout;
extern PIXTYPE *tabsigma;
extern float *res;
extern float *mode;
extern float *sigma;

PIXTYPE ***alloc_square_trish();

FILE *in,*open_file();
float x;
struct IMAGE read_FITS_header(),test;

   /* first check the number of files and file access and all parameters */
   list.nfile=0;
   in=open_file(list.name,"r"); /* open list */
   while(fscanf(in,"%s",test.name)!=-1) {
	fscanf(in,"%f",&x);
	fscanf(in,"%f",&x);
	list.nfile++;
	test=read_FITS_header(test);
   }
   fclose(in);

   /* Set initial values */
   if(buffer.height>test.ny) {
      printf("\n-> setting buffer height to %d instead of %d\n\n",
             test.ny,buffer.height);
      buffer.height=test.ny;
   }

   /* data buffers and parameters buffer */
   bufim=alloc_square_trish(list.nfile,buffer.height+1,test.nx);

   lineout=(PIXTYPE *) malloc(test.nx*sizeof(PIXTYPE));

   tabsigma=(PIXTYPE *) malloc(list.nfile*sizeof(float));
   res     =(float *) malloc(list.nfile*sizeof(float));
   mode    =(float *) malloc(list.nfile*sizeof(float));
   sigma   =(float *) malloc(list.nfile*sizeof(float));
   highrej =(float *) malloc(list.nfile*sizeof(float));
   lowrej  =(float *) malloc(list.nfile*sizeof(float));

   /* manage files for the FITS binary data */
   infile=(struct IMAGE *) malloc(list.nfile*sizeof(struct IMAGE));
}

/*----------------------------------------------------------------------------*/

void fitsclose()
{
extern struct IMAGE *infile,output;

extern PIXTYPE ***bufim;
extern PIXTYPE *lineout;
extern PIXTYPE *tabsigma;
extern float *res;
extern float *mode;
extern float *sigma;

int i,z;
char blockfill[blocksize];

void free_square_trish();

   /* fill the output file remaining block with 0s */
   /* don't care if swapping or not */
   z = blocksize-(output.bytepix*output.nx*output.ny)%blocksize;

   if(z) 
   {
      for(i=0; i<=z-1; i++) 
      {
	blockfill[i]=0.0;
      }
   }

   if((fwrite((char *)blockfill, z, 1, output.imagefile)) !=1 ) 
   {
     printf("\nCannot pad %d bytes to %s, exiting\n\n",z,output.name);
     exit(1);
   }

   /* close all files */
   fclose(output.imagefile);

   for(i=0;i<list.nfile;i++)
   {
     fclose(infile[i].imagefile);
   }

   /* free memory */
   if (bufim!=NULL)    free_square_trish(bufim, list.nfile,buffer.height+1,output.nx);
   if (lineout!=NULL)  free(lineout);
   if (tabsigma!=NULL) free(tabsigma);
   if (res!=NULL)      free(res);
   if (mode!=NULL)     free(mode);
   if (sigma!=NULL)    free(sigma);
   if (highrej!=NULL)    free(highrej);
   if (lowrej!=NULL)    free(lowrej);
   if (infile!=NULL)   free(infile);
}
