#include <stdlib.h>
#include "imcombflat.h"

/* 19.11.00:
   set the variables clipping.noise and clipping.gain (routine
   init_para() to default values to avoid possible floating exceptions
   during the coaddition process.

   10.03.01:
   removed all compiler warnings under DEC

   16.09.01:
   - an argument giving the name of the ini-file has now to be
     given as argument (this was necessary to make the code
     useable in parallel mode

   24.04.03:
   included the stdlib library to avoid warnings due to the
   undefined 'exit' function.

   31.05.04:
   when checking whether a list is given, we now take the
   basename for the name provided. Thus, also paths can be
   given as file names containing lists.

   23.01.05:
   I rearranged definitions of global variables to avoid
   double definitions during the linking process.

   09.09.05:
   I changed the standard C basename function to an own get_basename
   routine. The standard version did not compile under IRIX64.
*/

/* global variables (also exported ones) */
struct LIST list;
struct IMAGE *infile,output;
struct CLIPPING clipping;
struct VERBOSE verbose;
struct BUFFER buffer;

PIXTYPE ***bufim;

float *lineout,*tabsigma,*res;
float *sigma,*mode;
float *highrej, *lowrej;

int start;			     /* current first line of bufim */

int rescale_sky;

/*----------------------------------------------------------------------------*/
int main(int argc, char **argv)
{
extern struct IMAGE output;
extern struct LIST list;

FILE *F_IN=NULL,*F_OUT=NULL;
int open_i,stop_i;

FILE *open_file();

   printf("\n-> imcombflat\n\n");

   /* load parameters from imcombine.in file */
   if(argc < 2)
   {
      printf("\nNo inifile given, exiting\n\n");
      exit(0);     
   }
   else
   {
     init_para(argv[1]);
   }

   /* list in: list of lists or single list ? */
   if(!(strcpr("@@",get_basename(list.name)))) { /* list of lists */
      printf("-> list of lists in    %s\n",list.name);
      if(strcpr("@",get_basename(output.name))) { /* not a list */
         printf("\nOutput %s is not a list, exiting.\n\n",output.name);
         exit(0);
      }
      else
	printf("-> list of images out  %s\n\n",output.name);

      F_IN=open_file(list.name,"r");
      F_OUT=open_file(output.name,"r");
      open_i=1;
      stop_i=0;
   }
   else { /* simple list */
      if(strcpr("@",get_basename(list.name))) { /* not a list */
         printf("\nInput %s is not a list, exiting.\n\n",list.name);
         exit(0);
      }
      if(!strcpr("@",get_basename(output.name))) { /* it is a list */
         printf("\nOutput %s should not be a list, exiting.\n\n",output.name);
         exit(0);
      }
      open_i=0;
      stop_i=0;
   }

   if((clipping.ccdclip == 0)&&(clipping.median != 2)) {
      printf("-> standard sigma clipping [-%2.1f;+%2.1f]\n",
              clipping.lsigmas, clipping.hsigmas);
   } 
   else if((clipping.ccdclip == 1)&&(clipping.median != 2)) {
      printf("-> CCD clipping [-%2.1f;+%2.1f] noise=%2.1fe-  gain=%2.2fe-/ADU  e_rqe=%2.3f\n",
              clipping.lsigmas, clipping.hsigmas,
              clipping.noise, clipping.gain, clipping.e_rqe);
   }

   if(clipping.median == 0)
   {
      printf("-> taking mean value\n");
   }
   else if (clipping.median == 1)
   {
      printf("-> taking median value\n");
   }
   else if (clipping.median == 2)
   {
      printf("-> taking minimum value\n");
   }

   printf("-> thresholding from %5.2f to %5.2f ADUs\n",
          clipping.lcut, clipping.hcut);

   printf("-> using a rejection sigma of %5.2f\n\n",
          clipping.rejectsigma);

   if(output.rescale==1)
      printf("-> output mode = %5.0f\n\n",output.mode);
   else if(output.rescale==2)
      printf("-> output mode = mean(input modes)\n\n");
   else if(output.rescale==3)
      printf("-> output mode = %5.0f\n\n",output.mode);
   else
      printf("-> no rescaling (output mode = %5.0f in empty zones)\n\n",
	    output.mode);


   do {
        if(open_i) { /* list of lists */
           stop_i=fscanf(F_IN,"%s",list.name);
           if(strcpr("@",get_basename(list.name))) { /* not a list */
              printf("\nInput %s is not a list, exiting.\n\n",list.name);
              exit(0);
           }
           if(stop_i!=-1) {
              fscanf(F_OUT,"%s",output.name);
           }
           if(!strcpr("@",get_basename(output.name))) { /* not an image  */
              printf("\nOutput %s is not an image, exiting.\n\n",output.name);
              exit(0);
           }
        }
        if(stop_i!=-1) {

           printf("-> input list      %s \n", list.name);
           printf("-> output image    %s \n", output.name);

           /* allocate memory for the files */
           fitsalloc();

           /* load FITS headers, copy FITS header and open the FITS files */
           fitsopen();

           /* create the median file */
           do_combine();

           /* close files and free memory */
           fitsclose();

           printf("\n");
        }
        if(open_i==0)
           stop_i=-1;
   }
   while(stop_i!=-1);

   if(open_i) 
   {
      fclose(F_IN);
      fclose(F_OUT);
   }

   return 0;
}

/*----------------------------------------------------------------------------*/
void init_para(char *ainifile)
{
FILE *IN,*open_file();
char word[80];
int i,done[20];

int in_int();
float in_float();

   for(i=0;i<20;i++)
      done[i]=1;

   IN=open_file(ainifile,"r");

   while(fread_header(IN,word)) {

        if((strcpr(word,"INPUT")==0)&&(done[0]==1)) 
        {
	   in_string(IN,list.name,"name");

           done[0]=fseek(IN,0,0);
        }

        if((strcpr(word,"OUTPUT")==0)&&(done[1]==1)) 
        {
           in_string(IN,output.name,"name");
	   output.bitpix=in_int(IN,"outbitpix");
	   output.bytepix=((int)(fabs(output.bitpix)/8.));
           output.rescale=in_int(IN,"rescale");
           output.mode=in_int(IN,"mode");
           in_line(IN,output.comment,"comment");

           done[1]=fseek(IN,0,0);
        }

        if((strcpr(word,"CLIPPING")==0)&&(done[2]==1)) 
        {
	   clipping.median=in_int(IN,"median");
	   clipping.lsigmas=in_float(IN,"lsigmas");
	   clipping.hsigmas=in_float(IN,"hsigmas");
           clipping.lcut=in_int(IN,"lcut");
           clipping.hcut=in_int(IN,"hcut");
	   clipping.hrej=in_int(IN,"hrej");
	   clipping.rejectsigma=in_float(IN,"sigmare");

	   if(clipping.rejectsigma < 1e-5)
	   {
             clipping.rejectsigma=1.0;
	   }

           done[2]=fseek(IN,0,0);
        }


        if((strcpr(word,"CCDCLIPPING")==0)&&(done[3]==1)) 
        {
	   /* set some defaukt values to avoid floating exceptions later */
           clipping.gain=1.0;
           clipping.ccdclip=in_int(IN,"ccdclip");
           clipping.noise=in_float(IN,"noise");
           clipping.gain=in_float(IN,"gain");
           clipping.e_rqe=in_float(IN,"e_rqe");

	   if(clipping.gain < 1e-5)
	   {
	     clipping.gain=1.0;
	   }
           done[3]=fseek(IN,0,0);
        }

        if((strcpr(word,"VERBOSE")==0)&&(done[4]==1)) 
        {
           verbose.verbose=in_int(IN,"verbose");
           verbose.x=in_int(IN,"x");
           verbose.y=in_int(IN,"y");

           done[4]=fseek(IN,0,0);
        }

        buffer.height=1;

   }
   fclose(IN);
}


