#include <stdlib.h>
#include "immode.h"

/* 10.03.01:
   removed all compiler warnings under DEC

   16.09.01:
   - an argument giving the name of the ini-file has now to be
     given as argument (this was necessary to make the code
     useable in parallel mode

   18.09.01:
   - the filenames of the output files are now given on the
     command line

   24.04.03:
   included the stdlib library to avoid warnings due to the
   undefined 'exit' function.

   31.05.04:
   when checking whether a list is given, we now take the
   basename for the name provided. Thus, also paths can be
   given as file names containing lists.

   17.01.05:
   I included a sanity check after a ftell command.

   23.01.05:
   I rearranged definitions of global variables to avoid
   double definitions during the linking process.

   09.09.05:
   I changed the standard C basename function to an own get_basename
   routine. The standard version did not compile under IRIX64.
*/

/* global variables (also exported ones) */
FILE *STATI,*STATA;

struct IMAGE   input;
struct IMAGE   output;
struct RASTER  raster;
struct STAT    stati;


/*----------------------------------------------------------------------------*/
int main(int argc, char **argv)
{
struct IMAGE read_FITS_header();
FILE *IN=NULL,*open_file();
int open,stop;

   /* load parameters from immode.in file */
   if(argc < 4)
   {
      printf("\nNo inifile and/or output file given\n");
      printf("Syntax: immode inifile output1 output2\n");
      exit(0);     
   }
   else
   {
     init_para(argv[1]);
   }

   if(raster.full)
      printf("\n-> immode\n\n-> statistic area: Full detector\n-> statistics on %d percent of the area\n",stati.percent);
   else
      printf("\n-> immode\n\n-> statistic area: Xc=%d Yc=%d Xs=%d Ys=%d\n-> statistics on  %d percent of the area\n",raster.xc,raster.yc,raster.sizex,raster.sizey,stati.percent);
   printf("-> Dynamic range: %f - %f ADUs\n",input.dyn_min,input.dyn_max);

   /* output text files */
   STATI=open_file(argv[2],"w");
   STATA=open_file(argv[3],"w");
   printf("-> statistic data out: %s + %s\n\n", argv[2], argv[3]);
   if(!raster.full) 
   {
      fprintf(STATI,"-> immode\n-> Xc=%d Yc=%d Xs=%d Ys=%d\n-> statistics on  %d percent of the area\n\n",raster.xc,raster.yc,raster.sizex,raster.sizey,stati.percent);
      fprintf(STATA,"-> immode\n-> Xc=%d Yc=%d Xs=%d Ys=%d\n-> statistics on  %d percent of the area\n-> Format:\tname\tmode\tmedian\tmean\tsigma\n\n",raster.xc,raster.yc,raster.sizex,raster.sizey,stati.percent);
   }

   /* list or single file ? */
   if(!strcpr("@",get_basename(input.name))) 
   { /* list */
      IN=open_file(input.name,"r");
      printf("-> image list input %s\n\n",input.name);
      open=1;
      stop=0;
   }
   else { /* single file */
      open=0;
      stop=0;
   }

   do 
   {
       if(open) /* list */
       {
          stop=fscanf(IN,"%s",input.name);
       }

       if(stop!=-1) 
       {
	  input=read_FITS_header(input);
          input.imagefile=open_file(input.name,"rb");
          if(fseek(input.imagefile, input.startdata, 0)!=0)
	  {
	    printf("error reading file %s; existing\n", input.name);
	    exit(1);
	  }
          find_size();

          stat_mediane();

	  fclose(input.imagefile);
       }
       if(!open)
          stop=-1;
   }
   while(stop!=-1);

   fclose(STATI);
   fclose(STATA);
   if(open)
   {
      fclose(IN);
   }

   printf("\n");

   return 0;
}

/*----------------------------------------------------------------------------*/
void find_size()
{

   if(raster.full) 
   {
      raster.xmin=0;
      raster.ymin=0;
      raster.xmax=input.nx;
      raster.ymax=input.ny;
   }
   else 
   {
      raster.xmin=raster.xc-raster.sizex/2;
      if(raster.xmin<0)
         raster.xmin=0;
 
      raster.ymin=raster.yc-raster.sizey/2;
      if(raster.ymin<0)
         raster.ymin=0;
 
      raster.xmax=raster.xc+raster.sizex/2;
      if(raster.xmax>input.nx)
         raster.xmax=input.nx;
 
      raster.ymax=raster.yc+raster.sizey/2;
      if(raster.ymax>input.ny)
         raster.ymax=input.ny;
   }

   output.nx=raster.xmax-raster.xmin; 
   output.ny=raster.ymax-raster.ymin; 
}

/*----------------------------------------------------------------------------*/
void init_para(char *ainifile)
{
FILE *IN,*open_file();
char word[80];
int i,done[20];

int in_int();
float in_float();

   for(i=0;i<20;i++)
       done[i]=1;

   IN=open_file(ainifile,"r");


   while(fread_header(IN,word)) {

	if((strcpr(word,"INPUT")==0)&&(done[0]==1)) 
        {
           in_string(IN,input.name,"name");
	   input.dyn_min=in_float(IN,"dyn_min");
	   input.dyn_max=in_float(IN,"dyn_max");

           done[0]=fseek(IN,0,0);
        }

	else if((strcpr(word,"STAT")==0)&&(done[1]==1)) 
        {
	   stati.percent=in_int(IN,"percent");

           done[1]=fseek(IN,0,0);
        }

	else if((strcpr(word,"RASTER")==0)&&(done[2]==1)) 
        {
           raster.xc=in_int(IN,"xc");
           raster.yc=in_int(IN,"yc");
           raster.sizex=in_int(IN,"sizex");
           raster.sizey=in_int(IN,"sizey");

	   if((raster.xc==0)&&(raster.yc==0)&&
              (raster.sizex==0)&&(raster.sizey==0))
	   {
	      raster.full = 1;
	   }
           else
	   {
	      raster.full = 0;
	   }

           done[2]=fseek(IN,0,0);
        }
   }
   fclose(IN);
}




