#include <string.h>
#include <stdlib.h>
#include "imred.h"

/* 28.07.00:
   included cutting of input images after the reduction

   01.08.00:
   added stuff for fringe correction

   17.08.:
   added initilisation for open_fr and stop_fr to zero.
   (it seemed to be initialised to a non zero value and
    hence led to a core dump when trying to close a never
    opened file).

   16.09.01:
   - an argument giving the name of the ini-file has now to be
     given as argument (this was necessary to make the code
     useable in parallel mode

   24.04.03:
   included the stdlib library to avoid warnings due to the
   undefined 'exit' function.

   25.07.03:
   - the occurences of 'mod2d' (member of the struct ovscan)
     where changed to the new name 'modline'
   - we read the new members 'minreject' and 'maxreject' as
     parameters for overscan correction.

   31.05.04:
   when checking whether a list is given, we now take the
   basename for the name provided. Thus, also paths can be
   given as file names containing lists.

   11.08.04:
   To the query whether some action has to be performed we
   added a test for cutting images. 

   23.01.05:
   I rearranged definitions of global variables to avoid
   double definitions during the linking process.

   05.02.05:
   I made the code more stable against overwriting fixed length
   caracter strings.

   09.09.05:
   I changed the standard C basename function to an own get_basename
   routine. The standard version did not compile under IRIX64.
*/

/* global variables (they are also exported to other files concerning imred) */
PIXTYPE *filein,*filedk,*filefl,*filemk,*filefr;          /* buffer for one image line */
PIXTYPE **overscan; 		                /* the overscan buffer */
struct IMAGE input,output,dark,flat,mask,fringe;
struct OVERSCAN ovscan;
struct DARKTAB darktab;
struct RASTER raster;
float current_dark; /* current dark current if value from a table */

/*----------------------------------------------------------------------------*/
int main(int argc, char **argv)
{
FILE *F_IN=NULL,*F_OUT=NULL;
FILE *F_FIN=NULL,*F_DIN=NULL,*F_MIN=NULL,*F_FRIN=NULL;
int open_i,stop_i;
int open_d,stop_d;
int open_f,stop_f;
int open_m,stop_m;
int open_fr,stop_fr;
char list_dark[MAXCHAR];
char list_flat[MAXCHAR];
char list_mask[MAXCHAR];
char list_fringe[MAXCHAR];

int i,j,k;
int current_darktab=0;

FILE *open_file();

   printf("\n-> imred\n\n");

   /* load parameters from imred.in file */
   if(argc < 2)
   {
      printf("\nNo inifile given, exiting\n\n");
      exit(0);     
   }
   else
   {
     init_para(argv[1]);
   }

   if((input.doover==0)&&(dark.dodark==0)&&(flat.doflat==0)&&
      (mask.domask==0)&&(fringe.dofringe==0)&&(raster.cut==0)) 
   {
      printf("\nNo operation requested, exiting\n\n");
      exit(0);
   }

   /* all files not yet openned */
   open_i=stop_i=open_d=stop_d=open_f=stop_f=open_m=stop_m=open_fr=stop_fr=0;

   /* image: list or single file ? */
   if(!strcpr("@",get_basename(input.name))) 
   { /* list */
      F_IN=open_file(input.name,"r");
      F_OUT=open_file(output.name,"r");
      printf("-> image list in   %s\n",input.name);
      printf("-> image list out  %s\n",output.name);
      open_i=1;
      stop_i=0;
   }
   else 
   { /* single file */
      open_i=0;
      stop_i=0;
   }

   /* dark: list or single file ? */
   if(dark.dodark) 
   {
      if(darktab.file) 
      {
         if(!strcpr("@",get_basename(dark.name))) 
         { /* list */
            F_DIN=open_file(dark.name,"r");
            printf("-> dark list in    %s\n",dark.name);
            open_d=1;
            strncpy(list_dark,dark.name, (MAXCHAR-1));
         }
         else { /* single file */
            open_d=0;
         }
      }
      else 
      {
         printf("-> dark table in   |");

         /* Check we have at least one value */
         if(darktab.darkadu[0] == ' ') 
         {
            printf("\nError: dark current table empty, exiting\n\n");
            exit(0);
         }

         j = strlen(darktab.darkadu); /* length of table in characters */
         darktab.nelt=1; /* total number of elements in the table */
         for(k=0;k<j;k++) 
         {
            if(darktab.darkadu[k] == ' ')
	    {
	       darktab.nelt++;
	    }
         }

         for(i=0;i<darktab.nelt;i++) 
         {
	    sscanf(darktab.darkadu,"%f",&darktab.darkval[i]);
            if(i != (darktab.nelt-1)) 
            { /* shift string */
	       do 
               {
                  for(k=0;k<j;k++)
		  {
                     darktab.darkadu[k]=darktab.darkadu[k+1];
		  }
               }
               while(darktab.darkadu[0] != ' ');
            }
         }

         for(i=0;i<darktab.nelt;i++)
	 {
	    printf("%.1f|",darktab.darkval[i]);
	 }
         printf("\n");

	 current_darktab=0; /* index of first value used */
      }
   }

   /* flat: list or single file ? */
   if(flat.doflat) 
   {
      if(!strcpr("@",get_basename(flat.name))) 
      { /* list */
         F_FIN=open_file(flat.name,"r");
         printf("-> flat list in    %s\n",flat.name);
         open_f=1;
         strncpy(list_flat,flat.name, (MAXCHAR - 1));
      }
      else 
      { /* single file */
         open_f=0;
      }
   }

   /* mask: list or single file ? */
   if(mask.domask) 
   {
      if(!strcpr("@",get_basename(mask.name))) 
      { /* list */
         F_MIN=open_file(mask.name,"r");
         printf("-> mask list in    %s\n",mask.name);
         open_m=1;
         strncpy(list_mask,mask.name, (MAXCHAR - 1));
      }
      else { /* single file */
         open_m=0;
      }
   }

   /* fringe: must be a list */
   if(fringe.dofringe) 
   {
      if(!strcpr("@",get_basename(fringe.name))) 
      { /* list */
         F_FRIN=open_file(fringe.name,"r");
         printf("-> fringe list in    %s\n",fringe.name);
         open_fr=1;
         strncpy(list_fringe,fringe.name, (MAXCHAR - 1));
      }
      else { /* single file */
         printf("Fringe images must be given in a list !!!!!!");
	 exit (1);
      }
   }

   do 
   {
	if(open_i) 
        { /* list */
	   printf("\n");
	   stop_i=fscanf(F_IN,"%s",input.name);
	   if(stop_i!=-1) 
           {
	      fscanf(F_OUT,"%s",output.name);
	   }
	}
	if(stop_i!=-1) 
        {
           printf("-> input image     %s \n", input.name);
           printf("-> output image    %s \n", output.name);

           if(mask.domask) 
           {
              if(open_m) 
              { /* list */
                 stop_m=fscanf(F_MIN,"%s",mask.name);
                 if((stop_m==-1)&&(stop_i!=-1)) 
                 {
                    fclose(F_MIN);
                    F_MIN=open_file(list_mask,"r");
                    stop_m=fscanf(F_MIN,"%s",mask.name);
                 }
              }
              printf("-> mask correction %s\n",mask.name);
           }

           if(input.doover && ovscan.modline)
	   {
              printf("-> overscan correction per line\n");
	   }
           else if(input.doover && !ovscan.modline)
	   {
              printf("-> Cte overscan correction (median value)\n");
	   }

	   if(dark.dodark && darktab.file) 
           {
	      if(open_d) 
              { /* list */
		 stop_d=fscanf(F_DIN,"%s",dark.name);
	         if((stop_d==-1)&&(stop_i!=-1)) 
                 {
		    fclose(F_DIN);
		    F_DIN=open_file(list_dark,"r");
		    stop_d=fscanf(F_DIN,"%s",dark.name);
	         }
	      }
	      printf("-> dark correction %s\n\t\t   with multiplicative factor %.2f \n",
		      dark.name,dark.fact);
           }
           else if(dark.dodark && !darktab.file) 
           {
	      current_dark=darktab.darkval[current_darktab++];
              if(current_darktab == darktab.nelt)
	      {
	         current_darktab=0; /* loop back to first value */
	      }
	      printf("-> dark correction -> constant value: %.2f\n",current_dark);
           }

	   if(flat.doflat) 
           {
	      if(open_f) 
              { /* list */
	         stop_f=fscanf(F_FIN,"%s",flat.name);
		 if((stop_f==-1)&&(stop_i!=-1)) 
                 {
		    fclose(F_FIN);
		    F_FIN=open_file(list_flat,"r");
		    stop_f=fscanf(F_FIN,"%s",flat.name);
		 }
	      }
	      printf("-> flat correction %s\n\t\t   with multiplicative factor %.2f\n", 
		      flat.name,flat.fact);
           }

	   if(fringe.dofringe) 
           {
	      if(open_fr) 
              { /* list */
		 fringe.fact=1.0;
	         stop_fr=fscanf(F_FRIN,"%s%f",fringe.name,&(fringe.fact));
		 if((stop_fr==-1)&&(stop_i!=-1)) 
                 {
		   printf("There must be as many Fringe images given as input images !!");
		   exit(1);
		 }
	      }
	      printf("-> fringe correction %s\n\t\t   with multiplicative factor %.2f\n", 
		      fringe.name,fringe.fact);
           }

           /* load FITS headers, copy FITS header and open the FITS files */
	   fitsopen();

	   /* allocate memory for the files */
	   fitsalloc();

           /* calculate the overscan median */
           if(input.doover)
	   {
              find_overscan();
	   }

	   /* process the image */
	   ccdred_image();

	   /* close files and free memory */
           fitsclose(1);
	}
	if(open_i==0)
	{
	   stop_i=-1;
	}
   }
   while(stop_i!=-1);

   if(open_i) 
   {
      fclose(F_IN);
      fclose(F_OUT);
   }
   if(open_d)
   {
      fclose(F_DIN);
   }
   if(open_f)
   {
      fclose(F_FIN);
   }
   if(open_m)
   {
      fclose(F_MIN);
   }
   if(open_fr)
   {
      fclose(F_FRIN);
   }

   return 0;
}

/*----------------------------------------------------------------------------*/
void init_para(char *ainifile)
{
FILE *IN,*open_file();
char word[MAXCHAR];
int i,done[20];

int in_int();
int in_int_m ();
float in_float();

   for(i=0;i<20;i++)
   {
      done[i]=1;
   }

   IN=open_file(ainifile,"r");

   while(fread_header(IN,word)) 
   {

        if((strcpr(word,"INPUT")==0)&&(done[0]==1)) 
        {
	   in_string(IN,input.name,"name");

           done[0]=fseek(IN,0,0);
        }

        if((strcpr(word,"OUTPUT")==0)&&(done[1]==1)) 
        {
           in_string(IN,output.name,"name");
	   output.fulldyn=in_int(IN,"fulldyn");
	   output.bitpix=in_int(IN,"outbitpix");
	   output.bytepix=((int)(fabs(output.bitpix)/8.));
           in_line(IN,output.comment,"comment");

           done[1]=fseek(IN,0,0);
        }

        if((strcpr(word,"OVERSCAN")==0)&&(done[2]==1)) 
        {
	   input.doover=in_int(IN,"doover");
	   
	   ovscan.modline=in_int(IN,"modline");
	   ovscan.xmin=in_int_m(IN,"ovscan",0);
           ovscan.xmax=in_int_m(IN,"ovscan",1);
	   ovscan.minreject=in_int_m(IN,"reject",0);
           ovscan.maxreject=in_int_m(IN,"reject",1);

           done[2]=fseek(IN,0,0);
        }

        if((strcpr(word,"DARK")==0)&&(done[3]==1)) 
        {
	   dark.dodark=in_int(IN,"dodark");

	   darktab.file=in_int(IN,"file");
	   in_line(IN,darktab.darkadu,"darkadu");
	   dark.fact=in_float(IN,"fact");
	   in_string(IN,dark.name,"name");

           done[3]=fseek(IN,0,0);
        }

        if((strcpr(word,"FLAT")==0)&&(done[4]==1)) 
        {
           flat.doflat=in_int(IN,"doflat");

           flat.fact=in_float(IN,"fact");
	   flat.flatsubst=in_float(IN, "subst");
           in_string(IN,flat.name,"name");

           done[4]=fseek(IN,0,0);
        }

        if((strcpr(word,"MASK")==0)&&(done[5]==1)) 
        {
           mask.domask=in_int(IN,"domask");

           in_string(IN,mask.name,"name");

           done[5]=fseek(IN,0,0);
        }

        if((strcpr(word,"FRINGE")==0)&&(done[6]==1)) 
        {
           fringe.dofringe=in_int(IN,"dofringe");

           in_string(IN,fringe.name,"name");

           done[6]=fseek(IN,0,0);
        }

        if((strcpr(word,"RASTER")==0)&&(done[7]==1)) 
        {
           raster.cut=in_int(IN,"cut");
	   raster.x=in_int(IN,"x");
	   raster.y=in_int(IN,"y");
	   raster.xmin=raster.x-1;
	   raster.ymin=raster.y-1;
	   raster.sizex=in_int(IN,"sizex");
	   raster.sizey=in_int(IN,"sizey");

           done[7]=fseek(IN,0,0);
        }

   }
   fclose(IN);
}
