#include "imcombflat.h"

/* 25.07.00:
   corrected a bug that did not allow minimum/maximum
   values lower/higher than the max SHORT range

   28.07.00:
   I disabled the verbosing
   If there are no input pixels, the output pixel is set to hcut now.

   20.11.00:
   If there is no input pixel the output is now the mode of the
   output image and not the max input value (hcut).

   10.03.01:
   removed all compiler warnings under DEC

   03.05.02:
   removed the superfluous shift_buffer() function from the code  
*/

/*----------------------------------------------------------------------------*/
void do_combine()
{
extern struct LIST list;
extern struct CLIPPING clipping;
extern struct IMAGE output;
extern PIXTYPE ***bufim;
extern PIXTYPE *lineout;
extern PIXTYPE *tabsigma;
extern float *mode;
extern int rescale_sky;

int verb=0;

int i,j,k;
int nbr_pix;
float min,max,cmb_pixel;
float x;

float ccdsigmaclip();

     min=clipping.lcut;
     max=clipping.hcut;

     /* Loop over the lines */
     for(i=0; i<output.ny; i++) 
     {
       load_buffers(0,i);

       /* loop over the columns */
       for(j=0; j<output.nx; j++) 
       {
	 nbr_pix=0;

         /* loop over the files */
         for(k=0; k<list.nfile; k++) 
         {
	     x=bufim[k][0][j];

	        if((x>=min)&&(x<=max)) 
                {
		  if((x>=lowrej[k]) &&
                     (x<=highrej[k]))
		  {
                   /* if output.rescale=0 then basic combination */
	           if(output.rescale==1) 
                   { /* multiplicative rescaling */
                      if(rescale_sky==1)
		      {
	                 x = ( x / mode[k] ) * output.mode;
		      }
	              /*else if(rescale_sky==2)
		      {
                        x = ( x - mode[k] ) / scale[k] + output.mode;
		      }*/
		   }
                 
	           else if(output.rescale==2) /* mean mode rescaling */
		   {
	              x = ( x / mode[k] ) * output.mode;
		   }
	           else if(output.rescale==3) /* additive offsetting */
		   {
	    	      x = x - mode[k] + output.mode;
		   }

	           tabsigma[nbr_pix]=x;
	           nbr_pix++;
		  }
	        }
	 }

	 /* VERBOSE */
         /* if(verbose.verbose) 
         {
            if((j==verbose.x)&&(i==verbose.y)) 
            {
               verb=1;
               for(k=0;k<nbr_pix;k++)
                  printf("-> cut selection: tab[%d] = %.2f\n",k+1,tabsigma[k]);
            }
         } */

         /* Process the set of values */
	 if(nbr_pix>1)
	 {
	   sort((unsigned long)nbr_pix, tabsigma-1);
	 }
	 nbr_pix=nbr_pix-clipping.hrej;

	 if(nbr_pix<=0)
	 {
           cmb_pixel=output.mode;
	 }
	 else
	 {
           cmb_pixel=ccdsigmaclip(tabsigma,nbr_pix,verb);
	 }

         /* Disable verbose mode */
         verb=0;

         /* Combined pixel value */
         lineout[j]=cmb_pixel;

       } /* end loop j */

       /* VERBOSE */
       /* if(verbose.verbose) 
       {
          if(i==verbose.y) 
          {
             if(clipping.median == 1)
                printf("-> sigma clipping median = %.2f\n",lineout[verbose.x]);
	     else if (clipping.median == 0)
                printf("-> sigma clipping mean = %.2f\n",lineout[verbose.x]);
	     else
		printf("-> sigma clipping minimum = %.2f\n",lineout[verbose.x]);
          }
       } */

       /* write these data in output file */
       write_file(&(output), lineout, output.nx);

       if(i%10==0)
       {
          fprintf(stderr," line %d to %d\r",i,output.ny);
       }

     } /* end loop i */

     fprintf(stderr,"                                                    \r");
}

/*----------------------------------------------------------------------------*/
void load_buffers(line_buf,line_image)
int line_buf,line_image;
{
extern struct IMAGE *infile;
extern PIXTYPE ***bufim;
int k;

   for(k=0;k<list.nfile;k++) 
   {
       read_file(&(infile[k]),bufim[k][line_buf],infile[0].nx);
   }
}
