#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "utils_globals.h"
#include "options_globals.h"
/* #include "sm_declare.h" */
/* #include "fitscat.h" */

/* 01.11.98: added pointtype
 *         added support for integer quantities
 * 15.12.98: now the number of points plotted is printed
 *
 * 19.10.2004: removed the superfluous inclusion of malloc.h
 */

#define __PROGRAMMVERSION__ "1.0"
#define __PROGRAMMNAME__ "ldacplot"

#define __USAGE__ "\n\
USAGE:\n\
       ldacplot -i input catalog\n\
                -p plot quantities\n\
              [ -c conditions ]\n\
              [ -l plotlimits xmin xmax ymin ymax ]\n\
              [ -t table with the objects (OBJECTS) ]\n\
              [ -b mark excluded objects with red color (1=yes) (0) ]\n\
              [ -y sm-pointtype (41) ]\n\
              [ -d sm-device for the plots (x11) ]\n\n\
DESCRIPTION:\n\
The program 'ldacplot' plots two quantities of a LDAC\n\
catalog against each other.\n\n\
AUTHOR: Thomas Erben \n"

void gibArrayminmax(const float *aArray, int aElemente, float *aMinimum,
                    float *aMaximum)
{
  int i;

  *aMaximum = aArray[0];
  *aMinimum = aArray[0];

  for(i=1; i< aElemente; i++)
  {
    if(aArray[i] < *aMinimum)
    {
      *aMinimum = aArray[i];
    }
    if(aArray[i] > *aMaximum)
    {
      *aMaximum = aArray[i];
    }
  }
}


void makeplot(float *axaxisgood, float *ayaxisgood, 
              int angoodobjects, char *axaxisname,
              char *ayaxisname,
              float *axaxisbad, float *ayaxisbad, 
              int anbadobjects, int acolor,
              float axmin, float axmax, 
              float aymin, float aymax,
              float apointtype,
              char *adevice);

#define MAXSTRINGLENGTH 256
#define EPSILON 1e-06

int main(int argc, char **argv)
{
  char incatname[MAXSTRINGLENGTH];
  char tablename[MAXSTRINGLENGTH]="OBJECTS";
  char device[MAXSTRINGLENGTH]="x11";
  int color=0;
  float pointtype=41.0;
  char xaxisname[MAXSTRINGLENGTH];
  char yaxisname[MAXSTRINGLENGTH];
  char tmpstring1[MAXSTRINGLENGTH];
  char tmpstring2[MAXSTRINGLENGTH];
  int nobjects;
  char **condnames;
  float *condmin, *condmax;
  int startconditions;
  int nconditions=0;
  int ngoodobjects=0, nbadobjects=0;
    
  catstruct *incat;
  tabstruct *tab;
  keystruct *xaxiskey, *yaxiskey;
  keystruct **conditionskey;
  
  float *goodxaxis, *goodyaxis;
  float *badxaxis, *badyaxis;
  int i, j;
  int objectgood;
  float xmin=0.0, xmax=0.0, ymin=0.0, ymax=0.0;

  /* print start message: */
  printf("\n %s  V%s (%s;%s)\n\n", __PROGRAMMNAME__, __PROGRAMMVERSION__, __DATE__, __TIME__);
  if(argc < 5)
  {
    printf("%s\n", __USAGE__);
    exit(1);
  }

  /* get the command line arguments: */
  for (i=1; i<argc; i++) 
  {
    if (argv[i][0] == '-') 
    {
       switch(tolower(argv[i][1])) 
       {
  	 case 'd': strncpy(device, argv[++i], MAXSTRINGLENGTH-1);
  		   break;
  	 case 'y': pointtype = atof(argv[++i]);
  		   break;
  	 case 'b': color = atoi(argv[++i]);
  		   break;
  	 case 'l': xmin = atof(argv[++i]);
                   xmax = atof(argv[++i]);
                   ymin = atof(argv[++i]);
                   ymax = atof(argv[++i]);
  		   break;
  	 case 'i': strncpy(incatname, argv[++i], MAXSTRINGLENGTH-1);
  		   break;
  	 case 't': strncpy(tablename, argv[++i], MAXSTRINGLENGTH-1);
  		   break;
  	 case 'p': strncpy(xaxisname, argv[++i], MAXSTRINGLENGTH-1);
                   strncpy(yaxisname, argv[++i], MAXSTRINGLENGTH-1);
  		   break;
	 case 'c': startconditions=i+1;
	           while (i+1<argc && !(argv[i+1][0] == '-' && strlen(argv[i+1]) == 2)) {
                      i++;
                      nconditions++;
                   }
  	 default:  
  		   break;
       }
    } 
    else 
    {
       syserr(FATAL, "ldacplot","Wrong Command line syntax\n");
    }
  }

  if((nconditions % 3) != 0)
  {
    syserr(FATAL, "ldacplot","Error in conditions");
  } 

  nconditions/=3;

  ED_MALLOC(condnames, "ldacplot", char *, nconditions);

  for(i=0; i< nconditions; i++)
    ED_MALLOC(condnames[i], "ldacplot", char, MAXSTRINGLENGTH);

  ED_MALLOC(condmin, "ldacplot", float, nconditions);
  ED_MALLOC(condmax, "ldacplot", float, nconditions);
  ED_MALLOC(conditionskey, "ldacplot", keystruct *, nconditions);

  for(i=0; i< nconditions; i++)
  {
    strncpy(condnames[i], argv[startconditions+i*3], MAXSTRINGLENGTH-1);
    condmin[i]=atof(argv[startconditions+i*3+1]);
    condmax[i]=atof(argv[startconditions+i*3+2]);
  }

  if (!(incat=read_cat(incatname)))
    syserr(FATAL, "ldacplot","Error opening input catalog");

  if (!(tab=name_to_tab(incat,tablename,0))) {
    sprintf(tmpstring1, "No %s table in catalog", tablename);
    syserr(FATAL, "ldacplot",tmpstring1);
  }

  xaxiskey=read_key(tab, xaxisname);
  if(xaxiskey == NULL)
  {
    sprintf(tmpstring1, "No %s key in catalog", xaxisname);
    syserr(FATAL, "ldacplot",tmpstring1);
  }
  
  yaxiskey=read_key(tab, yaxisname);
  if(yaxiskey == NULL)
  {
    sprintf(tmpstring1, "No %s key in catalog", yaxisname);
    syserr(FATAL, "ldacplot",tmpstring1);
  }
  
  for(i=0; i<nconditions; i++)
  {
    conditionskey[i]=read_key(tab, condnames[i]);
    if(conditionskey[i] == NULL)
    {
      sprintf(tmpstring1, "No %s key in catalog", condnames[i]);
      syserr(FATAL, "ldacplot",tmpstring1);
    } 
  }

  nobjects=xaxiskey->nobj;

  ED_MALLOC(goodxaxis, "ldacplot", float, nobjects);
  ED_MALLOC(goodyaxis, "ldacplot", float, nobjects);
  ED_MALLOC(badxaxis, "ldacplot", float, nobjects);
  ED_MALLOC(badyaxis, "ldacplot", float, nobjects);

  for(i=0; i<nobjects; i++)
  {
    objectgood=0;
    for(j=0; j<nconditions; j++)
    {
      switch(conditionskey[j]->ttype)
      {
        case T_SHORT:
          if((float)((short int *)conditionskey[j]->ptr)[i] < condmin[j] ||
             (float)((short int *)conditionskey[j]->ptr)[i] > condmax[j])
          {
            objectgood=1;
          }
          break;
        case T_LONG:
          if((float)((long int *)conditionskey[j]->ptr)[i] < condmin[j] ||
             (float)((long int *)conditionskey[j]->ptr)[i] > condmax[j])
          {
            objectgood=1;
          }
          break;
        case T_DOUBLE:
          if((float)((float *)conditionskey[j]->ptr)[i] < condmin[j] ||
             (float)((float *)conditionskey[j]->ptr)[i] > condmax[j])
          {
            objectgood=1;
          }
          break;
        case T_FLOAT:
          if((float)((float *)conditionskey[j]->ptr)[i] < condmin[j] ||
             (float)((float *)conditionskey[j]->ptr)[i] > condmax[j])
          {
            objectgood=1;
          }
          break;
        default:
          syserr(FATAL, "ldacplot","Only float's and float's supported");
          break;
      }
    }
    if(objectgood == 0)
    {
      switch(xaxiskey->ttype)
      {
        case T_SHORT:
          goodxaxis[ngoodobjects]=(float)(((short int *)xaxiskey->ptr)[i]);
          break;
        case T_LONG:
          goodxaxis[ngoodobjects]=(float)(((long int *)xaxiskey->ptr)[i]);
          break;
        case T_DOUBLE:
          goodxaxis[ngoodobjects]=(float)(((float *)xaxiskey->ptr)[i]);
          break;
        case T_FLOAT:
          goodxaxis[ngoodobjects]=(float)(((float *)xaxiskey->ptr)[i]);
          break;
        default:
          syserr(FATAL, "ldacplot","Only float's and float's supported");
          break;
      }
      switch(yaxiskey->ttype)
      {
        case T_SHORT:
          goodyaxis[ngoodobjects]=(float)(((short int *)yaxiskey->ptr)[i]);
          break;
        case T_LONG:
          goodyaxis[ngoodobjects]=(float)(((long int *)yaxiskey->ptr)[i]);
          break;
        case T_DOUBLE:
          goodyaxis[ngoodobjects]=(float)(((float *)yaxiskey->ptr)[i]);
          break;
        case T_FLOAT:
          goodyaxis[ngoodobjects]=(float)(((float *)yaxiskey->ptr)[i]);
          break;
        default:
          syserr(FATAL, "ldacplot","Only float's and float's supported");
          break;
      }
      ngoodobjects++;
    }
    else
    {
      switch(xaxiskey->ttype)
      {
        case T_SHORT:
          badxaxis[nbadobjects]=(float)(((short int *)xaxiskey->ptr)[i]);
          break;
        case T_LONG:
          badxaxis[nbadobjects]=(float)(((long int *)xaxiskey->ptr)[i]);
          break;
        case T_DOUBLE:
          badxaxis[nbadobjects]=(float)(((float *)xaxiskey->ptr)[i]);
          break;
        case T_FLOAT:
          badxaxis[nbadobjects]=(float)(((float *)xaxiskey->ptr)[i]);
          break;
        default:
          syserr(FATAL, "ldacplot","Only float's and float's supported");
          break;
      }
      switch(yaxiskey->ttype)
      {
        case T_SHORT:
          badyaxis[nbadobjects]=(float)(((short int *)yaxiskey->ptr)[i]);
          break;
        case T_LONG:
          badyaxis[nbadobjects]=(float)(((long int *)yaxiskey->ptr)[i]);
          break;
        case T_DOUBLE:
          badyaxis[nbadobjects]=(float)(((float *)yaxiskey->ptr)[i]);
          break;
        case T_FLOAT:
          badyaxis[nbadobjects]=(float)(((float *)yaxiskey->ptr)[i]);
          break;
        default:
          syserr(FATAL, "ldacplot","Only float's and float's supported");
          break;
      }
      nbadobjects++;
    }
  }

  printf("I plot %d good points\n", ngoodobjects);

  makeplot(goodxaxis, goodyaxis, 
           ngoodobjects, xaxisname, yaxisname,
           badxaxis, badyaxis, 
           nbadobjects, color, 
           xmin, xmax, ymin, ymax, pointtype, device);

  /* release memory and bye */
  free_cat(incat, 1);  
  ED_FREE(condmin);
  ED_FREE(condmax);
  for(i=0; i< nconditions; i++)
  {
    ED_FREE(condnames[i]);
  }
  ED_FREE(condnames);
  ED_FREE(goodxaxis);
  ED_FREE(goodyaxis);
  ED_FREE(badxaxis);
  ED_FREE(badyaxis);
  free(conditionskey);
}

void makeplot(float *axaxisgood, float *ayaxisgood, 
              int angoodobjects, char *axaxisname,
              char *ayaxisname,
              float *axaxisbad, float *ayaxisbad, 
              int anbadobjects, int acolor,
              float axmin, float axmax, 
              float aymin, float aymax,
              float apointtype,
              char *adevice)
{
  int i;
  float plotxmin, plotxmax, plotymin, plotymax;
  char tmpString[MAXSTRINGLENGTH];


  if((fabs(axmin) < EPSILON) && (fabs(axmax) < EPSILON) ||
     (fabs(aymin) < EPSILON) && (fabs(aymax) < EPSILON))
  {
    gibArrayminmax(axaxisgood, angoodobjects, &plotxmin, &plotxmax);
    gibArrayminmax(ayaxisgood, angoodobjects, &plotymin, &plotymax);
  }
  else
  {
    plotxmin=axmin;
    plotxmax=axmax;
    plotymin=aymin;
    plotymax=aymax;
  }

  if(sm_device(adevice) < 0) {
     syserr(FATAL, "ldacplot","unable to start sm");
  }

  sm_graphics();
  sm_expand(1.0001);
  sm_window(1,1,1,1,1,1);
  sm_limits(plotxmin, plotxmax, plotymin, plotymax);
  /* sm_ctype_i(2); */   /* black */
  sm_box(1,2,0,0);
  sm_ptype(&(apointtype), 1);
  sm_xlabel(axaxisname);
  sm_ylabel(ayaxisname);
  sm_ctype_i(1); /* ?? */
  sm_points(axaxisgood, ayaxisgood, angoodobjects);
  if(acolor == 1)
  {
    sm_ctype_i(3); /* red */
    sm_points(axaxisbad, ayaxisbad, anbadobjects);
  }

  sm_gflush();
  if(strcmp(adevice, "x11")==0)
  {
    sm_redraw(0);  
  }
  sm_hardcopy();
  sm_alpha();     
}
