/**
 ** input and output routines for catalogues
 **
 ** int		read_cat_head(cathead *catheadptr);
 ** int		write_cat_head(cathead *catheadptr);
 ** int		read_object(object *objectptr);
 ** int		write_object(object *objectptr);
 ** void	set_cat_ipf(FILE *ipf);
 ** void	set_cat_opf(FILE *opf);
 ** void	add_cat_comment(int argc, char **argv, cathead *catheadptr);

 **
 ** read_cat_head(), write_cat_head() & write_object() all error_exit()
 ** if they fail. read_object() just returns 0/1 for fail/success
 **/

/*
  23.01.01: substituted the "theipf" variable by stdin everywhere
            and the "theopf" by stdout. The static definitions in the
            program are not accepted by Linux. This disables the possibility <              to change the input/output stream.
            included some return 0 to avoid warnings when leaving int
            functions without returning a value

  19.01.01: substituted back the theipf and theopf in place for stdin, stdout.
            The input/output streams have to be set in the programs explicitely
            now (the old versions did not work uner Linux/Alpha)

  23.01.05: I changed the definition of theipf and theopf to extern
            (This avoids linking problems due to double definitions).
            For all programs that this is file is used in, they are 
            defined in fits.c already. I plan to get rid of newpeaks
            and hcat2ldac needing this file anyway.             
*/
 
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "stats_stuff.h"
#include "object_stuff.h"
#include "error.h"
#include "catio.h"

extern FILE	*theipf, *theopf;



int		read_cat_head(cathead *catheadptr)
{
	int result;
	
	result = fread(catheadptr, sizeof(cathead), 1, theipf);
	/* check version number */
	if (catheadptr->version_number != VERSION_NUMBER)
		error_exit("read_cat_head: old catalogue version\n");
	if (result != 1)
		error_exit("read_cat_head: bad read\n");
	return (result);
}

 
 

int		write_cat_head(cathead *catheadptr)
{
	int result;
	
	catheadptr->version_number = VERSION_NUMBER;
	result = fwrite(catheadptr, sizeof(cathead), 1, theopf);
	if (result != 1)
		error_exit("write_cat_head: bad write\n");
	return (result);
}





int		read_object(object *objectptr)
{
	return (fread(objectptr, sizeof(object), 1, theipf));
}






int		write_object(object *objectptr)
{
	int result;
	
	result = fwrite(objectptr, sizeof(object), 1, theopf);
	if (result != 1)
		error_exit("write_object: bad write\n");
	return (result);
}


/* next four functions are for converting from binary to text */
/* format. These are used to transfer between machines with */
/* discordant byte ordering */

int		text_read_cat_head(cathead *catheadptr)
{
	int	i, j;

	fscanf(theipf, "%s\n", catheadptr->image);
	fscanf(theipf, "%e\n", &(catheadptr->zeropoint));
	fscanf(theipf, "%e\n", &(catheadptr->nu));
	fscanf(theipf, "%e\n", &(catheadptr->rf));
	fscanf(theipf, "%e\n", &(catheadptr->f_thresh));
	/* now the statsrec */
	fscanf(theipf, "%e\n", &(catheadptr->srec.fmin));
	fscanf(theipf, "%e\n", &(catheadptr->srec.fmax));
	fscanf(theipf, "%e\n", &(catheadptr->srec.fmedian));
	fscanf(theipf, "%e\n", &(catheadptr->srec.fupperquartile));
	fscanf(theipf, "%e\n", &(catheadptr->srec.flowerquartile));
	fscanf(theipf, "%e\n", &(catheadptr->srec.fmode));
	fscanf(theipf, "%e\n", &(catheadptr->srec.sigma));
	fscanf(theipf, "%ld\n", &(catheadptr->srec.badpix));
	/* and the rest of the header */
	fscanf(theipf, "%d\n", &(catheadptr->N));
	fscanf(theipf, "%d\n", &(catheadptr->version_number));
	fscanf(theipf, "%e\n", &(catheadptr->rpsf));
	fscanf(theipf, "%d\n", &(catheadptr->comc));
	for (i = 0; i < catheadptr->comc; i++) {
		j = 0;
		while (1) {
			catheadptr->com[i][j] = fgetc(theipf);
			if (catheadptr->com[i][j] == '\n') {
				catheadptr->com[i][j] = '\0';
				break;
			}
			j++;
		}
	}
	fscanf(theipf, "%d\n", &(catheadptr->hassky));
	for (i = 0; i < 4; i++) {
		fscanf(theipf, "%d\n", &(catheadptr->ispare[i]));
	}
	for (i = 0; i < 4; i++) {
		fscanf(theipf, "%e\n", &(catheadptr->fspare[i]));
	}

	return 0;
}

int		text_write_cat_head(cathead *catheadptr)
{
	int	i;

	fprintf(theopf, "%s\n", catheadptr->image);
	fprintf(theopf, "%e\n", catheadptr->zeropoint);
	fprintf(theopf, "%e\n", catheadptr->nu);
	fprintf(theopf, "%e\n", catheadptr->rf);
	fprintf(theopf, "%e\n", catheadptr->f_thresh);
	/* now the statsrec */
	fprintf(theopf, "%e\n", catheadptr->srec.fmin);
	fprintf(theopf, "%e\n", catheadptr->srec.fmax);
	fprintf(theopf, "%e\n", catheadptr->srec.fmedian);
	fprintf(theopf, "%e\n", catheadptr->srec.fupperquartile);
	fprintf(theopf, "%e\n", catheadptr->srec.flowerquartile);
	fprintf(theopf, "%e\n", catheadptr->srec.fmode);
	fprintf(theopf, "%e\n", catheadptr->srec.sigma);
	fprintf(theopf, "%ld\n", catheadptr->srec.badpix);
	/* and the rest of the header */
	fprintf(theopf, "%d\n", catheadptr->N);
	fprintf(theopf, "%d\n", catheadptr->version_number);
	fprintf(theopf, "%e\n", catheadptr->rpsf);
	fprintf(theopf, "%d\n", catheadptr->comc);
	for (i = 0; i < catheadptr->comc; i++) {
		fprintf(theopf, "%s\n", catheadptr->com[i]);
	}
	fprintf(theopf, "%d\n", catheadptr->hassky);
	for (i = 0; i < 4; i++) {
		fprintf(theopf, "%d\n", catheadptr->ispare[i]);
	}
	for (i = 0; i < 4; i++) {
		fprintf(theopf, "%e\n", catheadptr->fspare[i]);
	}

	return 0;
}

int		text_read_object(object *objectptr)
{
	int	i;

	if (EOF == fscanf(theipf, "%d\n", &(objectptr->i)))
		return (0);
	fscanf(theipf, "%d\n", &(objectptr->j));
	fscanf(theipf, "%e\n", &(objectptr->nu));
	fscanf(theipf, "%e\n", &(objectptr->fs));
	fscanf(theipf, "%e\n", &(objectptr->rg));
	fscanf(theipf, "%e\n", &(objectptr->Psm12));
	fscanf(theipf, "%d\n", &(objectptr->rnumax));
	fscanf(theipf, "%e\n", &(objectptr->numax));
	fscanf(theipf, "%d\n", &(objectptr->rmax));
	fscanf(theipf, "%e\n", &(objectptr->rh));
	fscanf(theipf, "%e\n", &(objectptr->l));
	fscanf(theipf, "%e\n", &(objectptr->lg));
	fscanf(theipf, "%e\n", &(objectptr->Psh11));
	fscanf(theipf, "%e\n", &(objectptr->Psh22));
	fscanf(theipf, "%e\n", &(objectptr->e[0]));
	fscanf(theipf, "%e\n", &(objectptr->e[1]));
	fscanf(theipf, "%e\n", &(objectptr->esigma[0]));
	fscanf(theipf, "%e\n", &(objectptr->esigma[1]));
	for (i = 0; i < 4; i++)
		fscanf(theipf, "%d\n", &(objectptr->sky.n[i]));
	for (i = 0; i < 4; i++)
		fscanf(theipf, "%e\n", &(objectptr->sky.i[i]));
	for (i = 0; i < 4; i++)
		fscanf(theipf, "%e\n", &(objectptr->sky.j[i]));
	fscanf(theipf, "%d\n", &(objectptr->nmagic));
	fscanf(theipf, "%d\n", &(objectptr->ncoinc));
	fscanf(theipf, "%d\n", &(objectptr->class));
	fscanf(theipf, "%d\n", &(objectptr->status));
	for (i = 0; i < 4; i++)
		fscanf(theipf, "%d\n", &(objectptr->ispare[i]));
	fscanf(theipf, "%e\n", &(objectptr->invr2));
	fscanf(theipf, "%e\n", &(objectptr->Psm11));
	fscanf(theipf, "%e\n", &(objectptr->Psm22));
	fscanf(theipf, "%e\n", &(objectptr->Psh12));
	return (1);
}

int		text_write_object(object *objectptr)
{
	int	i;

	fprintf(theopf, "%d\n", objectptr->i);
	fprintf(theopf, "%d\n", objectptr->j);
	fprintf(theopf, "%e\n", objectptr->nu);
	fprintf(theopf, "%e\n", objectptr->fs);
	fprintf(theopf, "%e\n", objectptr->rg);
	fprintf(theopf, "%e\n", objectptr->Psm12);
	fprintf(theopf, "%d\n", objectptr->rnumax);
	fprintf(theopf, "%e\n", objectptr->numax);
	fprintf(theopf, "%d\n", objectptr->rmax);
	fprintf(theopf, "%e\n", objectptr->rh);
	fprintf(theopf, "%e\n", objectptr->l);
	fprintf(theopf, "%e\n", objectptr->lg);
	fprintf(theopf, "%e\n", objectptr->Psh11);
	fprintf(theopf, "%e\n", objectptr->Psh22);
	fprintf(theopf, "%e\n", objectptr->e[0]);
	fprintf(theopf, "%e\n", objectptr->e[1]);
	fprintf(theopf, "%e\n", objectptr->esigma[0]);
	fprintf(theopf, "%e\n", objectptr->esigma[1]);
	for (i = 0; i < 4; i++)
		fprintf(theopf, "%d\n", objectptr->sky.n[i]);
	for (i = 0; i < 4; i++)
		fprintf(theopf, "%e\n", objectptr->sky.i[i]);
	for (i = 0; i < 4; i++)
		fprintf(theopf, "%e\n", objectptr->sky.j[i]);
	fprintf(theopf, "%d\n", objectptr->nmagic);
	fprintf(theopf, "%d\n", objectptr->ncoinc);
	fprintf(theopf, "%d\n", objectptr->class);
	fprintf(theopf, "%d\n", objectptr->status);
	for (i = 0; i < 4; i++)
		fprintf(theopf, "%d\n", objectptr->ispare[i]);
	fprintf(theopf, "%e\n", objectptr->invr2);
	fprintf(theopf, "%e\n", objectptr->Psm11);
	fprintf(theopf, "%e\n", objectptr->Psm22);
	fprintf(theopf, "%e\n", objectptr->Psh12);

	return 0;
}




void	set_cat_ipf(FILE *ipf)
{
	theipf = ipf;
}



void	set_cat_opf(FILE *opf)
{
	theopf = opf;
}



void	add_cat_comment(int argc, char **argv, cathead *cath)
{
	int	i, j, pos = 0, len;
	char	*string;
	
	if (cath->comc > MAX_CAT_COMMENTS - 1)
		return;

	string = cath->com[cath->comc];
	for (i = 0; i < argc; i++) {
		len = strlen(argv[i]);
		if (pos + len < CAT_COM_LENGTH - 3) {
			for (j = 0; j < len; j++)
				string[pos++] = argv[i][j];
			string[pos++] = ' ';
		}
	}
	string[pos] = '\0';
	cath->comc++;
}



