/* 
   December 6, 1998: deleted calculation of first guess ellipticities;
                     introduced storing of first guess centroid positions
   December 14,1998: added hfindpeaks table to hold sky parameters that
                     analyse needs; integrated writeldactabs into the main
                     program
   December 16,1998: added FIELD_POS key in OBJECTS table; added also a sequence
                     number for the objects.
   March 16, 1999: included types.h

   Jamuary 10, 2001: removed the ED_FREE's after the adding to the table
                     (problems under OSF); removed unused variables

   Tue May 22 2001 replaced all doubles by floats. The accuracy is sufficient,
                   the program needs less memory and the type is consistent
                   with SExtractor.

   Sat Jan 19 2002 included 'set_cat_ipf(stdin)' to set explicitely the
                   input catalog stream as is now necessary after the changes
                   made in catio.c, fits.c
   Sun Jan 29 2005 I now add HISTORY information to the output catalog.		   
*/

#define	usage "\n\n\n\
SYNOPSIS\n\
		hcat2ldac [options...] <hcatfile >catfile\n\
			-a		# fire on all local maxima\n\
                       [-o              # set name of output catalog (output.cat)]\n\
DESCRIPTION\n\
		\"hcat2ldac\" reads a set of peak trajectories in \"hcat\"\n\
		format from stdin and applies an algorithm to pick out\n\
		particular points (e.g. points of max significance).\n\
		Default is to pick only the most significant local maxima\n\
		along a peak trajectory, but -a option finds all local maxima.\n\
		Standard format id LDAC catalog format.\n\
\n\n\n"		

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>

#include "error.h"
#include "stats_stuff.h"
#include "object_stuff.h"
#include "catio.h"
#include "newpeaks.h"
#include "fitscat.h"
#include "utils_macros.h"
#include "utils_globals.h"
#include "tabutil_globals.h"

#define BESTPEAK	0
#define ALLPEAKS	1
#define MAXSTRINGLENGTH 256

void	process(peak *thepeak, int peakcount, int mode);
float	interp(float d, float x1, float x2, float x3);
void    createldaccat(char *);
void    write_ldac_object(object *);   

/* global variables: */

LONG *ixpos, *iypos;
float *dxpos, *dypos;
float *drg, *dnu;
short *ifieldpos;
LONG *inumber;
LONG nobjects=0;
LONG nobjectsmemory=1000;

catstruct *outcat;

int main(int argc, char *argv[])	
{
	cathead		thecat;
	line		*linehead = NULL, *theline;
	peak		*newpeak, *thepeak;
	int		arg = 1, peakcount = 0, mode;
	char            catalogname[MAXSTRINGLENGTH]="output.cat"; 

        /* stuff for the LDAC catalog */
        tabstruct       *fieldstab, *objectstab, *hfindpeakstab; 
        LONG        obj_pos, obj_count, chan_nr;
        char            chan_name[4]="KSB";

	/* defaults */
	mode = BESTPEAK;

	while (arg < argc) {
		if (argv[arg][0] != '-')
			error_exit(usage);
		switch (argv[arg++][1]) {
			case 'a':
				mode = ALLPEAKS;
				break;
			case 'o':
			        strncpy(catalogname, argv[arg++], MAXSTRINGLENGTH-1);
				break;
			default:
				error_exit(usage);
		}
	}

	set_cat_ipf(stdin);
	read_cat_head(&thecat);
	add_cat_comment(argc, argv, &thecat);
	ED_MALLOC(ixpos, "main", LONG, nobjectsmemory);
	ED_MALLOC(iypos, "main", LONG, nobjectsmemory);
	ED_MALLOC(ifieldpos, "main", short, nobjectsmemory);
	ED_MALLOC(inumber, "main", LONG, nobjectsmemory);
	ED_MALLOC(dxpos, "main", float, nobjectsmemory);
	ED_MALLOC(dypos, "main", float, nobjectsmemory);
	ED_MALLOC(drg, "main", float, nobjectsmemory);
	ED_MALLOC(dnu, "main", float, nobjectsmemory);

	theline = linehead;
	newpeak = (peak *) calloc(1, sizeof(peak));
	thepeak = NULL;
	while(1 == fread(newpeak, sizeof(peak), 1, stdin)) {
		peakcount++;
		if (newpeak->next == NULL) {	/* end of a line */
			newpeak->next = thepeak;
			thepeak = newpeak;
			process(thepeak, peakcount, mode);
			peakcount = 0;
			thepeak = NULL;
		} else {
			newpeak->next = thepeak;
			thepeak = newpeak;
		}
		newpeak = (peak *) calloc(1, sizeof(peak));
	}

  	createldaccat(catalogname);

 	/* make the fields table */
 	obj_pos = 1;
 	obj_count = nobjects;
 	chan_nr = 1;

 	fieldstab=new_tab("FIELDS");
      
 	add_key_to_tab(fieldstab, "OBJECT_POS", 1, &(obj_pos), T_LONG, 1, "",
 				 "Starting position of field");
 	add_key_to_tab(fieldstab, "OBJECT_COUNT", 1, &(obj_count), T_LONG, 1, "",
 				 "Number of objects in field");
 	add_key_to_tab(fieldstab, "CHANNEL_NR", 1, &(chan_nr), T_LONG, 1, "",
 				 "Channel numbers");
 	add_key_to_tab(fieldstab, "CHANNEL_NAME", 1, chan_name, T_STRING, 4, "",
 				 "Channel name");
 	update_tab(fieldstab);
 	if (add_tab(fieldstab,outcat,0) == RETURN_ERROR) 
 	{
 	   syserr(FATAL, "main", "Error adding FIELDS table to %s\n",
 	     outcat->filename);
 	}

        /* make the hfindpeaks table */
 	hfindpeakstab=new_tab("HFINDPEAKS");

 	add_key_to_tab(hfindpeakstab, "IMAGE", 1, thecat.image, T_STRING, 128, "",
 				 "the image that was processed");
 	add_key_to_tab(hfindpeakstab, "SKY_MODE", 1, &(thecat.srec.fmode), T_FLOAT, 1, "",
 				 "Mode of all pixels");
 	add_key_to_tab(hfindpeakstab, "SKY_SIGMA", 1, &(thecat.srec.sigma), T_FLOAT, 1, "",
 				 "Sigma of all pixels");

 	if (add_tab(hfindpeakstab,outcat,0) == RETURN_ERROR) 
 	{
 	   syserr(FATAL, "main", "Error adding HFINDPEAKS table to %s\n",
 	     outcat->filename);
 	}

 	/* make the objects table */

 	objectstab=new_tab("OBJECTS");
 	add_key_to_tab(objectstab, "FIELD_POS", nobjects, ifieldpos,
 			      T_SHORT, 1, "", "Reference number to field parameters");  

	/*ED_FREE(ifieldpos);*/
 	add_key_to_tab(objectstab, "SeqNr", nobjects, inumber,
 			      T_LONG, 1, "", "running sequence number");  
	/*ED_FREE(inumber);*/
 	add_key_to_tab(objectstab, "x", nobjects, ixpos,
 			      T_LONG, 1, "pixel", "x pixel of maximal sign. (hfindpeaks)");  
	/*ED_FREE(ixpos);*/
 	add_key_to_tab(objectstab, "y", nobjects, iypos,
 			      T_LONG, 1, "pixel", "y pixel of maximal sign. (hfindpeaks)");  
	/*ED_FREE(iypos);*/
 	add_key_to_tab(objectstab, "xbad", nobjects, dxpos,
 			      T_FLOAT, 1, "pixel", "x centroid position (hfindpeaks)");
	/*ED_FREE(dxpos);*/
 	add_key_to_tab(objectstab, "ybad", nobjects, dypos,
 			      T_FLOAT, 1, "pixel", "y centroid position (hfindpeaks)");
	/*ED_FREE(dypos);*/
 	add_key_to_tab(objectstab, "rg", nobjects, drg,
 			      T_FLOAT, 1, "pixel", "smoothing scale of max. sign. (hfindpeaks)");
	/*ED_FREE(drg);*/
 	add_key_to_tab(objectstab, "nu", nobjects, dnu,
 			      T_FLOAT, 1, "", "maximum significance (hfindpeaks)");
	/*ED_FREE(dnu);*/
 	
 	if (add_tab(objectstab,outcat,0) == RETURN_ERROR) 
 	{
 	   syserr(FATAL, "main", "Error adding OBJECTS table to %s\n",
 	     outcat->filename);
 	}

	/* add some HISTORY information to the output catalg */
        historyto_cat(outcat, "hcat2ldac", argc, argv);

	save_cat(outcat, catalogname);
	/*free_cat(outcat, 1);*/

	return 0;
}




void	process(peak *thepeak, int peakcount, int mode)
{
	peak		**pk;
	int		i = 0;
	static	object	*obj = NULL, *bestobj, *tempobj;
	float		d, y1, y2, y3, bestnu = 0.0;

	if (!obj)
		obj = (object *) calloc(1, sizeof(object));
	if (!bestobj)
		bestobj = (object *) calloc(1, sizeof(object));

	pk = (peak **) calloc(peakcount, sizeof(peak));

	while (thepeak) {
		pk[i++] = thepeak;
		thepeak = thepeak->next;
	}

	for (i = 1; i < peakcount - 1; i++) {
		if (pk[i]->nu > pk[i-1]->nu && pk[i]->nu > pk[i+1]->nu) {
			y1 = pk[i-1]->nu;
			y2 = pk[i]->nu;
			y3 = pk[i+1]->nu;
			d = 0.5 * (y3 - y1) / (2 * y2 - y1 - y3);
			d = (d > 1 ? 1 : d);
			d = (d < -1 ? -1 : d);
			obj->i = pk[i]->i;
			obj->j = pk[i]->j;
			obj->xbad[0] = pk[i]->x[0];
			obj->xbad[1] = pk[i]->x[1];
			obj->rg = interp(d, pk[i-1]->rf, pk[i]->rf, pk[i+1]->rf);
			obj->nu = interp(d, pk[i-1]->nu, pk[i]->nu, pk[i+1]->nu);
			if (mode == ALLPEAKS) {
				write_ldac_object(obj);
				continue;
			} else {
				if (obj->nu > bestnu) {
					bestnu = obj->nu;
					/* swap them */
					tempobj = obj;
					obj = bestobj;
					bestobj = tempobj;
				}
			}
		}
	}

	if (mode == BESTPEAK && bestnu > 0) 
		write_ldac_object(bestobj);

	for (i = 0; i < peakcount; i++)
		free(pk[i]);
	free(pk);
}


float	interp(float d, float x1, float x2, float x3)
{
	if (d > 0)
		return((1 - d) * x2 + d * x3);
	else
		return((1 + d) * x2 - d * x1);
}

void    createldaccat(char *aname)
{
    outcat = new_cat(1);
    init_cat(outcat);
    strcpy(outcat->filename, aname);
    if (open_cat(outcat, WRITE_ONLY) != RETURN_OK)
    {
      error_exit("Cannot open output catalog");
    }
}

void write_ldac_object(object* aobj)
{
  if(nobjects==nobjectsmemory)
  {
    nobjectsmemory+=1000;

    ED_REALLOC(ixpos, "write_ldac_object", LONG, nobjectsmemory);
    ED_REALLOC(iypos, "write_ldac_object", LONG, nobjectsmemory);
    ED_REALLOC(ifieldpos, "write_ldac_object", short, nobjectsmemory);
    ED_REALLOC(inumber, "write_ldac_object", LONG, nobjectsmemory);
    ED_REALLOC(dxpos, "write_ldac_object", float, nobjectsmemory);
    ED_REALLOC(dypos, "write_ldac_object", float, nobjectsmemory);
    ED_REALLOC(drg, "write_ldac_object", float, nobjectsmemory);
    ED_REALLOC(dnu, "write_ldac_object", float, nobjectsmemory);

  }

  ixpos[nobjects]=aobj->j;
  iypos[nobjects]=aobj->i;
  ifieldpos[nobjects]=1;
  inumber[nobjects]=nobjects+1;
  drg[nobjects]=aobj->rg;
  dnu[nobjects]=aobj->nu;
  dxpos[nobjects]=aobj->xbad[0];
  dypos[nobjects]=aobj->xbad[1];

  nobjects ++;
}




