// Program to determine the mean of quantities using the
// N nearest neughbours of an object in a user defined 
// space.
//
// The program is largely a copy of sigmaellip.cpp
//
// 20.04,03:
// - commented the freeing of mean, median and sigma at
//   the end of the code as this is done probably
//   when freeing the catalog keys
//
//   05.05.03:
//   included the tsize.h file
//
// 10.01.05:
// I removed compiler warnings under gcc with optimisation
// enabled (-O3).
//
// 29.01.2005:
// I now add HISTORY information to the output catalog.
//
// 13.08.2008:
// I changed the inclusion of iostream.h to iostream (the first
// is no longer supported by gcc 4.3)

#include <iostream>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "ANN.h"
#include "fitscat.h"
#include "tsize.h"
#include "utils_globals.h"
#include "tabutil_globals.h"

using namespace std;

// HISTORY:
// 26.10.1999: project started
//             
// TODO: check that all memory if freed at the end.
//
// STRANGE crashes with two arguments and free cat's
//
// 15.07.2001:
// included "tabutil_globals.h" to declare add_key_to tab

#define __PROGRAMMVERSION__ "1.0"
#define __PROGRAMMNAME__ "neighbourmean"

#define __USAGE__ "\n\
USAGE:\n\
       neighbourmean -i input catalog\n\
                     -o output catalog\n\
                     -c space coordinates\n\
                     -m mean quantities\n\
                   [ -t table with the objects (OBJECTS) ]\n\
                   [ -n number of nearest neighbours (10) ]\n\n\
DESCRIPTION:\n\
The program calculates the mean of all 'mean quantities' using\n\
N nearest neighbours in a user defined parameter space. The new\n\
quantities are appended to the table with names 'mean...', 'sigma...'\n\
where '...' stands for the original names of the quantities.\n\n\
EXAMPLE:\n\
  neighbourmean -i in.cat -o out.cat -c rg mag -m Pgamma_11 Pgamma_22 -n 20\n\n\
This would calculate for every object the mean of Pgamma_11 and\n\
Pgamma_22 using 20 nearest neighbours in the rg, mag plane.\n\n\
AUTHOR:\n\
      Thomas Erben\n"

#ifndef MAXSTRINGLENGTH
#define MAXSTRINGLENGTH 256
#endif

// Global array: needed for median determination
double *Gsortarray;

int quancmp(double *obj1, double *obj2);

int main(int argc, char **argv)
{
  char incatname[MAXSTRINGLENGTH];
  char outcatname[MAXSTRINGLENGTH]="out.cat";
  char tablename[MAXSTRINGLENGTH]="OBJECTS";
  char tmpstring[MAXSTRINGLENGTH];

  char **keynames;
  int startcoords = 0;
  int ncoords=0;     // the number of dimensions for the
                     // user defined space 
  int startmeanquan = 0;  
  int nmeanquan=0;   // the number of quantities for which
                     // the mean shall be calculated      
 
  catstruct *incat;
  catstruct *outcat;
  tabstruct *tab;
  keystruct **keys;

  double **keyptr;
  double **mean;
  double **median;
  double **sigma;

  int calcmedian=1; // how to calculate the median (dependent whether
                    // nneighbours is odd or even)
                    // (1 means that the mean of the two middle
                    // values is taken (even number of elements))
  int middle;       // nneighbours/2 (int value)

  // variables fro the next neighbour search
  int nobj;  // the number of objects in the catalog
  int nneighbours = 10; // the number of next neighbours
  ANNpointArray data_pts;
  ANNpoint      query_pt;
  ANNidxArray   nn_idx;
  ANNdistArray  dists;
  ANNkd_tree    *the_tree;

  // other variables
  int i, j, k, l;

  // read command line arguments
  for (i=1; i<argc; i++) 
  {
    if (argv[i][0] == '-') 
    {
       switch(tolower(argv[i][1])) 
       {
         case 'i': strncpy(incatname, argv[++i], MAXSTRINGLENGTH-1);
                   break;
         case 'o': strncpy(outcatname, argv[++i], MAXSTRINGLENGTH-1);
                   break;
         case 't': strncpy(tablename, argv[++i], MAXSTRINGLENGTH-1);
                   break;
         case 'n': nneighbours = atoi(argv[++i]);
                   break;
         case 'c': startcoords=i+1;
                   while (i+1<argc && argv[i+1][0] != '-') 
                   {
                      i++;
                      ncoords++;
                   }
                   break;
         case 'm': startmeanquan=i+1;
                   while (i+1<argc && argv[i+1][0] != '-') 
                   {
                      i++;
                      nmeanquan++;
                   }
                   break;
 
         default:  
                   break;
       }
    } 
    else 
    {
       syserr(FATAL, "main", "Wrong Command line syntax\n");
    }
  }

  // print start message:
  cout << "\n" << __PROGRAMMNAME__ << "  V" << __PROGRAMMVERSION__ 
       << " (" << __DATE__ << ";" << __TIME__ << ")" << endl << endl;

  if(argc < 8)
  {
    cout << __USAGE__ << endl;
    exit(1);
  }

  if(ncoords < 1)
  {
    syserr(FATAL, "main", "no coordinate given\n");
  }

  if(nmeanquan < 1)
  {
    syserr(FATAL, "main", "nothing to do!!\n");
  }

  // we need the coordinate keys and the keys for the quantity
  // to calculate the mean (median)
  ED_CALLOC(keynames, "main", char *, ncoords+nmeanquan);
  for (i=0;i<ncoords+nmeanquan;i++) 
  {
    ED_CALLOC(keynames[i], "main", char, MAXSTRINGLENGTH);
  }

  for(i=0; i< ncoords; i++)
  {
    strncpy(keynames[i], argv[startcoords+i], MAXSTRINGLENGTH-1);
  }

  for(i=0; i< nmeanquan; i++)
  {
    strncpy(keynames[ncoords+i], argv[startmeanquan+i], MAXSTRINGLENGTH-1);
  }

  
  // read catalog and quantities:
  if((incat=read_cat(incatname))==NULL)
  {
    syserr(FATAL, "main", "Error opening input catalog\n");
  }

  outcat=new_cat(1);
  inherit_cat(incat, outcat);
  copy_tabs(incat,outcat);
  
  if((tab=name_to_tab(outcat,tablename,0))==NULL)
  {
    syserr(FATAL, "main", "No %s table in catalog\n", tablename);
  }
  
  // read keys:
  ED_CALLOC(keys, "main", keystruct *, ncoords+nmeanquan);
  ED_CALLOC(keyptr, "main", double *, ncoords+nmeanquan);

  read_keys(tab, keynames, keys, ncoords+nmeanquan, NULL);
  
  for(j=0; j<ncoords+nmeanquan; j++)
  {
    if(!keys[j])
    {
      syserr(FATAL, "main", "No key %s in table\n", keynames[j]);
    }

    switch (keys[j]->ttype)
    {
    case T_FLOAT:
      ED_GETVEC(keyptr[j], "main", keys[j], 
                keys[j]->name, double, float);  
      break;
    case T_DOUBLE:
      ED_GETVEC(keyptr[j], "main", keys[j], 
                keys[j]->name, double, double);
      break;
    case T_LONG:
      ED_GETVEC(keyptr[j], "main", keys[j], 
                keys[j]->name, double, long);
      break;
    case T_SHORT:
      ED_GETVEC(keyptr[j], "main", keys[j], 
                keys[j]->name, double, short);
      break;
    default:
       syserr(FATAL, "main", "Unsupported key type key type in %s\n", keys[j]->name);        
      break;
    }
  }

  nobj = keys[0]->nobj;

  // prepare stuff for the next neighbour search:
  query_pt = annAllocPt(ncoords);
  data_pts = annAllocPts(nobj, ncoords);
  nn_idx = new ANNidx[nneighbours];
  dists = new ANNdist[nneighbours];
  ED_CALLOC(mean, "main", double*, nmeanquan);
  ED_CALLOC(median, "main", double*, nmeanquan);
  ED_CALLOC(sigma, "main", double*, nmeanquan);
  ED_CALLOC(Gsortarray, "main", double, nneighbours);

  // how is the median calculated?
  if((middle = nneighbours/2)*2 == nneighbours)
  {
    calcmedian = 0;
  }
  
  for(i=0; i< nmeanquan; i++)
  {
    ED_CALLOC(mean[i], "main", double, nobj);
    ED_CALLOC(median[i], "main", double, nobj);
    ED_CALLOC(sigma[i], "main", double, nobj);
  }

  // copy the coordinates into the required matrix structure
  // of the ANNpointArray

  for(i=0; i< nobj; i++)
  {
    for(j=0; j< ncoords; j++)
    {
      data_pts[i][j] = keyptr[j][i];
    }
  }

  cout << "building search tree ..." << endl;

  the_tree = new ANNkd_tree((ANNpointArray)(data_pts), nobj, ncoords);

  for(i=0; i< nobj; i++)
  {
    for(j=0; j<ncoords; j++)
    {
      query_pt[j] = keyptr[j][i];
    }

    the_tree->annkSearch(query_pt, nneighbours, nn_idx, dists, 0.0);

    for(j=0; j< nneighbours; j++)
    {
      for(k=0; k < nmeanquan; k++)
      {
        mean[k][i] += keyptr[ncoords+k][nn_idx[j]];
        sigma[k][i] += keyptr[ncoords+k][nn_idx[j]]*keyptr[ncoords+k][nn_idx[j]];
      }
    }

    for(k=0; k < nmeanquan; k++)
    {
      sigma[k][i] = sigma[k][i]/nneighbours-
          	    (mean[k][i]/nneighbours)*(mean[k][i]/nneighbours);
      sigma[k][i] = sqrt(sigma[k][i]);
      mean[k][i] /= nneighbours;
    } 
    
    // calculate median
    for(k=0; k < nmeanquan; k++)
    {
    	// store the values of the quantity in a dummy array
    	// and sort it.
        for(l=0; l < nneighbours; l++)
        {
          Gsortarray[l] = keyptr[ncoords+k][nn_idx[l]];
        }
        qsort(Gsortarray, nneighbours, sizeof(double), 
              (int(*)(const void*, const void*))quancmp);
    
    	if(calcmedian == 1)
    	{
    	  median[k][i] = Gsortarray[middle];
    	}
    	else
    	{
    	  median[k][i] = 0.5*(Gsortarray[middle-1]+Gsortarray[middle]);
    	}
    }
  }

  cout << "adding quantities to catalogs ...." << endl;

  for(i=0; i< nmeanquan; i++)
  {  
    strcpy(tmpstring, "mean");
    // The '-4' comes from "mean"
    strncat(tmpstring, keynames[ncoords+i], MAXSTRINGLENGTH-1-4); 
    add_key_to_tab(tab, tmpstring, nobj, mean[i], T_DOUBLE,
                          1, "", "");


    strcpy(tmpstring, "median");
    // The '-6' comes from "median"
    strncat(tmpstring, keynames[ncoords+i], MAXSTRINGLENGTH-1-6); 
    add_key_to_tab(tab, tmpstring, nobj, median[i], T_DOUBLE,
                          1, "", "");

    strcpy(tmpstring, "sigma");
    // The '-5' comes from "sigma"
    strncat(tmpstring, keynames[ncoords+i], MAXSTRINGLENGTH-1-5); 
    add_key_to_tab(tab, tmpstring, nobj, sigma[i], T_DOUBLE,
                          1, "", "");

  }

  // free memory and bye
  delete [] nn_idx;
  delete [] dists;

  annDeallocPt(query_pt);
  annDeallocPts(data_pts);

  delete the_tree;

  /*for(i=0; i< nmeanquan; i++)
  {
    ED_FREE(mean[i]);
    ED_FREE(median[i]);
    ED_FREE(sigma[i]);
  }

  ED_FREE(mean);
  ED_FREE(median);
  ED_FREE(sigma);
  */

  ED_FREE(Gsortarray);

  for (i=0;i<ncoords;i++) 
  {
    ED_FREE(keynames[i]);
  }
  ED_FREE(keynames);

  /* add some history information to the output catalog */
  historyto_cat(outcat, "neighbourmen", argc, argv);

  save_cat(outcat, outcatname);

  free_cat(incat, 1);
  free_cat(outcat, 1);
    
  return 0;
}  

int quancmp(double *obj1, double *obj2)
{
  return (*obj1 > *obj2 ? 1 : (*obj1 == *obj2 ? 0 : -1));
}
