/* 15.06.2001:
   removing compiler warnings under DEC (including
   explicit casts to (char *) for the return value of two functions

   20.07.2004:
   I removed compiler warnings due to possibly undefined
   variables (compilation with optimisation turned on)

   24.01.2005:
   I added explicit casts at several points. As these routines
   are currenty not used, it is not clear whether they
   work correctly.

   25.09.2009:
   removing compiler warnings on unused function parameters

   30.12.2012:
   I removed compiler warnings on set bot never used variables
*/

#include <stdio.h>
#include <math.h>
#if defined(__GNUC__)
#include <sys/types.h>
#include <unistd.h>
#endif
#include "casts_globals.h"
#include "options_globals.h"
#include "fits_image_io_globals.h"
#include "utils_globals.h"

#define FITSBLOCK 2880

char *decomp_basebyte(unsigned char *b, int nrecs, int npix,
                    double bscale, double bzero)
{
     unsigned char a, aa[2], aaaa[4];
     int  icur, irec, ipc, lsum, char_offset, ipix;
     int  ipixmax;
     float *map;
     
     ED_MALLOC(map, "decomp_basebyte", float, npix);
     nrecs = nrecs/FITSBLOCK;
     icur = 0;
     for (irec=0;irec<nrecs;irec++) {
        lsum = 0;
        ipc = 6;
        char_offset = irec*FITSBLOCK;
        ipixmax = short_cast(&b[char_offset+4]);
        for (ipix=0; ipix < ipixmax; ipix++) {
           a = b[char_offset+ipc++];
           if ((char)a != -128) {
              map[icur] = (float) a * bscale + bzero;
              lsum += (int) a;
           } else {
              aa[0] = b[char_offset+ipc++];
              aa[1] = b[char_offset+ipc++];
              if (short_cast(aa) == -32768) {
                 aaaa[0] = b[char_offset+ipc++];
                 aaaa[1] = b[char_offset+ipc++];
                 aaaa[2] = b[char_offset+ipc++];
                 aaaa[3] = b[char_offset+ipc++];
                 map[icur] = (float) int_cast(aaaa) * bscale + bzero;
                 lsum += int_cast(aaaa);
              } else {
                 map[icur] = (float) short_cast(aa) * bscale + bzero;
                 lsum += (int) short_cast(aa);
              }
           }
           icur++;
        }
/*
        csum = int_cast(&b[char_offset+ipc]);
        if (lsum != csum) {
           syserr(WARNING,"decomp_basebyte",
                          "Something wrong in base_byte decompression\n");
           return NULL;
        }
 */
     }
     return (char *)map;
}

char *decomp_prevpix(unsigned char *b, int nrecs, int npix, 
                     double bscale, double bzero)
{
     unsigned char a,aa[2];
     int  icur, irec, ipc, lsum, char_offset, ipix;
     float *map;

     ED_MALLOC(map, "decomp_prepix", float, npix);
     nrecs = nrecs/FITSBLOCK;
     icur = 0;
     lsum = 0;
     for (irec=0;irec<nrecs;irec++) {
        ipc = 8;
        char_offset = irec*FITSBLOCK;
        aa[0] = b[char_offset];
        aa[1] = b[char_offset+1];
        map[icur++] = (float) short_cast(aa) * bscale + bzero;
        for (ipix=1; ipix < short_cast(&b[char_offset+1]); ipix++) {
           a = b[char_offset+ipc++];
           if ((char)a != -128) {
              map[icur] = (float) a * bscale + bzero;
              lsum = (int) b[char_offset+ipc++];
           } else {
              aa[0] = b[char_offset+ipc++];
              aa[1] = b[char_offset+ipc++];
              map[icur] = (float) short_cast(aa) * bscale + bzero;
              lsum = (int) short_cast(aa);
           }
           icur++;
        }
        if ((int) short_cast(&b[2]) != lsum) {
           syserr(WARNING,"decomp_prevpix",
                          "Something wrong in prev_pix decompression\n");
        }
     }
     return (char *)map;
}

char *read_compressed_image(fitsfile_struct *ff)
{
     unsigned char	*b;
     int	imlen = ff->code_len, imtype = ff->comp_type,i;
     double	bscale = ff->bscale, bzero = ff->bzero;
     int	npix = ff->xwid * ff->ywid;
     char	*map = NULL;

     ED_MALLOC(b,"read_compressed_image", unsigned char, imlen);
     if ((i=fread(b,1,imlen,ff->fd)) != imlen) {
        syserr(FATAL,"read_compressed_image",
                     "Error in reading compressed FITS file\n");
     }
     switch (imtype) {
     case 1:
              map = decomp_basebyte(b, imlen, npix,bscale, bzero);
              break;
     case 2:
              map = decomp_prevpix(b, imlen, npix, bscale,bzero);
              break;
     default: 
              syserr(FATAL,"read_compressed_image",
                           "Error in compression type\n");
     }
     ED_FREE(b);
     return map;
}

