#include <math.h>
#include "utils_globals.h"
#include "utils_macros.h"
#include "gaussj_globals.h"
#include "options_globals.h"

/*
   20.07.2004:
   I removed compiler warnings due to possibly
   undefined variables (compilation with optimisation
   turned on)
*/

#define SWAP(a,b) {float temp=(a);(a)=(b);(b)=temp;}

void gaussj(float **a,int n,float **b,int m)
{
	int *indxc,*indxr,*ipiv;
	int i,icol = 0,irow = 0,j,k,l,ll;
	float big,dum,pivinv;

        ED_CALLOC(indxc, "gaussj", int, n);
        ED_CALLOC(indxr, "gaussj", int, n);
        ED_CALLOC(ipiv, "gaussj", int, n);
	for (j=0;j<n;j++) ipiv[j]=0;
	for (i=0;i<n;i++) {
		big=0.0;
		for (j=0;j<n;j++)
			if (ipiv[j] != 1)
				for (k=0;k<n;k++) {
					if (ipiv[k] == 0) {
						if (fabs(a[j][k]) >= big) {
							big=fabs(a[j][k]);
							irow=j;
							icol=k;
						}
					} else if (ipiv[k] > 1) syserr(FATAL, "gaussj", "Singular Matrix-1");
				}
		++(ipiv[icol]);
		if (irow != icol) {
			for (l=0;l<n;l++) SWAP(a[irow][l],a[icol][l])
			for (l=0;l<m;l++) SWAP(b[l][irow],b[l][icol])
		}
		indxr[i]=irow;
		indxc[i]=icol;
		if (a[icol][icol] == 0.0) syserr(FATAL,"gaussj","Singular Matrix-2");
		pivinv=1.0/a[icol][icol];
		a[icol][icol]=1.0;
		for (l=0;l<n;l++) a[icol][l] *= pivinv;
		for (l=0;l<m;l++) b[l][icol] *= pivinv;
		for (ll=0;ll<n;ll++)
			if (ll != icol) {
				dum=a[ll][icol];
				a[ll][icol]=0.0;
				for (l=0;l<n;l++) a[ll][l] -= a[icol][l]*dum;
				for (l=0;l<m;l++) b[l][ll] -= b[l][icol]*dum;
			}
	}
	for (l=n-1;l>=0;l--) {
		if (indxr[l] != indxc[l])
			for (k=0;k<n;k++)
				SWAP(a[k][indxr[l]],a[k][indxc[l]]);
	}
        ED_FREE(ipiv);
        ED_FREE(indxr);
        ED_FREE(indxc);
}

#undef SWAP
