#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "pos_globals.h"
#include "pos_defs.h"

/*
   20.07.2004:
   I removed compiler warnings due to possibly
   undefined variables (compilation with optimisation
   turned on)
*/

static posstruct radec_to_pos(double ra, double dec)
{
    unsigned int ira, idec;
    unsigned int a;
    int		 i;
    posstruct	 p;

    ira  = (unsigned int) (ra * accur);
    idec = (unsigned int) ((dec + 90.0) * accur);

    p.lsb = p.msb = 0;
    for (i=0;i<32;i++) {
       a = ira >> i;
       a = a & 0001;
       if (i<16) {
          a = a << (i*2);
          p.lsb |= a;
       } else {
          a = a << ((i-16)*2);
          p.msb |= a;
       }
    } 
    for (i=0;i<32;i++) {
       a = idec >> i;
       a = a & 0001;
       if (i<16) {
          a = a << (i*2+1);
          p.lsb |= a;
       } else {
          a = a << ((i-16)*2+1);
          p.msb |= a;
       }
    }


    return p;
}

void pos_to_radec(posstruct p, double *ra, double *dec)
{
    unsigned int ira, idec;
    unsigned int a;
    int		 i;

    ira = 0;
    for (i=0;i<32;i++) {
       if (i<16) {
          a = p.lsb >> (i*2);
       } else {
          a = p.msb >> ((i-16)*2);
       }
       a = a & 0001;
       a = a << i;
       ira |= a;
    }
    idec = 0;
    for (i=0;i<32;i++) {
       if (i<16) {
          a = p.lsb >> (i*2+1);
       } else {
          a = p.msb >> ((i-16)*2+1);
       }
       a = a & 0001;
       a = a << i;
       idec |= a;
    }
    *ra  = ((double) ira) / accur;
    *dec = ((double) idec) / accur - 90.0;
}

double pos_box_side(int clipped_bits)
{
    double	val;

    val = 1.0 * pow(2, clipped_bits/2) *accur;

    return val;
}
    
static void indexhash(int n, posstruct *arrin, int *indx, int nul)
{
    int l,j,ir,indxt,i;
    posstruct q;

    if (n == 1) return;
    if (nul == 0) {
       nul = 1;
    } else {
       nul = 0;
    }
    for (j=1;j<=n;j++) indx[j-nul]=j-nul;
    l=(n >> 1) + 1;
    ir=n;
    for (;;) {
       if (l > 1) {
          q.lsb=arrin[(indxt=indx[--l-nul])].lsb;
          q.msb=arrin[indxt].msb;
       } else {
          q.lsb=arrin[(indxt=indx[ir-nul])].lsb;
          q.msb=arrin[indxt].msb;
          indx[ir-nul]=indx[1-nul];
          if (--ir == 1) {
             indx[1-nul]=indxt;
             return;
          }
       }
       i=l;
       j=(l << 1);
       while (j <= ir) {
          if ((j < ir) && 
              ((arrin[indx[j-nul]].msb < arrin[indx[j+1-nul]].msb) ||
               ((arrin[indx[j-nul]].msb == arrin[indx[j+1-nul]].msb) &&
                (arrin[indx[j-nul]].lsb < arrin[indx[j+1-nul]].lsb)))) j++;
          if ((q.msb < arrin[indx[j-nul]].msb) ||
              ((q.msb == arrin[indx[j-nul]].msb) && 
               (q.lsb < arrin[indx[j-nul]].lsb))) {
             indx[i-nul]=indx[j-nul];
             j += (i=j);
          } else 
             j=ir+1;
       }
       indx[i-nul]=indxt;
    }
}

static int ibin_search_hashed(int n, posstruct *x, int *idx, posstruct key, 
                        int *where, int root)
{
    int  min, max, middle = 0;
    posstruct xx;

    if (root == 0) {
       max = n-1;
       min = 0;
    } else {
       max = n;
       min = 1;
    }
    if (n == 2) {
       *where = 2;
    }
    if ((x[idx[min]].msb > key.msb) ||
        ((x[idx[min]].msb == key.msb) && (x[idx[min]].lsb > key.lsb))) {
       *where = min;
       return 0;
    }
    if ((x[idx[max]].msb < key.msb) ||
        ((x[idx[max]].msb == key.msb) && (x[idx[min]].lsb < key.lsb))) {
       *where = max;
       return 0;
    }
    while (((max + min)/2) != middle) {
      middle  = (min + max ) / 2;
      xx.msb = x[idx[middle]].msb;
      xx.lsb = x[idx[middle]].lsb;
      if ((xx.msb > key.msb) ||
         ((xx.msb == key.msb) && (xx.lsb > key.lsb))) 
         max = middle;
      else
         min = middle;
   }
   *where = middle;
   return 1;
}

hashstruct *init_hash(double *ra, double *dec, int n)
{
    hashstruct	*a;
    int		i;

    a      = (hashstruct *) calloc(1, sizeof(hashstruct));
    a->pos = (posstruct *) calloc(n, sizeof(posstruct));
    a->idx = (int *) calloc(n, sizeof(int));
    a->n   = n;
    
    for (i=0;i<a->n;i++) 
       a->pos[i] = radec_to_pos(ra[i], dec[i]);

    indexhash(a->n, a->pos, a->idx, 0);
  
    return a;
}

idxstruct *find_hash(hashstruct *p, 
                     double ra1, double dec1, double ra2, double dec2)
{
    idxstruct	*a;
    int		nidx, nidxblock = 100;
    int		i, j, start;
    double	ra, dec;
    posstruct	t[3];

    nidx = nidxblock;
    a = (idxstruct *) calloc(1, sizeof(idxstruct));
    a->idx = (int *) calloc(nidx, sizeof(int));

    ra = (ra1 + ra2) / 2.0;
    dec = (dec1 + dec2) / 2.0;
    t[0] = radec_to_pos(ra, dec);
    t[1] = radec_to_pos(ra1, dec1);
    t[2] = radec_to_pos(ra2, dec2);
    j = 0;
    ibin_search_hashed(p->n, p->pos, p->idx, t[0], &start, 0);
       for (i=start+1;i<p->n;i++) {
          if (((t[2].msb > p->pos[p->idx[i]].msb) ||
              ((t[2].msb == p->pos[p->idx[i]].msb) && 
               (t[2].lsb > p->pos[p->idx[i]].lsb)))) {
/*
             pos_to_radec(p->pos[p->idx[i]], &ra, &dec);
             if (ra > ra1 && ra < ra2 && dec > dec1 && dec < dec2) {
 */
                if (j >= nidx) {
                   nidx += nidxblock;
                   a->idx = (int *) realloc(a->idx, sizeof(int) * nidx);
                }
                a->idx[j++] = p->idx[i];
/*
             }
 */
          } else
             break;
       }
       for (i=start;i>=0;i--) {
          if (((t[1].msb < p->pos[p->idx[i]].msb) ||
              ((t[1].msb == p->pos[p->idx[i]].msb) && 
               (t[1].lsb < p->pos[p->idx[i]].lsb)))) {
/*
             pos_to_radec(p->pos[p->idx[i]], &ra, &dec);
             if (ra > ra1 && ra < ra2 && dec > dec1 && dec < dec2) {
 */
                if (j >= nidx) {
                   nidx += nidxblock;
                   a->idx = (int *) realloc(a->idx, sizeof(int) * nidx);
                }
                a->idx[j++] = p->idx[i];
/*
             }
 */
          } else 
             break;
       }
    a->n = j;

    return a;
}

void free_idx(idxstruct *a)
{
    free(a->idx);
    free(a);
}

void free_hash(hashstruct *a)
{
    free(a->pos);
    free(a->idx);
    free(a);
}
