/* 21.06.01:
   removed compiler warnings under DEC

   05.05.03:
   included the tsize.h file

   10.01.05:
   removed compiler warnings under gcc with optimisation
   (-O3) enabled.

   29.01.2005:
   - I added history information to the output catalog
   - I added argc and argv to the arguments of function
     save_photo_cat to write HISTORY info to output catalogs.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "tsize.h"

#define NBANDS 3
#define DUMPHOT -98.0

#include "fitscat.h"
#include "tabutil_globals.h"
#include "utils_globals.h"
#include "utils_macros.h"
#include "precess_globals.h"
#include "options_globals.h"
#include "photom_vers.h"
#include "photom_globals.h"
#include "moments_globals.h"
#include "stdcat_globals.h"

char	header[MAXCHAR];

static float	**phot, **photerr;
static int	iphots, nphots, nstdmags;
static short int *flag, *artifact;
static short int arti_exist;
static long int	*imaflag;
static char	**phot_names, **photerr_names, **stdmag_names, **stdmag_limits;
static char	**exptime_names, **exptime_units;
static int	nexptimes;


#define STRING_LEN 16

static short int        *fpos;
static long int		*std_seqnr, *night_nr;
static double   *airmass, *obstime;
static char	*std_name;
static double	*exptime, *exptime_def;
static long int	*lexptime;

option my_opts[] = {
            {"FLAG_MASK", INT, &control.flag_mask, 0, 65535, 0.0, 0.0,
              0, 0.0, NO, "",
             {"", "", "", "", ""}, 0,
             "The mask to AND with the flag, if true than do not use data"},
            {"IMAFLAG_MASK", INT, &control.imaflags_mask, 0, 65535, 0.0, 0.0,
              0, 0.0, NO, "",
             {"", "", "", "", ""}, 0,
             "The mask to AND with the imaflags_iso, if true than do not use data"},
            {"ARTI_MASK", INT, &control.arti_mask, 0, 65535, 0.0, 0.0,
              0, 0.0, NO, "",
             {"", "", "", "", ""}, 0,
             "The mask to AND with the artifact, if true than do not use data"},
            {"COEFS", FARRAY, control.coef, 0, 0, 0.0, 100.0,
              0, 0.03, NO, "",
             {"", "", "", "", ""}, 0,
             "List of extinction coefficients (# = number of input cats)"},
            {"ZP_ESTIMATES", FARRAY, control.zp_estimate, 0, 0, 0.0, 100.0,
              0, 0.08, NO, "",
             {"", "", "", "", ""}, 0,
             "Estimate of the instrumental zeropoint (# = number of input cats)" },
            {"ZP_ERRORS", FARRAY, control.zp_error, 0, 0, 0.0, 100.0,
              0, 0.12, NO, "",
             {"", "", "", "", ""}, 0,
             "Estimate of the instr zeropoint error (# = number of input cats)"},
            {"PHOT", TEXTVEC, control.phots, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "",
             {"", "", "", "", ""}, 0,
             "Name of the OBJECTS table column containing the FLUX estimator"},
            {"PHOTERR", TEXTVEC, control.photerrs, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "",
             {"", "", "", "", ""}, 0,
             "Name of the OBJECTS table column containing the FLUX error estimator"},
            {"STDMAG", TEXTVEC, control.stdmags, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "",
             {"", "", "", "", ""}, 0,
             "The names of the standard star magnitude columns to be read from the standard star catalog"},
            {"STDLIM", TEXTVEC, control.stdlims, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "",
             {"", "", "", "", ""}, 0,
             "The brightest magnitude of a standard star to still be used for photometry"},
            {"AIRMASS", TEXT, control.airmass, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "2MUAIRM",
             {"", "", "", "", ""}, 0,
             "The name of the FIELDS table column containing the airmass value"},
            {"EXPTIME_KEYWORD", TEXTVEC, control.exptime_keyw, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "",
             {"", "", "", "", ""}, 0,
             "Name of the FIELDS table column containing the exposure time"},
            {"EXPTIME_UNIT", TEXTVEC, control.exptime_unit, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "",
             {"", "", "", "", ""}, 0,
             "The unit in which the EXPTIME_KEYWORD is to be expressed"},
            {"OBSTIME", TEXT, control.obstime, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "2MUUTIME",
             {"", "", "", "", ""}, 0,
             "Name of the FIELDS table column containing the observation time"},
            {"NIGHT_NR", TEXT, control.night_nr, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "2MUNIGHT",
             {"", "", "", "", ""}, 0,
             "Name of the FIELDS table column containing the night number"},
            {"FIX_COEF", BOOLEAN, &control.fix_coef, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "",
             {"YES", "NO", "", "", ""}, 0, 
             "Do we use the extinction coeff. estimator for photometry"},
            {"FIX_ZP", BOOLEAN, &control.fix_zp, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "",
             {"YES", "NO", "", "", ""}, 0, 
             "Do we use the zeropoint estimator for photometry"},
            {"DO_FIT", BOOLEAN, &control.do_fit, 0 , 0, 0.0, 0.0,
              0, 0.0, YES, "",
             {"YES", "NO", "", "", ""}, 0, 
             "Do we fit the extinction law?"},
            {""},
            };

void usage(char *me)
{
    fprintf(stderr,"Usage: %s \n",me);
    fprintf(stderr,"                 -i incat1 [[incat2] incat3] ... \n");
    fprintf(stderr,"                 -o outcat1 [[outcat2] outcat3] ... \n");
    fprintf(stderr,"                 -s standard_star_catalog\n");
    fprintf(stderr,"                 [-c conffile]\n");
    fprintf(stderr,"	Program to derive the photometric parameters for a night\n");
    fprintf(stderr,"	The derived paramaters will in principle be both:\n");
    fprintf(stderr,"		extinction coefficients and\n");
    fprintf(stderr,"		instrumental zero-points\n");
    fprintf(stderr,"	In case of too few standards in the catalog, or too large\n");
    fprintf(stderr,"	variations in the extinction coefficient derivation, the\n");
    fprintf(stderr,"	extinction coefficients are taken from the configuration file\n");
    fprintf(stderr,"	and the instrumental zero-points are derived\n\n");

  

    exit(1);
}
 
void init(int argc, char *argv[])
/****** init ***************************************************
 *
 *  NAME	
 *      init - Initialize using command line arguments and options
 *
 *  SYNOPSIS
 *      void init(int argc, char *argv[])
 *
 *  FUNCTION
 *      Fill the global variables with the arguments and options given
 *      on the command line.
 *
 *  INPUTS
 *      argc  - The number of command line elements
 *      argv  - The list of command line elements
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *      The global structure control is set. The following are set:
 *      phot_names    - The keyword names for photometric parameters
 *      photerr_names - The keyword names for photometric parameter errors
 *      stdmag_names  - The keyword names for standard star magnitudes
 *      nphots        - The number of individual photometric parameters
 *      iphots        - The number of array elements for the current phot par
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 20-08-1997
 *
 ******************************************************************************/
{
   int i, j, k;

   control.chan_nr = -1;
   if (argc == 1) usage(argv[0]);
   init_options(0, my_opts);
   for (i=1; i<argc; i++) {
     if (argv[i][0] == '-') {
        switch(argv[i][1]) {
        case 'c': strcpy(control.conffile,argv[++i]);
                  break;
        case 's': strcpy(control.stdcatfile,argv[++i]);
                  break;
        case 'i': control.ilist = 0;
                  strcpy(control.listfile[control.ilist],argv[++i]);
                  while (i+1<argc && argv[i+1][0] != '-') {
                     strcpy(control.listfile[++control.ilist],argv[++i]);
                  }
                  ++control.ilist;
                  break;
        case 'o': strcpy(control.outfile[0],argv[++i]);
                  break;
        case '@': print_options(argv[0]);
                  exit(0);
        default:  parse_options(control.conffile);
                  j = i++; k = i;
                  set_command_option(argv[j], argv[k]);
                  break;
        }
     } else {
        syserr(WARNING,"init","Command line syntax error\n");
        usage(argv[0]);
     }
  }
  finish_options(control.conffile);
  phot_names    = copy_textvec(control.phots,    &nphots);
  photerr_names = copy_textvec(control.photerrs, &i);
  if (i != nphots)
     syserr(FATAL, "init", "Number of %s parameters differs from %s\n",
             control.photerrs, control.phots);
  stdmag_names  = copy_textvec(control.stdmags,     &nstdmags);
  stdmag_limits = copy_textvec(control.stdlims,     &i);
  if (i != nstdmags)
     syserr(FATAL, "init", "Number of %s parameters differs from %s\n",
             control.stdlims, control.stdmags);
  exptime_names = copy_textvec(control.exptime_keyw, &nexptimes);
  exptime_units = copy_textvec(control.exptime_unit, &i);
  if (i != nexptimes)
     syserr(FATAL, "init", "Number of %s parameters differs from %s\n",
             control.exptime_unit, control.exptime_keyw);
  ED_CALLOC(exptime_def, "init", double, nexptimes);
  for (i=0;i<nexptimes;i++) 
     exptime_def[i] = atof(exptime_units[i]);
}

/**/
int load_photo_fields(char *name, char *photname, char *photerrname)
/****** load_photo_fields ***************************************************
 *
 *  NAME	
 *      load_photo_fields - Load the photometric parameter(s) for all fields
 *
 *  SYNOPSIS
 *      int load_photo_fields(char *name, char *photname)
 *
 *  FUNCTION
 *      Load the photometric parameter (array) for all fields in the strip.
 *      Also additional information such as standard star magnitudes and
 *      field position are loaded. In addition airmass and observation
 *      times are read. All information is saved in global variables.
 *
 *  INPUTS
 *      name        - The name of the catalog to read
 *      photname    - The name of the photometric parameter to read
 *      photerrname - The name of the photometric error parameter to read
 *
 *  RESULTS
 *
 *  RETURNS
 *
 *  MODIFIED GLOBALS
 *      The following global variables are filled with information:
 *      phot      - array (multidim) of photometric parameters
 *      photerr   - array (multidim) of photoemtric parameter errors
 *      stdmag    - array (multidim) of standard star magnitudes
 *      fpos      - array of field positions
 *      airmass   - array of airmasses (per field)
 *      obstime   - array of observation times (per field)
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 20-08-1997
 *
 ******************************************************************************/
{
    int		nobjs, nfields, i, len;
    int		NKEYS_O = 7, FIELD_POS = 0, NPHOT = 1, NKEYS_F = 4;
    int		NPHOTERR = 2, STDSEQNR = 3, NFLAG = 4, NIMAFLAG = 5;
    int		NARTI = 6;
    float	*fptr;
    catstruct	*cat;
    tabstruct	*tab, *ftab;
    char	**keynames_o;
    keystruct	*key, **keys_o, **keys_f;
    char	**keynames_f;
    VPRINTF(NORMAL,"Loading photometric parameter %s from catalog %s\n", 
                    photname, name);
    if ((cat = read_cat(name)) == NULL) {
       syserr(FATAL, "load_photo_fields", "Could not read catalog: %s\n",
              name);
    }
    if (!(ftab = name_to_tab(cat, "FIELDS", 0))) 
       syserr(FATAL, "No table FIELDS in %s catalog\n", name);

    if (!(tab = name_to_tab(cat, "OBJECTS", 0))) 
       syserr(FATAL, "No table OBJECTS in %s catalog\n", name);

    if (!(key = name_to_key(tab, photname)))
       syserr(FATAL, "load_photo_fields", "No Key %s in OBJECT table\n",
             photname);
    iphots = 1;
    if (key->naxis > 1)
       syserr(FATAL, "load_strip_phots",
            "Too large a dimension (%d) for %s (maximum of 1 only)\n",
             key->naxis, photname);
    for (i=0;i<key->naxis;i++) iphots *= key->naxisn[i];
/*
 *  First load the standard star magnitude values and phot params
 */
    ED_CALLOC(keys_o,     "load_photo_fields", keystruct *, NKEYS_O);
    ED_CALLOC(keynames_o, "load_photo_fields", char *,      NKEYS_O);
    for (i=0;i<NKEYS_O;i++)
       ED_CALLOC(keynames_o[i], "load_photo_fields", char, MAXCHAR);
    strcpy(keynames_o[FIELD_POS], "FIELD_POS");
    strcpy(keynames_o[NPHOT],     photname);
    strcpy(keynames_o[NPHOTERR],  photerrname);
    strcpy(keynames_o[STDSEQNR],  "STD_SeqNr");
    strcpy(keynames_o[NFLAG],     "Flag");
    strcpy(keynames_o[NIMAFLAG],  "IMAFLAGS_ISO");
    strcpy(keynames_o[NARTI],     "ARTIFACT");

    read_keys(tab, keynames_o, keys_o, NKEYS_O, NULL);

    for (i=0;i<NKEYS_O-1;i++) 
       if (!keys_o[i])
         syserr(FATAL,"load_photo_fields", "No key %s in table\n",
            keynames_o[i]);

    fpos    = keys_o[FIELD_POS]->ptr;
    nobjs   = keys_o[FIELD_POS]->nobj;
    ED_CALLOC(phot,    "load_phot_fields", float *, iphots);
    ED_CALLOC(photerr, "load_phot_fields", float *, iphots);
    for (i=0;i<iphots;i++) {
       len = strlen(keynames_o[NPHOT]);
       keynames_o[NPHOT][len+1] = i+1;
       ED_GETVEC(fptr, "load_phot_fields", keys_o[NPHOT],
                  keynames_o[NPHOT], float, float);
       phot[i] = fptr;
    }
    for (i=0;i<iphots;i++) {
       len = strlen(keynames_o[NPHOTERR]);
       keynames_o[NPHOTERR][len+1] = i+1;
       ED_GETVEC(fptr, "load_phot_fields", keys_o[NPHOTERR],
                  keynames_o[NPHOTERR], float, float);
       photerr[i] = fptr;
    }
    std_seqnr = keys_o[STDSEQNR]->ptr;
    flag      = keys_o[NFLAG]->ptr;
    imaflag   = keys_o[NIMAFLAG]->ptr;
    if (keys_o[NARTI]) {
       artifact  = keys_o[NARTI]->ptr;
       arti_exist = TRUE;
    } else {
       artifact = NULL;
       arti_exist = FALSE;
    }

    ED_FREE(keynames_o[FIELD_POS]);
    ED_FREE(keynames_o[NPHOT]);
    ED_FREE(keynames_o[STDSEQNR]);
    ED_FREE(keynames_o[NFLAG]);
    ED_FREE(keynames_o[NIMAFLAG]);
    ED_FREE(keynames_o[NARTI]);
    ED_FREE(keynames_o);
    ED_FREE(keys_o[NPHOT]->ptr);
    ED_FREE(keys_o[NPHOTERR]->ptr);
    if (control.chan_nr != -1) {
       if (get_channel_number(ftab) != control.chan_nr) {
          printf("Expected channel %d, found channel %d\n",
              control.chan_nr, get_channel_number(ftab));
          syserr(FATAL, "Catalog %s contains different channel info\n", name);
       }
    }
    control.chan_nr = get_channel_number(ftab);
    strcpy(control.chan_name, get_channel_name(ftab));
    if (control.chan_nr >= nexptimes)
       syserr(FATAL, "load_photo_fields", 
              "Too few (%d) EXPTIME_KEYWORD and EXPTIME_UNIT parameters\n",
              nexptimes);
/*
 *  Read the airmass and observation times from the FIELDS table
 */
    ED_CALLOC(keynames_f, "load_photo_fields", char *,      NKEYS_F);
    ED_CALLOC(keys_f,     "load_photo_fields", keystruct *, NKEYS_F);
    for (i=0;i<NKEYS_F;i++) 
       ED_CALLOC(keynames_f[i], "load_photo_fields", char, MAXCHAR);
    strcpy(keynames_f[0],control.airmass);
    strcpy(keynames_f[1],control.obstime);
    strcpy(keynames_f[2],exptime_names[control.chan_nr]);
    strcpy(keynames_f[3],control.night_nr);
    read_keys(ftab, keynames_f, keys_f, NKEYS_F, NULL);
    for (i=0;i<NKEYS_F;i++) {
       if (!keys_f[i])
         syserr(FATAL,"load_photo_fields", "No key %s in FIELDS table\n",
            keynames_f[i]);
    }
    airmass = keys_f[0]->ptr;
    nfields = keys_f[0]->nobj;
    obstime = keys_f[1]->ptr;
    if (keys_f[2]->ttype == T_LONG) {
       lexptime = keys_f[2]->ptr;
       ED_CALLOC(exptime, "load_photo_fields", double, nfields);
       for (i=0;i<nfields;i++) exptime[i] = (double) lexptime[i];
       ED_FREE(lexptime);
    } else {
       exptime = keys_f[2]->ptr;
    }
    night_nr = keys_f[3]->ptr;
    for (i=0;i<NKEYS_F;i++) 
       ED_FREE(keynames_f[i]);
    ED_FREE(keynames_f); 
    close_cat(cat);
    return nobjs;
}
/**/
void close_photo_fields()
/****** close_photo_fields ***************************************************
 *
 *  NAME	
 *      close_photo_fields - Close the global data structures
 *
 *  SYNOPSIS
 *      void close_photo_fields()
 *
 *  FUNCTION
 *      This function deallocates the memory requirements for a photometric
 *      deterimination.
 *
 *  INPUTS
 *      None
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *      Global variable memory allocation is free-ed
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 20-08-1997
 *
 ******************************************************************************/
{
    int	i;
   
    for (i=0;i<iphots;i++) {
       ED_FREE(phot[i]);
       ED_FREE(photerr[i]);
    }
    ED_FREE(phot);
    ED_FREE(photerr);
    ED_FREE(airmass);
    ED_FREE(obstime);
    ED_FREE(exptime);
    ED_FREE(std_seqnr);
    ED_FREE(fpos);
    ED_FREE(flag);
    ED_FREE(imaflag);
    ED_FREE(artifact);
}
/**/
photostruct *select_standards(int *n)
/****** select_standards ***************************************************
 *
 *  NAME	
 *      select_standards - Seelect the standard star extractions
 *
 *  SYNOPSIS
 *      photostruct *select_standards(int *n)
 *
 *  FUNCTION
 *      This routine selects the standard star extractiuon from the input
 *      data arrays and copies the necessary information to the output
 *      structure.
 *
 *  INPUTS
 *      n  - The number of extracted objects in the data arrays
 *
 *  RESULTS
 *      n  - The number of selected objects from the data arrays
 *
 *  RETURNS
 *      An array of standard star data eleents
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 20-08-1997
 *
 ******************************************************************************/
{
    int		i, j, l = 0, nblk=1, blksize=100;
    int		skip;
    photostruct	*std;
    float	*stdmags;
    char	stdname[MAXCHAR];
/*
 *  Blockwise allocate the output data structure
 */
    ED_MALLOC(std, "select_standards", photostruct, (nblk*blksize));
    ED_CALLOC(stdmags, "select_standards", float, 3);
    for (i=0;i<*n;i++) {
/*
 *     Check if there is standard star information available for this object
 */
       skip = TRUE;
       get_std_star(std_seqnr[i], &stdmags[0],&stdmags[1],&stdmags[2], stdname);
       for (j=0;j<nstdmags;j++) 
          if (stdmags[j] > DUMPHOT) skip = FALSE;
       if (!skip) {
/*
 *        Is there stil space in the current memory block
 */
          if (l >= nblk*blksize) { 
             nblk++;
             ED_REALLOC(std, "select_standards", photostruct, (nblk*blksize));
          }
/*
 *        Allocate space to store the in and output photometry and copy/init
 */
          ED_CALLOC(std[l].phot,    "select_standards", float, iphots);
          ED_CALLOC(std[l].photerr, "select_standards", float, iphots);
          ED_CALLOC(std[l].extmags, "select_standards", float, iphots);
          ED_CALLOC(std[l].zp,      "select_standards", float, iphots);
          for (j=0;j<iphots;j++) {
             std[l].phot[j]    = phot[j][i];
             std[l].photerr[j] = photerr[j][i];
             std[l].extmags[j] = 0.0;
          }
          ED_CALLOC(std[l].stdmag, "select_standards", float, nstdmags);
          for (j=0;j<nstdmags;j++) 
             std[l].stdmag[j] = stdmags[j];
          strcpy(std[l].stdname, stdname);
          std[l].airm      = airmass[fpos[i]-1];
          std[l].fpos      = fpos[i]-1;
          std[l].time      = obstime[fpos[i]-1];
          std[l].flag      = flag[i];
          std[l].imaflag   = imaflag[i];
          if (artifact) 
             std[l].arti   = artifact[i];
          l++;
       }
    }
    *n = l;
    ED_FREE(stdmags);
    return std;
}
/**/
void exclude_errors(photostruct *std, int n, control_struct c)
/****** exclude_errors ***************************************************
 *
 *  NAME	
 *      exclude_errors - Remove extranous measurements from the input
 *
 *  SYNOPSIS
 *      void exclude_errors(photostruct *std, int n, control_struct c)
 *
 *  FUNCTION
 *      This routine removes pairings of extracted objects and standard
 *      star magnitudes for obvious errors. When the estimated zero point
 *      is well outside the bound of the allowed/expected zero point
 *      range, the pairing is removed. Data is not physically removed, 
 *      but the structure flag is set FALSE in that case
 *
 *  INPUTS
 *      std - The array of standard star data points
 *      n   - The number of elements in the std array
 *      c   - The current channel
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 20-08-1997
 *
 ******************************************************************************/
{
    int		i, j, chan = c.chan_nr;

    for (i=0; i<n; i++) {
       std[i].lflag = std[i].flag & control.flag_mask ||
                      std[i].imaflag & control.imaflags_mask ||
                      std[i].arti & control.arti_mask ||
                      std[i].stdmag[chan] < atof(stdmag_limits[chan]);
       if (std[i].lflag)
         VPRINTF(VERBOSE,"Discarding entry for %s at obstime: %lf, Airm: %lf for fpos %d (Fl:%d, If:%d, Af:%d, M:%f)\n", 
             std[i].stdname, obstime[std[i].fpos], std[i].airm, std[i].fpos,
              std[i].flag, std[i].imaflag, std[i].arti, std[i].stdmag[chan]);
       for (j=0;j<iphots;j++) {
          if (std[i].phot[j] > 0) {
/*
             std[i].zp[j] = std[i].airm * c.coef[chan] + 
                         2.5 * log10((double)std[i].phot[j]/
                          exptime[std[i].fpos]*exptime_def[chan])
                             + std[i].stdmag[chan];
 */
             std[i].zp[j] = 2.5 * log10((double)std[i].phot[j]/
                            exptime[std[i].fpos]*exptime_def[chan])
                             + std[i].stdmag[chan];
          } else {
             std[i].zp[j] = 99.0;
          }
/*
 *
 */
          VPRINTF(DEBUG, "Checking ZP %f against estimate %f +/- %f\n", 
                         std[i].zp[j], c.zp_estimate[chan],c.zp_error[chan]);
          if (fabs(std[i].zp[j] - c.zp_estimate[chan]) > c.zp_error[chan]) {
             std[i].lflag = TRUE;
          }
       }
       if (!std[i].lflag)
         VPRINTF(VERBOSE,"Accept entry for %s at obstime: %lf, Airm: %lf for fpos %d\n", 
             std[i].stdname, obstime[std[i].fpos], std[i].airm, std[i].fpos);
    }
}
/**/
static int	nblks, blksize, photdata_nobjs;
static int	*photdata_chan_nr, *photdata_phot_id;
static short int *photdata_flag, *photdata_arti;
static long int	*photdata_imaflag;
static char	*photdata_chan_name, *photdata_phot_name, *photdata_stdname;
static float	*photdata_phot, *photdata_photerr, *photdata_zp,
                *photdata_stdmags[3];
static double	*photdata_airmass, *photdata_obstime;

void add_photdata_tab(char *name, float **Phots, float **Photerrs, float **zps,
                      float **mags, double *airm, double *obs_time, 
                      char *stdname, short int *fl, long int *imafl,
                      short int *art, int n, int *phot_id)
/****** add_photdata_tab ***************************************************
 *
 *  NAME	
 *      add_photdata_tab - Add data to the output photometry calibration table
 *
 *  SYNOPSIS
 *      void create_photdata_tab(char *name, float **Phots, 
 *                               float **zps, 
 *                               double *airm, double *obs_time, int n)
 *
 *  FUNCTION
 *      This routine adds data to the photometric calibration table
 *
 *  INPUTS
 *      Phots    - The array (multidim) of photometric parameters used
 *      Photerrs - The array (multidim) of photometric error parameters used
 *      zps      - The array (multidim) of estimated zero-points
 *      mags     - The array of standard star magnitudes
 *      airm     - The array of arimassed (per field)
 *      obs_time - The array of observation times (per field)
 *      fl       - The array of flags
 *      imafl    - The array of imaflags_iso
 *      art      - The array of artifacts
 *      stdname  - The array of standard star names
 *      n        - The number of data points
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      Table structure of the just created table
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 20-08-1997
 *
 ******************************************************************************/
{
    int		i, j, nn, lphotid=0;

/*
 *  Add the number of required output data blocks
 */
    while (photdata_nobjs + n*iphots >= nblks*blksize) {
       nblks++;
       ED_REALLOC(photdata_flag,      "init_photdata_tab", short int,  
                       (nblks*blksize));
       ED_REALLOC(photdata_arti,      "init_photdata_tab", short int,  
                       (nblks*blksize));
       ED_REALLOC(photdata_imaflag,   "init_photdata_tab", long int,  
                       (nblks*blksize));
       ED_REALLOC(photdata_chan_nr,   "init_photdata_tab", int,  
                       (nblks*blksize));
       ED_REALLOC(photdata_phot_id,   "init_photdata_tab", int,  
                       (nblks*blksize));
       ED_REALLOC(photdata_chan_name, "init_photdata_tab", char, 
                       (nblks*blksize*STRING_LEN));
       ED_REALLOC(photdata_phot_name, "init_photdata_tab", char, 
                       (nblks*blksize*STRING_LEN));
       ED_REALLOC(photdata_stdname, "init_photdata_tab", char, 
                       (nblks*blksize*STRING_LEN));
       ED_REALLOC(photdata_phot,      "init_photdata_tab", float, 
                       (nblks*blksize));
       ED_REALLOC(photdata_photerr,   "init_photdata_tab", float, 
                       (nblks*blksize));
       ED_REALLOC(photdata_zp,        "init_photdata_tab", float, 
                       (nblks*blksize));
       ED_REALLOC(photdata_stdmags[0],   "init_photdata_tab", float, 
                       (nblks*blksize));
       ED_REALLOC(photdata_stdmags[1],   "init_photdata_tab", float, 
                       (nblks*blksize));
       ED_REALLOC(photdata_stdmags[2],   "init_photdata_tab", float, 
                       (nblks*blksize));
       ED_REALLOC(photdata_airmass,   "init_photdata_tab", double, 
                       (nblks*blksize));
       ED_REALLOC(photdata_obstime,   "init_photdata_tab", double, 
                       (nblks*blksize));
    }
    nn = 0;
    for (i=0;i<n;i++) {
       lphotid = *phot_id;
       for (j=0;j<iphots;j++) {
          if (iphots > 1)
             sprintf(&photdata_phot_name[(nn+photdata_nobjs)*STRING_LEN],
                     "%s(%-d)",name,j+1);
          else
             strcpy(&photdata_phot_name[(nn+photdata_nobjs)*STRING_LEN],name);
          photdata_chan_nr[nn+photdata_nobjs]   = control.chan_nr;
          photdata_flag[nn+photdata_nobjs]      = fl[i];
          if (art)
             photdata_arti[nn+photdata_nobjs]      = art[i];
          photdata_imaflag[nn+photdata_nobjs]   = imafl[i];
          photdata_phot_id[nn+photdata_nobjs]   = lphotid;
          strcpy(&photdata_chan_name[(nn+photdata_nobjs)*STRING_LEN],
                 control.chan_name);
          photdata_phot[nn+photdata_nobjs]    = Phots[j][i];
          photdata_photerr[nn+photdata_nobjs] = Photerrs[j][i];
          photdata_zp[nn+photdata_nobjs]      = zps[j][i];
          photdata_stdmags[0][nn+photdata_nobjs] = mags[0][i];
          photdata_stdmags[1][nn+photdata_nobjs] = mags[1][i];
          photdata_stdmags[2][nn+photdata_nobjs] = mags[2][i];
          photdata_airmass[nn+photdata_nobjs] = airm[i];
          photdata_obstime[nn+photdata_nobjs] = obs_time[i];
          strcpy(&photdata_stdname[(nn+photdata_nobjs)*STRING_LEN],
                    &std_name[i*STRING_LEN]);
          nn++;
          lphotid++;
       }
    }
    photdata_nobjs += nn;
    *phot_id = lphotid;
}
/**/
void save_photdata_tab(tabstruct *tab)
/****** save_photdata_tab ***************************************************
 *
 *  NAME	
 *      save_photdata_tab - Save the PHOTDATA table to the output catalog
 *
 *  SYNOPSIS
 *      void save_photdata_tab(tabstruct *tab)
 *
 *  FUNCTION
 *      This function save the global PHOTDATA data into the table
 *      structure
 *
 *  INPUTS
 *      tab - Pointer to the table structure
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 20-08-1997
 *
 ******************************************************************************/
{
    add_key_to_tab(tab, "PHOT_NAME", photdata_nobjs, photdata_phot_name, 
                      T_STRING, STRING_LEN, "","Photometric parameter name");
    add_key_to_tab(tab, "STD_NAME", photdata_nobjs, photdata_stdname, 
                      T_STRING, STRING_LEN, "","Standard Star Name");
    add_key_to_tab(tab, "CHANNEL_NR", photdata_nobjs, photdata_chan_nr , T_LONG,
                      1, "","Channel number");
    add_key_to_tab(tab, "CHANNEL_NAME", photdata_nobjs, photdata_chan_name, 
                      T_STRING, STRING_LEN, "","Channel name");
    add_key_to_tab(tab, "PHOT", photdata_nobjs, photdata_phot, T_FLOAT,
                      1, "","Photometric Parameter");
    add_key_to_tab(tab, "PHOTERR", photdata_nobjs, photdata_photerr, T_FLOAT,
                      1, "","Photometric Error Parameter");
    add_key_to_tab(tab, "ZP", photdata_nobjs, photdata_zp, T_FLOAT,
                      1, "","Instrumental Zeropoint");
    add_key_to_tab(tab, "STDMAG1", photdata_nobjs, photdata_stdmags[0], T_FLOAT,
                      1, "","Standard Star Magnitude");
    add_key_to_tab(tab, "STDMAG2", photdata_nobjs, photdata_stdmags[1], T_FLOAT,
                      1, "","Standard Star Magnitude");
    add_key_to_tab(tab, "STDMAG3", photdata_nobjs, photdata_stdmags[2], T_FLOAT,
                      1, "","Standard Star Magnitude");
    add_key_to_tab(tab, "Airmass", photdata_nobjs, photdata_airmass, T_DOUBLE,
                      1, "","Airmass during observation");
    add_key_to_tab(tab, "ObsTime", photdata_nobjs, photdata_obstime, T_DOUBLE,
                      1, "","Observation time at acquisition");
    add_key_to_tab(tab, "Flag", photdata_nobjs, photdata_flag, T_SHORT,
                      1, "","Extraction Flag");
    add_key_to_tab(tab, "IMAFLAGS_ISO", photdata_nobjs, photdata_imaflag, 
                      T_LONG , 1, "","Isophotal image FLAG");
    if (arti_exist)
       add_key_to_tab(tab, "ARTIFACT", photdata_nobjs, photdata_arti,  T_SHORT,
                      1, "","Image Artifact marker");
    update_tab(tab);
    return;
}


tabstruct *init_photdata_tab()
/****** init_photdata_tab ***************************************************
 *
 *  NAME	
 *      init_photdata_tab - Initialize the PHOTDATA table 
 *
 *  SYNOPSIS
 *      tabstruct *init_photdata_tab()
 *
 *  FUNCTION
 *      This routine initializes the table structure and allocates the
 *      global variables required to store the data
 *
 *  INPUTS
 *      None
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      A pointer to the table structure
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 20-08-1997
 *
 ******************************************************************************/
{
    tabstruct	*tab;

    tab = new_tab("PHOTDATA");

    nblks = 1; blksize = 100; photdata_nobjs = 0;
    ED_CALLOC(photdata_chan_nr,   "init_photdata_tab", int,  
                       (nblks*blksize));
    ED_CALLOC(photdata_flag,      "init_photdata_tab", short int,  
                       (nblks*blksize));
    ED_CALLOC(photdata_imaflag,   "init_photdata_tab", long int,  
                       (nblks*blksize));
    ED_CALLOC(photdata_arti,      "init_photdata_tab", short int,  
                       (nblks*blksize));
    ED_CALLOC(photdata_phot_id,   "init_photdata_tab", int,  
                       (nblks*blksize));
    ED_CALLOC(photdata_chan_name, "init_photdata_tab", char, 
                       (nblks*blksize*STRING_LEN));
    ED_CALLOC(photdata_phot_name, "init_photdata_tab", char, 
                       (nblks*blksize*STRING_LEN));
    ED_CALLOC(photdata_stdname, "init_photdata_tab", char, 
                       (nblks*blksize*STRING_LEN));
    ED_CALLOC(photdata_phot,      "init_photdata_tab", float, 
                       (nblks*blksize));
    ED_CALLOC(photdata_photerr,   "init_photdata_tab", float, 
                       (nblks*blksize));
    ED_CALLOC(photdata_zp,        "init_photdata_tab", float, 
                       (nblks*blksize));
    ED_CALLOC(photdata_stdmags[0],   "init_photdata_tab", float, 
                       (nblks*blksize));
    ED_CALLOC(photdata_stdmags[1],   "init_photdata_tab", float, 
                       (nblks*blksize));
    ED_CALLOC(photdata_stdmags[2],   "init_photdata_tab", float, 
                       (nblks*blksize));
    ED_CALLOC(photdata_airmass,   "init_photdata_tab", double, 
                       (nblks*blksize));
    ED_CALLOC(photdata_obstime,   "init_photdata_tab", double, 
                       (nblks*blksize));
    return tab;
}

void get_photdata(int phot_id, float **airm, float **zps, float **zpsdev,
                  int *num)
{
    int		i, n;
    float	*a, *z, *zs, zsum;

    ED_CALLOC(a,  "get_photdata", float, (photdata_nobjs+1));
    ED_CALLOC(z,  "get_photdata", float, (photdata_nobjs+1));
    ED_CALLOC(zs, "get_photdata", float, (photdata_nobjs+1));
    VPRINTF(DEBUG,"Examining %d datapoints for phot %d\n",photdata_nobjs,phot_id);
    for (i=0,zsum=0.0,n=0;i<photdata_nobjs;i++) {
       if (photdata_phot_id[i] == phot_id) {
          zsum += photdata_zp[i];
          n++;
       }
    }
    if (n>0)
       zsum /= (float) n;
    for (i=0,n=0;i<photdata_nobjs;i++) {
       VPRINTF(DEBUG,"Checking Phot %d id %d <-> %d at %f near %f\n", 
               i, photdata_phot_id[i], phot_id, photdata_zp[i], zsum);
       if (photdata_phot_id[i] == phot_id 
           && fabs(photdata_zp[i]-zsum) < 0.4
          ) {
       VPRINTF(DEBUG,"ok\n");
          a[n]  = photdata_airmass[i];
          z[n]  = photdata_zp[i];
          zs[n] = 1.086 * photdata_photerr[i] / photdata_phot[i];
          n++;
       }
    }
    *airm   = a;
    *zps    = z;
    *zpsdev = zs;
    *num    = --n;
}

static int	mblks, photom_nobjs;
static int	*photom_chan_nr, *photom_phot_id, *photom_nite;
static char	*photom_chan_name, *photom_phot_name, *photom_stdname;
static float	*photom_phot_ave, *photom_phot_sdev;
static float	*photom_zp_ave, *photom_zp_sdev;
static float	*final_zp, *final_zp_sdev, *exptime_unit;
static float	*final_coef, *final_coef_sdev;

tabstruct *init_photom_tab()
/****** init_photom_tab ***************************************************
 *
 *  NAME	
 *      init_photom_tab - Initialize the photom table and allocate variables
 *
 *  SYNOPSIS
 *      tabstruct *init_photom_tab()
 *
 *  FUNCTION
 *      This routine initializes the table structure and allocates the
 *      global variables required to store the data
 *
 *  INPUTS
 *      None
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      A pointer to the table structure is returned
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 21-08-1997
 *
 ******************************************************************************/
{
    tabstruct	*tab;

    tab = new_tab("PHOTOM");

    mblks = 1; photom_nobjs = 0;
    ED_CALLOC(photom_chan_nr,   "init_photom_tab", int,  
                       (mblks*blksize));
    ED_CALLOC(photom_phot_id,   "init_photom_tab", int,  
                       (mblks*blksize));
    ED_CALLOC(photom_nite,      "init_photom_tab", int,  
                       (mblks*blksize));
    ED_CALLOC(photom_chan_name, "init_photom_tab", char, 
                       (mblks*blksize*STRING_LEN));
    ED_CALLOC(photom_phot_name, "init_photom_tab", char, 
                       (mblks*blksize*STRING_LEN));
    ED_CALLOC(photom_stdname,   "init_photim_tab", char, 
                       (nblks*blksize*STRING_LEN));
    ED_CALLOC(photom_phot_ave,  "init_photom_tab", float,
                       (mblks*blksize));
    ED_CALLOC(photom_phot_sdev, "init_photom_tab", float,
                       (mblks*blksize));
    ED_CALLOC(photom_zp_ave,    "init_photom_tab", float,
                       (mblks*blksize));
    ED_CALLOC(photom_zp_sdev,   "init_photom_tab", float,
                       (mblks*blksize));
    ED_CALLOC(final_zp,           "init_photdata_tab", float, 
                       (nblks*blksize));
    ED_CALLOC(final_zp_sdev,      "init_photdata_tab", float, 
                       (nblks*blksize));
    ED_CALLOC(final_coef,         "init_photdata_tab", float, 
                       (nblks*blksize));
    ED_CALLOC(final_coef_sdev,    "init_photdata_tab", float, 
                       (nblks*blksize));
    ED_CALLOC(exptime_unit,       "init_photdata_tab", float, 
                       (nblks*blksize));
    return tab;
}

void add_photom_tab(char *name, char *std_name, float *phot_ave, 
                    float *phot_sdev, float *zp_ave, float *zp_sdev, 
                    int *phot_id, long int *nite)
/****** add_photom_tab ***************************************************
 *
 *  NAME	
 *      add_photom_tab - Add iformation to the PHOTOM table structure
 *
 *  SYNOPSIS
 *      void add_photom_tab(char *name, float *phot_ave, float *phot_sdev,
 *                             float *zp_ave, float *zp_sdev)
 *
 *  FUNCTION
 *      Add information to the global variables allocated for the storage
 *      of PHOTOM table data.
 *
 *  INPUTS
 *      name        - Name of the photometric parameter
 *      std_name    - Name of tha standard star
 *      phot_aver   - Average photometric parameter data
 *      phot_sdev   - Standard deviation of average photometric parameter data
 *      zp_ave      - Average zeropoint data
 *      zp_sdev     - Standard deviation of average zeropoint data
 *      phot_id     - Photometric parameters sequence number
 *      nite        - The night number
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *     The global variables that store the data for the PHOTOM table are
 *     updated with the new additions
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 21-08-1997
 *
 ******************************************************************************/
{
    int		j, nn;

    while (photom_nobjs + iphots >= mblks*blksize) {
       mblks++;
       ED_REALLOC(photom_chan_nr,   "init_photom_tab", int,  
                          (mblks*blksize));
       ED_REALLOC(photom_phot_id,   "init_photom_tab", int,  
                          (mblks*blksize));
       ED_REALLOC(photom_nite,      "init_photom_tab", int,  
                          (mblks*blksize));
       ED_REALLOC(photom_chan_name, "init_photom_tab", char, 
                          (mblks*blksize*STRING_LEN));
       ED_REALLOC(photom_phot_name, "init_photom_tab", char, 
                          (mblks*blksize*STRING_LEN));
       ED_REALLOC(photom_stdname,   "init_photom_tab", char, 
                          (mblks*blksize*STRING_LEN));
       ED_REALLOC(photom_phot_ave,  "init_photom_tab", float,
                          (mblks*blksize));
       ED_REALLOC(photom_phot_sdev, "init_photom_tab", float,
                          (mblks*blksize));
       ED_REALLOC(photom_zp_ave,    "init_photom_tab", float,
                          (mblks*blksize));
       ED_REALLOC(photom_zp_sdev,   "init_photom_tab", float,
                          (mblks*blksize));
       ED_REALLOC(final_zp,         "init_photom_tab", float, 
                       (nblks*blksize));
       ED_REALLOC(final_zp_sdev,    "init_photom_tab", float, 
                       (nblks*blksize));
       ED_REALLOC(final_coef,       "init_photom_tab", float, 
                       (nblks*blksize));
       ED_REALLOC(final_coef_sdev,  "init_photom_tab", float, 
                       (nblks*blksize));
       ED_REALLOC(exptime_unit,     "init_photom_tab", float, 
                       (nblks*blksize));
    }
    nn = 0;
       for (j=0;j<iphots;j++) {
          if (iphots > 1)
             sprintf(&photom_phot_name[(nn+photom_nobjs)*STRING_LEN],
                     "%s(%-d)",name,j+1);
          else
             strcpy(&photom_phot_name[(nn+photom_nobjs)*STRING_LEN],name);
          photom_chan_nr[nn+photom_nobjs]     = control.chan_nr;
          photom_phot_id[nn+photom_nobjs]     = *phot_id;
          photom_nite[nn+photom_nobjs]        = *nite;
          strcpy(&photom_chan_name[(nn+photom_nobjs)*STRING_LEN],
                 control.chan_name);
          photom_phot_ave[nn+photom_nobjs]     = phot_ave[j];
          photom_phot_sdev[nn+photom_nobjs]    = phot_sdev[j];
          photom_zp_ave[nn+photom_nobjs]       = zp_ave[j];
          photom_zp_sdev[nn+photom_nobjs]      = zp_sdev[j];
          strcpy(&photom_stdname[(nn+photom_nobjs)*STRING_LEN], std_name);
          nn++;
          *phot_id += 1;
       }
    photom_nobjs += nn;
}
/**/
void add_photom_calib(int photid, float f_zp, float f_coef, float f_zp_sdev, 
                      float f_coef_sdev, float exp_time)
{
    int		i;

    for (i=0;i<photom_nobjs;i++) {
       if (photom_phot_id[i] == photid) {
          VPRINTF(DEBUG,"Saving %f %f %f %f at %d for %d\n", f_zp, f_zp_sdev,
                 f_coef, f_coef_sdev, i, photid);
          final_zp[i]        = f_zp;
          final_zp_sdev[i]   = f_zp_sdev;
          final_coef[i]      = f_coef;
          final_coef_sdev[i] = f_coef_sdev;
          exptime_unit[i]    = exp_time;
       }
    }
}

/**/
void save_photom_tab(tabstruct *tab)
/****** save_photom_tab ***************************************************
 *
 *  NAME	
 *      save_photom_tab - Save the PHOTOM table to the output catalog
 *
 *  SYNOPSIS
 *      void save_photom_tab(tabstruct *tab, catstruct *cat)
 *
 *  FUNCTION
 *      This routine saves the global photom table data into the table
 *      structure
 *
 *  INPUTS
 *      tab  - Pointer to the table structure
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 20-08-1997
 *
 ******************************************************************************/
{
    add_key_to_tab(tab, "PHOT_NAME", photom_nobjs, photom_phot_name, 
                      T_STRING, STRING_LEN, "","Photometric parameter name");
    add_key_to_tab(tab, "STD_NAME", photom_nobjs, photom_stdname, 
                      T_STRING, STRING_LEN, "","Standard Star Name");
    add_key_to_tab(tab, "CHANNEL_NR", photom_nobjs, photom_chan_nr , T_LONG,
                      1, "","Channel number");
    add_key_to_tab(tab, "CHANNEL_NAME", photom_nobjs, photom_chan_name, 
                      T_STRING, STRING_LEN, "","Channel name");
    add_key_to_tab(tab, "2MUNIGHT", photom_nobjs, photom_nite, T_LONG,
                      1, "","Night number");
    add_key_to_tab(tab, "PHOT_AVE", photom_nobjs, photom_phot_ave, T_FLOAT,
                      1, "","Photometric Parameter Average");
    add_key_to_tab(tab, "PHOT_SDEV", photom_nobjs, photom_phot_sdev, T_FLOAT,
                      1, "","Photometric Parameter Standard Deviation");
    add_key_to_tab(tab, "ZP_AVE", photom_nobjs, photom_zp_ave, T_FLOAT,
                      1, "","Instrumental Zeropoint Average");
    add_key_to_tab(tab, "ZP_SDEV", photom_nobjs, photom_zp_sdev, T_FLOAT,
                      1, "","Instrumental Zeropoint Standard Deviation");
    add_key_to_tab(tab, "FINAL_ZP", photom_nobjs, final_zp, T_FLOAT,
                      1, "","Final Nights zeropoint");
    add_key_to_tab(tab, "FINAL_ZP_SDEV", photom_nobjs, final_zp_sdev, T_FLOAT,
                      1, "","Final Nights zeropoint standard deviation");
    add_key_to_tab(tab, "FINAL_COEF", photom_nobjs, final_coef, T_FLOAT,
                      1, "","Final Nights extinction coefficient");
    add_key_to_tab(tab, "FINAL_COEF_SDEV", photom_nobjs, final_coef_sdev, 
                      T_FLOAT, 1, "",
                      "Final Nights extinction coefficient standard deviation");
    add_key_to_tab(tab, "EXPTIME_UNIT", photom_nobjs, exptime_unit, T_FLOAT,
                      1, "","Photometric Calibration Exposure Time Unit");
    update_tab(tab);
    return;
}


void save_photo_cat(char *name, tabstruct *photom_tab, tabstruct *photdata_tab,
                    int argc, char **argv)
/****** save_photo_cat ***************************************************
 *
 *  NAME	
 *      save_photo_cat - Save the photometric calibration tables
 *
 *  SYNOPSIS
 *      void save_photo_cat(char *name, tabstruct *photom_tab, 
 *                                      tabstruct *photdata_tab)
 *
 *  FUNCTION
 *      This routine saves the PHOTOM and PHOTDATA tables to the disk.
 *
 *  INPUTS
 *      name         - The name of the output catalog
 *      photom_tab   - Pointer to the PHOTOM table
 *      photdata_tab - Pointer to the PHOTDATA table
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *      Data is written to disk
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 21-08-1997
 *
 ******************************************************************************/
{
    catstruct	*outcat;

    outcat = new_cat(1);
    init_cat(outcat);
    historyto_cat(outcat, "photom", argc, argv);
    if (add_tab(photom_tab, outcat, 0) != RETURN_OK)
       syserr(FATAL, "save_photom_cat", "Error saving PHOTOM table");
    if (add_tab(photdata_tab, outcat, 0) != RETURN_OK)
       syserr(FATAL, "save_photom_cat", "Error saving PHOTDATA table");
    VPRINTF(NORMAL, "Saving catalog %s\n", name);
    save_cat(outcat, name);
    return;
}

void derive_night_calib(int maxphots)
{
    float	*airmass, *airmsdev, *zps, *zpsdev, chi2, q;
    float	f_zp, f_coef, f_zp_sdev, f_coef_sdev;
    float	*l_zps, *l_coefs, adev, svar, skew, curt, airmin, airmax, dairm;
    int		i, j, k, n, nobj;

    VPRINTF(DEBUG,"Number of phots to fit: %d \n", maxphots);
    for (i=0;i<maxphots;i++) {
       get_photdata(i, &airmass, &zps, &zpsdev, &nobj);
       VPRINTF(DEBUG,"Fitting Photom sequence %d for nobj: %d\n", i, nobj);
       ED_CALLOC(airmsdev, "derive_night_calib", float, nobj);
       for (j=0,airmin=5.0,airmax=0.0;j<nobj;j++) { 
          airmin=MIN(airmin,airmass[j]);
          airmax=MAX(airmax,airmass[j]);
          airmsdev[j] = 0.001;
       }
       dairm = airmax - airmin;
       if (nobj>2 && control.do_fit == YES) {
          fit0(airmass, zps, nobj, zpsdev, 1, &f_zp, &f_coef, 
              &f_zp_sdev, &f_coef_sdev, &chi2, &q);
       } else {
          f_coef = (zps[1]-zps[0]) / (airmass[0]-airmass[1]);
          f_zp   = zps[0] - f_coef * airmass[0];
          f_coef_sdev = f_zp_sdev = 0.0;
       }
       f_zp = f_zp - 1.0 * f_coef;
       VPRINTF(VERBOSE,"ZP: %f (+/- %f), COEF: %f (+/- %f), Dairm: %f\n", 
               f_zp, f_zp_sdev, f_coef, f_coef_sdev, dairm);
/*
 *     Only save significant derivations of the extinction coefficient
 *       Coeffient derived (difference with default) is significant
 *       w.r.t. sigma of derivation
 *       Resulting coefficient is positive
 *       Range in airmass is large enough
 */
       if (control.fix_coef == NO && control.fix_zp == NO &&
           fabs(f_coef) > 3.0 * f_coef_sdev &&
           ((fabs(f_coef) < 2.0 * control.coef[control.chan_nr]) &&
            (control.coef[control.chan_nr] > 2 * fabs(f_coef))) &&
           (f_coef+control.coef[control.chan_nr]) > 0.0 &&
           dairm > 0.2 ) {
          f_coef = f_coef+control.coef[control.chan_nr];
       } else {
          f_coef = control.coef[control.chan_nr];
          f_zp = control.zp_estimate[control.chan_nr];
          f_zp_sdev = 1.0;
          if (control.fix_coef == YES) {
             for (k=0;k<3;k++) {
                ED_CALLOC(l_zps, "derive_night_calib", float, nobj);
                for (j=0,n=0;j<nobj;j++) {
                   if (fabs(zps[j]-f_zp) < 3.0 * f_zp_sdev)
/*
                      l_zps[n++] = zps[j] + airmass[j] * f_coef;
 */
                      l_zps[n++] = zps[j];
                }
                VPRINTF(DEBUG,"Moments of %d objs\n", n);
                moments(l_zps,n,&f_zp,&adev,&f_zp_sdev,&svar,&skew,&curt);
                ED_FREE(l_zps);
             }
          } else {
             if (control.fix_zp == YES ) {
                for (k=0;k<3;k++) {
                   ED_CALLOC(l_coefs, "derive_night_calib", float, nobj);
                   for (j=0,n=0;j<nobj;j++) {
                      if (fabs(zps[j]-f_zp) < 3.0 * f_zp_sdev)
                      l_coefs[n++] = (zps[j] - f_zp) / airmass[j] ;
                   }
                   VPRINTF(DEBUG,"Moments of %d objs\n", n);
                   moments(l_coefs,n,
                         &f_coef,&adev,&f_coef_sdev,&svar,&skew,&curt);
                   ED_FREE(l_coefs);
                }
              }
          }
       }
       VPRINTF(VERBOSE,"2ZP: %f (+/- %f), COEF: %f (+/- %f), Dairm: %f\n", 
               f_zp, f_zp_sdev, f_coef, f_coef_sdev, dairm);
       add_photom_calib(i, f_zp, f_coef, f_zp_sdev, f_coef_sdev, 
                        exptime_def[0]);
       ED_FREE(zps);
       ED_FREE(zpsdev);
       ED_FREE(airmass);
       ED_FREE(airmsdev);
    }
}

int main(int argc, char *argv[])
/****** photom ***************************************************
 *
 *  NAME	
 *      photom - Derive Photometric solution
 *
 *  SYNOPSIS
 *      photom -i incat -o outcat 
 *            [-c configfile] [CONFOPTION] [-b band]
 *
 *  FUNCTION
 *      This program derives a unique photometric solution for all standard
 *      star observations in the input catalog. It makes use of the pairing
 *      as derived by the prephotom program
 *
 *  INPUTS
 *      -i incat       - The name of the input catalog 
 *      -o outcat      - The name of the output  catalog 
 *     [-c configfile] - The name of the configuration file. The default
 *                       name is /denisoft/conf/photom.conf
 *     [CONFOPTION]    - Pairs of configuration VARIABLE VALUE
 *                       sequences that allow dynamically
 *                       overwritting of configuration file values.
 *     [-b band]       - The bandpass number for which to determine the 
 *                       solution. The input catalog must contain the
 *                       same band as the specified one!
 *
 *  RESULTS
 *
 *  RETURNS
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *      The photometric calibration process has not been fully implemented
 *      yet.
 *
 *  BUGS
 *
 *  AUTHOR
 *     E.R. Deul - dd 01-08-1996
 *
 ******************************************************************************/
{ 
    int		i, j, k, l, n, m, photid, lphotid, maxphots;
    photostruct	*std;
    short int   *lflag, *larti;
    long int	*limaflags;
    float	**phots, **photerrs, **zps;
    double	*airm, *obs_time;
    float       *phot_ave, phot_adev, *phot_sdev, phot_svar,
                phot_skew, phot_cur, *zp_ave, *zp_sdev, *stdmags[3];
    tabstruct	*photdata_tab, *photom_tab;
    char	outline[512];

    init(argc, argv);

    sprintf(header,"%s Version: %s (%s)",NAME,VERS,DATE);
    VPRINTF(NORMAL,"\n   %s\n\n",header);

    photdata_tab = init_photdata_tab();
    photom_tab   = init_photom_tab();
    read_stdcat(control.stdcatfile, stdmag_names, nstdmags);
    maxphots = 0;
    for (i=0;i<control.ilist;i++) {
       photid = 0;
       for (j=0;j<nphots;j++) {
          n = load_photo_fields(control.listfile[i], phot_names[j],
                                photerr_names[j]);
          std = select_standards(&n);
          exclude_errors(std, n, control);
          m = 0;
          for (k=0;k<n;k++) {
             if (!std[k].lflag) m++;
          }
          VPRINTF(DEBUG,"\n\n Found %d matches\n\n", m);
          if (m>0) {
             ED_CALLOC(phot_ave,  "photom", float  , iphots);
             ED_CALLOC(phot_sdev, "photom", float  , iphots);
             ED_CALLOC(zp_ave,    "photom", float  , iphots);
             ED_CALLOC(zp_sdev,   "photom", float  , iphots);
             ED_CALLOC(phots,     "photom", float *, iphots);
             ED_CALLOC(photerrs,  "photom", float *, iphots);
             ED_CALLOC(zps,       "photom", float *, iphots);
             for (k=0;k<iphots;k++) {
                ED_CALLOC(phots[k],     "photom", float,  m);
                ED_CALLOC(photerrs[k],  "photom", float,  m);
                ED_CALLOC(zps[k],       "photom", float,  m);
             }
             ED_CALLOC(lflag,    "photom", short int, m);
             ED_CALLOC(larti,    "photom", short int, m);
             ED_CALLOC(limaflags,"photom", long int , m);
             ED_CALLOC(airm,     "photom", double, m);
             ED_CALLOC(stdmags[0],  "photom", float, m);
             ED_CALLOC(stdmags[1],  "photom", float, m);
             ED_CALLOC(stdmags[2],  "photom", float, m);
             ED_CALLOC(obs_time, "photom", double, m);
             ED_CALLOC(std_name, "photom", char,   (m*STRING_LEN));
             m = 0;
             for (l=0;l<n;l++) {
                if (!std[l].lflag) {
                   for (k=0;k<iphots;k++) {
                      phots[k][m]    = std[l].phot[k];
                      photerrs[k][m] = std[l].photerr[k];
                      zps[k][m]      = std[l].zp[k];
                   }
                   lflag[m]    = std[l].flag;
                   larti[m]    = std[l].arti;
                   limaflags[m]= std[l].imaflag;
                   stdmags[0][m]  = std[l].stdmag[0];
                   stdmags[1][m]  = std[l].stdmag[1];
                   stdmags[2][m]  = std[l].stdmag[2];
                   airm[m]     = std[l].airm;
                   obs_time[m] = std[l].time;
                   strcpy(&std_name[m*STRING_LEN], std[l].stdname);
                   m++;
                   sprintf(&outline[0],"%d", l);
                   for (k=0;k<iphots;k++) 
                      sprintf(&outline[0],"%s %f",outline, std[l].phot[k]);
                   for (k=0;k<iphots;k++) 
                      sprintf(&outline[0],"%s %f",outline, std[l].zp[k]);
                   sprintf(&outline[0],"%s %f %f %f %f %f\n", outline,
                        std[l].stdmag[0], std[l].stdmag[1], 
                        std[l].stdmag[2], std[l].airm, std[l].time);
                   VPRINTF(VERBOSE,"%s", outline);
                }
             }
             if (m > 1) {
                for (k=0;k<iphots;k++) {
                   moments(phots[k], m, &phot_ave[k], &phot_adev,&phot_sdev[k], 
                                        &phot_svar, &phot_skew,&phot_cur);
                   VPRINTF(VERBOSE,"Ave: %f %f \n", phot_ave[k], 
                                                      phot_sdev[k]);
                   moments(zps[k], m, &zp_ave[k], &phot_adev, &zp_sdev[k], 
                                      &phot_svar, &phot_skew,&phot_cur);
                   VPRINTF(VERBOSE,"Ave: %f %f \n", zp_ave[k], 
                                                    zp_sdev[k]);
                }
             } else {
                for (k=0;k<iphots;k++) {
                   phot_ave[k] = phots[k][0];
                   zp_ave[k] = zps[k][0];
                }
             }
             if (m > 0) {
                lphotid = photid;
                add_photdata_tab(phot_names[j], phots, photerrs, zps, stdmags,
                                 airm, obs_time, std_name, lflag, limaflags, 
                                 larti, m, &lphotid);
             }
             if (m > 0) {
                lphotid = photid;
                add_photom_tab(phot_names[j], std_name, phot_ave, phot_sdev, 
                               zp_ave, zp_sdev, &lphotid, &night_nr[0]);
             }
             photid = lphotid;
             for (k=0;k<iphots;k++) {
                ED_FREE(phots[k]);
                ED_FREE(photerrs[k]);
                ED_FREE(zps[k]);
             }
             ED_FREE(phots);
             ED_FREE(photerrs);
             ED_FREE(zps);
             ED_FREE(lflag);
             ED_FREE(larti);
             ED_FREE(limaflags);
             ED_FREE(airm);
             ED_FREE(stdmags[0]);
             ED_FREE(stdmags[1]);
             ED_FREE(stdmags[2]);
             ED_FREE(std_name);
             ED_FREE(obs_time);
             ED_FREE(phot_ave);
             ED_FREE(phot_sdev);
             ED_FREE(zp_ave);
             ED_FREE(zp_sdev);
          }
       }
       close_photo_fields();
       maxphots = MAX(maxphots, photid);
    }
    derive_night_calib(maxphots);
    save_photom_tab(photom_tab);
    save_photdata_tab(photdata_tab);
    save_photo_cat(control.outfile[0], photom_tab, photdata_tab,
                   argc, argv);
    return 0;
}
