/* 21.06.01:
   removed compiler warnings under DEC

   05.05.03:
   included the tsize.h file

   29.01.2005:
   - I added history information to the output catalog
   - I added argc and argv to the arguments of function
     save_photo_cat to write HISTORY info to output catalogs.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "stdcat_globals.h"
#include "fitscat.h"
#include "tsize.h"
#include "tabutil_globals.h"
#include "utils_globals.h"
#include "utils_macros.h"
#include "options_globals.h"
#include "prephotom_globals.h"
#include "prephotom_vers.h"

char	header[MAXCHAR];

static char     **stdmag_names;
static int	nstdmags;

control_struct control;

option my_opts[] = {
            {"EPOCH", FLOAT , &control.epoch, 0, 0, 1950.0, 2050.0,
              0, 2000.0, NO, "",
             {"", "", "", "", ""}, 0,
             "The value of the epoch to use for standard star catalog positions"},
            {"MULTIPLE", BOOLEAN, &control.multiple, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "",
             {"YES", "NO", "", "", ""}, 0,
             "Allow multiple detections of standards in one frame"},
            {"PHOT", TEXT, control.phot, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "FLUX_APER",
             {"", "", "", "", ""}, 1,
             "The name of the OBJECTS table column containing the flux estimator"},
            {"RA", TEXT, control.ra, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "Ra",
             {"", "", "", "", ""}, 0,
             "The name of the OBJECTS table column containing the Right Ascension"},
            {"DEC", TEXT, control.dec, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "Dec",
             {"", "", "", "", ""}, 0,
             "The name of the OBJECTS table column containing the Declination"},
            {"FLAG", TEXT, control.flag, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "Flag",
             {"", "", "", "", ""}, 0,
             "The name of the OBJECTS table column containing the extraction flag"},
            {"PHOTERR", TEXT, control.photerr, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "FLUXERR_APER",
             {"", "", "", "", ""}, 1,
             "The name of the OBJECTS table column containing the error in the flux estimator"},
            {"STDMAG", TEXTVEC, control.stdmags, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "",
             {"", "", "", "", ""}, 0,
             "The names of the standard star magnitude columns to be read from the standard star catalog"},
            {"TOL", FLOAT , &control.tol, 0, 0, 0.0, 100.0,
              0, 0.1, NO, "",
             {"", "", "", "", ""}, 0,
             "The position accuracy with which to search for the extracted object [pixels]"},
              {""} 
            };

void usage(char *me)
{
    fprintf(stderr,"Usage: %s \n",me);
    fprintf(stderr,"		-i incat1 [[incat2] incat3] ... \n");
    fprintf(stderr,"		-o outcat \n");
    fprintf(stderr,"		[-s std_cat]\n");
    fprintf(stderr,"		[-r|-p] (relative | photom processing)\n");

    fprintf(stderr,"            [-OPTION1 OPTVAL1 [-OPTION OPTVAL2] ...]\n");
    fprintf(stderr,"	Program to associate extracted objects on a frame to frame bases\n");
    fprintf(stderr,"	with standard stars from the standard star catalog.  The program\n");
    fprintf(stderr,"	merges several isochromatic catalogs into one `night' photometric catalog\n\n");
    exit(1);
}
 
void init(int argc, char *argv[])
{
   int i, j, k;

   strcpy(control.chan_name, "Oops");
   control.proc_type = PHOTO;
   if (argc == 1) usage(argv[0]);
   init_options(0, my_opts);
   for (i=1; i<argc; i++) {
     if (argv[i][0] == '-') {
        switch(argv[i][1]) {
        case 'c': strcpy(control.conffile,argv[++i]);
                  break;
        case 's': strcpy(control.stdcatfile,argv[++i]);
                  break;
        case 'i': strcpy(control.listfile[0],argv[++i]);
                  control.ilist = 1;
                  break;
        case 'o': strcpy(control.outfile,argv[++i]);
                  break;
        case 'p': control.proc_type = PHOTO;
                  break;
        case 'r': control.proc_type = RELATIVE;
                  break;
        case '@': print_options(argv[0]);
                  exit(0);
        default:  parse_options(control.conffile);
                  j = i++; k = i;
                  set_command_option(argv[j], argv[k]);
                  break;
        }
     } else {
        syserr(WARNING,"init","Command line syntax error\n");
        usage(argv[0]);
     }
  }
  if (control.ilist == 0)
     usage(argv[0]);
  finish_options(control.conffile);
  stdmag_names  = copy_textvec(control.stdmags,     &nstdmags);
}

#define STRING_LEN 16
#define MAXPHOTS 40
/* Number of keys is 4 + NPHOT */
#define NKEYS 5
#define FKEYS 4

static catstruct *photo_cat;
static tabstruct *photo_tab, *photo_ftab;
static int	*SeqNr;
static double	*Ra, *Dec;
static float	*phot;
static short int	*fpos;
static char	*flags;

int read_photo_cat(char *name)
{
    keystruct	*key; 
    int		n, *objs;
    char	*chan_name;

    if ((photo_cat = read_cat(name)) == NULL) {
       syserr(FATAL, "read_photo_cat","Could not read catalog: %s\n",
              name);
    }
    photo_ftab = name_to_tab(photo_cat, "FIELDS", 0);
    ED_GETKEY(photo_ftab, key, "OBJECT_POS", objs, n, "read_photo_cat");
    photo_tab = name_to_tab(photo_cat, "OBJECTS", 0);
    chan_name = get_channel_name(photo_ftab);
    if (!strcmp(control.chan_name, "Oops")) strcpy(control.chan_name,chan_name);
    if (!strcmp(control.chan_name,chan_name)) {
       syserr(FATAL, "read_photom_cat", 
              "Multicolor input! %s has filter names %s instead of %s\n", 
              name, chan_name, control.chan_name);
    }
    strcpy(control.chan_name, chan_name);
    strcpy(stdmag_names[0], chan_name);
    return n;
}

int load_field(int nfield)
{
    keystruct	**keys;
    int		n, i, NPHOT = 1;
    char	**keynames;


    ED_MALLOC(keynames, "load_field", char *,      (NKEYS+NPHOT));
    ED_MALLOC(keys,     "load_field", keystruct *, (NKEYS+NPHOT));
    for (i=0;i<NKEYS+NPHOT;i++)
       ED_MALLOC(keynames[i], "load_field", char, MAXCHAR);
    strcpy(keynames[0], "SeqNr");
    strcpy(keynames[1], control.ra);
    strcpy(keynames[2], control.dec);
    strcpy(keynames[3], "FIELD_POS");
    strcpy(keynames[4], control.flag);
    strcpy(keynames[5], control.phot);

    n = read_field(photo_tab, keynames, keys, NKEYS+NPHOT, nfield, photo_ftab);
    for (i=0;i<NKEYS;i++) {
       if (!keys[i])
          syserr(FATAL,"load_field","No key %s in table\n",keynames[i]);
    }
    SeqNr   = keys[0]->ptr;
    Ra      = keys[1]->ptr;
    Dec     = keys[2]->ptr;
    fpos    = keys[3]->ptr;
    flags   = keys[4]->ptr;
    ED_GETFIELDVEC(phot,   "load_perm_field",keys[5], control.phot, n, float, float);

    for (i=0;i<NKEYS+NPHOT;i++)
       ED_FREE(keynames[i]);
    ED_FREE(keynames);
    ED_FREE(keys);
    return n;
}

void close_photo_cat()
{
    close_cat(photo_cat);
}

void save_photo_cat(control_struct c, int **seqnr,
                    int *n, int argc, char **argv)
{
 
    catstruct	*incat, *outcat;
    tabstruct	*tab;
    int		i;

    outcat = new_cat(1);
    for (i=0;i<c.ilist;i++) {
       if ((incat = read_cat(c.listfile[i])) == NULL) {
          syserr(FATAL, "save_photo_cat", "Could not read catalog: %s\n",
                 c.listfile[i]);
       }
       if (i == 0) {
          inherit_cat(incat, outcat);
          historyto_cat(outcat, "prephotom", argc, argv);
       }
       copy_tabs(incat, outcat);
       if (!(tab = name_to_tab(outcat, "OBJECTS", i+1)))
          syserr(FATAL,"save_photo_cat",
                       "No OBJECTS segment %d in catalog %s",i+1,c.listfile[i]);
       add_key_to_tab(tab, "STD_SeqNr", n[i], seqnr[i], T_LONG, 1, 
                      "Mag", "Standard star sequence number");
       update_tab(tab);
       close_cat(incat);
    } 
    VPRINTF(NORMAL, "Saving catalog %s", c.outfile);
    save_cat(outcat, c.outfile);
}

int main(int argc, char *argv[])

/****** prephotom ***************************************************
 *
 *  NAME	
 *      prephotom - Perform prephotometric pairing
 *
 *  SYNOPSIS
 *      prephotom -i incat1 incat2 ... -o outcat
 *                -s stdcat_file [CONFOPTION]
 *
 *  FUNCTION
 *      This program performs the pairing between the photometric sequence
 *      set of frames and the standard stars catalog entries. The
 *      frame-to-frame pairing has already been performed by the preastrom
 *      program. In addition the Ra,Dec information is derived and stored
 *      by the astrom and aplastrom programs.
 *
 *  INPUTS
 *      -i incat1 incat2 ... - The list of input catalogs (single passband)
 *                             that are to be combined to form a single
 *                             photometric entity.
 *      -o outcat            - The output catalog name combining the
 *                             information from all input catalogs.
 *      -s stdcat_file       - The name of the standard star catalog file.
 *                             This file has the same format as the data
 *                             catalog files.
 *     [-c configfile]       - The name of the configuration file. The default
 *                             name is /denisoft/conf/photom.conf
 *     [CONFOPTION]          - Pairs of configuration VARIABLE VALUE
 *                             sequences that allow dynamically
 *                             overwritting of configuration file values.
 *
 *  RESULTS
 *
 *  RETURNS
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *      The photometric calibration process has not been fully implemented
 *      yet.
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 01-08-1996
 *
 ******************************************************************************/
{ 
    float	t_Im, t_Jm, t_Km, t_phot;
    int		**STD_SeqNr;
    int		i, j, k, l, *n, nfields;
    char	std_name[MAXCHAR];
    int		seqnr;

    init(argc, argv);

    sprintf(header,"%s Version: %s (%s)",NAME,VERS,DATE);
    VPRINTF(NORMAL,"\n   %s\n\n",header);

    VPRINTF(VERBOSE,"Interpreting Standard Star Catalog file\n");
    ED_MALLOC(STD_SeqNr, "prephotom", int *, control.ilist);
    ED_CALLOC(n,         "prephotom", int,   control.ilist);
    for (i=0;i<control.ilist;i++) {
       nfields = read_photo_cat(control.listfile[i]);
       VPRINTF(VERBOSE,"Processing %d fields of %s\n", nfields, 
                        control.listfile[i]);
       if (i==0) read_stdcat(control.stdcatfile, stdmag_names, nstdmags);
       for (j=0;j<nfields;j++) {
          l = load_field(j+1);
          if (j==0) {
             ED_MALLOC(STD_SeqNr[i],  "prephotom", int, l);
          } else {
             ED_REALLOC(STD_SeqNr[i], "prephotom", int, (l+n[i]));
          }
          for (k=n[i];k<l+n[i];k++) {
             STD_SeqNr[i][k] = 0;
          }
          t_phot = 0.0;
          for (k=0;k<l;k++) {
             if (select_std_star(Ra[k], Dec[k], (float) control.tol, 
                            &t_Im, &t_Jm, &t_Km, std_name, &seqnr)) {
                if (t_phot < phot[k] && control.multiple == NO) {
                   t_phot = phot[k];
                   STD_SeqNr[i][n[i]+k] = seqnr;
                   VPRINTF(NORMAL,"Found standard %s in field %d object %d\n",
                               std_name, j, k); 
                } else {
                   STD_SeqNr[i][n[i]+k] = seqnr;
                   VPRINTF(NORMAL,"Found standard %s in field %d object %d\n",
                               std_name, j, k); 
                }
             }
          }
          n[i] += l;
       }
    }
    VPRINTF(VERBOSE,"Creating concatinated output catalog %s\n", 
            control.outfile);
    save_photo_cat(control, STD_SeqNr, n, argc, argv);
    ED_FREE(n);
    return 0;
}
