#!/usr/bin/env python

"""
absmag.py - script to calculate absolute magnitudes

SYNOPSIS:
   absmag.py input_catalogue with z and M_0
             M_0_filter
             spectrum_file
             absmag_filter

DESCRIPTION:
   The program expects an ASCII input output catalogue with redshifts
   and magnitudes.  The magnitudes are apparent magnitudes of galaxies
   at a redshift z having the pectrum given in 'spectrum_file'.  The
   filter in which the magnitude is measured is given in
   'M_0_filter'. With this input the script calculates absolute magnitudes
   in the filter 'absmag_filter'. The result is posted to STDOUT.
   The files 'M_0_filter', 'spectrum_file' and 'absmag_filter' are part
   of the photometric redshift code 'BPZ'. The script uses routines
   within this code to perfom its calculations.

AUTHOR:   
   Thomas Erben (terben@astro.uni-bonn.de)
   
"""

import sys
import numpy as np
# the cosmology module is part of (the THELI modified) BPZ:
import cosmology

# crude sanity check on command line arguments:
if len(sys.argv) != 5:
    print __doc__
    sys.exit(1)

data = np.loadtxt(sys.argv[1])

absmags = cosmology.m_abs(data[: ,1], data[: ,0],
                          sys.argv[3], sys.argv[2],
                          (0.3,.7,.7), sys.argv[4])

for mag in absmags:
    print "%.2f" % (mag)

    
