#!/bin/bash -xv

# The script estimates colour transformations between filters.
# We have a target filter f_tar, a source filter f_sou and two
# filters in the source system providing a colour term c1_sou and
# c2_sou. The script estimates a colour term 'c' in the equation:
#
# f_tar = f_sou + c * {c1_sou - c2_sou). 
#  
# The script needs the python program 'coltrans.py'.

#$1: catalogue with an OBJECTS table having (model) magnitudes
#    for all considered filters.
#$2: target filter
#$3: source filter
#$4: first colour term filter
#$5: second colour term filter

INCAT=$1
TARFIL=$2
SOURCEFIL=$3
COLFIL1=$4
COLFIL2=$5

# create ASCII catalogue for the fit which is done by a python
# script. We saw that stars with colour terms larger than 1 typically
# significantly alter (falsify) the result:
ldactoasc -b -i ${INCAT} -t OBJECTS \
          -k ${TARFIL} ${SOURCEFIL} ${COLFIL1} ${COLFIL2} |\
   awk '{print $1 - $2, $3 - $4}' | \
   awk '$2 > -0.9 && $2 < 0.9'>  tmp.txt_$$

SHORTTAR=`echo ${TARFIL} | sed -e 's/OMEGACAM_//' | \
          sed -e 's/MEGAPRIME_//'`
SHORTSOUR=`echo ${SOURCEFIL} | sed -e 's/OMEGACAM_//' | \
           sed -e 's/MEGAPRIME_//'`
SHORTCOL1=`echo ${COLFIL1} | sed -e 's/OMEGACAM_//' | \
           sed -e 's/MEGAPRIME_//'`
SHORTCOL2=`echo ${COLFIL2} | sed -e 's/OMEGACAM_//' | \
           sed -e 's/MEGAPRIME_//'`

./coltrans.py tmp.txt_$$ ${SHORTTAR} ${SHORTSOUR} ${SHORTCOL1} ${SHORTCOL2} \
  ${SHORTTAR}_${SHORTSOUR}_${SHORTCOL1}_${SHORTCOL2}.png

rm *_$$
exit 0;


