#!/bin/bash

# the script "check_anisotropy.sh" substitutes a former C++
# program with the same name.
# The script assumes that the ldac executables 
# and sm are in your path.

# 
# 23.05.01: initial version

if [ $# -lt 2 ]; then
  echo ""
  echo "USAGE:"
  echo "       check_anisotropy -i input catalog"
  echo "	              [ -s scaling for the stick plots (2000) ]"
  echo "	              [ -c conditions for object selection; ldacfilter syntax (\"(cl=2);\"]"
  echo "		      [ -t table with the objects (OBJECTS) ]"
  echo "		      [ -x position quantities (Xpos, Ypos) ]"
  echo "		      [ -d sm-device for the plots (psfile) ]"
  echo ""
  echo "DESCRIPTION:"
  echo "The script 'check_anisotropy' makes a control plot"
  echo "for the fits of the ellipticity done by the 'anisotropy"
  echo "program. The device given by the optional '-d' option"
  echo "should be a postscript device in the sm-graphcap file."
  echo "The name of the output plots are '...PSFanisotropy.ps'"
  echo "'...' stands for the catalog"
  echo "name without the '.cat' extension."
  echo ""
  echo "AUTHOR: Thomas Erben" 

  exit 1;
fi

CAT=""
CAT_TABLE=OBJECTS
XPOS=Xpos
YPOS=Ypos
SCALE=2000
CONDITION="(cl=2);"
DEVICE="psfile"

GoOn=0
while [ $GoOn = 0 ]
do
   case $1 in
   -x)
       XPOS=${2}
       YPOS=${3}	
       shift
       shift
       shift	
       ;;
   -i)
       CAT=${2}
       shift
       shift
       ;;
   -t)
       CAT_TABLE=${2}
       shift
       shift
       ;;
   -s)
       SCALE=${2}
       shift
       shift
       ;;
   -c)
       CONDITION=${2}
       shift
       shift
       ;;
   -d)
       DEVICE=${2}
       shift
       shift
       ;;
    *)
       GoOn=1
       ;;
    esac
done

# test whether input catalog has the necessary keys
# if not, leave the script immediately

ldactestexist -i $1 -t ${CAT_TABLE} \
              -k ${XPOS} ${YPOS} e1 e2 e1anisocorr e2anisocorr e1corrpol e2corrpol\
              >& /dev/null

if [ "$?" -gt "0" ]; then
  echo ""
  echo "ERROR: Your catalog does not contain all the keys:"
  echo "e1 e2 e1anisocorr e2anisocorr e1corrpol e2corrpol"
  echo ""
  exit 1;
fi

ldacfilter -i ${CAT} -o tmp.cat -t ${CAT_TABLE} -c ${CONDITION}
ldactoasc -i tmp.cat -t ${CAT_TABLE} -b -k ${XPOS} ${YPOS} \
          e1 e2 e1anisocorr e2anisocorr e1corrpol e2corrpol \
          > tmp.asc

BASE=`basename ${CAT} .cat`

{
  echo 'device "'${DEVICE} ${BASE}PSFanisotropy.ps'"'
  echo 'toplabel "'${CAT}'"'
  echo "define TeX_strings 1"
  echo "expand 1.2"  
  echo "data tmp.asc"
  echo "read { x 1 y 2 e1 3 e2 4 e1a 5 e2a 6 e1p 7 e2p 8 }"
  echo "window 2 2 1 2"
  echo "limits -0.1 0.1 -0.1 0.1"
  echo "box"
  echo "xlabel e1"
  echo "ylabel e2"
  echo "points e1 e2"
  echo "window 2 2 2 2"
  echo "limits -0.1 0.1 -0.1 0.1"
  echo "box"
  echo "xlabel e1'"
  echo "ylabel e2'"
  echo "points e1a e2a"
  echo "window 2 2 1 1"
  echo "limits x y"
  echo "box"
  echo "xlabel ${XPOS}"
  echo "ylabel ${YPOS}"
  echo "set ang=0.5*atan2(e2,e1)"
  echo "set eps=sqrt(e1*e1+e2*e2)"
  echo "set epsx=eps*cos(ang)"
  echo "set epsy=eps*sin(ang)"
  echo "define n (dimen(x))"
  echo "do i=0,\$n-1 {"
  echo "  define xp (x[\$i]-0.5*epsx[\$i]*${SCALE})"
  echo "  define yp (y[\$i]-0.5*epsy[\$i]*${SCALE})"
  echo "  relocate \$xp \$yp"
  echo "  define xp (\$xp+epsx[\$i]*${SCALE})"
  echo "  define yp (\$yp+epsy[\$i]*${SCALE})"
  echo "  draw \$xp \$yp }"
  echo "stats e1 m s k"
  echo "relocate (500 700)"
  echo "define mean ('<e1> ='+sprintf(' %.3f ',\$m)+'+/-'+sprintf(' %.3f ',\$s))"
  echo "label \$mean"
  echo "stats e2 m s k"
  echo "relocate (500 0)"
  echo "define mean ('<e2> ='+sprintf(' %.3f ',\$m)+'+/-'+sprintf(' %.3f ',\$s))"
  echo "label \$mean"
  echo "window 2 2 2 1"
  echo "limits x y"
  echo "box"
  echo "xlabel ${XPOS}"
  echo "ylabel ${YPOS}"
  echo "set ang=0.5*atan2(e2p,e1p)"
  echo "set eps=sqrt(e1p*e1p+e2p*e2p)"
  echo "set epsx=eps*cos(ang)"
  echo "set epsy=eps*sin(ang)"
  echo "define n (dimen(x))"
  echo "do i=0,\$n-1 {"
  echo "  define xp (x[\$i]-0.5*epsx[\$i]*${SCALE})"
  echo "  define yp (y[\$i]-0.5*epsy[\$i]*${SCALE})"
  echo "  relocate \$xp \$yp"
  echo "  define xp (\$xp+epsx[\$i]*${SCALE})"
  echo "  define yp (\$yp+epsy[\$i]*${SCALE})"
  echo "  draw \$xp \$yp }"
  echo "stats e1a m s k"
  echo "relocate (16500 700)"
  echo "define mean ('<e1> ='+sprintf(' %.3f ',\$m)+'+/-'+sprintf(' %.3f ',\$s))"
  echo "label \$mean"
  echo "stats e2a m s k"
  echo "relocate (16500 0)"
  echo "define mean ('<e2> ='+sprintf(' %.3f ',\$m)+'+/-'+sprintf(' %.3f ',\$s))"
  echo "label \$mean"
  echo "hardcopy"
  echo "quit"
} | sm

{
  echo 'device "'${DEVICE} ${BASE}PSFres.ps'"'
  echo 'toplabel "'${CAT}'"'
  echo "define TeX_strings 1"
  echo "data tmp.asc"
  echo "read { x 1 y 2 e1 3 e2 4 e1a 5 e2a 6 e1p 7 e2p 8 }"
  echo "window 1 1 1 1"
  echo "limits x y"
  echo "box"
  echo "xlabel ${XPOS}"
  echo "ylabel ${YPOS}"
  echo "set ang=0.5*atan2(e2a,e1a)"
  echo "set eps=sqrt(e1a*e1a+e2a*e2a)"
  echo "set epsx=eps*cos(ang)"
  echo "set epsy=eps*sin(ang)"
  echo "define n (dimen(x))"
  echo "do i=0,\$n-1 {"
  echo "  define xp (x[\$i]-0.5*epsx[\$i]*${SCALE})"
  echo "  define yp (y[\$i]-0.5*epsy[\$i]*${SCALE})"
  echo "  relocate \$xp \$yp"
  echo "  define xp (\$xp+epsx[\$i]*${SCALE})"
  echo "  define yp (\$yp+epsy[\$i]*${SCALE})"
  echo "  draw \$xp \$yp }"
  echo "stats e1a m s k"
  echo "relocate (500 700)"
  echo "define mean ('<e1> ='+sprintf(' %.3f ',\$m)+'+/-'+sprintf(' %.3f ',\$s))"
  echo "label \$mean"
  echo "stats e2a m s k"
  echo "relocate (500 0)"
  echo "define mean ('<e2> ='+sprintf(' %.3f ',\$m)+'+/-'+sprintf(' %.3f ',\$s))"
  echo "label \$mean"
  echo "hardcopy"
  echo "quit"
} | sm






