#!/bin/bash -xv

# this script creates a KSB catalog from the coadded
# images. first the coadded sections are pasted and then
# the catalog is created from this pasted image

#$1: main dir.
#$2: science dir.
#$3: image ending  including the completed ending for the 
#                  images with that coaddition was done
#                  (e.g. "OFCSFF.wcs")
#$4: name of coadd script
#$5: name of coadded image (without FITS extension)
#$6: name of the catalog (if not given it is the same
#    as the name of the coadded image

. lensing.ini

# first paste the coadded sections if not yet done

if [ "$#" -eq "6" ]; then
  CATNAME=$6
else
  CATNAME=$5
fi

if [ ! -f /$1/$2/coadd_$4/$5.fits ]; then
  LIMITS=`${P_FITSLIMITS} /$1/$2/*$3.fits -r /$1/$2/coadd_$4/$4CA+00+00.fits`
  
  ${P_FITSPASTE} -i /$1/$2/coadd_$4/$4CA??????.fits \
			  -o /$1/$2/coadd_$4/$5.fits \
			  -x ${LIMITS}
  
  ${P_FITSPASTE} -i /$1/$2/coadd_$4/$4CA??????.weight.fits \
			  -o /$1/$2/coadd_$4/$5.weight.fits \
			  -x ${LIMITS}
fi

if [ ! -d /$1/$2/lensing ]; then
  mkdir /$1/$2/lensing
  mkdir /$1/$2/lensing/cat
  mkdir /$1/$2/lensing/plots
fi

if [ ! -d /$1/$2/lensing/cat ]; then
  mkdir /$1/$2/lensing/cat
fi

if [ ! -d /$1/$2/lensing/plots ]; then
  mkdir /$1/$2/lensing/plots
fi


# now run Sextractor and KSB
${P_SEX} -c ${DATACONF}/postcoadd.conf.sex \
	       /$1/$2/coadd_$4/$5.fits \
	       -CATALOG_NAME sex.cat \
	       -WEIGHT_IMAGE /$1/$2/coadd_$4/$5.weight.fits \
	       -WEIGHT_TYPE MAP_WEIGHT\
	       -DETECT_MINAREA 5 \
	       -DETECT_THRESH 2.5 \
	       -ANALYSIS_THRESH 2.5\
               -FILTER_NAME ${DATACONF}/gauss_4.0_7x7.conv
	       -BACK_TYPE AUTO
#	       -FILTER N

#	       -WEIGHT_IMAGE "" \
#	       -DETECT_MINAREA 3 \
#	       -DETECT_THRESH 1.0 \
#	       -ANALYSIS_THRESH 1.0\



${P_LDACCONV} -i sex.cat -o tmp.cat -b 1 -c "sex" -f R

${P_LDACFILTER} -i tmp.cat -o /$1/$2/lensing/cat/${CATNAME}_ldac.cat\
                -t OBJECTS -c "(Flag<4);"

rm sex.cat
tm rmp.cat

${P_LDACADDKEY} -i /$1/$2/lensing/cat/${CATNAME}_ldac.cat -o tmp.cat\
			-t OBJECTS -k nu 1.0 FLOAT ""

${P_LDACCALC} -i tmp.cat -o tmp1.cat -t OBJECTS \
			  -c "(FLUX_RADIUS);" -n "rg" "" -k FLOAT \
			  -c "(Xpos-1.0);" -n "x" "" -k LONG \
			  -c "(Ypos-1.0);" -n "y" "" -k LONG \
			  -c "(Xpos);" -n "xbad" "" -k FLOAT \
			  -c "(Ypos);" -n "ybad" "" -k FLOAT

${P_LDACTOASC} -i tmp1.cat -b -t FIELDS \
			   -k FITSFILE SEXBKGND SEXBKDEV -s > tmp.asc

# create a config file for asctoldac on the fly

echo "VERBOSE = DEBUG"       >  asctoldac_tmp.conf
echo "COL_NAME  = IMAGE"     >> asctoldac_tmp.conf
echo "COL_TTYPE = STRING"    >> asctoldac_tmp.conf
echo "COL_HTYPE = STRING"    >> asctoldac_tmp.conf
echo 'COL_COMM = ""'         >> asctoldac_tmp.conf
echo 'COL_UNIT = ""'         >> asctoldac_tmp.conf
echo 'COL_DEPTH = 128'       >> asctoldac_tmp.conf
echo "COL_NAME  = SKY_MODE"  >> asctoldac_tmp.conf
echo "COL_TTYPE = FLOAT"     >> asctoldac_tmp.conf
echo "COL_HTYPE = FLOAT"     >> asctoldac_tmp.conf
echo 'COL_COMM = ""'         >> asctoldac_tmp.conf
echo 'COL_UNIT = ""'         >> asctoldac_tmp.conf
echo 'COL_DEPTH = 1'         >> asctoldac_tmp.conf
echo "COL_NAME  = SKY_SIGMA" >> asctoldac_tmp.conf
echo "COL_TTYPE = FLOAT"     >> asctoldac_tmp.conf
echo "COL_HTYPE = FLOAT"     >> asctoldac_tmp.conf
echo 'COL_COMM = ""'         >> asctoldac_tmp.conf
echo 'COL_UNIT = ""'         >> asctoldac_tmp.conf
echo 'COL_DEPTH = 1'         >> asctoldac_tmp.conf
		 
${P_ASCTOLDAC} -i tmp.asc -c asctoldac_tmp.conf -t HFINDPEAKS \
				 -o hfind.cat -b 1 -n "KSB"
		  

# now transfer the HFINDPEAKS table to the SEX catalog
${P_LDACADDTAB} -i tmp1.cat -o tmp2.cat \
			    -t HFINDPEAKS -p hfind.cat

${P_LDACFILTER} -i tmp2.cat \
			    -o /$1/$2/lensing/cat/${CATNAME}_pos.cat \
			    -c "(rg>0.0)AND(rg<30.0);"

# now run analyseldac
${P_ANALYSELDAC} -i /$1/$2/lensing/cat/${CATNAME}_pos.cat \
			       -o /$1/$2/lensing/cat/${CATNAME}_ksb.cat \
			       -x 1 -r -3 -f /$1/$2/coadd_$4/$5.fits

# merge KSB and SEX catalogs

${P_LDACJOINKEY} -i /$1/$2/lensing/cat/${CATNAME}_ksb.cat -o tmp.cat\
     -p /$1/$2/lensing/cat/${CATNAME}_pos.cat -t OBJECTS -k DELTA_J2000\
		ALPHA_J2000 FLUX_RADIUS MAG_AUTO 

${P_LDACFILTER} -i tmp.cat -o clean.cat\
     -c "(cl=0);"

if [ -f tmp ]; then
  rm tmp
fi

#${P_PREANISOTROPY} -i clean.cat -t OBJECTS \
#     -k rh mag -s 0.16 -c snratio 70.0 10000.0 >& tmp
${P_PREANISOTROPY} -i clean.cat -t OBJECTS \
     -k rh mag -s 0.16 -c snratio 100.0 10000.0 >& tmp

LINE=`${P_GAWK} '($2=="propose") { print $10,$8,$12,$16,$14,$18}' tmp`
LINE2=`${P_GAWK} '($2=="propose") { print $8,$12,$14,$18}' tmp`
MINRH=`${P_GAWK} '($2=="propose") { print $12}' tmp`
MINMAG=`${P_GAWK} '($2=="propose") { print $14}' tmp`

#LINE="rh 1.5 2.2 mag 20.0 25.0"
#LINE2="1.5 2.2 20.0 25.0"
#MINRH=1.5
#MINMAG=20.0

#${P_ANISOTROPY} -i clean.cat -o /$1/$2/lensing/cat/${CATNAME}_anisocorr.cat \
#			    -c ${LINE} -j 12.25 -e 3.0
${P_ANISOTROPY} -i clean.cat -o /$1/$2/lensing/cat/${CATNAME}_anisocorr.cat \
			    -c ${LINE} -j 12.25 -e 2.0
${LENSSCRIPTS}/check_anisotropy.sh \
		-i /$1/$2/lensing/cat/${CATNAME}_anisocorr.cat -s 20000
mv ${CATNAME}_*.ps /$1/$2/lensing/plots

if [ -f ${CATNAME}result.txt ]; then
  rm ${CATNAME}result.txt
fi
 
${LENSSCRIPTS}/stars.sh /$1/$2/lensing/cat/${CATNAME}_anisocorr.cat \
			    ${LINE2} /$1/$2/coadd_$4/$5.fits
${LENSSCRIPTS}/addPshPsm.sh /$1/$2/lensing/cat/${CATNAME}_anisocorr.cat \
			      ${CATNAME}result.txt 
${P_LDACCALC} -i /$1/$2/lensing/cat/${CATNAME}_final.cat \
			      -o t.cat \
			      -c "sqrt(e1iso_RG*e1iso_RG+e2iso_RG*e2iso_RG);" \
			      -n "eiso" "" -k DOUBLE
${P_LDACFILTER} -i t.cat -o  /$1/$2/lensing/cat/${CATNAME}_clus.cat \
			      -c "(((cl=0)AND(mag>${MINMAG})))AND(rh>${MINRH});"

# create ASCII catalog for skycat:
${P_LDACTOSKYCAT} -i /$1/$2/lensing/cat/${CATNAME}_clus.cat -t OBJECTS\
 -k SeqNr ALPHA_J2000 DELTA_J2000 MAG_AUTO -s -l id_col SeqNr ra_col \
ALPHA_J2000 dec_col DELTA_J2000 mag_col MAG_AUTO \
> /$1/$2/lensing/cat/${CATNAME}.skycat.asc

#rm clean.cat 
#rm t.cat
#rm tmp*cat
#rm tmp*asc
#rm asctoldac_tmp.conf




