#!/freeware/bin/bash 

# the script transforms an ASCII context file
# (drizzle output) to a LDAC catalog. It assumes
# that the "awk" program and "asctoldac" are in your
# path.

#$1: ASCII context table from EIS drizzle
#$2: output LDAC catalog

# first make a pass with awk determining the maximum
# number of images involved:

NMAX=`awk 'BEGIN {n=0} ( $1 != "#") { if(NF>n) { n=NF } } END {print n}' $1`

# now go through the file creating a file for asctoldac
awk '( $1 != "#") { printf("%d %d %.0f ", $1, $2, $3)
                    for(i=4; i<NF+1; i++) {
                      printf("%d ", $i)
                    }
                    for(i=NF; i< '${NMAX}'; i++) {
                      printf("%d ", -1)
                    }
                    printf("\n") 
                  }' $1 > tmp.asc

# now create a config file for asctoldac
echo "COL_NAME = CONTEXTNR" > asctoldac.conf
echo "COL_TTYPE= LONG" >> asctoldac.conf
echo "COL_HTYPE= INT" >> asctoldac.conf
echo "COL_COMM = Context number" >> asctoldac.conf
echo "COL_UNIT = \"\"" >> asctoldac.conf
#
echo "COL_NAME = NIMA" >> asctoldac.conf
echo "COL_TTYPE= LONG" >> asctoldac.conf
echo "COL_HTYPE= INT" >> asctoldac.conf
echo "COL_COMM = Number of images" >> asctoldac.conf
echo "COL_UNIT = \"\"" >> asctoldac.conf
#
echo "COL_NAME = NPIX" >> asctoldac.conf
echo "COL_TTYPE= LONG" >> asctoldac.conf
echo "COL_HTYPE= INT" >> asctoldac.conf
echo "COL_COMM = Number of pixels" >> asctoldac.conf
echo "COL_UNIT = \"\"" >> asctoldac.conf

i=1
NIMA=$(( ${NMAX} -3 ))
while [ "${i}" -le "${NIMA}" ]
do
  echo "COL_NAME = IMAGE_${i}" >> asctoldac.conf
  echo "COL_TTYPE= LONG" >> asctoldac.conf
  echo "COL_HTYPE= INT" >> asctoldac.conf
  echo "COL_COMM = Unique image IDs" >> asctoldac.conf
  echo "COL_UNIT = \"\"" >> asctoldac.conf  

  i=$(( $i + 1 ))
done

# now run asctoldac
asctoldac -i tmp.asc -c asctoldac.conf -t CONTEXT -o $2




