#!/bin/bash

# script to extract masks for individual, resampled
# images from the region file of a co-added image 
# result region files will go into $1/$2/maks_$3
# 
# The script relies on the script
# ./convert_mask.sh 

#$1: basedir.
#$2: set dir.
#$3: coadd ident
#$4: name of co-added image (assumed to
#    reside in /$1/$2/coadd_$3)
#$5: master mask file from co-added image 
#    (/$1/$2/masks_$3)

BASECOADD=`basename $4 .fits`

imhead -b < $1/$2/coadd_$3/$4 > ${BASECOADD}.head

FILES=`find $1/$2/coadd_$3 -name \*$3.resamp.fits`

# get rid of 'yellow' underdense masks
# They typically apply to co-added images only
gawk '$0 !~ /yellow/' /$1/$2/masks_$3/$5 > tmp.reg_$$

for FILE in ${FILES}
do
  BASE=`basename ${FILE} .fits`
  imhead -b < ${FILE} > ${BASE}.head
  ./convert_mask.sh tmp.reg_$$ \
    ${BASECOADD}.head ${BASE}.head \
    /$1/$2/masks_$3/${BASE}.reg
done

test -f tmp.reg_$$ && rm tmp.reg_$$
