#!/bin/bash

# the script runs SExtractor over an image
# (HAVING CORRECT ASTROMETRIC HEADERS) and 
# creates an icat catalog ready for the 
# preastrom tool.
# the script uses the executables in the
# PIPEBIN directory and takes the config files
# from the PIPESOFT/config/others directory.
#
# 03.01.2005:
# I changed the location of the do_make_icat
# config files.
#
# 11.04.2005:
# I introduced the possibility to use weight images
# in the extraction process.

# $1: image name
# $2: weight image (if this argument is 'NONE' then no weight image is used)
# $3: icat catalog name

# preliminary stuff:
if [ -f tmp.cat ]; then
  rm tmp.asc
fi

# first run sextractor

if [ "$2" != "NONE" ]; then
  ${PIPEBIN}/${EXT}/sex -c ${PIPESOFT}/conf/others/do_make_icat.sex $1\
                        -WEIGHT_IMAGE $2 -WEIGHT_TYPE MAP_WEIGHT
else
  ${PIPEBIN}/${EXT}/sex -c ${PIPESOFT}/conf/others/do_make_icat.sex $1
fi

# now ldacconv, ldacfilter

${PIPEBIN}/${EXT}/ldacconv -i make_icat_sex.cat -o make_icat_ldac.cat -f I\
                           -c "test" -b 1
${PIPEBIN}/${EXT}/ldacfilter -i make_icat_ldac.cat -o make_icat_good.cat \
                      -c "(Flag < 1);"
${PIPEBIN}/${EXT}/ldactoasc -i make_icat_good.cat -t OBJECTS \
                     -k FIELD_POS SeqNr ALPHA_J2000 DELTA_J2000 MAG_AUTO\
                     > tmp.asc
${PIPEBIN}/${EXT}/make_icat -i tmp.asc -o $3 -c ${PIPESOFT}/conf/others/do_make_icat.conf


