#!/bin/bash -u

# Main installation script for the pipeline

#
# 17.07.03:
# updated scripts are now copied to a machine dependent
# subdirectory under ${PD}/scripts. Hence, there is no
# more problem with overwriting when scripts are updated
# for different architectures. Also, all final scripts
# are in a single place from where they can be copied. 
#
# 04.09.03:
# the testmachine now also compiles and executes the
# sysconf program from the eclipse package. It creates
# a config.h file necessary to compile the eclipse and
# qfits libraries.
#
# 18.02.2004:
# included path for the photometric standard star catalog.
#
# 07.06.2004:
# included the TEMPDIR in creating the progs.ini file
#
# 24.11.2004:
# Python scripts are copied to the machine
# dependent script directory.
#
# 07.12.2004:
# I corrected a bug in the copying of Python scripts
# to the machine script directory.
#
# 18.03.2005:
# I included '.include' that collect bash script
# functions and need to be copied to machine dependent
# script directories.
#
# 18.07.2005:
# Astrometrix scripts are included in the installation
# process. The global version number is now assigned
# from this script.
#
# 02.08.2005:
# Increase of pipeline global version number
#
# 14.08.2005:
# I included the UNIX programs 'find" and 'sort' in the
# installation process (find needs to support the '-maxdepth'
# and sort the '-g' option.)
#
# 24.10.2005:
# Increase of pipeline global version number
#
# 06.12.2005:
# - I added specific CFH12K scripts to the pipeline setup
# - Increase of the global pipeline version number
#
# 30.12.2005:
# Increase of the global pipeline version number
#
# 31.12.2005:
# Increase of the global pipeline version number
#
# 13.02.2006:
# Increase of the global pipeline version number
#
# 09.03.2006:
# I included MAC/Darwin systems to the known architectures.
#
# 14.03.2006:
# I changed the installation scheme for ASTROMETRIX: instead
# of overriding the original script files with modified ones,
# I keep the modified, renamed, ones in the pipeline structure
# and add the corresponding directory the PERL5LIB variable. 
# This allows us to directly call the modified ASTROMETRIX files 
# with the perl interpreter. 
#
# 17.05.2006:
# Increase of the minor pipeline version number
#
# 14.06.2006:
# Increase of the minor pipeline version number
#
# 24.06.2006:
# I added MMT Megacam configuration files and processing scripts
#
# 28.08.2006:
# The SM program is now queried by a 'which' command (before
# the complete path had to be given by the user).
#
# 30.08.2006:
# Increase of the minor pipeline version number
#
# 23.09.2006:
# Increase of semi major pipeline version number
#
# 06.11.2006:
# The version number goes to 0.12.0
#
# 15.09.2007:
# I included code for the possibility to use scamp within
# THELI
#
# 03.12.2007:
# I introduced Darwin_PPC and Darwin_INTEL systems; the old
# 'Darwin' was removed. This was necessary as the Intel and PPC
# MACS come with different byte swap for floating point numbers.
#
# 10.06.2008:
# - I added the scripts/science dir. to the script-update
#   directories.
# - Internal version number pushed to 0.12.23
#
# 25.07.2008:
# Data from two MEGAPRIME config directories (reduction and science)
# are copied to the machine script directories (UPDATE step).
#
# 28.11.2008:
# I slightly reformated the script to make it more readable
#
# 21.12.2008:
# The interpreters for python and perl are defined now
# also via variables. The main problem was that 'alias'
# deinitions were not taken into account in pipeline
# script calls leading to problems when the script 
# interpreters were privately installed.
#
# 08.06.2009:
# I added files for LBT data processing
#
# 15.09.2009:
# - I added files from the MEGAPRIME_science config dir.
#   to the final script/config file collection
# - In the update scripts step we now consider Perl and Python
#   files in addition to shell scripts. We only took into account
#   Python files up to now.
#
# 31.08.2010:
# Version number goes to 1.2.1
#
# 16.09.2010
# Version number goes to 1.2.2 (internal production version)
#
# 30.09.2010
# Version number goes to 1.3.0 (new WWW release)
#
# 08.10.2010
# Version number goes to 1.3.1 (internal production version)
#
# 26.11.2010:
# Version number goes to 1.4.0 (internal production version);
# new swarp version and associated changes
#
# 01.17.2010:
# version number goes to 1.4.1 (internal production version)
#
# 24.05.2010:
# version number goes to 1.5.0 (release version; important bug fixes)
#
# 13.06.2011:
# Version number goes to 1.5.1; I included Makefiles for Darwin_INTEL 64
# architectures.
#
# 27.07.2011:
# Version number goes to 1.5.2 (minor bug fixes)
#
# 05.09.2011:
# Version number goes to 1.5.3 (minor bug fixes/enhancements)
#
# 26.01.2012:
# Version number goes to 1.6.0 (bug fixes/enhancements)
#
# 31.01.2012:
# Version number goes to 1.6.1 (bug fixes/enhancements)
#
# 19.11.2012:
# Version number goes to 1.6.3 as preparation for V1.7.0
# (this should be the version to process the KIDS survey)
#
# 23.11.2012:
# Omegacam files are now copied to the machine scripts directory;
# version number goes to 1.6.5
#
# 26.11.2012:
# I implemented photometric calibration on a run basis.
# version number goes to 1.6.6
#
# 30.12.2012:
# many small updates for a first V0.1 run of KIDS data.
# version number goes to 1.6.7
#
# 10.01.2013:
# introduction of new error reporting and logging scheme
# version number goes to 1.7.0
#
# 18.01.2013:
# lots of small bugs for KIDS processing fixed.
# version number goes to 1.7.1
#
# 11.02.2013:
# again lots of small bugs for KIDS processing fixed.
# version number goes to 1.7.2
#
# 20.02.2013:
# again lots of small bugs for KIDS processing fixed.
# version number goes to 1.7.4
#
# 03.04.2013:
# again lots of small bugs for KIDS processing fixed.  I added scripts
# for crosstalk correction (thanks to A. Buddendiek) version number
# goes to 1.7.5
#
# 16.05.2013:
# and again a lot of small bug fixes; version number goes to 1.7.6.
#
# 24.05.2013:
# upgrade to SExtractor 2.5.0 to use windowed object positions in various
# parts of the pipeline; version number goes to 1.8.0.
#
# 22.09.2013:
# I start to integrate windowed SExtractor quantities; this is done
# within version 1.9.0.
#
# 22.01.2014:
# - The code did not compile on MACOS 10.9 - fixed. Code version goes to 1.9.1
# - We now used windowed position quantities for astrometric calibration.
#
# 03.04.2014:
# - The TEMPDIR var. for progs.ini is no longer modified in this script.
#   We sat it to `pwd` in the include script itself. If the user wants to
#   change it this is done in that script anyway.
# - Version number goes to 1.9.3 - again many small fixes/refinements for
#   KIDS/ATLAS processing.
#
# 15.05.2014:
# Version number goes to 1.9.5 - again many small bug fixes during KIDS data
# processing.
#
# 24.06.2014:
# Version number goes to 1.9.6. Main improvement is a consequent consideration
# of bad CCDs in the preprocessing phase.
#
# 16.09.2014:
# I included the DECam instrument - version number goes to 1.9.7.
#
# 29.09.2014:
# Numerous small refinements with the inclusion of the DECam
# instrument. Version number goes to 1.9.8.
#
# 10.11.2014:
# The global pipeline version number now has to be transfered to
# bash_functions.include to be included in THELI logging messages -
# version number goes to 1.9.9.
#
# 21.01.2015:
# Numerous refinements in teh course of a new KIDS/ATLAS processing run.
# Version number goes to 1.9.10.
#
# 23.02.2015:
# I included treatment of the 'conv_WFI_filter_names.include'. It
# contains an alias definition to perform filter name translation for
# the WFI@MPG/ESO2.2m instrument.
#
# 16.06.2015:
# Inclusion of scripts and modifications to process MEGAPRIME@CFHT
# data with its new 40-chip mode. Version number pushed to 1.9.11.
#
# 05.08.2015:
# - Instead for 'python' I now test for an executable 'python2' to
#   ensure we use python 2 and not python 3.
# - We now test for the number of available CPUs / cores and put this
#   number automatically into the progs.ini file (NPARA variable).
#
# 11.08.2015:
# Version number pushed to 1.10.0.

# IMPORTANT:
#
# The script checks machine type, updates pipeline scripts (progs.ini;
# pipe_env ...)  and compiles pipeline sources the script needs the
# source file 'endian.c' in the directory it is called. Modify the
# paths listed below before running this script.

#
# Modes of the script
# 
# ./install.sh -m "TEST"
# tests the machine type and gives information whether
# you can compile sources without problems or whether
# you have to create new Makefiles (see the documentation
# for more details)
#
# ./install.sh -m "ALL"
# updates paths in all pipeline scripts and compiles
# pipeline sources
#
# ./install.sh -m "UPDATE"
# only updates paths in pipeline scripts
#
# ./install.sh -m "COMPILE"
# only compiles pipeline sources

##
## stuff that has to be modified by the user
##

# needed directories (absolutely necessary)
#
# PD is the main directory with the pipeline sources
PD=/vol/aibn234/data1/thomas/software/cvswork/ldacpipeline
# The directory with the standard star catalog
STANDARDSTARCAT=/aibn202_2/thomas/USNO-A2/
# The directory with the photometric standard star catalog
PHOTSTANDARDSTARCAT=/aibn85_b/terben/software/cvswork/Stetson_std.cat
#
# The IRAFDIR only needs to be edited if eisdrizzle
# is used for the coaddition
IRAFDIR=/aibn85_b/terben/iraf

# needed programs/scripts not included in the pipeline distribution
# (absolutely necessary)
#
# The path for the bash shell (you need version 2.0 or higher)
P_BASH=`which bash`
#
# The path to the GNU awk program (awk is NOT sufficient)
P_GAWK=`which gawk`
#
# The path of the UNIX find program (the '-maxdepth' option has to be
# supported; use for instance the GNU find !!)
P_FIND=`which find`
#
# The path of the UNIX sort program (the '-g' option has to be
# supported; use for instance the GNU sort !!)
P_SORT=`which sort`
#
# The perl and python interpreters:
P_PYTHON=`which python2`
P_PERL=`which perl`

# the following programs are not absolutely necessary (see the manual
# to learn what restrictions you have in this case)
#
# Path to the IRAL cl script
P_CL=/scisoft/bin/cl
#
# Path to the Super Mongo (sm) program
P_SM=`which sm`
#
# Path to the scamp program. The current pipeline
# suuports version V1.7.0 of scamp
P_SCAMP=`which scamp`
#
# Path to the aclient program. It is mandatory
# if scamp is used within THELI
P_ACLIENT=`which aclient`

# Pipeline version
PIPEVERS="Pipeline Global Version: 1.10.0 (Internal Version)"

# C-compiler used in this script
# (this affects ONLY this script; not the
# C-compiler used in the compilation of the sources)
CC=gcc

# known architectures for source compilation
KNOWNARCH="AIX Darwin_PPC Darwin_INTEL Darwin_INTEL_64 HP-UX IRIX64 Linux Linux_64 OSF1 SunOS"

##
## here the stuff that should ever be edited ends!!!
##

# by default the architecture is not known !!
KNOWN=0
MACHINE=""
NCPUS=1 # default for the number of available CPUs / cores

function updatescripts() {
  DIR=`pwd`
  cd $1
  #
  # deal with the bash shell scripts where we substitute the first
  # line by the actual location of the bash shell
  ls -1 *.sh.raw > tmp.$$

  while read FILE
  do
    BASE=`basename ${FILE} .sh.raw`
    sed -e 's!/bin/bash!'${P_BASH}'!' \
           ${FILE} > ${PD}/scripts/${MACHINE}/${BASE}.sh	  
    chmod a+x ${PD}/scripts/${MACHINE}/${BASE}.sh
  done < tmp.$$

  rm tmp.$$
  cd ${DIR}
}

#
# test the machine type run also sysconf from eclipse
#
function testmachine() {
  # first test if the endian program is alreay available; if not,
  # compile it!
  if [ ! -f endian.c ]; then
    echo "need the source file 'endian.c'"
    exit 1
  else
    ${CC:?} -o endian endian.c	 
  fi

  MACHINE=`uname`

  # test for LINUXALPHA architecture if the machine is Linux
  if [ "${MACHINE}" = "Linux" ]; then
    LONG=`./endian | ${P_GAWK:?} '($1=="Sizes:") {print substr($5,3,1)}'`

    if [ "${LONG}" = "8" ]; then
      MACHINE="Linux_64"
    fi
  fi

  # test for the different MAC/Darwin systems (PPC or Intel)
  if [ "${MACHINE}" = "Darwin" ]; then
    INTEL=`./endian | ${P_GAWK:?} '($2=="machine") {print $5}'`

    MACHINE="Darwin_PPC"
    if [ "${INTEL}" = "Intel" ]; then
      MACHINE="Darwin_INTEL"
    fi

    LONG=`./endian | ${P_GAWK:?} '($1=="Sizes:") {print substr($5,3,1)}'`

    if [ "${LONG}" = "8" ]; then
      MACHINE="${MACHINE}_64"
    fi
  fi

  # test for known architecture
  for arch in ${KNOWNARCH}
  do
    if [ "${MACHINE}" = "${arch}" ]; then
	KNOWN=1
    fi 
  done    

  if [ "${KNOWN}" = "0" ]; then
    echo "Your machinetype \"${MACHINE}\" has not yet been tested !!"
    echo "Please refer to the manual how to proceed"
    echo ""
    ENDIAN=`./endian |  ${P_GAWK:?}'($1 == "this") {print $5}'`

    if [ "${ENDIAN}" = "Intel" ]; then
       echo "You need byte swap !!! (-DBSWAP flag)"
    else
       echo "You do not need byte swap !!!"
    fi

    LONG=`./endian | ${P_GAWK:?}  '($1=="Sizes:") {print substr($5,3,1)}'`

    if [ "${LONG}" = "8" ]; then
       echo "Long is represented by 8 bytes (-DINT64 flag)"
    else
       echo "Long is represented by 4 bytes"
    fi
    exit 1
  else
    echo "Your machine has type \"${MACHINE}\"; compilation should"
    echo "work with no problems"
  fi
  #
  # running eclipse's system configuration:
  #
  if [ ! -f sysconf.c ]; then
    echo "need the source file 'sysconf.c'"
    exit 1
  else
    ${CC:?} -o sysconf sysconf.c	 
  fi
  ./sysconf

  cp ./config.h ${PD}/preprocess/eclipse/include
  mv ./config.h ${PD}/preprocess/qfits

  # test for the number of CPUs / cores
  if [ -x "${P_PYTHON}" ]; then
    NCPUS=`${P_PYTHON} -c 'import multiprocessing; \
                print multiprocessing.cpu_count()'`

    echo "Your machine has ${NCPUS} CPUs / cores."
  else
    echo "I cannot determine your number of CPUs / cores."
  fi

}

# read command line argument(s)

# 4 MODES available:
# - TEST: the the machine type
# - ALL: do script update and compile sources
# - UPDATE: do script update
# - COMPILE: compile sources

MODE="ALL"

while [ $# -gt 0 ]
do
   case $1 in
   -m)
       MODE=${2}
       shift
       shift
       ;;
    *)
       # there might be an 'empty string' argument which we
       # can ignore:
       if [ ! -z "$1" ]; then
         theli_error "Unknown command line option: ${1}"    
         exit 1;
       else
         shift  
       fi
       ;;
   esac
done

#
# test machine type
#
for mode in ${MODE}
do
  if [ "${mode}" = "TEST" ]; then
     testmachine
  fi    
done

#
# update scripts
#
for mode in ${MODE}
do
  if [ "${mode}" = "UPDATE" ] || [ "${mode}" = "ALL" ]; then

    # first test the machine type if necessary
    if [ "${MACHINE}" = "" ]; then
      testmachine
    fi

    # create machine dependent subdirectory in the scripts directory
    if [ ! -d ${PD}/scripts/${MACHINE} ]; then
      mkdir ${PD}/scripts/${MACHINE}
    fi

    # first the pipe_env scripts    
    sed -e 's!PIPESOFT=!PIPESOFT='${PD}'!' \
        -e 's!EXT=!EXT='${MACHINE}'!' \
        ${PD}/scripts/pipe_install/pipe_env.bash.raw \
        > ${PD}/scripts/pipe_install/pipe_env.bash.${MACHINE}
    
    sed -e 's! PIPESOFT! PIPESOFT '${PD}'!' \
        -e 's! EXT! EXT '${MACHINE}'!' \
        ${PD}/scripts/pipe_install/pipe_env.csh.raw \
        > ${PD}/scripts/pipe_install/pipe_env.csh.${MACHINE}
    
    # now the progs.ini file    
    sed -e 's!NPARA=1!NPARA='${NCPUS}'!' \
        -e 's!PIPESOFT=!PIPESOFT='${PD}'!' \
        -e 's!^PHOTSTANDARDSTARSCAT=!PHOTSTANDARDSTARSCAT='${PHOTSTANDARDSTARCAT}'!' \
        -e 's!^STANDARDSTARSCAT=!STANDARDSTARSCAT='${STANDARDSTARCAT}'!' \
        -e 's!BIN=${PIPESOFT}/bin/!BIN=${PIPESOFT}/bin/'${MACHINE}'/!' \
        -e 's!UTILSCRIPTS=${PIPESOFT}/scripts/utilities!UTILSCRIPTS=${PIPESOFT}/scripts/'${MACHINE}'/!' \
        -e 's!IRAFDIR\[1\]=!IRAFDIR\[1\]='${IRAFDIR}'!' \
        -e 's!P_PYTHON=!P_PYTHON='${P_PYTHON}'!' \
        -e 's!P_PERL=!P_PERL='${P_PERL}'!' \
        -e 's!P_GAWK=!P_GAWK='${P_GAWK}'!' \
        -e 's!P_FIND=!P_FIND='${P_FIND}'!' \
        -e 's!P_SORT=!P_SORT='${P_SORT}'!' \
        -e 's!P_CL=!P_CL='${P_CL}'!' \
        -e 's!P_SM=!P_SM='${P_SM}'!' \
        -e 's!P_SCAMP=!P_SCAMP='${P_SCAMP}'!' \
        -e 's!P_ACLIENT=!P_ACLIENT='${P_ACLIENT}'!' \
        -e 's!S_ASTROMETRIX=!S_ASTROMETRIX='${PD}'/scripts/astrometrix/astrom_theli!' \
        -e 's!S_PHOTOMETRIX=!S_PHOTOMETRIX='${PD}'/scripts/astrometrix/photom_theli!' \
        ${PD}/scripts/reduction/progs.ini.raw \
        > ${PD}/scripts/${MACHINE}/progs.ini
    
    # include pipeline version for the start logging message in
    # bash_functions.include:
    sed -e 's!__PIPEVERS__!'"${PIPEVERS}"'!' \
       ${PD}/scripts/reduction/bash_functions.include.raw > \
       ${PD}/scripts/${MACHINE}/bash_functions.include

    # now update all the scripts    
    SCRIPTDIRS="${PD}/scripts/reduction 
                ${PD}/scripts/CFH12K_reduction
                ${PD}/scripts/LBT_reduction
                ${PD}/scripts/Megacam_reduction
                ${PD}/scripts/MEGAPRIME_reduction
                ${PD}/scripts/WFI_reduction
                ${PD}/scripts/OMEGACAM_reduction
                ${PD}/scripts/DECam_reduction
    	        ${PD}/scripts/parallel 
                ${PD}/scripts/utilities
                ${PD}/scripts/science"
    
    for SCRIPTDIR in ${SCRIPTDIRS}
    do
      updatescripts ${SCRIPTDIR}

      # just copy possible Perl and Python scripts (they are called
      # via the env command and hence we do not need to hardcode the
      # location of the Python executable):
      PERLPYFILES=`find ${SCRIPTDIR} \( -name \*.pl -o -name \*.py \)`

      for perlpyfile in ${PERLPYFILES}
      do
	BASE=`basename ${perlpyfile}`
	chmod a+x ${perlpyfile}
        cp ${perlpyfile} ${PD}/scripts/${MACHINE}
      done

      # copy include files
      INCLUDE=`find ${SCRIPTDIR} -name \*.include`

      for include in ${INCLUDE}
      do
        cp ${include} ${PD}/scripts/${MACHINE}
      done
    done

    # copy files from the instrument specific directories to the
    # machine directories
    cp ${PD}/scripts/WFI_reduction/WFI* ${PD}/scripts/${MACHINE}
    cp ${PD}/scripts/WFI_reduction/conv_WFI* ${PD}/scripts/${MACHINE}
    cp ${PD}/scripts/CFH12K_reduction/CFH12K* ${PD}/scripts/${MACHINE}
    cp ${PD}/scripts/CFH12K_reduction/cfh12k* ${PD}/scripts/${MACHINE}
    cp ${PD}/scripts/Megacam_reduction/MMT* ${PD}/scripts/${MACHINE}
    cp ${PD}/scripts/MEGAPRIME_reduction/*ini ${PD}/scripts/${MACHINE}
    cp ${PD}/scripts/LBT_reduction/*ini ${PD}/scripts/${MACHINE}
    cp ${PD}/scripts/OMEGACAM_reduction/*ini ${PD}/scripts/${MACHINE}
    cp ${PD}/scripts/DECam_reduction/*ini ${PD}/scripts/${MACHINE}
    
    cp ${PD}/conf/WFI_reduction/WFI* ${PD}/scripts/${MACHINE}
    cp ${PD}/conf/LBT_reduction/LBT* ${PD}/scripts/${MACHINE}
    cp ${PD}/conf/MEGAPRIME_reduction/*MEGAPRIME* \
       ${PD}/conf/MEGAPRIME_reduction/megacam.ret ${PD}/scripts/${MACHINE}
    cp ${PD}/conf/MEGAPRIME_science/* ${PD}/scripts/${MACHINE}
    cp ${PD}/conf/OMEGACAM_reduction/*OMEGACAM* ${PD}/scripts/${MACHINE}
    cp ${PD}/conf/DECam_reduction/*DECam* ${PD}/scripts/${MACHINE}
    
    chmod a+x ${PD}/export/test_dir
    chmod a+x ${PD}/export/vraag

    # now deal with the perl scripts from ASTROMETRIX:
    sed -e "s!GaBoDSVersion=!GaBoDSVersion=\"${PIPEVERS}\";!" \
      ${PD}/scripts/astrometrix/src/photom_theli.raw >\
      ${PD}/scripts/astrometrix/photom_theli
    chmod +x ${PD}/scripts/astrometrix/photom_theli
    sed -e "s!GaBoDSVersion=!GaBoDSVersion=\"${PIPEVERS}\";!" \
      ${PD}/scripts/astrometrix/src/astrom_theli.raw >\
      ${PD}/scripts/astrometrix/astrom_theli
    chmod +x ${PD}/scripts/astrometrix/astrom_theli
    sed -e "s!GaBoDSVersion=!GaBoDSVersion=\"${PIPEVERS}\";!" \
      ${PD}/scripts/astrometrix/src/nldacsub_theli.pm.raw >\
      ${PD}/scripts/astrometrix/nldacsub_theli.pm
    cp ${PD}/scripts/astrometrix/src/AstSubs_theli.pm.raw \
       ${PD}/scripts/astrometrix/AstSubs_theli.pm
    cp ${PD}/scripts/astrometrix/src/AstConf_theli.pm.raw \
       ${PD}/scripts/astrometrix/AstConf_theli.pm
    cp ${PD}/scripts/astrometrix/src/catsubs_theli.pm.raw \
       ${PD}/scripts/astrometrix/catsubs_theli.pm
    cp ${PD}/scripts/astrometrix/src/PhotConf_theli.pm.raw \
       ${PD}/scripts/astrometrix/PhotConf_theli.pm
  fi
done

# compile sources
for mode in ${MODE}
do
  if [ "${mode}" = "COMPILE" ] || [ "${mode}" = "ALL" ]; then

    # do a test first whether the machine type is already known
    if [ "${MACHINE}" = "" ]; then
      testmachine
    fi

    # set environment variables necessary for compilation
    . ${PD}/scripts/pipe_install/pipe_env.bash.${MACHINE}

    # write the correct global_vers.h file:
    sed -e "s!#define __PIPEVERS__!#define __PIPEVERS__ \"${PIPEVERS}\"!" \
	  ${PD}/scripts/pipe_install/global_vers.h.raw >\
	  ${PD}/library/global_vers.h

    cd ${PD}/export/
    make all
    cd ${PD}/scripts/pipe_install
  fi
done
