#!/bin/bash -u

# the script creates a THELI archive out of our CVS ingested
# ldacpipeline structure.  It only contains a subset. Especially LDAC
# astrometry and photometry and related modules are NOT YET included.

# The subprogram structure is changed from the original one. All
# programs developed by Erik are now grouped in the ldactools
# subdirectory The script has to be called in the pipe_install
# subdirectory of the original ldacpipeline structure.

# main directory. Here the THELI directory and its structure are
# created.

# first obtain version number information:
DIR=`pwd`
VERS=`grep "PIPEVERS=" install.sh.extern |\
      awk '{len = length($4); print substr($4, 1, len - 1)}'`

# version numbers of programs from Emmanuel Bertin that we
# currently use in THELI:

# swarp:
SWARPVER=2.19.1

# now create name for the theli main directory:
cd ../../../
MD=`pwd`
cd ${DIR}
MD=${MD}/theli-${VERS}

# create theli main directory and copy files:
if [ ! -d ${MD} ]; then
  mkdir ${MD}
else
  rm -rf ${MD}/*
fi

# the LDAC tools:
mkdir ${MD}/ldactools
mkdir ${MD}/ldactools/library
mkdir ${MD}/ldactools/common
mkdir ${MD}/ldactools/tools
mkdir ${MD}/ldactools/associate
mkdir ${MD}/ldactools/make_ssc
mkdir ${MD}/ldactools/select_overlap

cp ../../READMES/CHANGES_THELI_ldactools.txt ${MD}/ldactools/CHANGES_THELI
cd ../../ldactools/common
cp *.c *.h Makefile \
   ${MD}/ldactools/common
cd ${DIR}

cd ../../ldactools/tools
cp *.c *.h \
   ${MD}/ldactools/tools
cp Makefile.extern ${MD}/ldactools/tools/Makefile
rm ${MD}/ldactools/tools/bileave*
rm ${MD}/ldactools/tools/ldacbutvec*
rm ${MD}/ldactools/tools/ldacshow*
rm ${MD}/ldactools/tools/ldacplot*
rm ${MD}/ldactools/tools/plot_glob.h
rm ${MD}/ldactools/tools/plplot.h
cd ${DIR}

cd ../../associate/new
cp *.c *.h Makefile \
   ${MD}/ldactools/associate
cd ${DIR}

cd ../../make_ssc/new
cp *.c *.h Makefile \
   ${MD}/ldactools/make_ssc
cd ${DIR}

cd ../../select_overlap
cp *.c *.h Makefile \
   ${MD}/ldactools/select_overlap
cd ${DIR}

cd ../../library
cp *.c *.h ${MD}/ldactools/library
cp Makefile.extern ${MD}/ldactools/library/Makefile
cp moments.c.extern ${MD}/ldactools/library/moments.c
cp moments_globals.h.extern ${MD}/ldactools/library/moments_globals.h

rm ${MD}/ldactools/library/sv*
rm ${MD}/ldactools/library/rel*
rm ${MD}/ldactools/library/gauss*
rm ${MD}/ldactools/library/fits*
rm ${MD}/ldactools/library/decomp*
rm ${MD}/ldactools/library/pos_*
rm ${MD}/ldactools/library/proj*
rm ${MD}/ldactools/library/prec*
rm ${MD}/ldactools/library/reject.c

cd ${DIR}


# the IMCAT utilities
mkdir ${MD}/imcattools
mkdir ${MD}/imcattools/tools

cd ../../imcatutils
cp *.c *.h Makefile \
   ${MD}/imcattools/tools
cp ../READMES/CHANGES_THELI_imcattools.txt ${MD}/imcattools/CHANGES_THELI
cd ${DIR}

# eclipse utilities
mkdir ${MD}/eclipsetools
mkdir ${MD}/eclipsetools/eclipse
mkdir ${MD}/eclipsetools/eclipse/include
mkdir ${MD}/eclipsetools/qfits
mkdir ${MD}/eclipsetools/tools

cp ../../READMES/CHANGES_THELI_eclipsetools.txt \
   ${MD}/eclipsetools/CHANGES_THELI
cd ../../preprocess/eclipse
cp *.c Makefile \
   ${MD}/eclipsetools/eclipse
cd include
cp *.h ${MD}/eclipsetools/eclipse/include
cd ${DIR}

cd ../../preprocess/qfits
cp *.c *.h Makefile \
   ${MD}/eclipsetools/qfits
cd ${DIR}

cd ../../preprocess/tools
cp *.c *.h \
   ${MD}/eclipsetools/tools
cp Makefile.extern ${MD}/eclipsetools/tools/Makefile
rm ${MD}/eclipsetools/tools/*linedetect*
cd ${DIR}

# TERAPIX utilities:
mkdir ${MD}/bertintools
mkdir ${MD}/bertintools/sextractor-2.5.0 
mkdir ${MD}/bertintools/sextractor-2.5.0/src 
mkdir ${MD}/bertintools/sextractor-2.5.0/src/wcs 
mkdir ${MD}/bertintools/sextractor-2.5.0/src/fits
mkdir ${MD}/bertintools/sextractor-2.5.0/config 
mkdir ${MD}/bertintools/swarp${SWARPVER}
mkdir ${MD}/bertintools/swarp${SWARPVER}/src
mkdir ${MD}/bertintools/swarp${SWARPVER}/src/fits
mkdir ${MD}/bertintools/swarp${SWARPVER}/src/wcs
mkdir ${MD}/bertintools/weightwatcher1.3
mkdir ${MD}/bertintools/weightwatcher1.3/config
mkdir ${MD}/bertintools/weightwatcher1.3/source

cd ../../sex/
cp Makefile.extern ${MD}/bertintools/Makefile
cp ../READMES/CHANGES_THELI_bertintools.txt  ${MD}/bertintools/CHANGES_THELI
cd ${DIR}

cd ../../sex/sextractor-2.5.0 
find . -maxdepth 1 -name \* -type f \
       -exec cp {} ${MD}/bertintools/sextractor-2.5.0 \;
cd ./config
find . -maxdepth 1 -name \* -type f \
       -exec cp {} ${MD}/bertintools/sextractor-2.5.0/config \;
cd ../src
find . -maxdepth 1 \( -name \*.c -o -name \*.h \) -type f \
       -exec cp {} ${MD}/bertintools/sextractor-2.5.0/src \;
cp Makefile.extern ${MD}/bertintools/sextractor-2.5.0/src/Makefile
cd ./wcs
find . -maxdepth 1 \( -name \*.c -o -name \*.h -o -name Makefile \) -type f \
       -exec cp {} ${MD}/bertintools/sextractor-2.5.0/src/wcs \;
cd ../fits
find . -maxdepth 1 \( -name \*.c -o -name \*.h -o -name Makefile \) -type f \
       -exec cp {} ${MD}/bertintools/sextractor-2.5.0/src/fits \;
cp Makefile.extern ${MD}/bertintools/sextractor-2.5.0/src/fits/Makefile
cd ${DIR}

cd ../../sex/swarp${SWARPVER} 
find . -maxdepth 1 -name \* -type f \
       -exec cp {} ${MD}/bertintools/swarp${SWARPVER} \;
cd ./src
find . -maxdepth 1 \( -name \*.c -o -name \*.h \) -type f \
       -exec cp {} ${MD}/bertintools/swarp${SWARPVER}/src \;
cp Makefile.extern ${MD}/bertintools/swarp${SWARPVER}/src/Makefile 
cd ./fits
find . -maxdepth 1 \( -name \*.c -o -name \*.h -o -name Makefile \) -type f \
       -exec cp {} ${MD}/bertintools/swarp${SWARPVER}/src/fits \;
cd ../wcs
find . -maxdepth 1 \( -name \*.c -o -name \*.h -o -name Makefile \) -type f \
       -exec cp {} ${MD}/bertintools/swarp${SWARPVER}/src/wcs \;
cd ${DIR}

cd ../../sex/weightwatcher1.2 
find . -maxdepth 1 -name \* -type f \
       -exec cp {} ${MD}/bertintools/weightwatcher1.3 \;
cd ./config
find . -maxdepth 1 -name \* -type f \
       -exec cp {} ${MD}/bertintools/weightwatcher1.3/config \;
cd ../source
find . -maxdepth 1 \( -name \*.c -o -name \*.h \) -type f \
       -exec cp {} ${MD}/bertintools/weightwatcher1.3/source \;
cp Makefile.extern ${MD}/bertintools/weightwatcher1.3/source/Makefile
cd ${DIR}

# lensing/PSF analysis utilities
mkdir ${MD}/psftools
cd ../../lensing/psfcorrection
cp preanisotropy.c ${MD}/psftools
cp anisotropy.c ${MD}/psftools
cp Makefile.extern ${MD}/psftools/Makefile
cd ../imcat
cp analyseldac* ${MD}/psftools
cp error* ${MD}/psftools
cp fits* ${MD}/psftools
cp nrutil* ${MD}/psftools
cp interpol* ${MD}/psftools
cp stats_stuff* ${MD}/psftools
cp arrays* ${MD}/psftools
cp object_stuff.h ${MD}/psftools
cp magic.h ${MD}/psftools
cp status.h ${MD}/psftools
cd ${DIR}

# lib directory:
SYSTEMS="Darwin_PPC Darwin_INTEL Darwin_INTEL_64 Linux Linux_64 HP-UX IRIX64 OSF1 SunOS"
mkdir ${MD}/lib

cd ../../lib

cp Makefile.extern ${MD}/lib/Makefile

for system in ${SYSTEMS}
do
  mkdir ${MD}/lib/${system}
  cp ./${system}/Macrodefs* ${MD}/lib/${system}
  awk '((substr($1, 1, 9) != "FLIPS_LIB") && (substr($1, 1, 7) != "CPP_LIB")) {
   print $0}' ./${system}/Makefile > ${MD}/lib/${system}/Makefile.tmp
  sed -e 's/-I$(CPP_LIB_INC)//' -e 's!/preprocess/!/eclipsetools/!' \
      -e 's!$(PIPESOFT)/library!$(PIPESOFT)/ldactools/library!' \
      ${MD}/lib/${system}/Makefile.tmp > ${MD}/lib/${system}/Makefile
  rm ${MD}/lib/${system}/Makefile.tmp
done
cd ${DIR}

# conf directories:
mkdir ${MD}/conf
mkdir ${MD}/conf/reduction
mkdir ${MD}/conf/WFI_reduction
mkdir ${MD}/conf/MEGAPRIME_reduction
mkdir ${MD}/conf/OMEGACAM_reduction
mkdir ${MD}/conf/others

cd ../../conf

# conf/reduction directory
find reduction -maxdepth 1 -name \* -type f \
       -exec cp {} ${MD}/conf/reduction \;
rm ${MD}/conf/reduction/fullphotom*
rm ${MD}/conf/reduction/fullastrom*

# conf/others directory
find others -maxdepth 1 -name \*astrometrix\* -type f \
       -exec cp {} ${MD}/conf/others \;
cp ./others/gauss_4.0_7x7.conv ${MD}/conf/others 

# instrument specific conf directories:
find WFI_reduction -maxdepth 1 -name \* -type f |\
  xargs cp --target-directory=${MD}/conf/WFI_reduction
find MEGAPRIME_reduction -maxdepth 1 -name \* -type f |\
  xargs cp --target-directory=${MD}/conf/MEGAPRIME_reduction
find OMEGACAM_reduction -maxdepth 1 -name \* -type f |\
  xargs cp --target-directory=${MD}/conf/OMEGACAM_reduction

cd ${DIR}

# utilities directory:
mkdir ${MD}/smalltools

cd ../../utilities
cp *.c Makefile ${MD}/smalltools
cp Makefile.extern ${MD}/smalltools/Makefile
rm ${MD}/smalltools/calib_photoz_data.c
cd ${DIR}

# compile directory
mkdir ${MD}/pipesetup

cp ../../export/Makefile.extern ${MD}/pipesetup/Makefile
cp ../../export/vraag ${MD}/pipesetup/
cp ../../export/test_dir ${MD}/pipesetup/
cp ./endian.c ./sysconf.c *.raw ${MD}/pipesetup
cp ./install.sh.extern ${MD}/pipesetup/install.sh
cp ./check_python_modules.sh ${MD}/pipesetup/
chmod +x ${MD}/pipesetup/install.sh

# scripts directory
mkdir ${MD}/scripts
mkdir ${MD}/scripts/astrometrix
mkdir ${MD}/scripts/astrometrix/src
mkdir ${MD}/scripts/WFI_reduction
mkdir ${MD}/scripts/MEGAPRIME_reduction
mkdir ${MD}/scripts/CFH12K_reduction
mkdir ${MD}/scripts/OMEGACAM_reduction
mkdir ${MD}/scripts/parallel
mkdir ${MD}/scripts/reduction
mkdir ${MD}/scripts/testdata
mkdir ${MD}/scripts/others
mkdir ${MD}/scripts/utilities
mkdir ${MD}/scripts/sm_macros

WFI_FILES="process_split_WFI_eclipse.sh.raw conv_WFI_filter_names.include WFI*"
MEGAPRIME_FILES="process_split_MEGAPRIME_eclipse.sh.raw MEGAPRIME*"
OMEGACAM_FILES="process_split_OMEGACAM_eclipse.sh.raw
                create_OMEGACAM_ct_weights.sh.raw OMEGACAM*"
RED_FILES="process_bias_eclipse_para.sh.raw process_flat_eclipse_para.sh.raw
           process_science_eclipse_para.sh.raw create_illumfringe_para.sh.raw
           process_science_illum_fringe_eclipse_para.sh.raw 
           process_science_illum_eclipse_para.sh.raw
           process_science_fringe_eclipse_para.sh.raw create_norm_para.sh.raw
           process_standard_eclipse_para.sh.raw cleanfiles.sh.raw
           create_absphot_header.sh.raw create_absphotom_photometrix.sh.raw
           create_binnedmosaics.sh.raw create_binnedmosaics_exposurepara.sh.raw
           create_weights_para.sh.raw
           distribute_sets.sh.raw create_astromcats_weights_para.sh.raw
           create_astrometrix_astrom.sh.raw create_astrometrix_astrom_run.sh.raw
           create_photometrix.sh.raw create_stats_table.sh.raw 
           make_checkplot_stats.sh.raw
           create_skysub_para.sh.raw prepare_coadd_swarp.sh.raw 
           resample_coadd_swarp_para.sh.raw
           perform_coadd_swarp.sh.raw create_modesub_para.sh.raw 
           transform_ds9_reg.sh.raw
           resolvelinks.sh.raw select_best.sh.raw check_files.sh.raw 
           bash_functions.include.raw
           create_runid_para.sh.raw create_run_list.sh.raw 
           create_weights_flags_para.sh.raw
           create_global_weights_flags_para.sh.raw update_coadd_header.sh.raw 
	   create_bitpixchange.sh.raw create_scamp_astrom_photom.sh.raw
           merge_scamp_cats.sh.raw create_abs_photo_info.sh.raw 
           create_stdphotom_prepare_para.sh.raw check_science_PSF_para.sh.raw
           create_stdphotom_merge_exposurepara.sh.raw
           check_science_PSF_plot.sh.raw merge_sex_ksb.sh.raw
           fix_stars_weights_para.sh.raw create_zero_flag_para.sh.raw
           create_catalogue.sh.raw create_mag_distribution_korr.sh.raw
           create_checkplot_rh_mag.sh.raw process_cut_eclipse_para.sh.raw
           measure_crosstalk.sh.raw apply_crosstalk_exposurepara.sh.raw
           create_crosstalk_coefficients_exposurepara.sh.raw"

# scripts/astrometrix dir.
find ../../scripts/astrometrix/src -maxdepth 1 -type f -name \*raw \
    -exec cp {} ${MD}/scripts/astrometrix/src \;
cp ../../READMES/CHANGES_THELI_astrometrix.txt \
   ${MD}/scripts/astrometrix/CHANGES_THELI

# scripts/WFI_reduction dir.
cd ../../scripts/WFI_reduction

for file in ${WFI_FILES}
do
  cp ${file} ${MD}/scripts/WFI_reduction 
done

# scripts/MEGAPRIME_reduction dir.
cd ../../scripts/MEGAPRIME_reduction

for file in ${MEGAPRIME_FILES}
do
  cp ${file} ${MD}/scripts/MEGAPRIME_reduction 
done

# scripts/OMEGACAM_reduction dir.
cd ../../scripts/OMEGACAM_reduction

for file in ${OMEGACAM_FILES}
do
  cp ${file} ${MD}/scripts/OMEGACAM_reduction 
done

cd ${DIR}

#scripts/CFH12K_reduction dir.
find ../../scripts/CFH12K_reduction -maxdepth 1 -type f \
    -exec cp {} ${MD}/scripts/CFH12K_reduction \;
rm ${MD}/scripts/CFH12K_reduction/check_science_PSF_plot_CFH12K.sh.raw

find ../../scripts/parallel -maxdepth 1 -type f \
    -exec cp {} ${MD}/scripts/parallel \;

# scripts/reductuin dir.
cd ../../scripts/reduction

for file in ${RED_FILES}
do
  cp ${file} ${MD}/scripts/reduction 
done

cp progs.ini.raw.extern ${MD}/scripts/reduction/progs.ini.raw
cd ${DIR}

# scripts/testdata dir.
cp ../../scripts/testdata/do_WFItestdata.sh ${MD}/scripts/testdata
cp ../../scripts/testdata/night*result.asc ${MD}/scripts/testdata
cp ../../scripts/testdata/WFI*.reg ${MD}/scripts/testdata

# scripts/others dir.
cp ../../scripts/others/do_astrometrix_cat.sh ${MD}/scripts/others

# scripts/utilities dir.
cp ../../scripts/utilities/scampcat.py ${MD}/scripts/utilities
cp ../../scripts/utilities/aplastrom_scamp.py ${MD}/scripts/utilities
cp ../../scripts/utilities/nmpfit.py ${MD}/scripts/utilities
cp ../../scripts/utilities/photo_abs.py ${MD}/scripts/utilities
cp ../../scripts/utilities/stellar_locus.pl ${MD}/scripts/utilities
cp ../../scripts/utilities/superflatexclusion.py ${MD}/scripts/utilities

# scripts/sm_macros dir.
cp ../../scripts/sm_macros/shearfield.sm ${MD}/scripts/sm_macros
cp ../../scripts/sm_macros/mag_distribution_korr.sm ${MD}/scripts/sm_macros
cp ../../scripts/sm_macros/magrh.sm ${MD}/scripts/sm_macros

# README files
# get the correct version number in the main README file:
sed -e 's/1.0.0/'${VERS}'/' ../../READMES/README.txt > ${MD}/README
cp ../../READMES/AUTHORS_theli.txt ${MD}/AUTHORS
cp ../../READMES/LICENSE_readmefirst.txt ${MD}/LICENSE_readmefirst
cp ../../READMES/LICENSE_eclipsetools.txt ${MD}/LICENSE_eclipsetools
cp ../../READMES/LICENSE_not_eclipsetools.txt ${MD}/LICENSE_not_eclipsetools

# create archive file
cd ${MD}/..
tar cvfz theli-${VERS}.tgz ./theli-${VERS}
