#!/bin/bash -xv

# utility script to create a ds9 region file for image overlay from
# an onject catalogue in LDAC format.

#$1: catalogue
#$2: output region file

INCAT=$1
OUTCAT=$2

# names of the keys:
XPOS=Xpos        # x pixel coordinate
YPOS=Ypos        # y pixel coordinate
A=A              # semi major axis in pix.
B=B              # semi minor axis in pix.
THETA=Theta      # position angle in degrees
MAG=MAG_AUTO     # magnitude estimate

echo ${!XPOS}

ldactoasc -i ${INCAT} -t OBJECTS \
   -k ${XPOS} ${YPOS} ${A} ${B} ${THETA} ${MAG} |\
   awk '($1 !~ /#/ ) { 
       printf("ellipse(%d,%d,%d,%d,%.2f) # text = { %.2f }\n", 
              int($1), int($2), 
              int($3 * 4.0), int($4 * 4.0), # the "4" scales the
                                            # ellipse for clarity
              $5, $6)
     }' > ${OUTCAT}

