#!/usr/bin/env python

# script to obtain modified julian dates for given civil dates.
# The civil dates have to be be provided in a file where each line
# contains a string like:
#
#  2011-08-03T13:22:00.734
#
# It represents the civil time in UT!
#
# The script complements the C-program mjd. I wrote it to quickly
# obtain the mjds for a bunch of civil dates in ISO format with just a
# few astropy calls!

import sys
import string
import astropy.time as at
import numpy as np

if len(sys.argv) != 3:
    print "Synopsis: sys.argv[0] file_with_civil_dates output_file"
    sys.exit(1)

input_file = sys.argv[1]
output_file = sys.argv[2]

try:
    dates = np.loadtxt(input_file, dtype='str')    
    astrotime = at.Time(dates, format='isot', scale='utc')
    mjd = astrotime.mjd
    np.savetxt(output_file, mjd, fmt="%.2f")
except IOError:
    print "Could not open file %s!" % (sys,argv[1])
    sys.exit(1)

sys.exit(0)

