 /*
 				define.h

*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*
*	Part of:	EYE
*
*	Author:		E.BERTIN, IAP & Leiden observatory
*
*	Contents:	global definitions.
*
*	Last modify:	10/05/97
*
*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

/*------------------------ what, who, when and where ------------------------*/

#define		BANNER		"E.Y.E."
#define		VERSION		"1.1 (May 11 1997)"
#define		COPYRIGHT	"Emmanuel BERTIN (bertin@iap.fr)"
#define		INSTITUTE	"IAP/Leiden"

/*----------------------------- Internal constants --------------------------*/

#define	OUTPUT			stderr		/* where all msgs are sent */
#define	MAXCHAR			256		/* max. number of characters */
#define	DATA_BUFSIZE		262144		/* data buffer size */
#define	BIG			1e+30		/* a huge number */
#define	MAX_LEARNRATE		50.0		/* default max. learn rate */

#ifndef PI
#define	PI			3.14159265359	/* never met before? */
#endif

/* NOTES:

One must have:	DATA_BUFSIZE >= 2880 with DATA_BUFSIZE%8 = 0
		MAXCHAR >= 16
*/

/*----- Set defines according to machine's specificities and customizing ----*/

#ifdef DEC_ALPHA
#define	BSWAP
#define	INT64
#endif

#ifdef HP_UX
#define	_HPUX_SOURCE
#endif

#ifdef PC_LINUX
#define	BSWAP
#endif

#ifdef SUN_ULTRASPARC
#define	INT64
#endif

#ifdef MEMORY_LINK
#define MEMORY_READ
#endif

/*--------------------- in case of missing constants ------------------------*/

#ifndef		SEEK_SET
#define		SEEK_SET	0
#endif
#ifndef		SEEK_CUR
#define		SEEK_CUR	1
#endif

#ifndef	EXIT_SUCCESS
#define	EXIT_SUCCESS		0
#endif
#ifndef	EXIT_FAILURE
#define	EXIT_FAILURE		-1
#endif

/*---------------------------- return messages ------------------------------*/

#define		RETURN_OK		0
#define		RETURN_ERROR		(-1)
#define		RETURN_FATAL_ERROR	(-2)

/*--------------------- in case of missing constants ------------------------*/

#ifndef RAND_MAX
#define	RAND_MAX	0x7fffffffL	/* default dynamic range of rand() */
#endif

#ifndef         SEEK_SET
#define         SEEK_SET        0
#endif
#ifndef         SEEK_CUR
#define         SEEK_CUR        1
#endif

#ifndef EXIT_SUCCESS
#define 	EXIT_SUCCESS	0
#endif
#ifndef EXIT_FAILURE
#define		EXIT_FAILURE	-1
#endif

/*------------------- a few definitions to read FITS parameters ------------*/

#define	FITSTOF(k, def)	((point = fitsnfind(buf, k, n))? \
					 atof(strncpy(st, &point[10], 70)) \
					:(def))
#define	FITSTOI(k, def)	((point = fitsnfind(buf, k, n))? \
					 atoi(strncpy(st, &point[10], 70)) \
					:(def))

#define	FITSTOS(k, str, def)	{ if (point = fitsnfind(buf, k, n)) \
					{ \
					for (i=0,point+=11; (*point)!='\'';) \
						str[i++] = *(point++); \
					str[i] = '\0'; \
					} \
				  else\
					strcpy(str, def); \
				}

/*------------------------------- Other Macros -----------------------------*/

#define QCALLOC(ptr, typ, nel) \
                if (!(ptr = (typ *)calloc((size_t)(nel),sizeof(typ)))) \
                  error(EXIT_FAILURE, "Not enough memory for ", \
                        #ptr " (" #nel " elements) !")

#define QMALLOC(ptr, typ, nel) \
                if (!(ptr = (typ *)malloc((size_t)(nel)*sizeof(typ)))) \
                  error(EXIT_FAILURE, "Not enough memory for ", \
                        #ptr " (" #nel " elements) !")

#define	QREALLOC(ptr, typ, nel) \
		{if (!(ptr = (typ *)realloc(ptr, (size_t)(nel)*sizeof(typ)))) \
		   error(EXIT_FAILURE, "Not enough memory for ", \
			#ptr " (" #nel " elements) !");;}

#define QFREAD(ptr, size, file, fname) \
                if (fread(ptr, (size_t)(size), (size_t)1, file)!=1) \
                  error(EXIT_FAILURE, "*Error* while reading ", fname)

#define QFWRITE(ptr, size, file, fname) \
                if (fwrite(ptr, (size_t)(size), (size_t)1, file)!=1) \
                  error(EXIT_FAILURE, "*Error* while writing ", fname)

#define QFSEEK(file, offset, pos, fname) \
                if (fseek(file, (offset), pos)) \
                  error(EXIT_FAILURE,"*Error*: file positioning failed in ", \
                        fname)

#define QFTELL(pos, file, fname) \
                if ((pos=ftell(file))==-1) \
                  error(EXIT_FAILURE,"*Error*: file position unknown in ", \
                        fname)

#define PIX(x, y)	field->strip[(((int)y)%field->stripheight) \
				*field->width +(int)x]

#define	NPRINTF		if (prefs.verbose_type == NORM) fprintf

#define	NFPRINTF(w,x)	{if (prefs.verbose_type == NORM) \
				fprintf(w, "\33[1M> %s\n\33[1A",x); \
			else if (prefs.verbose_type == FULL) \
				fprintf(w, "%s.\n", x);}

#define	FPRINTF		if (prefs.verbose_type == FULL)	fprintf

#define	RINT(x)		(int)(floor(x+0.5))

