 /*
 				field.c

*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*
*	Part of:	EYE
*
*	Author:		E.BERTIN, IAP & Leiden observatory
*
*	Contents:	Handling of field structures.
*
*	Last modify:	10/05/97
*
*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

#include	"define.h"
#include	"globals.h"
#include	"fitscat.h"
#include	"back.h"

/********************************* newfield **********************************/
/*
Returns a pointer to a new field, ready to go!
*/
picstruct	*newfield(char *filename)

  {
   picstruct	*field;

/* First allocate memory for the new field (and nullify pointers) */
  QCALLOC(field, picstruct, 1);
  strcpy (field->filename, filename);
/* A short, "relative" version of the filename */
  if (!(field->rfilename=strrchr(field->filename, '/')))
    field->rfilename = field->filename;

  sprintf(gstr, "Looking for %s", field->rfilename);
  NFPRINTF(OUTPUT, gstr);
/* Check the image exists and read important info (image size, etc...) */
  readimagehead(field);
  if (prefs.verbose_type != QUIET)
    fprintf(OUTPUT, "Frame: \"%.20s\" / %d x %d / %d bits %s data\n",
	field->ident,
	field->width, field->height, field->bytepix*8,
	field->bitpix>0?
		(field->compress_type!=COMPRESS_NONE?"COMPRESSED":"INTEGER")
		:"FLOATING POINT");

/* Provide a buffer for compressed data */
  if (field->compress_type != COMPRESS_NONE)
    QMALLOC(field->compress_buf, char, FBSIZE);

/* Important initializations according to the prefs */

/* Background */
  field->backw = prefs.backsize[0]<field->width ? prefs.backsize[0]
						  : field->width;
  field->backh = prefs.backsize[1]<field->height ? prefs.backsize[1]
						   : field->height;
  field->nbackp = field->backw * field->backh;
  if ((field->nbackx = (field->width-1)/field->backw + 1) < 1)
    field->nbackx = 1;
  if ((field->nbacky = (field->height-1)/field->backh + 1) < 1)
    field->nbacky = 1;
  field->nback = field->nbackx * field->nbacky;
  field->nbackfx = field->nbackx>1 ? prefs.backfsize[0] : 1;
  field->nbackfy = field->nbacky>1 ? prefs.backfsize[1] : 1;
/* Now make the background map */
  makeback(field);

/* Prepare the image buffer */
  field->stripheight = prefs.retina_size[1];
  field->stripmargin = (prefs.retina_size[1]-1)/2;

  if (!(field->strip=(PIXTYPE *)malloc(field->stripheight*field->width
	*sizeof(PIXTYPE))))
    error(EXIT_FAILURE,"Not enough memory for the image buffer in ",
	field->rfilename);

  if (prefs.verbose_type != QUIET)
    fprintf(OUTPUT, "Background: %-10g RMS: %-10g\n",
	field->backmean, field->backsig);

  return field;
  }


/********************************* endfield **********************************/
/*
Free and close everything related to a field structure.
*/
void	endfield(picstruct *field)

  {
#ifndef MEMORY_READ
  fclose(field->file);
#endif /* ifndef MEMORY_READ */

  free(field->fitshead);
  free(field->strip);
  free(field->compress_buf);
  endback(field);
  free(field);

  return;
  }

