 /*
 				main.c

*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*
*	Part of:	EYE
*
*	Author:		E.BERTIN, IAP & Leiden observatory
*
*	Contents:	parsing and main loop.
*
*	Last modify:	11/05/97
*
*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

#include	"define.h"
#include	"globals.h"

#define		SYNTAX \
"eye -i Input_frame(s) -o Output_frame(s) \n\
           [-c <configuration_file>] [-<keyword> <value>]\n\n"

/********************************** main ************************************/

main(int argc, char *argv[])
  {
   int		a,abi,aei,nai, abo,aeo,nao, narg;
   char		**argkey, **argval;

  if (argc<2)
    {
    fprintf(OUTPUT, "\n		%s  Version %s\n", BANNER, VERSION);
    fprintf(OUTPUT, "\nFor information, please contact: %s\n", COPYRIGHT);
    error(EXIT_SUCCESS, "SYNTAX: ", SYNTAX);
    }

  QMALLOC(argkey, char *, argc);
  QMALLOC(argval, char *, argc);

/* default parameters */
  abi=aei=abo=aeo=nai=nao=0;
  narg = 0;
  strcpy(prefs.prefs_name, "default.eye");
  prefs.verbose_type = NORM;

  for (a=1; a<argc; a++)
    {
    if (argv[a][0] == '-' && a<(argc-1))
      {
      if (strlen(argv[a])<3)
        switch((int)tolower((int)argv[a][1]))
          {
          case 'c':
            strcpy(prefs.prefs_name, argv[++a]);
            break;
          case 'i':
            for(abi = ++a; (a<argc) && (*argv[a]!='-'); a++);
            aei = a--;
            nai = aei - abi;
            break;
          case 'o':
            for(abo = ++a; (a<argc) && (*argv[a]!='-'); a++);
            aeo = a--;
            nao = aeo - abo;
            break;
          default:
            error(EXIT_SUCCESS,"SYNTAX: ", SYNTAX);
          }
      else
        {
        argkey[narg] = &argv[a][1];
        argval[narg++] = argv[++a];
        }       
      }
    else
      error(EXIT_SUCCESS,"SYNTAX: ", SYNTAX);
    }

  NFPRINTF(OUTPUT, "Reading configuration...");
  readprefs(prefs.prefs_name, argkey, argval, narg);
  useprefs();
  free(argkey);
  free(argval);

  if (nao!=nai)
    error(EXIT_FAILURE, "*Error*: different number of images and catalogs",
	"");

  makeit(argv+abi, argv+abo, nai);

  NFPRINTF(OUTPUT, "All done");
  NPRINTF(OUTPUT, "\n");

  exit(EXIT_SUCCESS);
  }
