/*
 				readimage.c

*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*
*	Part of:	EYE
*
*	Author:		E.BERTIN, IAP & Leiden observatory
*
*	Contents:	functions for input of image data.
*
*	Last modify:	11/05/97
*
*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

#include	"define.h"
#include	"globals.h"
#include	"fitscat.h"
#include	"back.h"

/******************************* loadstrip ***********************************/
/*
Load a new strip of pixel data into the buffer.
*/
PIXTYPE	*loadstrip(picstruct *field)

  {
   int		y, w;
   PIXTYPE	*data;

  w = field->width;

  if (!field->y)
    {
/*- First strip */
     int	nbpix;

    nbpix = w*field->stripheight;
#  ifdef MEMORY_READ
    memcpy(field->strip, field->memlink_ima, nbpix*sizeof(PIXTYPE));
    field->memlink_imaoffset = nbpix;
#  else
    readdata(field, field->strip, nbpix);
#  endif /* ifdef MEMORY_READ */
    data = field->strip;
    for (y=0; y<field->stripheight; y++, data += w)
      {
/*
      if (prefs.check_type==CHECK_IDENTICAL)
        writecheck(data, w);
*/
/*---- This is the only place where one can pick-up safely the current bkg */
      subbackline(field, y, data);
/*---- Check-image stuff */
/*
      if (prefs.check_type!=CHECK_NONE && (field->flags&MEASURE_FIELD))
        switch (prefs.check_type) 
          {
          case CHECK_BACKGROUND:
            writecheck(field->backline, w);
            break;
          case CHECK_SUBTRACTED:
          case CHECK_APERTURES:
            writecheck(data, w);
            break;
          default:
            break;
          }
*/
      }

    field->ymax = field->stripheight;
    if (field->ymax < field->height)
      field->stripysclim = field->stripheight - field->stripmargin;
    }
  else
    {
/*- other strips */
    data = field->strip + field->stripylim*w;
/*---- copy to Check-image the "oldest" line before it is replaced */
/*
    if (prefs.check_type == CHECK_SUBOBJECTS)
       writecheck(data, w);
*/
#  ifdef MEMORY_READ
    memcpy(data,field->memlink_ima+field->memlink_imaoffset,w*sizeof(PIXTYPE));
    field->memlink_imaoffset += w;
#  else
    readdata(field, data, w);
#  endif /* ifdef MEMORY_READ */
/*
    if (prefs.check_type==CHECK_IDENTICAL)
      writecheck(data, w);
*/
/*-- Interpolate and subtract the background at current line */
    subbackline(field, field->ymax, data);
/*---- Check-image stuff */
/*
    if (prefs.check_type!=CHECK_NONE && (field->flags&MEASURE_FIELD))
      switch (prefs.check_type) 
        {
        case CHECK_BACKGROUND:
          writecheck(field->backline, w);
          break;
        case CHECK_SUBTRACTED:
        case CHECK_APERTURES:
          writecheck(data, w);
          break;
        default:
          break;
        }
*/
    field->stripylim = (++field->ymin)%field->stripheight;
    if ((++field->ymax)<field->height)
      field->stripysclim = (++field->stripysclim)%field->stripheight;
    }

  return field->strip + field->stripy*w;
  }


/******************************** readdata **********************************/
/*
read and convert input data stream in PIXTYPE (float) format.
*/
void	readdata(picstruct *field, PIXTYPE *ptr, int size)
  {
  static char	bufdata0[DATA_BUFSIZE];
  char		*bufdata;
  short		val16;
  int		i, bowl, spoonful, npix, curval, dval;
  PIXTYPE	bs,bz;

  bs = (PIXTYPE)field->bscale;
  bz = (PIXTYPE)field->bzero;
  switch(field->compress_type)
    {
/*-- Uncompressed image */
    case COMPRESS_NONE:
      bowl = DATA_BUFSIZE/field->bytepix;
      spoonful = size<bowl?size:bowl;
      for(; size>0; size -= spoonful)
        {
        if (spoonful>size)
          spoonful = size;
        bufdata = bufdata0;
        QFREAD(bufdata, spoonful*field->bytepix, field->file, field->filename);
        switch(field->bitpix)
          {
          case BP_BYTE:
            if (field->bitsgn)
              for (i=spoonful; i--;)
                *(ptr++) = *(bufdata++)*bs + bz;
            else
              for (i=spoonful; i--;)
                *(ptr++) = *((unsigned char *)bufdata++)*bs + bz;
            break;

          case BP_SHORT:
#          ifdef BSWAP
            swapbytes(bufdata, 2, spoonful);
#          endif
            if (field->bitsgn)
              for (i=spoonful; i--; bufdata += sizeof(short))
                *(ptr++) = *((short *)bufdata)*bs + bz;
            else
              for (i=spoonful; i--; bufdata += sizeof(USHORT))
                *(ptr++) = *((USHORT *)bufdata)*bs + bz;
            break;

          case BP_LONG:
#          ifdef BSWAP
            swapbytes(bufdata, 4, spoonful);
#          endif
            if (field->bitsgn)
              for (i=spoonful; i--; bufdata += sizeof(long))
                *(ptr++) = *((long *)bufdata)*bs + bz;
            else
              for (i=spoonful; i--; bufdata += sizeof(unsigned long))
                *(ptr++) = *((unsigned long *)bufdata)*bs + bz;
              break;

          case BP_FLOAT:
#          ifdef BSWAP
            swapbytes(bufdata, 4, spoonful);
#          endif
            for (i=spoonful; i--; bufdata += sizeof(float))
              *(ptr++) = *((float *)bufdata)*bs + bz;
            break;

          case BP_DOUBLE:
#          ifdef BSWAP
            swapbytes(bufdata, 8, spoonful);
#          endif
            for (i=spoonful; i--; bufdata += sizeof(double))
              *(ptr++) = *((double *)bufdata)*bs + bz;
            break;

          default:
            error(EXIT_FAILURE,"*FATAL ERROR*: unknown BITPIX type in ",
				"readdata()");
            break;
          }
        }
      break;

/*-- Compressed image */
    case COMPRESS_BASEBYTE:
      bufdata = field->compress_bufptr;
      curval = field->compress_curval;
      npix = field->compress_npix;
      while (size--)
        {
        if (!(npix--))
          {
          if (curval != field->compress_checkval)
            error(EXIT_FAILURE, "*Error*: invalid BASEBYTE checksum in ",
		field->filename);
          bufdata = field->compress_buf;
          QFREAD(bufdata, FBSIZE, field->file, field->filename);
          curval = 0;
#        ifdef BSWAP
          swapbytes(bufdata, 4, 1);
#        endif
          field->compress_checkval = *((int *)bufdata);
         bufdata += 4;
#        ifdef BSWAP
          swapbytes(bufdata, 2, 1);
#        endif
          npix = (int)(*((short *)bufdata))-1;
          bufdata+=2;
          }
        if ((dval=(int)*(bufdata++))==-128)
          {
#        ifdef BSWAP
          swapbytes(bufdata, 2, 1);
#        endif
          memcpy(&val16, bufdata, 2);
          dval = (int)val16;
          if (dval==-32768)
            {
            bufdata += 2;
#          ifdef BSWAP
            swapbytes(bufdata, 4, 1);
#          endif
            memcpy(&dval,bufdata,4);
            bufdata += 4;
            }
          else
            bufdata += 2;
          }
        *(ptr++) = dval*bs + bz;
        curval += dval;
        }
      field->compress_curval = curval;
      field->compress_bufptr = bufdata;
      field->compress_npix = npix;
      break;

    case COMPRESS_PREVPIX:
      bufdata = field->compress_bufptr;
      curval = field->compress_curval;
      npix = field->compress_npix;
      while (size--)
        {
        if (!(npix--))
          {
          if (curval != field->compress_checkval)
            error(EXIT_FAILURE, "*Error*: invalid PREV_PIX checksum in ",
		field->filename);
          bufdata = field->compress_buf;
          QFREAD(bufdata, FBSIZE, field->file, field->filename);
#        ifdef BSWAP
          swapbytes(bufdata, 2, 3);
#        endif
          curval = (int)*(short *)bufdata;
          npix = (int)*(short *)(bufdata+=2)-1;
          field->compress_checkval = (int)(*(short *)(bufdata+=2));
          bufdata+=4;
          }
        if ((dval=(int)*(bufdata++))==-128)
          {
#        ifdef BSWAP
          swapbytes(bufdata, 2, 1);
#        endif
          memcpy(&val16, bufdata, 2);
          curval = (int)val16;
          bufdata += 2;
          }
        else
          curval += dval;
        *(ptr++) = curval*bs + bz;
        }
      field->compress_curval = curval;
      field->compress_bufptr = bufdata;
      field->compress_npix = npix;
      break;

    default:
      error(EXIT_FAILURE,"*Internal Error*: unknown compression mode in ",
				"readdata()");
    }

  return;
  }


/******************************* readimagehead *******************************/
/*
extract some data from the FITS-file header
*/
void	readimagehead(picstruct *field)
  {
   int		i, n;
   char		*buf, st[80], *point;

/*Open the file */
 
#ifndef MEMORY_READ
  if (!(field->file = fopen(field->filename, "rb")))
    error(EXIT_FAILURE,"*Error*: cannot open ", field->filename);
#endif /* ifndef MEMORY_READ */

  buf = readfitshead(field->file, field->filename, &n);
  if(FITSTOI("NAXIS   ", 0) < 2)
    error(EXIT_FAILURE, field->filename, " does NOT contain 2D-data!");

  field->bitpix = FITSTOI("BITPIX  ", 0);
  if (field->bitpix != BP_BYTE
	&& field->bitpix != BP_SHORT
	&& field->bitpix != BP_LONG
	&& field->bitpix != BP_FLOAT
	&& field->bitpix != BP_DOUBLE)
    error(EXIT_FAILURE, "Sorry, I don't know that kind of data.", "");

  field->bytepix = (field->bitpix>0?field->bitpix:-field->bitpix)>>3;
  field->width = FITSTOI("NAXIS1  ", 0);
  field->height = FITSTOI("NAXIS2  ", 0);
  field->npix = (size_t)field->width*field->height;
  field->bscale = FITSTOF("BSCALE  ", 1.0);

  field->bzero = FITSTOF("BZERO   ", 0.0);
  field->bitsgn = FITSTOI("BITSGN  ", 1);

  FITSTOS("OBJECT  ", field->ident, "Unnamed");

/* Check if the image is in compressed format */
  if (fitsread(buf, "IMAGECOD", st, H_STRING, T_STRING)==RETURN_OK)
    {
    if (!strncmp(st, "NONE", 4))
      field->compress_type = COMPRESS_NONE;
    else if (!strncmp(st, "BASEBYTE", 8))
      field->compress_type = COMPRESS_BASEBYTE;
    else if (!strncmp(st, "PREV_PIX", 8))
      field->compress_type = COMPRESS_PREVPIX;
    else
      warning("Compression skipped: unknown IMAGECOD parameter:", st);
    }

/*---------------------------------------------------------------------------*/

  field->fitshead = buf;
  field->fitsheadsize = n*FBSIZE;

  return;
  }


/******************************* readfitshead ********************************/
/*
read data from the FITS-file header
*/
char    *readfitshead(FILE *file, char *filename, int *nblock)

  {
   int     n;
   char    *buf;

  if (!(buf=(char *)malloc((size_t)FBSIZE)))
    error(EXIT_FAILURE, "*Error*: Not enough memory in ", "readfitshead()");

/* Find the number of FITS blocks of the header while reading it */
#ifdef MEMORY_READ
  memcpy(buf, prefs.memlink_head, FBSIZE);
#else
  QFREAD(buf, FBSIZE, file, filename);
#endif /* ifdef MEMORY_READ */

  if (strncmp(buf, "SIMPLE  ", 8))
    {
/* Ugly but necessary patch to handle this stupid DeNIS compressed format! */
    if (strncmp(buf, "XTENSION", 8))
      error(EXIT_FAILURE, filename, " is NOT a FITS file!");
    else
      {
      memset(buf, ' ', 80);
      strncpy(buf,
	"SIMPLE  =                    T / Decompressed by SExtractor", 59);
      }
    }

  for (n=1; !fitsnfind(buf,"END     ", n); n++)
    {
    if (!(buf=(char *)realloc(buf, (size_t)(FBSIZE*(n+1)))))
      error(EXIT_FAILURE, "*Error*: Not enough memory in ", "readfitshead()");
#  ifdef MEMORY_READ
    memcpy(buf+n*FBSIZE, prefs.memlink_head+n*FBSIZE, FBSIZE);
#  else
    QFREAD(buf+n*FBSIZE, FBSIZE, file, filename);
#  endif /* ifdef MEMORY_READ */
    }

  *nblock = n;
  return  buf;
  }

