/*
                                  makeit.c

*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*
*       Part of:        fitslimits
*
*       Author:         E.BERTIN (IAP)/ Thomas Erben
*
*       Contents:       Main loop
*
*       Last modify:    04/03/2001
*
*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "define.h"
#include "globals.h"
#include "fitscat.h"
#include "fitswcs.h"
#include "field.h"

/********************************** makeit ***********************************/
void	makeit(int ninfield, char **infield_name, char *refimage_name)
  {
   fieldstruct		**infield, *outfield,
			*field, *wfield;
   catstruct		*cat;
   tabstruct		*tab;
   double		w,w1,w2, invscale;
   static int	       	list[MAXINFIELD],next[MAXINFIELD];
   int			i,j,k, ntinfield;
   int                  minabsx=100000, maxabsx=-100000, minabsy=100000, maxabsy=-100000;

/* First check input files and count FITS extensions when available */
  ntinfield = k = 0;
  for (i=0; i<ninfield; i++)
    {
    if (!(cat=read_cat(infield_name[i])))
      {
      sprintf(gstr, "*Error*: %s not found", infield_name[i]);
      error(EXIT_FAILURE, gstr,"");
      }
/*-- Examine all extensions */
    tab=cat->tab;
    for (j=0; j<cat->ntab; j++)
      {
      if (tab->naxis
	&& (!tab->tfields))
        {
        list[k++] = j;
        next[i]++;
        }
      tab = tab->nexttab;
      }
    ntinfield += next[i];
    if (!next[i])
      warning("No suitable data found in ", cat->filename);
    free_cat(cat, 1);
    }

/* Now load the ``fields'' */
  QMALLOC(infield, fieldstruct *, ntinfield);
  /*NFPRINTF(OUTPUT, "Examining input data...")*/
  k = 0;
  for (i=0; i<ninfield; i++)
    {
    for (j=0; j<next[i]; j++, k++)
      {
      infield[k] = load_field(infield_name[i], list[k], i);
/*---- Now we can close input files */
      close_cat(infield[k]->cat);
      }
    }

/* Load or create output image */
  outfield = load_field(refimage_name, 0, ninfield+1);


/* Compute projected limits and scaling in output frame (avoid unnecessary */
/* computations) */
  /*NFPRINTF(OUTPUT, "Framing and scaling images...")*/
  for (i=0; i<ntinfield; i++)
    {
    frame_wcs(infield[i]->wcs, outfield->wcs);
    }

  for (i=0; i<ninfield; i++)
  {
      if(infield[i]->wcs->outmin[0] < minabsx)
      {
	  minabsx = infield[i]->wcs->outmin[0]-50;
      }
      if(infield[i]->wcs->outmax[0] > maxabsx)
      {
	  maxabsx = infield[i]->wcs->outmax[0]+50;
      }
      if(infield[i]->wcs->outmin[1] < minabsy)
      {
	  minabsy = infield[i]->wcs->outmin[1]-50;
      }
      if(infield[i]->wcs->outmax[1] > maxabsy)
      {
	  maxabsy = infield[i]->wcs->outmax[1]+50;
      }
  }

  
  printf("%d %d %d %d\n", (int)((float)minabsx-outfield->wcs->crpix[0]), 
                          (int)((float)maxabsx-outfield->wcs->crpix[0]), 
                          (int)((float)minabsy-outfield->wcs->crpix[1]), 
                          (int)((float)maxabsy-outfield->wcs->crpix[1]));

  return;
  }





