/* 15.06.2001:
   removed compiler warnings under DEC
   int -> unsigned int and explicit casts
   to (unsigned char *) in calls to save_fits

   24.01.2005:
   I added explicit casts at several points. As these routines
   are currenty not used, it is not clear whether they
   work correctly.
*/

#ifndef TIFF_IO

#include <stdio.h>
#include <string.h>
#include "image_io_globals.h"
#include "fits_image_io_globals.h"
#include "utils_globals.h"

static char	*last_header=NULL;
static int	curpos = 0;

int read_map(char *file, image_struct *imag, char *flag_map, double satur, double dead, bool invert)
{
    unsigned int        i;
    int			retcode = -1;
    fitsfile_struct     *ffd;
    float		*data;
    char		*vlag, *flag;

    ffd = open_fits_file(file);
    ffd->pix_type = T_FLOAT;
    imag->map = interp_fits_image(ffd, invert);
    imag->xwid     = ffd->xwid;
    imag->ywid     = ffd->ywid;
    imag->bitpix   = ffd->bitpix;
    imag->bscale   = ffd->bscale;
    imag->bzero    = ffd->bzero;
    imag->px0      = ffd->px0;
    imag->py0      = ffd->py0;
    imag->dx       = ffd->dx;
    imag->dy       = ffd->dy;
    imag->xcen     = ffd->xcen;
    imag->ycen     = ffd->ycen;
    if (last_header != NULL) ED_FREE(last_header);
    ED_CALLOC(last_header, "read_map", char, FITSBLKSIZ * ffd->head->nblocks);
    memcpy(last_header, ffd->head->ptr, FITSBLKSIZ * ffd->head->nblocks);
    curpos = 0;
    close_fits(ffd, NO);

    ED_CALLOC(imag->flag, "read_map", char, imag->xwid * imag->ywid);
    flag = flag_map;
    vlag = imag->flag;
    data = imag->map;
    if (flag == NULL) {
       for (i=0;i<imag->ywid*imag->xwid;i++,data++,vlag++) {
          if (*data > satur) *vlag |= SATURATED;
          if (*data < dead) *vlag |= DEAD;
       }
    } else {
       for (i=0;i<imag->ywid*imag->xwid;i++,data++,vlag++,flag++) {
          if (*flag) *vlag = DEAD;
          if (*data > satur) *vlag |= SATURATED;
          if (*data < dead) *vlag |= DEAD;
       }
    }
    retcode = 0;
    return retcode;
}

char *read_flag(char *file)
{
    fitsfile_struct     *fff;
    char		*flag;

    fff = open_fits_file(file);
    fff->pix_type = T_BYTE;
    flag = interp_fits_image(fff, NO);
    close_fits(fff, NO);
    
    return flag;
}

int write_map(char *file, field_struct *field, int type)
{
    fitsfile_struct	*ff;
    image_struct	image = field->image;
    float		*data, *back;
    unsigned int	i;
  
    ff = new_fits();
    init_fits(ff);
    switch (type) {
    case BACKMAP:
    case CORRECTED:
       image.bitpix = PIX_FLOAT;
       break;
    case THRESHOLD:
       image.bitpix = PIX_BYTE;
       break;
    case OBJECTNR:
       image.bitpix = PIX_SHORT;
       break;
    }
    w_fits_bool  ("SIMPLE",  ff->head, TRUE);
    w_fits_int   ("BITPIX",  ff->head, (ff->bitpix = image.bitpix));
    w_fits_int   ("NAXIS",   ff->head, 2);
    w_fits_int   ("NAXIS1",  ff->head, (ff->xwid   = image.xwid));
    w_fits_int   ("NAXIS2",  ff->head, (ff->ywid   = image.ywid));
    w_fits_int   ("PCOUNT",  ff->head, 0);
    w_fits_int   ("GCOUNT",  ff->head, 1);
    w_fits_int   ("TFIELDS", ff->head, 1);
    w_fits_double("BSCALE",  ff->head, (ff->bscale = image.bscale));
    w_fits_double("BZERO",   ff->head, (ff->bzero  = image.bzero));
    w_fits_double("CRPIX1",  ff->head, (ff->px0    = image.px0));
    w_fits_double("CRPIX2",  ff->head, (ff->py0    = image.py0));
    w_fits_double("CDELT1",  ff->head, (ff->dx     = image.dx));
    w_fits_double("CDELT2",  ff->head, (ff->dy     = image.dy));
    w_fits_double("CRVAL1",  ff->head, (ff->xcen   = image.xcen));
    w_fits_double("CRVAL2",  ff->head, (ff->ycen   = image.ycen));
    w_fits_pad(ff->head);
    switch (type) {
    case BACKMAP:
       ff->pix_type = T_FLOAT;
       save_fits(ff, (unsigned char *)image.bck, file);
       break;
    case CORRECTED:
       ff->pix_type = T_FLOAT;
       data = image.map;
       back = image.bck;
       for (i=0;i<image.ywid*image.xwid;i++,data++,back++) {
          *data -= *back;
       }
       save_fits(ff, (unsigned char *)image.map, file);
       break;
    case THRESHOLD:
       ff->pix_type = T_BYTE;
       save_fits(ff, (unsigned char *)image.info, file);
       break;
    case OBJECTNR:
       ff->pix_type = T_SHORT;
       save_fits(ff, (unsigned char *)image.info, file);
       break;
    }
    return 0;
}

int get_header_card(char *record)
{
    int retcode = 1;

    while (retcode == 1) {
       strncpy(record, &last_header[curpos], 80);
       record[80] = '\0';
       if (strncmp(record,"XTENSION",8) != 0 &&
           strncmp(&last_header[curpos],"SIMPLE",6) != 0 &&
           strncmp(&last_header[curpos],"PCOUNT",6) != 0 &&
           strncmp(&last_header[curpos],"GCOUNT",6) != 0 &&
           strncmp(&last_header[curpos],"TFIELDS",7) != 0) {
          retcode = -1;
       }
       if (strncmp(record,"END",3) == 0) {
          retcode = 0;
       }
       curpos += 80;
    }
    return retcode;
}

int num_header_cards()
{
    int number = 0;

    curpos = 0;
    while (strncmp(&last_header[curpos],"END",3) != 0) {
       if (strncmp(&last_header[curpos],"XTENSION",8) != 0 &&
           strncmp(&last_header[curpos],"SIMPLE",6) != 0 &&
           strncmp(&last_header[curpos],"PCOUNT",6) != 0 &&
           strncmp(&last_header[curpos],"GCOUNT",6) != 0 &&
           strncmp(&last_header[curpos],"TFIELDS",7) != 0) {
          number++;
       }
       curpos += 80;
    }
    curpos = 0;
    return number;
}
#endif
