#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#ifndef DPI
#define DPI 3.14159265358979323846
#endif
double arcsin(double x)
{
    double val;


    if (x < -1.0) {
       val = -DPI*0.5;
    } else {
       if (x > 1.0) {
          val = DPI*0.5;
       } else {
          val = asin(x);
       }
    }
    return val;
}
/**/
double arccos(double x)
{
    double val;

    if (x < -1.0) {
       val = DPI;
    } else {
       if (x > 1.0) {
          val = 0.0;
       } else {
          val = acos(x);
       }
    }
    return val;
}
/**/
void rel_to_abs(double a0, double d0, double pangle, 
                double *x, double *y, double *a, double *d, int n)
/*
 * MAP	Internal routine
 *	Convert a set of plane position to spherical coordinates
 *	according to the gnomonic projection formulae
 *
 *	This routine has been carefully programmed to allow
 *	equivalencing (X,A) and (Y,D) arrays. This feature is
 *	used in CONVERT.
 *
 * Arguments :
 *       A0      R*8     RA like coordinate of projection center  Input
 *       D0      R*8     Dec like coordinate of projection center Input
 *       PANGLE  R*8     Projection angle                         Input
 *	X	R*8 (*)	Array of offsets in X		         Input
 *	Y	R*8 (*)	Array of offsets in Y		         Input
 *	A	R*8 (*)	Array of R.A. (radians)		         Output
 *	D	R*8 (*)	Array of declinations		         Output
 *	N	I	Size of arrays			         Input
 */
{
    int		i;
    double	precision, sind0, cosd0, sina0, cosa0, npole, spole;
    double	r, p, sinr, cosr, sinp, cosp, sind;
/*
 * Setup necessary constants
 */
    precision = 1.0E-10;
    sind0 = sin(d0);
    cosd0 = cos(d0);
    sina0 = sin(a0);
    cosa0 = cos(a0);
/*
 * Compute POLE position 
 */
   if (d0 < -1.0E-30) {
      spole = 1.0/tan(d0);
      npole = 1.0E38;
   } else {
      if (d0 > 1.0E-30) {
         npole = 1.0/tan(d0);
         spole = 1.0E38;
      } else {
         npole = 1.0E38;
         spole = 1.0E38;
      }
   }
/*
 * (A,D) = f(A0,D0,R,P)
 *	D = ASIN (SIN(D0)*COS(R) + COS(D0)*SIN(R)*COS(P))
 *	A = A0 + ATAN2( SIN(R)*SIN(P)*SIN(D0) ,
 *	1	 (SIN(D)*COS(D0)-SIN(R)*COS(P)) )
 *	ou A = A0 + ASIN ( SIN(R)*SIN(P)/COS(D) )
 *
 * Gnomonic projection
 */
    if (sind0 > precision) {
       for (i=0;i<n;i++) {
          r = atan(sqrt(x[i]*x[i]+y[i]*y[i]));
          if (r > precision) {
             p = atan2(x[i], y[i]) - pangle;
             sinr = sin(r);
             cosr = cos(r);
             sinp = sin(p);
             cosp = cos(p);
             sind = (sind0 * cosr + cosd0 * sinr * cosp);
             if (y[i] < npole) {
                d[i] = arcsin(sind);
                a[i] = a0 + arcsin(sinr * sinp / cos(d[i]));
             } else {
                d[i] = arcsin(sind);
                a[i] = a0 - arcsin(sinr * sinp / cos(d[i])) + DPI;
             }
          } else {
             a[i] = a0;
             d[i] = d0;
          }
       }
    } else {
       if (sind0 < -precision) {
          for(i=0;i<n;i++) {
             r = atan(sqrt(x[i]*x[i]+y[i]*y[i]));
             if (r > precision) {
                p = atan2(x[i], y[i]) - pangle;
                sinr = sin(r);
                cosr = cos(r);
                sinp = sin(p);
                cosp = cos(p);
                sind = (sind0 * cosr + cosd0 * sinr * cosp);
                if (y[i] > spole) {
                   d[i] = arcsin(sind);
                   a[i] = a0 + arcsin(sinr * sinp / cos(d[i]));
                } else {
                   d[i] = arcsin(sind);
                   a[i] = a0 - arcsin(sinr * sinp / cos(d[i])) + DPI;
                }
             } else {
                a[i] = a0;
                d[i] = d0;
             }
          }
       } else {
          for(i=0;i<n;i++) {
             r = atan(sqrt(x[i]*x[i]+y[i]*y[i]));
             if (r > precision) {
                p = atan2(x[i], y[i]) - pangle;
                sinr = sin(r);
                cosr = cos(r);
                sinp = sin(p);
                cosp = cos(p);
                sind = (sind0 * cosr + cosd0 * sinr * cosp);
                d[i] = arcsin(sind);
                a[i] = a0 + arcsin(sinr * sinp / cos(d[i]));
             } else {
                a[i] = a0;
                d[i] = d0;
             }
          }
       }
    }
}
/**/
void abs_to_rel(double a0, double d0, double pangle,
                double *a, double *d, double *x, double *y, int n)
/*
 * MAP	Internal routine
 *	Convert a set of plane position to spherical coordinates
 *	according to the gnomonic projection formulae
 *
 *	This routine has been carefully programmed to allow
 *	equivalencing (A,X) and (D,Y) arrays. This feature is
 *	used in CONVERT.
 *
 * Arguments :
 *       A0      R*8     RA like coordinate of projection center  Input
 *       D0      R*8     Dec like coordinate of projection center Input
 *       PANGLE  R*8     Projection angle                         Input
 *	A	R*8 (*)	Array of R.A. (radians)		Input
 *	D	R*8 (*)	Array of declinations		Input
 *	X	R*8 (*)	Array of offsets in X		Output
 *	Y	R*8 (*)	Array of offsets in Y		Output
 *	N	I	Size of arrays			Input
 */
{
    int         i;
    double      precision, sind0, cosd0, sina0, cosa0;
    double      r, p, sind, cosd, sina, cosa, expr;
/*
 * Setup necessary constants
 */
    precision = 1.0E-10;
    sind0 = sin(d0);
    cosd0 = cos(d0);
    sina0 = sin(a0);
    cosa0 = cos(a0);
/*
 * (R,P) = f(A0,D0,A,D)
 *	R = ACOS ( (SIN(D) - COS(D0) *
 *	1 ( SIN(D)*COS(D0)-COS(D)*SIN(D0)*COS(A-A0) ) ) / SIN(D0) )
 *	P = ATAN2 ( COS(D)*SIN(A-A0) ,
 *	1 ( SIN(D)*COS(D0) - COS(D)*SIN(D0)*COS(A-A0) ) )
 *
 * Case D0 very small
 *	R = ACOS (COS(D)*COS(A-A0))
 *	P = A-A0
 *
 * Gnomonic
 */
    if (fabs(sind0) >= precision) {
       for (i=0;i<n;i++) {
          sind = sin(d[i]);
          cosd = cos(d[i]);
          sina = sin(a[i]-a0);
          cosa = cos(a[i]-a0);
          expr = sind * cosd0 - cosd * sind0 * cosa;
          r = arccos((sind - cosd0 * expr) / sind0);
          if (r > precision) {
             p = atan2(cosd * sina, expr) + pangle;
             expr = tan(r);
             x[i] = expr * sin(p);
             y[i] = expr * cos(p);
          } else {
             x[i] = 0.0;
             y[i] = 0.0;
          }
       }
    } else {
       for (i=0;i<n;i++) {
          sind = sin(d[i]);
          cosd = cos(d[i]);
          sina = sin(a[i]-a0);
          cosa = cos(a[i]-a0);
          r = arccos(cosd * cosa);
          if (r > precision) {
             p = atan2(cosd * sina, sind) + pangle;
             expr = tan(r);
             x[i] = expr * sin(p);
             y[i] = expr * cos(p);
          } else {
             x[i] = 0.0;
             y[i] = 0.0;
          }
       }
    }
}
