#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "fitscat.h"
#include "solve.h"
#include "options_globals.h"
#include "utils_globals.h"
#include "lists_globals.h"
#include "relphotom_globals.h"
#include "relphotom_vers.h"
#include "fixed_phot_globals.h"
#include "derive_relzps_globals.h"
#include "relphotom_tab_globals.h"
/**/

/* 04.08.00:
   The line if (isum[i] != 0 && err[i] != 0.0)
   was replaced by if (isum[i] != 0 && fabs(err[i]) > 1.0e-05 && 
   fabs(sum[i]) > 1.0e-05)
   at three places. err[i] is a float and it is divided by sum[i]
   that can happen to be zero.

   21.06.01:
   I made sure that in the averaging no "pairs" of the same image are used
   (This led to floating exceptions under alpha). Included "string.h"
   (compiler warnings)

   13.09.01:
   - changed long int to LONG 
   - The weighting of summands in the calculation of the mean magnitude
     difference between frames is now done with the inverse of the magnitude
     error.
   - for the moment I set the weighting of all the summands in te chi2
     sum (relative ZP) to 1.0. HAS TO BE REVISITED
   - removed an error in the calculation of the chi2 that is meaningless
     in the moment where the sigmas of all summands in the chi2 sum are
     set to 1.0.

   26.10.01:
   function derive_relzps:
   - changed all occurences of "isum[i] != 0" with "isum[i] > control.minobj"
     so that only overlapps consisting of at least minobj objects in common
     are considered in the solution.
   - the program stops now if there is an image that has no overlap
     with any other frame

   10.01.05:
   removed compiler warnings under gcc with optimisation
   (-O3) enabled.

   13.03.2005:
   - In the RELPHOTOM table we store two new columns indicating
     whether a frame was under the photometric ones and we also
     store a corrected zeropoint in this case.
     Tthe corrected zeropoint is the zeropoint at zero 
     airmass corrected for extinction and estimated relative zeropoint.
     For this also calls to derive_abs_phot and save_relphot_elem
     have been adapted.
   - We estimate the mean of relative zeropoints now BEFORE the absolute
     zeropoint is determined. Hence, the relative zeropoints do not
     change if we use a different set of photometric pegs.
     In this respect, only the case where the absolute photometric calibration
     is a simple scalar is correct (the old EIS scheme of rows and columns
     is not supported by our data anyway). 
   - I removed argument out_cat from derive_relzps as it was never used.

   09.03.2006:
   I removed the last argument from the NR free_... functions.
   That argument is not used in the functions.
*/

extern control_struct control;

float gammq(float a, float x);

void derive_relzps(pair_struct *pairs, 
                   pair_struct *refs, int nrefs, 
                   char *name, int imags, int nfields)
/****** derive_relzps ***************************************************
 *
 *  NAME	
 *      derive_relzps - Derive zeropints for the current magnitude type
 *
 *  SYNOPSIS
 *
 *  FUNCTION
 *
 *  INPUTS
 *      pairs    - Pointer to primary pairs catalog information
 *      refs     - List of reference pairs catalog information
 *      nrefs    - Number of reference pairs catalogs
 *      name     - The name of the magnitude array
 *      imags    - The number of magnitudes in the cuurent mag array
 *      nfields  - The number of fields in the input catalog
 *
 *  RESULTS
 *
 *  RETURNS
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *
 *****************************************************************************/
{
    float	*x, *sx, *c=NULL, **cov=NULL, *p, chi2, parsum, tmp;
    int		*ix, *iy, *ic=NULL;
    int		i, ii, j, k, ndata, nref;
    int		*ifield, iter;
    float	*params, *params_sdev, *covar, *zp;
    float       *framephot; /* is a given frame chosen to be photometric ? (1=YES) */
    float       *corrzp;    /* the corrected zeropoint for photometric frames. It is
                               the standardstar ZP corrected for extinction and relative
                               photometry
			    */
    double	*sum, *sigsum, *ave, *err, temp, temp_f, temp_t;
    LONG	*isum;
    char	outline[MAXCHAR*MAXCHAR], nname[MAXCHAR];
    int		kkk, ndiag, nc,*check;
/*
 *  Step along the mags list
 */

    VPRINTF(NORMAL,"Deriving overlap averages for %s\n", name);
    ndiag = pairs->nover;
    nc    = pairs->nc;
    for (kkk=0;kkk<imags;kkk++) {
       if (imags > 1)
          sprintf(nname,"%s(%-d)",name,kkk+1);
       else
          strcpy(nname,name);
       ED_CALLOC(zp, "relphotom", float, nc);
/*
    goto bigjump;
 */
/* 
 *     Sum pair observations of the same overlap
 */
       sum    = dvector(1, ndiag*2);
       err    = dvector(1, ndiag*2);
       ave    = dvector(1, ndiag*2);
       sigsum = dvector(1, ndiag*2);
       isum   = lvector(1, ndiag*2);
/*
 *     Build an iteration here
 *     ==================================================
 */
       for (iter=0;iter<control.niter;iter++) {
/*
 *        Clear the averaging arrays first
 */
          for (i=1;i<=ndiag;i++) 
          {
             ave[i] = sum[i] = sigsum[i] = err[i] =  0;
             isum[i] = 0;
          }
/* 
 *        Get the allowed pairs into the averaging. First do a rough
 *        estimate and then with `known' averages a better selection.
 */
          ndata = pairs->nobjs;
          for (i=0;i<ndata;i++) {
             if (pairs->from_f[i] != pairs->to_f[i])
	     {
  	       if (pairs->from_f[i]>pairs->to_f[i]) {
  		 temp_f = pairs->from_m[kkk][i] + zp[pairs->from_f[i]-1];
  		 temp_t = pairs->to_m[kkk][i] + zp[pairs->to_f[i]-1];
  		 ii = pairs->indx[pairs->from_f[i]-1 + (pairs->to_f[i]-1)*nc];
  	       } else {
  		 temp_t = pairs->from_m[kkk][i] + zp[pairs->from_f[i]-1];
  		 temp_f = pairs->to_m[kkk][i] + zp[pairs->to_f[i]-1];
  		 ii = pairs->indx[pairs->from_f[i]-1 + (pairs->to_f[i]-1)*nc];
  	       }
  	       if (((temp_f - control.maxmag) > 0.0 &&
  		   (temp_t - control.maxmag) > 0.0) ||
  		   ((control.minmag - temp_f) > 0.0 &&
  		   (control.minmag - temp_t) > 0.0)) continue;
  	       if (control.use_flag == YES &&
  		   ((pairs->from_flag[i]&control.flag_mask) ||
  		    (pairs->to_flag[i]&control.flag_mask) ||
  		    (pairs->from_imaflag[i]&control.imaflags_mask) ||
  		    (pairs->to_imaflag[i]&control.imaflags_mask))) continue;
  	       if (pairs->from_f[i] != pairs->to_f[i] ) {

                  /* the factor of 0.01 should prevents too large numbers
                     It cancels out in the final calculation of sum/err
                  */
  		  temp = 0.01/(pairs->from_e[kkk][i]*pairs->from_e[kkk][i]+
                             pairs->to_e[kkk][i]*pairs->to_e[kkk][i]);
  		  sum[ii]    += (temp_f - temp_t)*temp;
  		  err[ii]    += temp;
  		  isum[ii]++;
  	       }
	     }
          }
          for (i=1;i<=ndiag;i++) 
          {
             if (isum[i] >=control.minobj && fabs(err[i]) > 1.0e-05 && fabs(sum[i]) > 1.0e-05) 
             {
                ave[i] = sum[i] / err[i];
                VPRINTF(VERBOSE,"Aver: %f, %d %d (%d,%d)\n",ave[i],
                pairs->iind[i], pairs->jind[i], i,isum[i]);
             }
          }
/*
 *        Do a better selection
 */
          for (i=1;i<=ndiag;i++) 
          {
             sum[i] = sigsum[i] = err[i] = 0;
             isum[i] = 0;
          }

          for (i=0;i<ndata;i++) 
          {
             if (pairs->from_f[i] != pairs->to_f[i])
	     {
  	       if (pairs->from_f[i]>pairs->to_f[i]) 
  	       {
  		 temp_f = pairs->from_m[kkk][i] + zp[pairs->from_f[i]-1];
  		 temp_t = pairs->to_m[kkk][i] + zp[pairs->to_f[i]-1];
  		 ii = pairs->indx[pairs->from_f[i]-1 + (pairs->to_f[i]-1)*nc];
  	       } 
  	       else 
  	       {
  		 temp_t = pairs->from_m[kkk][i] + zp[pairs->from_f[i]-1];
  		 temp_f = pairs->to_m[kkk][i] + zp[pairs->to_f[i]-1];
  		 ii = pairs->indx[pairs->from_f[i]-1 + (pairs->to_f[i]-1)*nc]; 
  	       }
  
  	       if (((temp_f - control.maxmag) > 0.0 &&
  		   (temp_t - control.maxmag) > 0.0) ||
  		   ((control.minmag - temp_f) > 0.0 &&
  		   (control.minmag - temp_t) > 0.0) ||
  		   ((iter>0) && (fabs((temp_f - temp_t)-ave[ii])) > control.magdif)) continue;
  
  	       if (control.use_flag == YES &&
  		   ((pairs->from_flag[i]&control.flag_mask) ||
  		    (pairs->to_flag[i]&control.flag_mask) ||
  		    (pairs->from_imaflag[i]&control.imaflags_mask) ||
  		    (pairs->to_imaflag[i]&control.imaflags_mask))) continue;
  
  	       if (pairs->from_f[i] != pairs->to_f[i] ) 
  	       {

                  /* the factor of 0.01 should prevents too large numbers
                     It cancels out in the final calculation of sum/err
                  */
  		  temp = 0.01/(pairs->from_e[kkk][i]*pairs->from_e[kkk][i]+
                             pairs->to_e[kkk][i]*pairs->to_e[kkk][i]);
  		  sum[ii]    += (temp_f - temp_t)*temp; 
  		  err[ii]    += temp;
  
  		  sigsum[ii] += 0.01/temp;
  		  isum[ii]++;
  	       }
	     }
          }

/*
 *        Count the number of real overlaps
 */
          j = 0;
          for (i=1;i<=ndiag;i++) {
            if (isum[i] >= control.minobj && fabs(err[i]) > 1.0e-05 && fabs(sum[i]) > 1.0e-05) 
            {
               j++;
                VPRINTF(VERBOSE,"Aver: %f, %d %d (%d,%d)\n",sum[i]/err[i],
                pairs->iind[i], pairs->jind[i], i,isum[i]);
            }
          }
          ndata = j;
/*
 *        Copy the average information to the data arrays for LSQ fitting.
 *        Add an extra belement to allow for an artificial pegging of the
 *        system.
 */
          x  = vector(1, ndata);
          sx = vector(1, ndata);
          ix = ivector(1, ndata);
          iy = ivector(1, ndata);
          c  = vector(1, nc);
          ic = ivector(1, nc);
          cov = matrix(1, nc, 1, nc);
          j = 1; 
          ED_CALLOC(check, " ", int, (nc+1));
          for (i=1;i<=ndiag;i++) {
             if (isum[i] >= control.minobj && fabs(err[i]) > 1.0e-05 && fabs(sum[i]) > 1.0e-05) 
              {
                x[j]  = (float) (sum[i]/err[i]);
                /*sx[j] = (float) (sigsum[i]/isum[i]);*/
                sx[j] = (float) (1.0);
                ix[j] = pairs->iind[i];
                iy[j] = pairs->jind[i];
                check[ix[j]-1] = 1;
                check[iy[j]-1] = 1;
                VPRINTF(VERBOSE,"Rel: %f %f %d %d\n", 
                                 x[j], sx[j], ix[j], iy[j]);
                j++;
/*
 *              Add an extra data element when we have a fixed frame. This
 *              data element will get a better weight than the original one.
 */
/*
                if (
                check_fixed(ix, iy, j-1, i, x, sx, pairs);
 */
/*
                )
                   j++;
 */
             }
          }
          for (i=0;i<nc;i++)
	  {
            if (!check[i])
	    {
              syserr(FATAL, "relphotom", "No relations with frame %d\n", i+1);
            }
	  }
          ED_FREE(check);
          VPRINTF(NORMAL,
            "Solving equations (iter=%-d) for %d overlaps and %d parameters for %s\n", 
                  iter,ndata,nc,nname);

          if (ndata == 1) 
          {
            zp[0] = x[1] * 0.5;
            zp[1] = - x[1] * 0.5;
          } 
          else 
          {
            relphotsolve(x, sx, ix, iy, ndata, nc, c, ic, cov, control.method);
/*
 *          Update the currently known zero points with the newly derived
 *          differences
 */
            for (i=0;i<nc;i++) 
            {
               VPRINTF(VERBOSE,"zpc[%d]: %f\n", i, c[i+1]);
               zp[i] -= c[i+1];
            }
            for (chi2=0.0,i=1;i<=ndata;i++) 
            {
              chi2 += (tmp=((x[i]-(c[iy[i]]-c[ix[i]]))/sqrt(sx[i])),tmp*tmp);
              VPRINTF(VERBOSE,"CHI2: %f (%f, %f)\n", chi2, x[i]-(c[iy[i]]-c[ix[i]]), sx[i]);
            }
            VPRINTF(VERBOSE,"CHI2: %f\n", chi2);
            chi2 = gammq(0.5*(ndata-nc), chi2*0.5);
          }
          for (i=0;i<nc;i++) 
	  {
            VPRINTF(VERBOSE,"zp[%d]: %f\n", i, zp[i]);
	  }
          VPRINTF(NORMAL,"CHI2 prob: %f\n", chi2);
          free_vector(x, 1);
          free_vector(sx, 1);
          free_ivector(ix, 1);
          free_ivector(iy, 1);
          if (ndata != 1) {
            if (iter != (control.niter-1)) {
               free_vector(c, 1);
               free_ivector(ic, 1);
               free_matrix(cov, 1, nc, 1);
            }
          }
/*
 *        End of iteration 
 *        =========================================
 */
       } 
       free_dvector(sum, 1);
       free_dvector(err, 1);
       free_dvector(ave, 1);
       free_dvector(sigsum, 1);
       free_lvector(isum, 1);
/*
 *     Copy the derived information to the outout arrays
 */
       ED_CALLOC(params,      "relphotom", float, nc);
       ED_CALLOC(params_sdev, "relphotom", float, nc);
       ED_CALLOC(framephot,   "relphotom", float, nc);
       ED_CALLOC(corrzp,      "relphotom", float, nc);
       ED_CALLOC(ifield,      "relphotom", int  , nc);
       ED_CALLOC(covar,       "relphotom", float, nc*(nc+1)/2);
       k = 0;
       for (i=0;i<nc;i++) {
          params[i]      = zp[i];
          params_sdev[i] = sqrt(cov[i+1][i+1]);
          ifield[i]      = ic[i+1];
          for (j=i;j<nc;j++) {
             covar[k++]  = cov[i+1][j+1];
          }
       }
       VPRINTF(NORMAL,"Chi2: %f\n", chi2);
       if (verbosity(VERBOSE)) {
         for (i=1;i<=nc;i++)
            VPRINTF(VERBOSE,"%d %f %f\n", ic[i], c[i], sqrt(cov[i][i]));
         for (i=1;i<=nc;i++) 
         {
            strcpy(outline,"");
            for (j=1;j<=nc;j++)
               sprintf(outline,"%s %f ", outline, cov[i][j]);
            VPRINTF(VERBOSE,"%s\n",outline);
         }
       }
/*
 *     Make sure the relative zeropoints are scaled to average = 0.0
 *     Doing this here instead of after the absolute calibration
 *     ensires that rel. zeropoints stay the same if photometric
 *     frames are changed.
 */
       for (i=0,parsum=0.0;i<nc;i++) 
          parsum += (double) params[i];
       parsum /= nc;
       for (i=0;i<nc;i++) 
          params[i] -= (float) parsum;


/*
 *     In case of reference information perform the polynomial fitting
 */
/*
  bigjump:
 */
       VPRINTF(NORMAL,"Checking Fixed\n");
       nref = make_reference_zps(refs, nrefs, kkk);
       if (nref) {
          p = vector(1, control.npol_param);
          derive_abs_phot(p, params, framephot, corrzp, control.npol_param);
          for (i=1;i<=control.npol_param;i++)
	  {
             VPRINTF(NORMAL,"p(%d): %f\n\n", i, p[i]);
	  }
	  /* for the moment we only treat the case with a scalar
             fit for the absolute zeropoint correctly !!
	  */
          for (i=0;i<nc;i++) {
             VPRINTF(VERBOSE,"zp(%d): %f ", i, params[i]);
             /*params[i] += polval((float) i, p, control.npol_param);*/
             VPRINTF(VERBOSE,"zp(%d): %f\n", i, params[i] + p[1]);
	  }
	  parsum = p[1];
       }
       VPRINTF(NORMAL,"Saving\n");
       save_relphot_elem(nname, params, params_sdev, framephot, 
                         corrzp, parsum, chi2, nfields);

       ED_FREE(params);
       ED_FREE(params_sdev);
       ED_FREE(ifield);
       ED_FREE(covar);

       free_vector(c,1);
       free_ivector(ic,1);
       free_matrix(cov,1,nc,1);

       ED_FREE(zp);
       ED_FREE(pairs->from_m[kkk]);
       ED_FREE(pairs->to_m[kkk]);
    }
    ED_FREE(pairs->from_m);
    ED_FREE(pairs->to_m);
}
/**/
