#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "get_epoch_globals.h"
#include "stdcat_globals.h"
#include "fitscat.h"
#include "tabutil_globals.h"
#include "utils_globals.h"
#include "utils_macros.h"
#include "precess_globals.h"
#include "options_globals.h"
#include "prephotom_vers.h"

/* 
   21.06.01:
   removed compiler warnings under DEC
*/

control_struct control;

void usage(char *me)
{
    fprintf(stderr,"Usage: %s \n",me);
    fprintf(stderr,"		-s std_cat\n");
    fprintf(stderr,"		-n standard_name\n");

    fprintf(stderr,"	Program to extract the Standard Star's Epoch from the catalog\n\n");
    exit(1);
}
 
void init(int argc, char *argv[])
{
   int i;

   if (argc == 1) usage(argv[0]);
   for (i=1; i<argc; i++) {
     if (argv[i][0] == '-') {
        switch(tolower((int)argv[i][1])) {
        case 's': strcpy(control.stdcatfile,argv[++i]);
                  break;
        case 'n': strcpy(control.std_name,argv[++i]);
                  break;
        default:  usage(argv[0]);
                  break;
        }
     } else {
        syserr(WARNING,"init","Command line syntax error\n");
        usage(argv[0]);
     }
  }
}

#define STRING_LEN 16
#define MAXPHOTS 40
#define NPHOT 4
/* Number of keys is 4 + NPHOT */
#define NKEYS 8

int main(int argc, char *argv[])

/****** prephotom ***************************************************
 *
 *  NAME	
 *      prephotom - Perform prephotometric pairing
 *
 *  SYNOPSIS
 *      prephotom -i incat1 incat2 ... -o outcat
 *                -s stdcat_file [CONFOPTION]
 *
 *  FUNCTION
 *      This program performs the pairing between the photometric sequence
 *      set of frames and the standard stars catalog entries. The
 *      frame-to-frame pairing has already been performed by the preastrom
 *      program. In addition the Ra,Dec information is derived and stored
 *      by the astrom and aplastrom programs.
 *
 *  INPUTS
 *      -i incat1 incat2 ... - The list of input catalogs (single passband)
 *                             that are to be combined to form a single
 *                             photometric entity.
 *      -o outcat            - The output catalog name combining the
 *                             information from all input catalogs.
 *      -s stdcat_file       - The name of the standard star catalog file.
 *                             This file has the same format as the data
 *                             catalog files.
 *     [-c configfile]       - The name of the configuration file. The default
 *                             name is /denisoft/conf/photom.conf
 *     [CONFOPTION]          - Pairs of configuration VARIABLE VALUE
 *                             sequences that allow dynamically
 *                             overwritting of configuration file values.
 *
 *  RESULTS
 *
 *  RETURNS
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *      The photometric calibration process has not been fully implemented
 *      yet.
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 01-08-1996
 *
 ******************************************************************************/
{ 
    float	epoch;
    char	header[MAXCHAR], *names[3] = {"Imag", "Jmag", "Kmag"};
    int		nn = 3;

    init(argc, argv);

    sprintf(header,"\n   %s Version: %s (%s)\n",NAME,VERS,DATE);
    VPRINTF(NORMAL,header);


    VPRINTF(VERBOSE,"Interpreting Standard Star Catalog file\n");
    read_stdcat(control.stdcatfile, names, nn);
    epoch = 4000.0 - epoch_std_star(control.std_name);
    printf("%f\n",epoch);
    return 0;
}
