#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "make_stdcat_globals.h"
#include "fitscat.h"
#include "tabutil_globals.h"
#include "utils_globals.h"
#include "utils_macros.h"

/* 10.01.2005:
   removed compiler warnings under gcc with optimisation
   (-O3) enabled.
 */

control_struct control;

void usage(char *me)
{
    fprintf(stderr,"Usage: %s \n",me);
    fprintf(stderr,"                 -i incat \n");
    fprintf(stderr,"                 -o outcat\n");
    exit(1);
}
 
void init(int argc, char *argv[])
{
   int i;

   if (argc == 1) usage(argv[0]);
   for (i=1; i<argc; i++) {
     if (argv[i][0] == '-') {
        switch(argv[i][1]) {
        case 'i': strcpy(control.incatfile,argv[++i]);
                  break;
        case 'o': strcpy(control.outcatfile,argv[++i]);
                  break;
        default:  syserr(WARNING,"init","Command line syntax error\n");
                  usage(argv[0]);
        }
     } else {
        syserr(WARNING,"init","Command line syntax error\n");
        usage(argv[0]);
     }
  }
}


static int kcol, buf_len = 100, nbuf;
static stdstruct *standards;

tabstruct *create_stdtab()
{
    kcol = -1; nbuf = 1;
    ED_MALLOC(standards, "create_stdtab", stdstruct, buf_len * nbuf);
    return new_tab("STDTAB");
}

void add_stdtab_line(stdstruct s)
{
    if (++kcol >= buf_len * nbuf) {
       nbuf++;
       ED_REALLOC(standards, "add_stdtab_line", stdstruct, buf_len * nbuf);
    }
    strcpy(standards[kcol].name,     s.name);
    strcpy(standards[kcol].Cat,      s.Cat);
    strcpy(standards[kcol].Q,        s.Q);
    strcpy(standards[kcol].Spectrum, s.Spectrum);
    strcpy(standards[kcol].Cross_ID, s.Cross_ID);
    strcpy(standards[kcol].Iref,     s.Iref);
    strcpy(standards[kcol].Jref,     s.Jref);
    strcpy(standards[kcol].Kref,     s.Kref);
    standards[kcol].Qual    = s.Qual;
    standards[kcol].ra      = s.ra;
    standards[kcol].dec     = s.dec;
    standards[kcol].epoch   = s.epoch;
    standards[kcol].V       = s.V;
    standards[kcol].Imag    = s.Imag;
    standards[kcol].Jmag    = s.Jmag;
    standards[kcol].Kmag    = s.Kmag;
    standards[kcol].Ierr    = s.Ierr;
    standards[kcol].Jerr    = s.Jerr;
    standards[kcol].Kerr    = s.Kerr;
    standards[kcol].Group   = s.Group;
}

#define NAMES_LEN    11
#define QUAL_LEN      1
#define CATS_LEN      3
#define Q_LEN         3
#define SPECTRUM_LEN 11
#define CROSS_ID_LEN 22
#define IREF_LEN      3
#define JREF_LEN      3
#define KREF_LEN      3

void save_stdcat(catstruct *cat, tabstruct *tab, char *filename)
{
    int		i;
    char	*Names, *Qual, *Cats, *Q, *Spectrum, *Cross_ID, 
                *Iref, *Jref, *Kref;
    double	*ra, *dec;
    float 	*epoch, *V, *Imag, *Jmag, *Kmag, *Ierr, *Jerr, *Kerr;
    long	*Group, *SeqNr;

    kcol++;
    ED_MALLOC(Names,    "save_stdcat", char  , kcol*NAMES_LEN);
    ED_MALLOC(Qual,     "save_stdcat", char  , kcol*QUAL_LEN);
    ED_MALLOC(Cats,     "save_stdcat", char  , kcol*CATS_LEN);
    ED_MALLOC(Q,        "save_stdcat", char  , kcol*Q_LEN);
    ED_MALLOC(Spectrum, "save_stdcat", char  , kcol*SPECTRUM_LEN);
    ED_MALLOC(Cross_ID, "save_stdcat", char  , kcol*CROSS_ID_LEN);
    ED_MALLOC(Iref,     "save_stdcat", char  , kcol*IREF_LEN);
    ED_MALLOC(Jref,     "save_stdcat", char  , kcol*JREF_LEN);
    ED_MALLOC(Kref,     "save_stdcat", char  , kcol*KREF_LEN);
    ED_MALLOC(ra,       "save_stdcat", double, kcol);
    ED_MALLOC(dec,      "save_stdcat", double, kcol);
    ED_MALLOC(epoch,    "save_stdcat", float , kcol);
    ED_MALLOC(V,        "save_stdcat", float , kcol);
    ED_MALLOC(Imag,     "save_stdcat", float , kcol);
    ED_MALLOC(Jmag,     "save_stdcat", float , kcol);
    ED_MALLOC(Kmag,     "save_stdcat", float , kcol);
    ED_MALLOC(Ierr,     "save_stdcat", float , kcol);
    ED_MALLOC(Jerr,     "save_stdcat", float , kcol);
    ED_MALLOC(Kerr,     "save_stdcat", float , kcol);
    ED_MALLOC(Group,    "save_stdcat", long  , kcol);
    ED_MALLOC(SeqNr,    "save_stdcat", long  , kcol);
    for (i=0;i<kcol;i++) {
       strcpy(&Names[i*NAMES_LEN],      standards[i].name);
       strcpy(&Cats[i*CATS_LEN],        standards[i].Cat);
       strcpy(&Q[i*Q_LEN],              standards[i].Q);
       strcpy(&Spectrum[i*SPECTRUM_LEN],standards[i].Spectrum);
       strcpy(&Cross_ID[i*CROSS_ID_LEN],standards[i].Cross_ID);
       strcpy(&Iref[i*IREF_LEN],        standards[i].Iref);
       strcpy(&Jref[i*JREF_LEN],        standards[i].Jref);
       strcpy(&Kref[i*KREF_LEN],        standards[i].Kref);
       Qual[i*QUAL_LEN]               = standards[i].Qual;
       SeqNr[i]                       = i+1;
       ra[i]                          = standards[i].ra;
       dec[i]                         = standards[i].dec;
       epoch[i]                       = standards[i].epoch;
       Imag[i]                        = standards[i].Imag;
       Jmag[i]                        = standards[i].Jmag;
       Kmag[i]                        = standards[i].Kmag;
       Ierr[i]                        = standards[i].Ierr;
       Jerr[i]                        = standards[i].Jerr;
       Kerr[i]                        = standards[i].Kerr;
       Group[i]                       = standards[i].Group;
    }
    add_key_to_tab(tab, "SeqNr", kcol, SeqNr, T_LONG, 1, "",
                   "Standard star sequence number");
    add_key_to_tab(tab, "Name", kcol, Names, T_STRING, NAMES_LEN, "",
                   "Catalog names of standard star");
    add_key_to_tab(tab, "Ra", kcol, ra, T_DOUBLE, 1, "",
                   "Right Ascension (at epoch)");
    add_key_to_tab(tab, "Dec", kcol, dec, T_DOUBLE, 1, "",
                   "Declination (at epoch)");
    add_key_to_tab(tab, "Epoch", kcol, epoch, T_FLOAT, 1, "",
                   "Epoch of reference coordinates");
    add_key_to_tab(tab, "Q", kcol, Q, T_STRING, 1, "",
                   "Quality of standard");
    add_key_to_tab(tab, "Group", kcol, Group, T_LONG, 1, "",
                   "Group of standard");
    add_key_to_tab(tab, "V", kcol, V, T_FLOAT, 1, "Mag",
                   "V magnitude of standard");
    add_key_to_tab(tab, "Imag", kcol, Imag, T_FLOAT, 1, "Mag",
                   "I magnitude of standard");
    add_key_to_tab(tab, "Jmag", kcol, Jmag, T_FLOAT, 1, "Mag",
                   "J magnitude of standard");
    add_key_to_tab(tab, "Kmag", kcol, Kmag, T_FLOAT, 1, "Mag",
                   "K magnitude of standard");
    add_key_to_tab(tab, "Catagory", kcol, Cats, T_STRING, CATS_LEN, "",
                   "Catagory of standard");
    add_key_to_tab(tab, "Quality", kcol, Qual, T_STRING, QUAL_LEN, "",
                   "Quality of standard");
    add_key_to_tab(tab, "Ierr", kcol, Ierr, T_FLOAT, 1, "Mag",
                   "Estimated error in I magnitude");
    add_key_to_tab(tab, "Jerr", kcol, Jerr, T_FLOAT, 1, "Mag",
                   "Estimated error in J magnitude");
    add_key_to_tab(tab, "Kerr", kcol, Kerr, T_FLOAT, 1, "Mag",
                   "Estimated error in K magnitude");
    add_key_to_tab(tab, "Iref", kcol, Iref, T_STRING, IREF_LEN, "",
                   "I references");
    add_key_to_tab(tab, "Jref", kcol, Jref, T_STRING, JREF_LEN, "",
                   "J references");
    add_key_to_tab(tab, "Kref", kcol, Kref, T_STRING, KREF_LEN, "",
                   "K references");
    add_key_to_tab(tab, "Spectrum", kcol, Spectrum, T_STRING, SPECTRUM_LEN, "",
                   "Spectral type");
    add_key_to_tab(tab, "Cross_ID", kcol, Cross_ID, T_STRING, CROSS_ID_LEN, "",
                   "Cross reference");
    update_tab(tab);
    if (add_tab(tab, cat, 0) == RETURN_ERROR) {
       syserr(FATAL,"main","Error adding table to catalog\n");
    }
    save_cat(cat, filename);
}

double HMStoDEG(float h, float m, float s)
{
    double Ra;

    Ra = 15.0*((double)h+((double)m+((double)s/60.0))/60.0);
    return (Ra);
}

double DMStoDEG(char teken, float d, float m, float s)
{
/*
    float sign = (d>=0?1.0:-1.0);
 */
    double sign = 1.0;
    if (teken == '-') sign = -1.0;

    d = fabs(d);
    return (sign*((double)d+((double)m+((double)s/60.0))/60.0));
}


int main(int argc, char *argv[])
{ 
    catstruct	*outcat;
    tabstruct 	*tab;
    FILE	*fd;
    stdstruct   star;
    float       hra, mra, sra, dec, mdec, sdec;
    char        teken;
    int		idum1, kk=0;

    init(argc, argv);

/*
 *  Changes because init_cat is available now
 *
 *   dumcat = read_cat(control.dumcatfile);
 *   outcat = new_cat(1);
 *   inherit_cat(dumcat, outcat);
 */
    outcat = new_cat(1);
    init_cat(outcat);
    tab = create_stdtab();

    ED_FOPEN(fd, "main", control.incatfile, "rt");
/*
    while (fscanf(fd, "%11c %f %f %f %1c%f %f %f %f %1c %d %f %f %f %f %3c %3c %f %f %f %3c %3c %3c %s %s \n",
             star.name,&hra, &mra, &sra, &teken, &dec, &mdec, &sdec, 
             &star.epoch, 
             &star.Qual,&star.Group,&star.V, &star.Imag, &star.Jmag, &star.Kmag,
             star.Cat, star.Q, &star.Ierr, &star.Jerr, &star.Kerr,
             star.Iref, star.Jref, star.Kref, star.Spectrum, star.Cross_ID)
             != EOF) {
*/
    while (fscanf(fd, "%d %f %f %f %1c%f %f %f %f %f %f %f %s\n",
             &idum1,&hra, &mra, &sra, &teken, &dec, &mdec, &sdec, 
             &star.epoch, &star.Imag, &star.Jmag, &star.Kmag, star.name)
             != EOF) {
         star.ra  = HMStoDEG(hra, mra, sra);
         star.dec = DMStoDEG(teken, dec, mdec, sdec);
         star.name[11]='\0';
         star.Qual='\0';
         star.Cat[3]='\0';
         star.Q[3]='\0';
         star.Iref[3]='\0';
         star.Jref[3]='\0';
         star.Kref[3]='\0';
         star.Spectrum[5]='\0';
         star.Cross_ID[22]='\0';
         printf("%d\n", kcol);
         add_stdtab_line(star);  
    }
    printf("%d\n", kk);
    save_stdcat(outcat, tab, control.outcatfile);

    return 0;
}
