#include <stdio.h>
#include <string.h>
#include <math.h>

#include "eclipse.h"
#include "prefs_linedetect.h"

#define __PROGRAM__ "linedetect"
#define __VERS__ "0.0.1"

#define __USAGE__ "\
USAGE:\n\
  test\n"

/*
 * 30.05.2004:
 * from the S/N radon map we subtract a smoothed version
 * to get rid of local Maxima (due to large scale sky background
 * variations for instance
 *
 * 23.01.2005:
 * I changed the location of the actual prefstruct definition
 * to avoid double existence of it during the link process.
 *
 * 27.05.2010:
 * removal of compiler warnings
 *
 * 24.08.2010:
 * - Calls to functions getpeaks were done with too many arguments
 *   (only noticed by Intels icc compiler!!)
 * - I resolved inconsistencies about the definition of the 'usage'
 *   function. It's surprising that the code ever compiled :-))
 */

#define MAX(x,y) ((x)>(y)?(x):(y))
#define MIN(x,y) ((x)<(y)?(x):(y))

static void usage(void) ;
image_t ** image_radon(image_t *input, 
                       float pmin, 
                       float pmax, 
                       float deltap, 
                       float deltatau);
float ** getpeaks(image_t *radon, int *npeaks);
prefstruct	prefs;

int main(int argc, char **argv)
{
  int             narg, a, part = 1;
  char	          **argkey, **argval;
  image_t         *input;
  image_t         *flag;
  image_t         *binned;
  image_t         **radon_orig;
  image_t         **radon_transpose;
  image_t         *transpose;
  qfits_header    *fh_radon_orig;
  qfits_header    *fh_radon_transpose;
  qfits_header    *fh_bin;
  qfits_header    *fh_transpose;
  char            tmpstr[MAXCHAR];
  int             npeaks, npeaks_transp, i, npeaksactu;
  int             accept = 0;
  float           **peakdata;
  float           **peakdata_transp;
  float           slope;
  float           deltaptranspose;
  float           thresholdorig;
  float           thresholdtranspose;
  FILE            *origpeaks;
  FILE            *tranpeaks;
  pixelvalue      median;

  narg = 0;
  QMALLOC(argkey, char *, argc);
  QMALLOC(argval, char *, argc);
  strcpy(prefs.prefs_name, "default.linedetect");


  /* read command line arguments */
  if(argc < 2)
  {
	usage();
  }

  for (a=1; a<argc; a++)
  {
    if (argv[a][0] == '-')
    {
      if (strlen(argv[a])<3)
      {
        switch((int)tolower((int)argv[a][1]))
        {
/*-------- Config filename */
          case 'h':	usage(); 
			break;
          case 'd':	dumpprefs();
       		        exit(0);
			break;
          case 'c':	if(a<(argc-1))
	                {
                          strncpy(prefs.prefs_name, argv[++a], MAXCHAR-1); 
			}
	                else
		        {
			    usage();  
			}
			break;
          default:	usage();
		        break;
        }
      }
      else
      {
/*------ Config parameters */
        argkey[narg] = &argv[a][1];
        argval[narg++] = argv[++a];
      }       
    }
    else
    {
/*---- The input image filename */
      strcpy(prefs.inname, argv[a]);
    }
  }

  readprefs(prefs.prefs_name, argkey, argval, narg);

  free(argkey);
  free(argval);
  
  /*
   * Initialize eclipse environment.
   */
  eclipse_init() ;
  set_verbose(1); /* print comment messages */

  /* load input frame */
  e_comment(0, "-> Part %d: loading input image: %s", part++, prefs.inname);
  input = image_load(prefs.inname);

  if (input==NULL) 
  {
      e_error("cannot load image [%s]: aborting", prefs.inname);
      return -1 ;
  }


  /* flag input image if asked for */
  if(prefs.flagflag == 1)
  {
    e_comment(0, "-> Part %d: flag input image with: %s", part++, prefs.flag);
    flag = image_load(prefs.flag);

    if (flag==NULL) 
    {
        e_error("cannot load flag image [%s]: aborting", prefs.flag);
        return -1 ;
    }

    /* invert flag map */
    for(i=0; i < flag->lx*flag->ly; i++)
    {
      if(flag->data[i] > 0)
      {
	flag->data[i] = 0;
      }
      else
      {
	flag->data[i] = 1;
      }
    }

    image_mul_local(input, flag);

    /* we do not need the flag image any more */
    image_del(flag);
  }

  /* bin image if asked for */
  binned=image_bin(input, prefs.binning);

  median=image_getmedian(binned);
  printf("median of binned image: %f\n", median);

  image_cst_op_local(binned, (double)median, '-');

  /* calculate the transposed from the original frame */
  e_comment(0, "-> Part %d: build transposed image image", part++);
  transpose=image_transpose(binned);

  /* estimate radon transform from original frame */
  e_comment(0, "-> Part %d: estimate radon transform from original frame", 
            part++);
  radon_orig=image_radon(binned, prefs.pmin, prefs.pmax, prefs.deltap, 1.0);

  median=image_getmedian(radon_orig[2]);
  printf("median of sn map: %f\n", median);

  /*image_cst_op_local(radon_orig[2], (double)median, '-');*/
  peakdata=getpeaks(radon_orig[2], &npeaks);
  printf("found %d peaks in original image\n", npeaks);

  origpeaks = fopen(prefs.origpeaksfile, "w");
  if(origpeaks == NULL)
  {
    e_error("Error opening file: %s", prefs.origpeaksfile);
    exit(1);
  }

  /* search peaks; do it so long until only maximal 10
     peaks are listed; if there are more we increase
     significance by 1 until we get below 10
  */
  thresholdorig=prefs.peakthreshold;
  accept = 0;

  for(i=0; i< npeaks; i++) printf("%d %f\n", npeaks, peakdata[2][i]);
  while(accept == 0)
  {
    npeaksactu = 0;

    for(i=0; i< npeaks; i++)
    {
      if(peakdata[2][i] > thresholdorig)
      {
	npeaksactu++;
      }
    }
    if(npeaksactu <= 8)
    {
      accept = 1;
    }
    thresholdorig=thresholdorig+1.;
  }

  thresholdorig=thresholdorig-1.;

  for(i=0; i< npeaks; i++)
  {
    if(peakdata[2][i] > thresholdorig)
    {
      slope = prefs.pmin+peakdata[0][i]*prefs.deltap;

      fprintf(origpeaks, "%f %f %f\n", slope,
                         (peakdata[1][i]-
                         slope*(int)(binned->lx / 2))*prefs.binning-
                         slope*prefs.xshift+prefs.yshift,
                         peakdata[2][i]);
    }
  }

  fclose(origpeaks);

  /* estimate radon transform from transposed frame */
  e_comment(0, "-> Part %d: estimate radon transform from transposed frame", part++);

  /* adapt deltap so that we sample the same distances */
  deltaptranspose=prefs.deltap/(transpose->lx/binned->lx);
  radon_transpose=image_radon(transpose, prefs.pmin, prefs.pmax, 
                              deltaptranspose, 1.0);

  median=image_getmedian(radon_transpose[2]);
  printf("median of sn map: %f\n", median);

  /*image_cst_op_local(radon_transpose[2], (double)median, '-');*/
  peakdata_transp=getpeaks(radon_transpose[2], &npeaks_transp);
  printf("found %d peaks in transposed image\n", npeaks_transp);

  tranpeaks = fopen(prefs.tranpeaksfile, "w");

  thresholdtranspose = prefs.peakthreshold*sqrt(transpose->lx/binned->lx);
  accept = 0;

  while(accept == 0)
  {
    npeaksactu = 0;

    for(i=0; i< npeaks_transp; i++)
    {
      if(peakdata_transp[2][i] > thresholdtranspose)
      {
	npeaksactu++;
      }
    }
    if(npeaksactu <= 8)
    {
      accept = 1;
    }
    thresholdtranspose=thresholdtranspose+1.;
  }

  thresholdtranspose=thresholdtranspose-1.;


  for(i=0; i< npeaks_transp; i++)
  {
    if(peakdata_transp[2][i] > thresholdtranspose)
    {
      slope = prefs.pmin+peakdata_transp[0][i]*deltaptranspose;

      fprintf(tranpeaks, "%f %f %f\n", slope, 
                         (peakdata_transp[1][i]-
                         slope*(int)(transpose->lx / 2))*prefs.binning- 
                         slope*prefs.yshift+prefs.xshift, 
                         peakdata_transp[2][i]);
    }
  }

  fclose(tranpeaks);

  /* save results */

  if(prefs.binnedsaveflag == 1)
  {
    e_comment(0, "-> Part %d: saving binned frame: %s", part++, prefs.binned);

    /* create image header for transposed frame */
    fh_bin = qfits_header_default();
  
    /* BITPIX */
    sprintf(tmpstr, "%d", BPP_IEEE_FLOAT);
    qfits_header_add(fh_bin, "BITPIX", tmpstr, "Bits per pixel", NULL);
    /* NAXIS* */
    qfits_header_add(fh_bin, "NAXIS", "2", "Number of axes", NULL);
    sprintf(tmpstr, "%d", binned->lx);
    qfits_header_add(fh_bin, "NAXIS1", tmpstr, "size in X", NULL);
    sprintf(tmpstr, "%d", binned->ly);
    qfits_header_add(fh_bin, "NAXIS2", tmpstr, "size in Y", NULL);

    image_save_fits_hdrdump(binned,
     			    prefs.binned,
     			    fh_bin,
     			    BPP_IEEE_FLOAT);

    /* delete data from transposed frame */
    qfits_header_destroy(fh_bin);
  }

  /* save transposed frame if asked for */

  if(prefs.transposesaveflag == 1)
  {
    e_comment(0, "-> Part %d: saving transposed frame: %s", 
              part++, prefs.transpose);

    /* create image header for transposed frame */
    fh_transpose = qfits_header_default();
  
    /* BITPIX */
    sprintf(tmpstr, "%d", BPP_IEEE_FLOAT);
    qfits_header_add(fh_transpose, "BITPIX", tmpstr, "Bits per pixel", NULL);
    /* NAXIS* */
    qfits_header_add(fh_transpose, "NAXIS", "2", "Number of axes", NULL);
    sprintf(tmpstr, "%d", transpose->lx);
    qfits_header_add(fh_transpose, "NAXIS1", tmpstr, "size in X", NULL);
    sprintf(tmpstr, "%d", transpose->ly);
    qfits_header_add(fh_transpose, "NAXIS2", tmpstr, "size in Y", NULL);

    image_save_fits_hdrdump(transpose,
     			    prefs.transpose,
     			    fh_transpose,
     			    BPP_IEEE_FLOAT);

    /* delete data from transposed frame */
    qfits_header_destroy(fh_transpose);
  }  


  /* create image header for radon and associated frames from
     original image 
  */
  fh_radon_orig = qfits_header_default();

  /* BITPIX */
  sprintf(tmpstr, "%d", BPP_IEEE_FLOAT);
  qfits_header_add(fh_radon_orig, "BITPIX", tmpstr, "Bits per pixel", NULL);
  /* NAXIS* */
  qfits_header_add(fh_radon_orig, "NAXIS", "2", "Number of axes", NULL);
  sprintf(tmpstr, "%d", radon_orig[0]->lx);
  qfits_header_add(fh_radon_orig, "NAXIS1", tmpstr, "size in X", NULL);
  sprintf(tmpstr, "%d", radon_orig[0]->ly);
  qfits_header_add(fh_radon_orig, "NAXIS2", tmpstr, "size in Y", NULL);

  /* create image header for radon and associated frames from
     transposed image 
  */
  fh_radon_transpose = qfits_header_default();

  /* BITPIX */
  sprintf(tmpstr, "%d", BPP_IEEE_FLOAT);
  qfits_header_add(fh_radon_transpose, "BITPIX", tmpstr, "Bits per pixel", NULL);
  /* NAXIS* */
  qfits_header_add(fh_radon_transpose, "NAXIS", "2", "Number of axes", NULL);
  sprintf(tmpstr, "%d", radon_transpose[0]->lx);
  qfits_header_add(fh_radon_transpose, "NAXIS1", tmpstr, "size in X", NULL);
  sprintf(tmpstr, "%d", radon_transpose[0]->ly);
  qfits_header_add(fh_radon_transpose, "NAXIS2", tmpstr, "size in Y", NULL);

  if(prefs.radonsaveflag == 1)
  {
    e_comment(0, "-> Part %d: saving radon transforms", part++);

    image_save_fits_hdrdump(radon_orig[0],
       			    prefs.radon_orig,
     			    fh_radon_orig,
     			    BPP_IEEE_FLOAT);    

    image_save_fits_hdrdump(radon_transpose[0],
     			    prefs.radon_tran,
     			    fh_radon_transpose,
     			    BPP_IEEE_FLOAT);
  }

  if(prefs.noisesaveflag == 1)
  {
    e_comment(0, "-> Part %d: saving noise estimates from radon transforms", 
              part++);

    image_save_fits_hdrdump(radon_orig[1],
     			    prefs.noise_orig,
     			    fh_radon_orig,
     			    BPP_IEEE_FLOAT);

    image_save_fits_hdrdump(radon_transpose[1],
     			    prefs.noise_tran,
     			    fh_radon_transpose,
     			    BPP_IEEE_FLOAT);
  }
  
  if(prefs.snsaveflag == 1)
  {
    e_comment(0, "-> Part %d: saving S/N estimates from radon transforms", 
              part++);

    image_save_fits_hdrdump(radon_orig[2],
     			    prefs.sn_orig,
     			    fh_radon_orig,
     			    BPP_IEEE_FLOAT);

    image_save_fits_hdrdump(radon_transpose[2],
     			    prefs.sn_tran,
     			    fh_radon_transpose,
     			    BPP_IEEE_FLOAT);
  }

  /* release memory and bye */
  image_del(input);  
  image_del(binned);
  image_del(transpose);
  qfits_header_destroy(fh_radon_orig);
  qfits_header_destroy(fh_radon_transpose);

  for(i=0; i<3; i++)
  {
    image_del(radon_orig[i]);
    image_del(radon_transpose[i]);
    free(peakdata[i]);
    free(peakdata_transp[i]);
  }

  free(radon_orig);
  free(radon_transpose);
  free(peakdata);
  free(peakdata_transp);

  xmemory_status() ;
  return 0;
}

image_t ** image_radon(image_t *input, 
                       float pmin, 
                       float pmax, 
                       float deltap, 
                       float deltatau)
{
  image_t **radon;
  image_t *smooth;
  float alpha, beta;
  int mmin, mmax;
  int pfinal, taufinal;
  int taumin, taumax;
  int xmin, xmax, ymin, ymax;
  int h, k, m, n, i;
  int npix, maxpix;
  float var;
  pixelvalue *values;
  pixelvalue sum;
  pixelvalue dummy;

  /* check input image */
  if(input == NULL) return NULL;
  
  maxpix = (int)(sqrt(input->lx*input->lx+input->ly*input->ly)) + 1;
  values=(pixelvalue *)(malloc(maxpix*sizeof(pixelvalue)));

  /* quantities for radon transform */
  taumin = 0;
  taumax = (int)(input->ly);
  /*taumin = -(int)(input->lx / 4);
    taumax = (int)(input->ly)+(int)(input->lx / 4);*/
  xmin = -(int)(input->lx / 2);
  xmax = (int)(input->lx)-(int)(input->lx / 2);
  ymin = 0;
  ymax = (int)(input->ly);

  pfinal = (int)((pmax - pmin)/deltap);
  taufinal = (int)((taumax - taumin)/deltatau);

  radon=(image_t **)(malloc(3*sizeof(image_t *)));

  radon[0] = image_new(pfinal, taufinal) ;
  radon[1] = image_new(pfinal, taufinal) ;
  radon[2] = image_new(pfinal, taufinal) ;

  for(k=0; k< pfinal; k++)
  {
    for(h=0; h < taufinal; h++)
    {
      alpha = pmin+(k*deltap);
      if(fabs(alpha) < 1.0e-06)
      {
	alpha=0.005;
      }
      beta = alpha*xmin+(taumin+(h*deltatau))-ymin;

      sum = 0.0;
      npix = 0;
      var = 0.0;

      if(fabs(alpha) < 1.0e-06)
      {
	mmin = 0;
	mmax = input->lx;
      }
      else
      {
        if(alpha > 0)
        {
          mmin = (int)MAX(0, (int)(-(beta/alpha))) + 1;
          mmax = (int)MIN(input->lx,(int)((input->ly-0.5-beta)/alpha)) - 1;
        }
	else
	{
	  mmin = (int)MAX(0, (int)((input->ly-0.5-beta)/alpha)) + 1;
	  mmax = (int)MIN(input->lx, (int)(-(beta/alpha))) - 1;
	}
      }

      for(m = mmin; m < mmax; m++)
      {
	n=(int)(alpha*m+beta);
	dummy = input->data[n*input->lx+m];
	if(fabs(dummy) > 1.0e-06 && fabs(dummy) < prefs.datathreshold)
	{
	  values[npix]=dummy;
          /*sum += dummy;
	    var += dummy*dummy;*/
	  npix += 1;
	}
      }

      pixel_qsort(values, npix);
      npix=(int)(npix*(100.-prefs.highcut)/100.);

      if(npix>50)
      {
        for(i=0; i < npix; i++)
        {
  	  sum+=values[i];
  	  var+=values[i]*values[i];
        }
  
        radon[0]->data[h*radon[0]->lx+k]=sum/npix;
        radon[1]->data[h*radon[1]->lx+k]=sqrt(var/npix-(sum*sum/(npix*npix)));
        radon[2]->data[h*radon[2]->lx+k]=(sum/sqrt(npix))/sqrt(var/npix-(sum*sum/(npix*npix)));
      }
      else
      {
        radon[0]->data[h*radon[0]->lx+k]=0.;
        radon[1]->data[h*radon[1]->lx+k]=0.;
        radon[2]->data[h*radon[2]->lx+k]=0.;
      }
    }
  }

  free(values);

  smooth=image_filter_flat(radon[2], 10);
  image_sub_local(radon[2], smooth);
  image_del(smooth);

  return radon;
}

float **getpeaks(image_t *radon, int *npeaks)
{
  static int in[8] = {-1, -1, -1, 0, 1, 1, 1, 0};
  static int jn[8] = {-1, 0, 1, 1, 1, 0, -1, -1};
  int hasbiggerneighbour;
  int peakmemory = 1000;
  float dummy;
  float **peakdata;
  float *dummypointer;

  int i, j, l, m, n, o;

  peakdata=(float **)malloc(3*sizeof(float *));
  peakdata[0]=(float *)malloc(peakmemory*sizeof(float));  
  peakdata[1]=(float *)malloc(peakmemory*sizeof(float));  
  peakdata[2]=(float *)malloc(peakmemory*sizeof(float));  

  *npeaks = 0;

  for (i = 0; i <  radon->lx; i++) 
  {
    for (j = 0; j < radon->ly; j++) 
    {
	
      hasbiggerneighbour = 0;
      for (n = 0; n < 8; n++) 
      {
	m=j + jn[n];
	o=i + in[n];

	if((o>=0) && (m>=0) && (o<radon->lx) && (m<radon->ly))
	{ 
          dummy = radon->data[(m)*radon->lx+(o)];
	
          if (dummy > radon->data[j*radon->lx+i])
          {
            hasbiggerneighbour = 1;
            break;
          }
	}
      }
      if(hasbiggerneighbour)
      {
        continue;
      }
      else
      {
        peakdata[0][*npeaks]=i;
        peakdata[1][*npeaks]=j;
        peakdata[2][*npeaks]=radon->data[j*radon->lx+i];

        (*npeaks)++;

        if(*npeaks == peakmemory)
        {
          peakmemory+=1000;
	  for(l=0; l<3; l++)
	  {
	    /* simulate realloc as it's not supported
               by the xmemory library */
	    dummypointer=peakdata[l];
	    peakdata[l]=(float *)malloc(peakmemory*sizeof(float));
	    memcpy(peakdata[l], dummypointer, (peakmemory-1000)*sizeof(float));
	    free(dummypointer);
	  }
        }
      }
    }
  }

  return peakdata;
}

static void usage(void)
{
    printf("%s (V%s)\n\n%s", __PROGRAM__, __VERS__, __USAGE__) ;
    exit(1) ;
} 






