#!/bin/bash

# 22.03.2000:
# changed the tensor calculation to the full tensor expression
#
# 23.11.2004:
# the initial filtering also filters for large rg now (stellar
# quantities are only estimated up to rg=10)
#
# 12.07.2005:
# Before estimating weights for galaxies, stars are now filtered.

. lensing.ini

BASE=`basename $1 anisocorr.cat`
DIR=`dirname $1`

${P_LDACFILTER} -i $1 -o ${DIR}/${BASE}anisocorrfilt.cat -c "(rg>1.0)AND(rg<9.0);"
${P_ASCTOLDAC} -i $2 -o PshPsm.cat -t PSHSTPSMST \
                     -c ${LENSCONF}/PshstPsmst.asctoldac.conf
${P_LDACADDTAB} -i  ${DIR}/${BASE}anisocorrfilt.cat \
                      -o ${DIR}/${BASE}pshpsm.cat -p PshPsm.cat -t PSHSTPSMST
${P_LDACCALC} -i ${DIR}/${BASE}pshpsm.cat -o tmp0_$$.cat \
                    -c "((rg+0.5));" -n pshpsmlink "" -k SHORT
${P_MAKEJOIN} -i tmp0_$$.cat -o tmp1_$$.cat \
                    -c ${LENSCONF}/addPshPsm.make_join.conf \
                    -m OBJECTS -r PSHSTPSMST

${P_LDACCALC} -i tmp1_$$.cat -o tmp2_$$.cat -t OBJECTS \
	   -c "0.5*(Psh11Psm11_MEAN+Psh22Psm22_MEAN);" -n PshstPsmst_RG ""\
           -k DOUBLE \
           -c "(sqrt(e1*e1+e2*e2));"                   \
           -n "e" "modulus of ellipticity" -k DOUBLE

${P_LDACCALC} -i tmp2_$$.cat -o tmp3_$$.cat -t OBJECTS \
         -c "(Psh11-PshstPsmst_RG*Psm11);"           -n "Pgamma11_RG" ""\
         -k DOUBLE \
         -c "(Psh22-PshstPsmst_RG*Psm22);"           -n "Pgamma22_RG" ""\
         -k DOUBLE \
         -c "(Psh21-PshstPsmst_RG*Psm21);"           -n "Pgamma21_RG" ""\
         -k DOUBLE \
         -c "(Psh12-PshstPsmst_RG*Psm12);"           -n "Pgamma12_RG" ""\
         -k DOUBLE

${P_LDACCALC} -i tmp3_$$.cat -o tmp4_$$.cat -t OBJECTS \
         -c "(0.5*(Pgamma11_RG+Pgamma22_RG));"       -n "Pgamma_RG" ""\
         -k DOUBLE \
         -c "(0.5*(Pgamma21_RG+Pgamma12_RG));"       -n "Pgamma_off_RG" ""\
         -k DOUBLE

${P_LDACCALC} -i tmp4_$$.cat -o tmp5_$$.cat -t OBJECTS \
	   -c "(e1anisocorr/Pgamma_RG);"               -n "e1iso_RG" ""\
           -k DOUBLE \
	   -c "(e2anisocorr/Pgamma_RG);"               -n "e2iso_RG" ""\
           -k DOUBLE \
	   -c "(e1/Pgamma_RG);"               -n "e1isonocorr_RG" ""\
           -k DOUBLE \
	   -c "(e2/Pgamma_RG);"               -n "e2isonocorr_RG" ""\
           -k DOUBLE \
	   -c "(e2anisocorr*Pgamma12_RG-e1anisocorr*Pgamma22_RG)/(Pgamma12_RG*Pgamma21_RG-Pgamma11_RG*Pgamma22_RG);" \
	   -n "e1isotensor_RG" ""        -k DOUBLE \
	   -c "(e1anisocorr*Pgamma21_RG-e2anisocorr*Pgamma11_RG)/(Pgamma12_RG*Pgamma21_RG-Pgamma11_RG*Pgamma22_RG);" \
	   -n "e2isotensor_RG" ""        -k DOUBLE \
	   -c "(Pgamma12_RG*Pgamma21_RG-Pgamma11_RG*Pgamma22_RG);" -n "det_RG" ""         -k DOUBLE

${P_LDACFILTER} -i tmp5_$$.cat -o tmp6_$$.cat \
                -c "((((e1iso_RG<1.5)AND(e1iso_RG>-1.5))AND(e2iso_RG<1.5))AND(e2iso_RG>-1.5))AND(cl=0);"
${P_SIGMAE} -i tmp6_$$.cat -o tmp7_$$.cat -c rg snratio -e e1iso_RG e2iso_RG -t OBJECTS -n 20

${P_SIGMAE} -i tmp7_$$.cat -o tmp8_$$.cat -c rg snratio -e e1isotensor_RG \
e2isotensor_RG -t OBJECTS -n 20 -s Sigmae_tensor

${P_LDACCALC} -i tmp8_$$.cat \
           -t OBJECTS -o ${DIR}/${BASE}final.cat \
           -c "(1./(Sigmae*Sigmae));"                -n "weight" ""\
           -k DOUBLE \
           -c "(1./(Sigmae_tensor*Sigmae_tensor));"  -n "weight_tensor" ""\
           -k DOUBLE

rm tmp*_$$.cat








