#!/bin/bash

# script to estimate photometric zeropoints for uncalibrated
# fields with overlap sources in calibrated data; it is done by simple
# magnitude comparison.

#$1: uncalibrated catalgue
#$2 .. $n: calibration catalogues

# source routines to create config files on the fly:
. calibrate_zeropoint_include

# we assume that all catalogues starting from the second
# have valid magnitudes:

ldacpaste -i $@ \
          -t OBJECTS -o pasted.cat_$$
associate -i pasted.cat_$$ \
          -o associated.cat_$$ -c associate_calibrate_zeropoint.conf_$$
make_ssc -i associated.cat_$$ \
         -o ssc.cat_$$ -c make_ssc_calibrate_zeropoint.conf_$$

test -f tmp.asc && rm tmp.asc

ldactoasc -b -i ssc.cat_$$ -t PAIRS -s |\
  awk 'BEGIN { n = 0; mean = 0; sigma = 0 } { 
          if((($1 > 16) && ($1 < 20) && ($2 < 0)) || 
             (($2 > 16) && ($2 < 20) && ($1 < 0))) {
            if($1 < 0) {
              print $2, $1, $2 - $1 - 1;
            } 
            else {
              print $1, $2, $1 - $2 - 1;
            }; 
          }
        }' > tmp.asc_$$
  awk 'BEGIN {n=1} {if($1 < 20.0 && $1 > 17.0 && $2 < 1.0) {
                       oldline[n] = $0;
                       newline[n] = $0; 
                       oldzp[n]   = $3; 
                       newzp[n]   = $3;
                       n++;
                      }} 
               END {if(n>20) {maxiter = 20} else {maxiter=0}
                    oldnelem = 0; 
                    newnelem = n; 
                    actuiter = 0;
                    while (actuiter <= maxiter && oldnelem != newnelem) {
                      zpmean = 0.0; zpsdev = 0.0;
                      for(i = 1; i <= newnelem; i++) {
                        zpmean += newzp[i]; 
                        zpsdev += newzp[i]*newzp[i];
                      } zpmean /= newnelem;
                      if(newnelem > 2) {
                        zpsdev = sqrt(zpsdev/newnelem-zpmean*zpmean)
                      } else {
                        zpsdev = 0.0
                      }
                      
                      oldnelem = newnelem;
                      if(zpsdev > 0.0 && actuiter < maxiter)
                      {
                        newnelem = 0;
                        for(i = 1; i <= oldnelem; i++) {
                          oldzp[i]   = newzp[i];
                          oldline[i] = newline[i];
                        }
                        for(i = 1; i <= oldnelem; i++) {
                          if((sqrt((oldzp[i]-zpmean)*(oldzp[i]-zpmean)) < 2.5 * zpsdev) &&
                             (sqrt((oldzp[i]-zpmean)*(oldzp[i]-zpmean)) < 1.0))
                          {
                            newzp[++newnelem] = oldzp[i];
                            newline[newnelem] = oldline[i];
                          }
                        }
                      }
                      actuiter++;
                    }
                    for(i=1; i < oldnelem; i++) {
                      printf("%s %.4f %.4f\n", newline[i], zpmean, zpsdev);
                    }
                   }' tmp.asc_$$ |\
  awk '{print $1, $3-$4, $4, $5}' > final_zp.asc

rm *_$$


