#!/bin/bash -u

# script to create model magnitudes with stellar Pickles models.
# These can be used to compare stellar tracks of measured magnitudes
# and to verify photometric calibration of multi-band observations.
#
# The script uses the Python module 'starcol.py' (within the BPZ directory).

#$1: instrument name
#$2: filters (in quotes) for which model magnitudes should be estimates,
#    e.g. "WFI_B123.res WFI_I.res"
#$3: AB or Vega (magnitude system used for the models)

INST=$1
FILTERS=$2
SYSTEM=$3

python starcol.py ${SYSTEM} ${FILTERS} \
   Pickles_${INST}_all.txt Pickles_${INST}_main.txt

# create a configuration file for asctoldac on the fly:
{
  echo 'COL_NAME  = SeqNr'
  echo 'COL_TTYPE = LONG'   
  echo 'COL_HTYPE = INT'     
  echo 'COL_COMM = ""'       
  echo 'COL_UNIT = ""'       
  echo 'COL_DEPTH = 1'
  echo '#'
  
  for FILT in ${FILTERS}
  do
    echo "COL_NAME  = MAG_${SYSTEM}_${FILT}"
    echo 'COL_TTYPE = DOUBLE'   
    echo 'COL_HTYPE = FLOAT'     
    echo 'COL_COMM = "model mag for '${FILT}'"'   
    echo 'COL_UNIT = "mag"'       
    echo 'COL_DEPTH = 1'
    echo '#'      
  done
} > asctoldac.conf_$$

awk 'BEGIN {n = 1} {if($0 !~ /#/) {print n, $0; n = n + 1}}' \
   Pickles_${INST}_all.txt > tmp.txt_$$
awk 'BEGIN {n = 1} {if($0 !~ /#/) {print n, $0; n = n + 1}}' \
   Pickles_${INST}_main.txt > tmp1.txt_$$

asctoldac -i tmp.txt_$$ -o Pickles_${SYSTEM}_${INST}_all.cat \
          -c asctoldac.conf_$$ -t ${INST}_${SYSTEM}

asctoldac -i tmp1.txt_$$ -o Pickles_${SYSTEM}_${INST}_main.cat \
          -c asctoldac.conf_$$ -t ${INST}_${SYSTEM}

# clean up and bye:
rm *_$$
exit 0;
