#!/usr/bin/env python

# -*- coding: utf-8 -*-

# script to create a list of moon phases.
# It outputs a list with entries like:
#
# -
# 09/01/2012 01f12
# 23/01/2012 02n12
# .
# read: The first full moon of the year 2012 is on 09/01.
# The first new moon in the year 2012 is on 23/01/2012
# We can use this list to sort astronomical data into runs
# that correspond to dark and bright moon phases; see e.g.
# 'moon_id.sh' and 'dogroup_OMEGACAM_KIDS.sh' for its usage
# within OMEGACAM@VST data processing.

# written by: Oliver Cordes 2012-07-06
# changed by: Oliver Cordes 2012-07-06

# standard python modules

import datetime
import sys

if len(sys.argv) < 3:
    print 'Too few parameters! %s start_year end_year ' % sys.argv[0]
    sys.exit(-2)

try:
    import ephem
except ImportError:
    print 'Please install the pyephem (for python 2.x) or ephem (for python >= 3)!'
    print 'The webpage is http://pypi.python.org/pypi/pyephem/ !'
    sys.exit(-1)

start_year = int(sys.argv[1])
end_year = int(sys.argv[2])

dt_start = datetime.datetime(
    start_year,
    1,
    1,
    0,
    0,
    0,
    )

dt_end = datetime.datetime(
    end_year,
    12,
    31,
    23,
    59,
    59,
    )

dt = dt_start
dt_new = ephem.next_new_moon(dt)
dt_full = ephem.next_full_moon(dt)

print dt_new, dt_full
if dt_new < dt_full:
    phase = 1
else:
    phase = 0

new_nr = 1
full_nr = 1
year = start_year
while dt <= dt_end:
    if phase == 0:
        edt = ephem.next_full_moon(dt)
    else:
        edt = ephem.next_new_moon(dt)

    dt = edt.datetime()

    if dt > dt_end:
        break

    if dt.year > year:
        year = dt.year
        new_nr = 1
        full_nr = 1

    if phase == 0:
        print '%s %02if%s' % (dt.strftime('%d/%m/%Y'), full_nr,
                              dt.strftime('%y'))
        full_nr += 1
        phase = 1
    else:
        print '%s %02in%s' % (dt.strftime('%d/%m/%Y'), full_nr,
                              dt.strftime('%y'))
        new_nr += 1
        phase = 0

# end

sys.exit(0)
