#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "fitscat.h"
#include "utils_globals.h"
#include "sort_globals.h"
#include "utils_macros.h"
#include "tabutil_globals.h"
#include "lists_globals.h"
#include "files_cat_globals.h"

/*
  15.07.01:
  modificantions to avoid compiler warnings
 
  29.01.05:
  I adapted calls to addhistoryto_cat to the new syntax of this function.

  27.03.2006:
  I renamed sorting and indexing functions to the new naming scheme
  described in the sort_globals.h file

  15.02.2010:
  removal of compiler warnings due to possibly undefined variables
*/

static char     *fitsfiles, *img_types, *obj_names, *filters, *instruments,
                *cameras;
static int      *night_nrs, *number_nrs, *seq_nrs, *field_nrs, *set_nrs;
static int      n_files, *strip_nrs;
catstruct       *incat;
tabstruct       *tab;

#ifndef MAXCHAR
#define MAXCHAR 256
#endif

static int      currl, blklen = 100, currblk = 1;
/**/
void create_files_tab()

/****** create_files_tab ***************************************************
 *
 *  NAME	
 *      create_files_tab - Create the table structure for files information
 *
 *  SYNOPSIS
 *      void create_files_tab()
 *
 *  FUNCTION
 *      This routine creates the table structure that will eventually 
 *      contain all the relavant information about all the files in a
 *      nights observation sequence.
 *
 *  INPUTS
 *      None
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *      The global table structure is created and the global variables
 *      are allocated.
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
{

/*
 *  Changed because init_cat is available now
 *
 *   catstruct   *dumcat;
 *
 *   dumcat = read_cat("/denisoft/data/dummy.cat");
 *   incat = new_cat(1);
 *   inherit_cat(dumcat, incat);
 *   close_cat(dumcat);
 */
    incat = new_cat(1);
    init_cat(incat);
    tab = new_tab("FILES");
    currl = 0;
    ED_CALLOC(fitsfiles,  "create_files_tab", char, (blklen*currblk*FILELEN));
    ED_CALLOC(img_types,  "create_files_tab", char, (blklen*currblk*IDENTLEN));
    ED_CALLOC(obj_names,  "create_files_tab", char, (blklen*currblk*OBJLEN));
    ED_CALLOC(filters,    "create_files_tab", char, (blklen*currblk*FILTLEN));
    ED_CALLOC(cameras,    "create_files_tab", char, (blklen*currblk*CAMLEN));
    ED_CALLOC(instruments,"create_files_tab", char, (blklen*currblk*INSTLEN));
    ED_CALLOC(night_nrs,  "create_files_tab", int,  (blklen*currblk));
    ED_CALLOC(strip_nrs,  "create_files_tab", int,  (blklen*currblk));
    ED_CALLOC(number_nrs, "create_files_tab", int,  (blklen*currblk));
    ED_CALLOC(seq_nrs,    "create_files_tab", int,  (blklen*currblk));
    ED_CALLOC(field_nrs,  "create_files_tab", int,  (blklen*currblk));
}
/**/
void add_file_to_tab(filestruct *f)

/****** add_file_to_tab ***************************************************
 *
 *  NAME	
 *      add_file_to_tab - Add the file information to the table structure
 *
 *  SYNOPSIS
 *      void add_file_to_tab(filestruct *f)
 *
 *  FUNCTION
 *      This routine appends the information retained in the arguments
 *      file structure to the already existing table structure and
 *      arrays of global variables.
 *
 *  INPUTS
 *
 *  RESULTS
 *
 *  RETURNS
 *
 *  MODIFIED GLOBALS
 *      The arrays of the global variables are appended with the provided
 *      information. If necessary, additional memory for global variables
 *      is allocated.
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *
 ******************************************************************************/
{
   
    if (currl >= currblk*blklen) {
       currblk++;
       ED_REALLOC(fitsfiles,  "add_file_to_tab", char,
                  (blklen*currblk*FILELEN));
       ED_REALLOC(img_types,  "add_file_to_tab", char,
                  (blklen*currblk*IDENTLEN));
       ED_REALLOC(obj_names,  "add_file_to_tab", char,
                  (blklen*currblk*OBJLEN));
       ED_REALLOC(filters,   "add_file_to_tab", char,
                  (blklen*currblk*FILTLEN));
       ED_REALLOC(cameras,   "add_file_to_tab", char,
                  (blklen*currblk*CAMLEN));
       ED_REALLOC(instruments,   "add_file_to_tab", char,
                  (blklen*currblk*INSTLEN));
       ED_REALLOC(night_nrs,  "add_file_to_tab", int,  (blklen*currblk));
       ED_REALLOC(strip_nrs,  "add_file_to_tab", int,  (blklen*currblk));
       ED_REALLOC(number_nrs, "add_file_to_tab", int,  (blklen*currblk));
       ED_REALLOC(seq_nrs,    "add_file_to_tab", int,  (blklen*currblk));
       ED_REALLOC(field_nrs,  "add_file_to_tab", int,  (blklen*currblk));
    }
    strncpy(&fitsfiles[currl*FILELEN],  f->fitsfile,   FILELEN);
    strncpy(&img_types[currl*IDENTLEN], f->ident,      IDENTLEN);
    strncpy(&obj_names[currl*OBJLEN],   f->object,     OBJLEN);
    strncpy(&filters[currl*FILTLEN],    f->filter,     FILTLEN);
    strncpy(&cameras[currl*CAMLEN],     f->camera,     CAMLEN);
    strncpy(&instruments[currl*INSTLEN],f->instrument, INSTLEN);
    night_nrs[currl]  = f->night;
    strip_nrs[currl]  = f->strip;
    number_nrs[currl] = f->number;
    seq_nrs[currl]    = f->seq_nr;
    field_nrs[currl]  = f->field;
    currl++;
}
/**/
void save_files_tab(char *filename, char *prog)

/****** save_files_tab **************************************************
 *
 *  NAME	
 *      save_files_tab - Save the table to the output catalog
 *
 *  SYNOPSIS
 *      void save_files_tab(char *filename, char *prog)
 *
 *  FUNCTION
 *      This routine actually saves the information stored in the table
 *      structure to the catalog file.
 *
 *  INPUTS
 *      filename  - The name of the output catalog
 *      prog      - The calling name of the program to be added to the history
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
{
    if (currl == 0) {
       syserr(FATAL, "save_files_tab",
        "No fits files found to save.\n  \
         Check pre- and postfix strings and/or configuration file\n");
    }
    add_key_to_tab(tab, "FITSFILE", currl, fitsfiles, T_STRING, FILELEN,
                   "", "Full path to fits file");
    add_key_to_tab(tab, "NIGHT", currl, night_nrs, T_LONG, 1,
                   "", "Night number");
    add_key_to_tab(tab, "STRIP", currl, strip_nrs, T_LONG, 1,
                   "", "Strip number");
    add_key_to_tab(tab, "Number", currl, number_nrs, T_LONG, 1,
                   "", "Running number");
    add_key_to_tab(tab, "OBSTYPE", currl, img_types, T_STRING, IDENTLEN,
                   "", "Observation type");
    add_key_to_tab(tab, "FILTER", currl, filters, T_STRING, FILTLEN,
                   "", "Filter Name");
    add_key_to_tab(tab, "CAMERA", currl, cameras, T_STRING, CAMLEN,
                   "", "Camera Name");
    add_key_to_tab(tab, "INSTRUMENT", currl, instruments, T_STRING, INSTLEN,
                   "", "Instrument Name");
    add_key_to_tab(tab, "OBJNAME", currl, obj_names, T_STRING, OBJLEN,
                   "", "Observation type");
    add_key_to_tab(tab, "Seq_Nr", currl, seq_nrs, T_LONG, 1,
                   "", "Running Sequence number (PHOTO)");
    add_key_to_tab(tab, "FIELD", currl, field_nrs, T_LONG, 1,
                   "", "Running Field number (REGULAR)");
    if (add_tab(tab,incat,0) == RETURN_ERROR) {
       syserr(FATAL,"save_files_tab","Cannot add FITSFILE table\n");
    }
    addhistoryto_cat(incat, prog, 1);
    save_cat(incat, filename);
}   
/**/
#define NKEYS 11

char *keynames[] = {"FITSFILE", "NIGHT", "STRIP", "Number", "OBSTYPE",
                    "FILTER", "CAMERA", "INSTRUMENT", "OBJNAME","Seq_Nr", 
                    "FIELD", "SET_NR"};

int read_files_tab(char *filename)

/****** read_files_tab **************************************************
 *
 *  NAME	
 *      read_files_tab - Read the files table and store everything globally
 *
 *  SYNOPSIS
 *      int read_files_tab(char *filename)
 *
 *  FUNCTION
 *      The routine reads the catalog from disk and stores the information
 *      from the FILES table into the global variables.
 *
 *  INPUTS
 *      filename - The pathname of the catalog file
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *     The number of elements in the table is returned.
 *
 *  MODIFIED GLOBALS
 *      The global variables are filled with the column information
 *      stored in the FILES table of the specified catalog.
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
{
/*
 *  Read in all the columns from the FILES table and store the results
 *  in global arrays.
 */
    keystruct	**keys, *key;
    int		nkeys = NKEYS;

    if ((incat = read_cat(filename)) == NULL) {
       syserr(FATAL, "read_files_tab", "Could not read catalog: %s\n",
              filename);
    }
    tab = name_to_tab(incat, "FILES", 0);
/*
 *   Check if the SET_NRs are included in the file
 */
    if ((key = name_to_key(tab, "SET_NR"))) {
       nkeys++;
    }
    ED_MALLOC(keys, "read_files_tab", keystruct *, nkeys);
    read_keys(tab, keynames, keys, nkeys, NULL);
    fitsfiles  = keys[0]->ptr;
    night_nrs  = keys[1]->ptr;
    strip_nrs  = keys[2]->ptr;
    number_nrs = keys[3]->ptr;
    img_types  = keys[4]->ptr;
    filters    = keys[5]->ptr;
    cameras    = keys[6]->ptr;
    instruments= keys[7]->ptr;
    obj_names  = keys[8]->ptr;
    seq_nrs    = keys[9]->ptr;
    field_nrs  = keys[10]->ptr;
    n_files = keys[0]->nobj;
    if (nkeys > NKEYS) {
       set_nrs = keys[11]->ptr;
    } else {
       ED_CALLOC(set_nrs, "read_files_tab", int, n_files);
    }

    return (n_files);
}
/**/
int number_of_sets()

/****** number_of_sets ***************************************************
 *
 *  NAME	
 *      number_of_sets - Return the number of sets in the FILES table
 *
 *  SYNOPSIS
 *      int number_of_sets()
 *
 *  FUNCTION
 *      This routine examines the table structure and returns the number of
 *      distinct sets stored in that table.
 *
 *  INPUTS
 *      None
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      The number of sets is returned. If an error occurs the routine
 *      calls the FATAL syserr function.
 *
 *  MODIFIED GLOBALS
 *      The number of elements global variable and the array of set numbers
 *      is used.
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
{
    int		*n, i, j;
    int         *l_set_nrs;

    ED_CALLOC(l_set_nrs, "number_of_sets", int, n_files);
    for (i=0,j=0;i<n_files;i++) 
       if (set_nrs[i] != 0) 
          l_set_nrs[j++] = set_nrs[i];
    n = uniq_int(l_set_nrs, &j);
    if (j == 1 && n[0] == 0)
       syserr(FATAL,"number_of_sets", "Catalog does not contain set numbers\n");
    ED_FREE(l_set_nrs);
    return (j);
}
/**/
char *get_filters(int n)

/****** get_filters ***************************************************
 *
 *  NAME	
 *      get_filter - Get the list of all different filters in the table
 *
 *  SYNOPSIS
 *      char *get_filters(int n)
 *
 *  FUNCTION
 *      This routine examines the global variable table structure and
 *      returns a list (concatinated string of characters) with the
 *      names of the different passbands.
 *
 *  INPUTS
 *      n  - Set number for which to get the channels
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      A null terminated string with passband names. The names consist of
 *      single characters. For instance for DENIS: IJK
 *
 *  MODIFIED GLOBALS
 *      The number of elements global variable and the array of channel 
 *      names is used.
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
{
    char	*s;
    int		i,j, k, num;

    ED_MALLOC(s, "get_filters", char, (n_files*FILTLEN));
    j = 0; num = 0;
    for (i=0;i<n_files;i++) {
       if (n == set_nrs[i] || n == 0 || n == UNKNOWN) {
          for (k=0;k<FILTLEN;k++) {
             s[j+k] = filters[i*FILTLEN+k];
          }
          j+=FILTLEN;
          num++;
       }
    }
    for (i=num;i<n_files;i++) {
       for (k=0;k<FILTLEN;k++) {
          s[i*FILTLEN+k] = s[k];
       }
    }
    return s;
}
/**/
char *get_cameras(int n)

/****** get_cameras ***************************************************
 *
 *  NAME	
 *      get_cameras - Get the list of all different cameras in the table
 *
 *  SYNOPSIS
 *      char *get_cameras(int n)
 *
 *  FUNCTION
 *      This routine examines the global variable table structure and
 *      returns a list (concatinated string of characters) with the
 *      names of the different cameras.
 *
 *  INPUTS
 *      n  - Set number for which to get the channels
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      A null terminated string with passband names. The names consist of
 *      single characters. For instance for DENIS: IJK
 *
 *  MODIFIED GLOBALS
 *      The number of elements global variable and the array of channel 
 *      names is used.
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
{
    char	*s;
    int		i,j, k, num;

    ED_MALLOC(s, "get_cameras", char, (n_files*CAMLEN));
    j = 0; num = 0;
    for (i=0;i<n_files;i++) {
       if (n == set_nrs[i] || n == 0 || n == UNKNOWN) {
          for (k=0;k<CAMLEN;k++) {
             s[j+k] = cameras[i*CAMLEN+k];
          }
          j+=CAMLEN;
          num++;
       }
    }
    for (i=num;i<n_files;i++) {
       for (k=0;k<CAMLEN;k++) {
          s[i*CAMLEN+k] = s[k];
       }
    }
    return s;
}
/**/
int *get_number_nrs()

/****** get_number_nrs ***************************************************
 *
 *  NAME	
 *      get_number_nrs - Get a list of all numbers from the table
 *
 *  SYNOPSIS
 *      int *get_number_nrs()
 *
 *  FUNCTION
 *      This routine returns a list of all numbers as stored in the
 *      global table structure.
 *
 *  INPUTS
 *      None
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      A list of numbers is returned.
 *
 *  MODIFIED GLOBALS
 *      The number of elements global variable and the array of number_nrs 
 *      is used.
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
{
    int		*n, i;

    ED_MALLOC(n, "get_number_nrs", int, n_files);
    for (i=0;i<n_files;i++) n[i] = number_nrs[i];

    return n;
}
/**/
void paste_set_nrs(char *file, char *prog)

/****** paste_set_nrs ***************************************************
 *
 *  NAME	
 *      paste_set_nrs - Add the set numbers to the FILES table and save it
 *
 *  SYNOPSIS
 *      void paste_set_nrs(char *file, char *prog)
 *
 *  FUNCTION
 *      This routine adds the list of set numbers as otherwise derived to
 *      the FILES table in the catalog and outputs this new updated catalog
 *      to disk.
 *
 *  INPUTS
 *      file   - The name of the output catalog
 *      prog   - The calling name of the program allowing addition to history
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *      The number of elements global variable and the array of set_nrs 
 *      is used.
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
{
    catstruct *outcat;
    keystruct *key;

    outcat = new_cat(1);
    inherit_cat(incat, outcat);
    copy_tabs(incat, outcat);
    tab = name_to_tab(outcat, "FILES", 0);
/*
 *  The set_nr may already be in the catalog. If so just overwrite it
 */
    if ((key = name_to_key(tab, "SET_NR")) == NULL) {
       add_key_to_tab(tab, "SET_NR", n_files, set_nrs, T_LONG, 1,
                      "", "Set sequence number");
    } else {
       key->ptr = set_nrs;
    }
    addhistoryto_cat(outcat, prog, 1);
    save_cat(outcat, file);
}
/**/
char *get_all_names(int n, int len, int *num)

/****** get_all_names ***************************************************
 *
 *  NAME	
 *      get_all_names - Get a list of all standard star names in the table
 *
 *  SYNOPSIS
 *      char *get_all_names(int n, int len, int *num)
 *
 *  FUNCTION
 *      This routine extracts a list of all standard star names that are
 *      recorded in the table structure.
 *
 *  INPUTS
 *      n      - The running number for which to get the list
 *      len    - The length of one single standard star name. Required
 *               for concatinating the names in the output array
 *
 *  RESULTS
 *      num    - The number of different names returned
 *
 *  RETURNS
 *
 *  MODIFIED GLOBALS
 *      The number of elements global variable and the array of obj_names
 *      is used.
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
{
/*
 *  Get all the standard star names for a given number
 */
    int         i;
    char        *names;

    *num = 0;
    ED_MALLOC(names, "get_all_names", char, (((*num)+1)*len));
    for (i=0;i<n_files;i++) {
        if (n == number_nrs[i] && obstype(i) != FLAG && obstype(i) != FLAT
            && obstype(i) != WEIGHT) {
           ED_REALLOC(names, "get_all_names", char, (((*num)+1)*len));
           strncpy(&names[(*num)*len],&obj_names[i*len],len);
           (*num)++;
       }
    }
    return (names);
}

/**/
file_id *get_files(int num, char *ch, int chlen, char *name, int namlen,
                   int type, int *n)

/****** get_files ***************************************************
 *
 *  NAME	
 *      get_files - Get a list of filenames that adhere to the arguments
 *
 *  SYNOPSIS
 *      file_id *get_files(int num, char *ch, int chlen, char *name, 
 *                         int namlen, int type, int *n)
 *
 *  FUNCTION
 *      This routine returns a list of file identifications that adhere
 *      to the values provided by the calling argument list.
 *
 *  INPUTS
 *      num    - The running number of the files requested
 *      ch     - The channel name of the files requested
 *      chlen  - The length of the channel name
 *      name   - The name of the files requested
 *      namlen - The length of type name string
 *      type   - The type of the files requested
 *
 *  RESULTS
 *      n      - The number of file identifications returned
 *
 *  RETURNS
 *      A list of file indetifications is returned. Currently the structure
 *      contains the following information:
 *      filename, and set number.
 *
 *  MODIFIED GLOBALS
 *      The number of elements global variable and the array of number_nrs,
 *      obj_names, channels, field_nrs and fitsfiles are used.
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *
 ******************************************************************************/
{
    int         i,j=0,add;
    file_id     *f;

    ED_MALLOC(f,    "get_all_files", file_id, (j+1));
    for (i=0;i<n_files;i++) {
       add = 0;
       if (num == number_nrs[i] && 
           strncmp(ch, &filters[i*chlen],chlen) == 0) {
          if (type == CALIB) {
             if (strncmp(name, &obj_names[i*namlen],namlen) == 0) {
                f[j].seq_nr = seq_nrs[i];
                add = 1;
             }
          }
          if (type == STRIP) {
             f[j].seq_nr = field_nrs[i];
             add = 1;
          }
          if (add == 1) {
             ED_MALLOC(f[j].filename, "get_all_files", char,   FILELEN);
             strncpy(f[j].filename, &fitsfiles[i*FILELEN], FILELEN);
             j++;
             ED_REALLOC(f, "get_all_files", file_id, (j+1));
          }
       }
    }
    *n = j;
    return f;
}
/**/
int obstype(int n)

/****** obstype ***************************************************
 *
 *  NAME	
 *      obstype - Return the observation type if the table element
 *
 *  SYNOPSIS
 *      int obstype(int n)
 *
 *  FUNCTION
 *      This routine returns the observation type of the nth table element.
 *      The observations types returned are CALIB or STRIP;
 *
 *  INPUTS
 *      n   - The line number of the table element to be inspected
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      CALIB if the element is a calibration measurement
 *      STRIP if the element is a regular strip measurement
 *
 *  MODIFIED GLOBALS
 *      The array of img_types global variable is used.
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1966
 *
 ******************************************************************************/
{

    if (strcmp("PHOTO", &img_types[n*IDENTLEN]) == 0) {
       return CALIB;
    }
    if (strcmp("REGULAR", &img_types[n*IDENTLEN]) == 0) {
       return STRIP;
    }
    if (strcmp("FLAG", &img_types[n*IDENTLEN]) == 0) {
       return FLAG;
    }
    if (strcmp("FLAT", &img_types[n*IDENTLEN]) == 0) {
       return FLAT;
    }
    if (strcmp("WEIGHT", &img_types[n*IDENTLEN]) == 0) {
       return WEIGHT;
    }
    return STRIP;
}
/**/
int obstype_set(int n)

/****** obstype_set ***************************************************
 *
 *  NAME	
 *      obstype_set - Get the observation type of the specified set
 *
 *  SYNOPSIS
 *      int obstype_set(int n)
 *
 *  FUNCTION
 *      This routine gets the observatoin type of the given set number
 *      from the global table of files.
 *
 *  INPUTS
 *      n   - The set number for which the observation type is requested
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      CALIB if the set is a calibration sequence
 *      STRIP if the set is a regular strip sequence
 *
 *  MODIFIED GLOBALS
 *      The variable n_files is used
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
{
    int         i;

    for (i=0;i<n_files;i++) {
       if (n == set_nrs[i] && obstype(i) != FLAG && obstype(i) != FLAT
           && obstype(i) != WEIGHT) {
          return obstype(i);
       }
    }
    return UNKNOWN;
}
/**/
int obstype_nr(int n)

/****** obstype_nr ***************************************************
 *
 *  NAME	
 *      obstype_nr - get the observation type of a given running number
 *
 *  SYNOPSIS
 *      int obstype_nr(int n)
 *
 *  FUNCTION
 *      This routine returns the obsetvation type of the specified running
 *      number element from the global table information.
 *
 *  INPUTS
 *      n    - The running number of the element  requested
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      CALIB if the running number is a calibration measurement
 *      STRIP if the running number is a regular strip measurement
 *      
 *
 *  MODIFIED GLOBALS
 *      The variable n_files is used
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
{
    int         i;

    for (i=0;i<n_files;i++) {
       if (n == number_nrs[i] && obstype(i) != FLAG&& obstype(i) != FLAT
          && obstype(i) != WEIGHT) {
          return obstype(i);
       }
    }
    return UNKNOWN;
}
/**/
void set_set_nrs(char *f, int n)
/******  set_set_nrs ***************************************************
 *
 *  NAME	
 *       set_set_nrs - Set the set numbers of the specified elements
 *
 *  SYNOPSIS
 *       void set_set_nrs(char *f, int n)
 *
 *  FUNCTION
 *       This routine sets the set number of the table element that matches
 *       to the filename f. The set number is set to the value of n.
 *
 *  INPUTS
 *       f  - Filename to which an element should match
 *       n  - The set number to which to set the set number table element
 *
 *  RESULTS
 *       None
 *
 *  RETURNS
 *       None
 *
 *  MODIFIED GLOBALS
 *      The number of elements global variable and the array of set_nrs
 *      is used and modified.
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
{
    int         i;

    for (i=0;i<n_files;i++) {
       if (match(&fitsfiles[i*FILELEN], f)) {
          set_nrs[i] = n;
          break;
       }
    }
}
int get_set_nr(int i)
{
      return set_nrs[i];
}
/**/
static char *objs_root(char *fits, char *std, int set, 
                       int nr, int type)

/****** objs_root ***************************************************
 *
 *  NAME	
 *      objs_root - Return the objectfile pathname of the specified element
 *
 *  SYNOPSIS
 *      static char *objs_root(char *fits, char *std, int set,
 *                             int nr, int type)
 *
 *  FUNCTION
 *      This routine returns a pathname construction for the file that
 *      is denoted by the calling argument values.
 *
 *  INPUTS
 *      fits    - The pathname of the fits file
 *      std     - The name of ths standard star for this fits file
 *      set     - The set number of the fits file
 *      nr      - The running number of the fits file
 *      type    - The observation type of the fits file
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      The root pathname to the objects file is returned
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *      The actual pathnaming deconvolution is done by decode_path
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
{
    char        *s, *apath;
    int         idum;
    
    ED_MALLOC(s, "objs_root_chan", char, MAXCHAR);
    switch (type) {
    case CALIB:
                if ((apath=getenv("LOCAL_PATH")))
                   sprintf(s,"%s/objects/%s%s",apath,std,
                                       leading_zeros(set,2));
                else
                   sprintf(s,"%s/objects/%s%s",decode_path(fits, &idum),std,
                                       leading_zeros(set,2));
                break;
    case STRIP:
                if ((apath=getenv("LOCAL_PATH")))
                   sprintf(s,"%s/objects/%s",apath,
                                        strip_name(nr));
                else
                   sprintf(s,"%s/objects/%s",decode_path(fits, &idum),
                                        strip_name(nr));
                break;
    }
    return s;
}
/**/
static char *objs_root_chan(char *fits, char *std, int set, 
                            char *filt, char *cam, int nr, int type)

/****** objs_root_chan ***************************************************
 *
 *  NAME	
 *      objs_root_chan - Return objectfile pathname with channelof the element
 *
 *  SYNOPSIS
 *      static char *objs_root_chan(char *fits, char *std, int set,
 *                                  char *filt, char *cam, int nr, int type)
 *
 *  FUNCTION
 *      This routine returns a pathname construction for the file that
 *      is denoted by the calling argument values. The pathname
 *      incorporates tha channel name.
 *
 *  INPUTS
 *      fits    - The pathname of the fits file
 *      std     - The name of ths standard star for this fits file
 *      set     - The set number of the fits file
 *      filt    - The filter name of the fits file
 *      cam     - The camera name of the fits file
 *      nr      - The running number of the fits file
 *      type    - The observation type of the fits file
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      The root pathname to the objects file including channel name is 
 *      returned
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *      The actual pathnaming deconvolution is done by decode_path
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
{
    char        *s;

    s = objs_root(fits, std, set, nr, type);
    if (filt[0] != '\0')  sprintf(s,"%s.%s",s,to_lower(filt));
    if (cam[0] != '\0')  sprintf(s,"%s.%s",s,to_lower(cam));
    return s;
}
/**/
static char *objs_path(char *fits, char *std, int set, int seq, int fld_nr, 
                       char *filt, char *cam, int nr, int type)
/****** objs_path ***************************************************
 *
 *  NAME	
 *      objs_path - Return objectfile full pathname of the specified element
 *
 *  SYNOPSIS
 *      static char *objs_path(char *fits, char *std, int set, int seq, 
 *                             int fld_nr, char *filt, char *cam, int nr, int type)
 *
 *  FUNCTION
 *      This routine returns a full pathname construction for the file that
 *      is denoted by the calling argument values.
 *
 *  INPUTS
 *      fits    - The pathname of the fits file
 *      std     - The name of ths standard star for this fits file
 *      set     - The set number of the fits file
 *      seq     - The sequence number of the fits file
 *      fld_nr  - The field number of the fits file
 *      filt    - The filter name of the fits file
 *      cam     - The camera name of the fits file
 *      nr      - The running number of the fits file
 *      type    - The observation type of the fits file
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      The root pathname to the objects file including channel name is 
 *      returned
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *      The actual pathnaming deconvolution is done by decode_path
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
{
    char        *s;
    int         idum = 0;

    ED_MALLOC(s, "objs_path", char, MAXCHAR);
    switch (type) {
    case CALIB:
                idum = ((seq-1) % OCTAGONAL);
                break;
    case STRIP:
                idum=fld_nr;
                break;
    }
    sprintf(s,"%s.%s.objs",objs_root_chan(fits, std, set, filt, cam, nr, type),
                            leading_zeros(idum, 10));
    return s;
}
/**/
void make_field_count(int s_n, char *filt_n, char *cam_n)
/****** make_field_count ***************************************************
 *
 *  NAME	
 *      make_field_count - Count the number of fields in a set
 *
 *  SYNOPSIS
 *      void make_field_count(int s_n, char *filt_n, char *cam_n)
 *
 *  FUNCTION
 *      This debugging routine prints the number of fields that are present
 *      is a specified set. The result is printed to stdout.
 *
 *  INPUTS
 *      s_n  - The set number for which to make the count
 *      filt_n - The channel name for which the count is requested
 *      cam_n - The channel name for which the count is requested
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 ******************************************************************************/
{
    int		i, count = 0;;

    for (i=0;i<n_files;i++) {
       if (set_nrs[i] == s_n &&
           strcmp(to_upper(filt_n), to_upper(&filters[i*FILTLEN])) == 0 &&
           strcmp(to_upper(cam_n), to_upper(&cameras[i*CAMLEN])) == 0) {
          count++;
       }
    }
    printf("%d\n", count);
}
/**/
void make_blend_list(int s_n, char *filt_n, char *cam_n)

/****** make_blend_list ***************************************************
 *
 *  NAME	
 *      make_blend_list - Generate a listing for the blend program
 *
 *  SYNOPSIS
 *      void make_blend_list(int s_n, char *filt_n, char *cam_n)
 *
 *  FUNCTION
 *      This routine makes a list of files as needed by the blend program
 *      to do the source extraction.
 *
 *  INPUTS
 *      s_n   - The set number of the list requested
 *      filt_n  - The filter name of thelist requested
 *      c_mn  - The camera name of thelist requested
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
{
    int		i;

    for (i=0;i<n_files;i++) {
       if (set_nrs[i] == s_n && obstype(i) != FLAG && obstype(i) != FLAT &&
	   obstype(i) != WEIGHT &&
           strcmp(to_upper(filt_n), to_upper(&filters[i*FILTLEN])) == 0 &&
           strcmp(to_upper(cam_n), to_upper(&cameras[i*CAMLEN])) == 0) {
          printf("%s %s\n", &fitsfiles[i*FILELEN], 
             objs_path(&fitsfiles[i*FILELEN], &obj_names[i*OBJLEN],
                  set_nrs[i], seq_nrs[i], field_nrs[i], &filters[i*FILTLEN], 
                  &cameras[i*CAMLEN], number_nrs[i], obstype(i)));
       }
    }
}

void make_eis_list(int s_n, char *filt_n, char *cam_n)

/****** make_eis_list ***************************************************
 *
 *  NAME	
 *      make_eis_list - Generate a listing for the blend program
 *
 *  SYNOPSIS
 *      void make_eis_list(int s_n, char *filt_n, char *cam_n)
 *
 *  FUNCTION
 *      This routine makes a list of files as needed by the blend program
 *      to do the source extraction.
 *
 *  INPUTS
 *      s_n   - The set number of the list requested
 *      filt_n  - The filter name of thelist requested
 *      cam_n  - The camera name of thelist requested
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
{
    int		i;

    for (i=0;i<n_files;i++) {
       if (set_nrs[i] == s_n && obstype(i) != FLAG &&  obstype(i) != FLAT 
           && obstype(i) != WEIGHT &&
           strcmp(to_upper(filt_n), to_upper(&filters[i*FILTLEN])) == 0 &&
           strcmp(to_upper(cam_n), to_upper(&cameras[i*CAMLEN])) == 0) {
          printf("%s %s %s.back ", &fitsfiles[i*FILELEN], 
             objs_path(&fitsfiles[i*FILELEN], &obj_names[i*OBJLEN],
                  set_nrs[i], seq_nrs[i], field_nrs[i], &filters[i*FILTLEN], 
                  &cameras[i*CAMLEN],number_nrs[i], obstype(i)),
                  &fitsfiles[i*FILELEN]);
 /*
  *       Now get the FLAG and WEIGHT files for this image file
  */
          make_file_path(s_n, filt_n, cam_n, FLAG);
          printf(" ");
          make_file_path(s_n, filt_n, cam_n, WEIGHT);
          printf("\n");
       }
    }
}

/**/
void make_astrom_name(int s_n)

/****** make_astrom_name ***************************************************
 *
 *  NAME	
 *      make_astrom_name - Create the name for an astrometric ref catalog
 *
 *  SYNOPSIS
 *      void make_astrom_name(int s_n)
 *
 *  FUNCTION 
 *      This routine prints to stdout the name of the astrometric reference
 *      catalog, in the LDAC naming convention. This name can be used with
 *      the preastrom and astrom programs.
 *
 *  INPUTS
 *      s_n  - The set number for which to produce the astrometric ref name
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
{
    make_root_name(s_n, "", "");
    printf("\n");
    return;
}
/**/
void make_std_name(int s_n)

/****** make_std_name ***************************************************
 *
 *  NAME	
 *      make_std_name - Create the name for an astrometric ref catalog
 *
 *  SYNOPSIS
 *      void make_std_name(int s_n)
 *
 *  FUNCTION 
 *      This routine prints to stdout the name of the photometric standard.
 *
 *  INPUTS
 *      s_n  - The set number for which to produce the photometric std name
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 22-10-1996
 *
 ******************************************************************************/
{
    int		i;

    for (i=0;i<n_files;i++) {
       if (set_nrs[i] == s_n) {
          printf("%s\n", &obj_names[i*OBJLEN]);
          break;
       }
    }
}
/**/
void make_number(int s_n)

/****** make_number ***************************************************
 *
 *  NAME	
 *      make_number - Give the running number
 *
 *  SYNOPSIS
 *      void make_number(int s_n)
 *
 *  FUNCTION 
 *
 *  INPUTS
 *      s_n  - The set number for which to produce the photometric std name
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 22-10-1996
 *
 ******************************************************************************/
{
    int		i;

    for (i=0;i<n_files;i++) {
       if (set_nrs[i] == s_n) {
          printf("%d\n", number_nrs[i]);
          break;
       }
    }
}
/**/
void make_instrument(int s_n)

/****** make_instrument ***************************************************
 *
 *  NAME	
 *      make_instrument - Give the instrument name
 *
 *  SYNOPSIS
 *      void make_instrument(int s_n)
 *
 *  FUNCTION 
 *
 *  INPUTS
 *      s_n  - The set number 
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 22-10-1996
 *
 ******************************************************************************/
{
    int		i;
    char	t[MAXCHAR];

    for (i=0;i<n_files;i++) {
       if (set_nrs[i] == s_n) {
          strncpy(t, &instruments[i*INSTLEN], INSTLEN);
          printf("%s\n",t);
          break;
       }
    }
}
/**/
void make_file_path(int s_n, char *filt_n, char *cam_n, int type)
{
    int		i, j;
    char	t[MAXCHAR];

    if (type == FLAG || type == FLAT) {
/*
 *     First get the number of the set/chan/cam
 */
       for (i=0;i<n_files;i++) {
          if (set_nrs[i] == s_n &&
              strcmp(to_upper(filt_n), to_upper(&filters[i*FILTLEN])) == 0 &&
              strcmp(to_upper(cam_n), to_upper(&cameras[i*CAMLEN])) == 0) {
             break;
          }
       }
       j = number_nrs[i];
/*
 *     Then get the file
 */
       for (i=0;i<n_files;i++) {
          if (number_nrs[i] == j && obstype(i) == type &&
             strcmp(to_upper(filt_n), to_upper(&filters[i*FILTLEN])) == 0 &&
             strcmp(to_upper(cam_n), to_upper(&cameras[i*CAMLEN])) == 0) {
             strncpy(t,&fitsfiles[i*FILELEN], FILELEN);
             printf("%s", t);
             break;
          }
       }
    } else {
       for (i=0;i<n_files;i++) {
          if (obstype(i) == type &&  set_nrs[i] == s_n &&
              strcmp(to_upper(filt_n), to_upper(&filters[i*FILTLEN])) == 0 &&
              strcmp(to_upper(cam_n), to_upper(&cameras[i*CAMLEN])) == 0) {
             strncpy(t,&fitsfiles[i*FILELEN], FILELEN);
             printf("%s", t);
             break;
          }
       }
    }
}
/*^L*/
void make_root_name(int s_n, char *filt_n, char *cam_n)
/******  make_root_name ***************************************************
 *
 *  NAME	
 *       make_root_name - Create a root pathname for a specific set and channel
 *
 *  SYNOPSIS
 *       void make_root_name(int s_n, char *ch_n)
 *
 *  FUNCTION
 *       This routine prints to stdout the root pathname for the provided
 *       set number and channel name. The naming is according to the LDAC
 *       naming convention.
 *
 *  INPUTS
 *       s_n  - The set number for the requested pathname
 *       ch_n - The channel name for the requested pathname
 *
 *  RESULTS
 *       None
 *
 *  RETURNS
 *       None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
{
    int		i, old_set_nr = 0, *s_idx;

    ED_CALLOC(s_idx, "make_root_name", int, n_files);
    iindex(n_files, set_nrs, s_idx);
    for (i=0;i<n_files;i++) {
       if (old_set_nr != set_nrs[s_idx[i]] && 
           (set_nrs[s_idx[i]] == s_n || s_n == 0) && 
            (obstype(s_idx[i]) == STRIP || obstype(s_idx[i]) == CALIB)) {
          printf(" %s",objs_root_chan(&fitsfiles[s_idx[i]*FILELEN], 
                  &obj_names[s_idx[i]*OBJLEN],
                  set_nrs[s_idx[i]], to_lower(filt_n), to_lower(cam_n), 
                  number_nrs[s_idx[i]], obstype(s_idx[i])));
          old_set_nr = set_nrs[s_idx[i]];
          if (s_n == 0) printf("\n");
       }
    }
    ED_FREE(s_idx);
}
/**/
void make_objs_dir(int s_n)

/****** make_objs_dir ***************************************************
 *
 *  NAME	
 *      make_objs_dir - Create the dirname in which to store object files
 *
 *  SYNOPSIS
 *      void make_objs_dir(int s_n)
 *
 *  FUNCTION
 *      This routine prints to stdout the name of the directory, in the
 *      LDAC naming convetion, in which normally all object files are 
 *      to be stored.
 *
 *  INPUTS
 *      s_n   - The set number for the objects directory name request
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
{
    int		i,j ;
    char	*apath;

    for (i=0;i<n_files;i++) {
       if (set_nrs[i] == s_n) {
          if ((apath = getenv("LOCAL_PATH")))
             printf("%s/objects\n",apath);
          else
             printf("%s/objects\n",decode_path(&fitsfiles[i*FILELEN], &j));
          break;
       }
    }
}
/**/
void get_strip_name(int s_n)

/****** get_strip_name ***************************************************
 *
 *  NAME	
 *      get_strip_name - Create the name of the strip for the given set
 *
 *  SYNOPSIS
 *      void get_strip_name(int s_n)
 *
 *  FUNCTION
 *      This routine prints to stdout the name of the strip for a given
 *      set number.
 *
 *  INPUTS
 *      s_n   - The set number for the objects directory name request
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 11-01-1997
 *      T. Erben  - dd 02-16-1998
 *      T. Erben  - dd 03-11-1998
 *
 ******************************************************************************/
{
    int		i,j ;

    if (s_n == 0) {
       /* changed from 5 to 7 (TE) */ 
       /* changed from 7 to 10 (TE) */ 
       printf("RS%s\n",leading_zeros(strip_nrs[0],10));
       return;
    }
    for (i=0;i<n_files;i++) {
       if (set_nrs[i] == s_n) {
          decode_path(&fitsfiles[i*FILELEN], &j);
          printf("%s\n",strip_name(j));
          return;
       }
    }
}
/**/
void make_set_names(int s_n, int equal)

/****** make_set_names ***************************************************
 *
 *  NAME	
 *      make_set_names - Create a (list of) pathname(s) for the specified set
 *
 *  SYNOPSIS
 *      void make_set_names(int s_n)
 *
 *  FUNCTION
 *      This routine prints to stdout a pathname (for a s_n > 0) or a full
 *      list of pathnames (for s_n = 0) of the root pathname for the provided
 *      set number at all channels.
 *
 *  INPUTS
 *      s_n  - The set number for which to procude the pathname(s)
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
{
    int		i,j,k,l;
    char	**filts, **cams;

    i = n_files;
    filts = uniq_strings(get_filters(s_n), FILTLEN, &i);
    j = n_files;
    cams  = uniq_strings(get_cameras(s_n), CAMLEN, &j);
    for (k=0;k<i;k++) {
      for (l=0;l<j;l++) {
        if ((equal && !strcmp(filts[k], cams[l])) || !equal) {
           make_root_name(s_n, filts[k], cams[l]);
        }
      }
    }
    printf("\n");
}
