#include <stdio.h>
#include <dirent.h>
#include <sys/stat.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include "fitscat.h"
#include "utils_globals.h"
#include "utils_macros.h"
#include "tabutil_globals.h"
#include "lists_globals.h"
#include "options_globals.h"
#include "files_cat_globals.h"
#include "make_files_vers.h"

#define STFMT(s)        ((s)->st_mode & S_IFMT)
#define ISDIR(f, s)     ((*statfun)(f, &s) == 0 && STFMT(&s) == S_IFDIR)
#define DIRECTORY 0
#define AFILE 1
#define UNKNOWN -1

/* 
 * 15.07.01:
 * modifications to avoid compiler warnings
 *
 * 21.04.03:
 * changes to make the code work under Linux; it is
 * NOT used at all for the moment
 */

typedef struct {
    char        dir_path[MAXCHAR], outfile[MAXCHAR];
    char	prefix[MAXCHAR], postfix[MAXCHAR];
    char	night[MAXCHAR], filter[MAXCHAR], instrument[MAXCHAR];
    char	ident[MAXCHAR], field[MAXCHAR], camera[MAXCHAR];
    char	object[MAXCHAR], conffile[MAXCHAR];
    char	photo[MAXCHAR], regular[MAXCHAR];
    char	strip[MAXCHAR], flag[MAXCHAR], flat[MAXCHAR], survey[MAXCHAR];
    char	weight[MAXCHAR];
    char	number[MAXCHAR], row[MAXCHAR], subrow[MAXCHAR];
    char	listfile[MAXCHAR];
    int		listdriven;
    int		subrow_seg;
    int         quiet;
} control_struct;

typedef struct {
   char	 **list;
   int   n;
} string_list;

static control_struct control;
static int field = 0;
static int nfiles;

option my_opts[] = {
            {"NIGHT_KEY", TEXT, control.night, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "2MUNIGHT",
             {"", "", "", "", ""}, 0},
            {"FILTER_KEY", TEXT, control.filter, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "2MUBAND",
             {"", "", "", "", ""}, 0},
            {"CAMERA_KEY", TEXT, control.camera, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "CHIP",
             {"", "", "", "", ""}, 0},
            {"INSTRUMENT_KEY", TEXT, control.instrument, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "INSTRUME",
             {"", "", "", "", ""}, 0},
            {"IDENT_KEY", TEXT, control.ident, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "2MUIDENT",
             {"", "", "", "", ""}, 0},
            {"FIELD_KEY", TEXT, control.field, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "2MUFIELD",
             {"", "", "", "", ""}, 0},
            {"OBJECT_KEY", TEXT, control.object, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "OBJECT",
             {"", "", "", "", ""}, 0},
            {"STRIP_KEY", TEXT, control.strip, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "2MUSTRIP",
             {"", "", "", "", ""}, 0},
            {"CALIB", TEXT, control.photo, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "PHOTO",
             {"", "", "", "", ""}, 0},
            {"STRIP", TEXT, control.regular, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "REGULAR",
             {"", "", "", "", ""}, 0},
            {"FLAG", TEXT, control.flag, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "FLAG",
             {"", "", "", "", ""}, 0},
            {"WEIGHT", TEXT, control.weight, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "WEIGHT",
             {"", "", "", "", ""}, 0},
            {"FLAT", TEXT, control.flat, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "FLAT",
             {"", "", "", "", ""}, 0},
            {"NUMBER", TEXT, control.number, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "EISPATCH",
             {"", "", "", "", ""}, 0},
            {"ROW", TEXT, control.row, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "EISROW",
             {"", "", "", "", ""}, 0},
            {"SUBROW", TEXT, control.subrow, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "EISSROW",
             {"", "", "", "", ""}, 0},
            {"SUBROW_SEG", INT, &control.subrow_seg, 1, 1000, 0.0, 0.0,
              10, 0.0, NO, "",
             {"", "", "", "", ""}, 0},
            {"SURVEY", STRING, control.survey, 0, 0, 0.0, 0.0,
              0, 0.0, NO, "DENIS",
             {"DENIS", "EIS", "", "", ""}, 0},
            {""}
       };

/**/
int filetype(char *path)

/****i* filetype ***************************************************
 *
 *  NAME	
 *      filetype - Obtain the type of a specified file
 *
 *  SYNOPSIS
 *      int filetype(char *path)
 *
 *  FUNCTION
 *      This routine returns the type of the file specified as the argument
 *      pathname.
 *
 *  INPUTS
 *      path   - The pathname of the file to inspect
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      DIRECTORY - Defined value denoting the file is a directory
 *      AFILE     - Defined value denoting the file is a regular file
 *      UNKNOWN   - Defined value denoting the file is neither a directory
 *                  nor a regular file.
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *      THe routine is taken from a public domain package
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 01-08-1996
 *
 ******************************************************************************/
{
    int f;
    struct stat st;

    if (stat(path, &st) == 0) {
       f = STFMT(&st);
       if (f == S_IFDIR)              /* Directory */
          return(DIRECTORY);
       else if (f == S_IFREG)         /* Regular file */
          return(AFILE);
       return(UNKNOWN);             /* Unknown type */
    } else {
       perror("Oops");
    }
    return(UNKNOWN);
} 
/**/
static char prev_object[1000], prev_chan[8];
static int seq_nr, prev_number;

void save_file(filestruct *f)

/****i* save_file ***************************************************
 *
 *  NAME	
 *      save_file - Save the file information in the table structure
 *
 *  SYNOPSIS
 *      void save_file(filestruct *f)
 *
 *  FUNCTION
 *      The information stored in the filestructure is saved to the
 *      table structure that will later be stored as the output catalog
 *      file.
 *
 *  INPUTS
 *      f  - The filestructure containing relavent information to be stored
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 01-08-1996
 *
 ******************************************************************************/
{
    if (f->type > 0) {
       if (control.quiet) 
          printf("%s %d %d %s %s %d %s %s %s %d\n",
             f->fitsfile, f->night, f->number, f->ident, f->object,
             f->seq_nr, f->filter, f->camera, f->instrument, f->field);
       add_file_to_tab(f);
    }
}
/**/

filestruct *examine_fits(char *fitsfile)

/****i* examine_fits ***************************************************
 *
 *  NAME	
 *      examine_fits - Examine the contents of a fits file
 *
 *  SYNOPSIS
 *      filestruct *examine_fits(char *fitsfile)
 *
 *  FUNCTION
 *      This routine interpretes the contents of a fits file and stores
 *      the relavent information into the filestructure which is returned
 *      on completion.
 *
 *      The current information stored is:
 *          filename, ident (REGULAR, PHOTO, or FLAG, FLAT), Instrument (I, J, or K),
 *          image_type (CALIB or STRIP), night number, sequence number,
 *          a running number, and the field number
 *
 *  INPUTS
 *      fitsfile - The pathname of the fits file to be examined
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      The filestructure filled with relavent fits file information is
 *      returned
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 01-08-1996
 *      T.   Erben- dd 02-16-1998: chamged the number for the EIS survey
 *
 ******************************************************************************/
{
    catstruct	*cat;
    tabstruct	*tab;
    char	keyword[16], comment[82], camera[82], *fitsline;
    char	object[82], filter[82], instrument[82], ident[82];
    int		night = 0, number = 0, strip = 0;
    h_type	htype;
    t_type	ttype;
    int		image_type = STRIP;
    filestruct  *a_file;

    ED_MALLOC(a_file, "examine_fits", filestruct, 1);
    strcpy(object, "");
    strcpy(ident, "");
    strcpy(filter, "");
    strcpy(camera, "");
    strcpy(instrument, "");
    if ((cat = read_cat(fitsfile)) == NULL) {
       syserr(FATAL, "examine_fits", "Could not read catalog: %s\n", fitsfile);
    }
/*
 *  Fix to allow reading the erroneous files without a primary header
 */
    if (cat->ntab == 1) {
      tab = cat->tab;
    } else {
      tab = cat->tab->nexttab;
    }
    tab = cat->tab;
    
    if ((fitsline = fitsnfind(tab->headbuf, control.ident, tab->headnblock))) {
       fitspick(fitsline, keyword, ident, &htype, &ttype, comment);
    } else {
       syserr(WARNING,"examine_fits","Could not find KEYWORD %s in FITS file %s\n",
            control.ident,fitsfile);
    }
    strcpy(ident,strip_space(ident));
    if ((fitsline = fitsnfind(tab->headbuf, control.night, tab->headnblock))) {
       fitspick(fitsline, keyword, &night, &htype, &ttype, comment);
    } else {
       syserr(WARNING,"examine_fits","Could not find KEYWORD %s in FITS file %s\n",
            control.night,fitsfile);
    }
    if (strstr(ident, control.photo)) {
       image_type = CALIB;
       sprintf(ident, "PHOTO");
    }
    if (strstr(ident, control.regular)) {
       image_type = STRIP;
       sprintf(ident, "REGULAR");
    }
    if (strstr(ident, control.flag)) {
       image_type = FLAG;
       sprintf(ident, "FLAG");
    }
    if (strstr(ident, control.weight)) {
       image_type = WEIGHT;
       sprintf(ident, "WEIGHT");
    }
    if (strstr(ident, control.flat)) {
       image_type = FLAT;
       sprintf(ident, "FLAT");
    }
    if (image_type == UNKNOWN)
       syserr(WARNING, "examine_fits", "%s = %s is unknown in %s\n Expected %s, %s, or %s\n", 
           control.ident, ident, fitsfile, 
           control.photo, control.regular, control.flag);
    if (image_type > 0) {
       if (strstr(control.survey,"DENIS") == 0) {
          int row, subrow;
          char patch[MAXCHAR];

          if ((fitsline = 
                  fitsnfind(tab->headbuf, control.object, tab->headnblock))) {
             fitspick(fitsline, keyword, object, &htype, &ttype, comment);
          }    else {
             syserr(WARNING,"examine_fits",
                "Could not find KEYWORD %s in FITS file %s\n",
                 control.object, fitsfile);
          }
          strcpy(object, strip_space(object));
          if ((fitsline=fitsnfind(tab->headbuf,control.strip,tab->headnblock))) {
             fitspick(fitsline, keyword, &patch, &htype, &ttype, comment);
          } else {
             syserr(WARNING,"examine_fits",
                "Could not find KEYWORD %s in FITS file %s\n",
                 control.number,fitsfile);
             sprintf(patch,"E");
          }
          if ((fitsline=fitsnfind(tab->headbuf,control.row,tab->headnblock))) {
             fitspick(fitsline, keyword, &row, &htype, &ttype, comment);
          } else {
             syserr(WARNING,"examine_fits",
                "Could not find KEYWORD %s in FITS file %s\n",
                 control.row,fitsfile);
             row = 0;
          }
          if ((fitsline=fitsnfind(tab->headbuf,control.subrow,tab->headnblock))) {
             fitspick(fitsline, keyword, &subrow, &htype, &ttype, comment);
          } else {
             syserr(WARNING,"examine_fits",
                "Could not find KEYWORD %s in FITS file %s\n",
                 control.subrow, fitsfile);
             subrow = 0;
          }
/*
 *        Let the field number be just a running number
 */
          field++;
          number = (patch[0] - 64)*1000000 + (row-1)*10000+(subrow/control.subrow_seg)*1000+night;
          strip =  (patch[0] - 64)*1000 + (row-1)*10;
       } else {
          decode_path(fitsfile, &number);
          if (image_type == CALIB) {
             if (strcmp(prev_object, object) == 0 && 
                 strcmp(prev_chan, filter) == 0 && 
                 prev_number == number) 
                seq_nr++;
             else
                seq_nr=1;
             strcpy(prev_object, object);
             strcpy(prev_chan, filter);
             prev_number = number;
             if ((fitsline = 
                  fitsnfind(tab->headbuf, control.object, tab->headnblock))) {
                fitspick(fitsline, keyword, object, &htype, &ttype, comment);
             }    else {
                syserr(WARNING,"examine_fits",
                   "Could not find KEYWORD %s in FITS file %s\n",
                    control.object, fitsfile);
             }
             strcpy(object, strip_space(object));
          }
          if (image_type == STRIP) {
             if ((fitsline=fitsnfind(tab->headbuf, control.field, tab->headnblock))) {
                fitspick(fitsline, keyword, &field, &htype, &ttype, comment);
             } else {
                syserr(WARNING,"examine_fits",
                   "Could not find KEYWORD %s in FITS file %s\n",
                    control.field, fitsfile);
             }
             if ((fitsline=fitsnfind(tab->headbuf, control.strip, tab->headnblock))) {
                fitspick(fitsline, keyword, &strip, &htype, &ttype, comment);
             } else {
                syserr(WARNING,"examine_fits",
                   "Could not find KEYWORD %s in FITS file %s\n",
                    control.field, fitsfile);
             }
          }
          if ((fitsline=fitsnfind(tab->headbuf, control.field, 
               tab->headnblock))) {
             fitspick(fitsline, keyword, &field, &htype, &ttype, comment);
          } else {
             syserr(WARNING,"examine_fits",
                "Could not find KEYWORD %s in FITS file %s\n",
                 control.field, fitsfile);
             field = 0;
          }
       }
       if ((fitsline = fitsnfind(tab->headbuf, control.filter,tab->headnblock))) {
          fitspick(fitsline, keyword, filter, &htype, &ttype, comment);
       } else {
          syserr(WARNING,"examine_fits",
             "Could not find KEYWORD %s in FITS file %s\n",
              control.filter, fitsfile);
       }
       if (ttype == T_LONG)
          sprintf(filter, "%d",((int *)filter)[0]);
       strcpy(filter, strip_space(filter));
       if ((fitsline = fitsnfind(tab->headbuf, control.camera,tab->headnblock))) {
          fitspick(fitsline, keyword, camera, &htype, &ttype, comment);
       } else {
          syserr(WARNING,"examine_fits",
             "Could not find KEYWORD %s in FITS file %s\n",
              control.camera, fitsfile);
          sprintf(camera, "0");
       }
       if (ttype == T_LONG)
          sprintf(camera, "%d",((int *)camera)[0]);
       strcpy(camera, strip_space(camera));
       if ((fitsline = fitsnfind(tab->headbuf, control.instrument,tab->headnblock))) {
          fitspick(fitsline, keyword, instrument, &htype, &ttype, comment);
       } else {
          syserr(WARNING,"examine_fits",
             "Could not find KEYWORD %s in FITS file %s\n",
              control.instrument, fitsfile);
       }
    }
    /* close_cat(cat); */
    strcpy(a_file->fitsfile, fitsfile);
    strcpy(a_file->object, object);
    strcpy(a_file->ident, ident);
    strcpy(a_file->filter, filter);
    strcpy(a_file->camera, camera);
    strcpy(a_file->instrument, instrument);
    a_file->type   = image_type;
    a_file->night  = night;
    a_file->strip  = strip;
    a_file->seq_nr = seq_nr;
    a_file->number = number;
    a_file->field  = field;
    return a_file;
}
/**/
void list_dir(char *dir)

/****i* list_dir ***************************************************
 *
 *  NAME	
 *      list_dir - Parse the directory structure recursively
 *
 *  SYNOPSIS
 *      void list_dir(char *dir)
 *
 *  FUNCTION
 *      This routine parses the directory tree and calls when appropriate
 *      the save_file routine to store for later use the information
 *      retained in interesting FITS files.
 *
 *  INPUTS
 *      dir  - The directory root path from which to parse the tree
 *
 *  RESULTS
 *     None
 *
 *  RETURNS
 *     None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *     This is a recursive routine that has the actual processing within.
 *
 *  BUGS
 *
 *  AUTHOR
 *     E.R. Deul - dd 01-08-1996
 *
 ******************************************************************************/
{
    DIR		*dirp;
    struct	dirent *dp;
    char	local_dir[1000], fits_file[255];

    if ((dirp = opendir(dir)) == NULL) {
       syserr(FATAL, "list_dir", "No such directory: %s\n", dir);
    }
    while ((dp = readdir(dirp)) != NULL) {
       strcpy(local_dir, dir);
       strcat(local_dir, "/");
       strcat(local_dir, dp->d_name);
       switch (filetype(local_dir)) {
       case DIRECTORY: if (!match(dp->d_name,"..") && !match(dp->d_name,".")) {
                          list_dir(local_dir);
                       } 
                       break;
       case AFILE:     sprintf(fits_file,"%s*%s", 
                               control.prefix,control.postfix);
                       if (match(dp->d_name,fits_file)) {
                          save_file(examine_fits(local_dir));
                       }
                       break;
       case UNKNOWN:   printf("Unkn: ");
                       printf("%s\n",dp->d_name);
                       break;
       }
    }
    (void) closedir(dirp);
} 
/**/
void list_files(string_list file_list)

/****i* list_files ***************************************************
 *
 *  NAME	
 *      list_files - Parse the files list and interpret the FITS files
 *
 *  SYNOPSIS
 *      void list_files(char **files_list)
 *
 *  FUNCTION
 *      This routine parses the directory tree and calls when appropriate
 *      the save_file routine to store for later use the information
 *      retained in interesting FITS files.
 *
 *  INPUTS
 *      dir  - The directory root path from which to parse the tree
 *
 *  RESULTS
 *     None
 *
 *  RETURNS
 *     None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *     This is a recursive routine that has the actual processing within.
 *
 *  BUGS
 *
 *  AUTHOR
 *     E.R. Deul - dd 01-08-1996
 *
 ******************************************************************************/
{
    int  i;

    for (i=0;i<file_list.n;i++)
       save_file(examine_fits(file_list.list[i]));
} 
/**/
void usage(char *me)

/****i* usage ***************************************************
 *
 *  NAME	
 *      usage - Output on stderr the program usage information
 *
 *  SYNOPSIS
 *      void usage(char *me)
 *
 *  FUNCTION
 *      User interface information printed to standard error. This allows
 *      for a more pleasant interaction with interactive users.
 *
 *  INPUTS
 *      me - The calling name of the program
 *
 *  RESULTS
 *      None
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *      This routine exits the program with an error code 1
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 01-08-1996
 *
 ******************************************************************************/
{
    fprintf(stderr,"Usage: %s\n", me);
    fprintf(stderr,"    -p root_night_path -o outcat \n");
    fprintf(stderr,"	   [-q] [-f fits_prefix -e fits_postfix]\n");
    fprintf(stderr,"       [-l file_list]\n\n");
    fprintf(stderr," This program examines the directory tree and interpretes all fits files\n");
    fprintf(stderr," When a file_list is provided it will interpret only those files\n");
    fprintf(stderr," present both in the file_list and in the directory tree\n");
    fprintf(stderr," The result is stored in an output files catalog\n\n");
    exit(1);
}
/**/
void init(int argc, char *argv[], control_struct *c)

/****i* init ***************************************************
 *
 *  NAME	
 *      init - Parse the command line arguments and fill the control structure
 *
 *  SYNOPSIS
 *      void init(int argc, char *argv[], control_struct *c)
 *
 *  FUNCTION
 *      This routine reads the command line arguments, interprets them
 *      in terms of program usage and fills the control structure with
 *      the appropriate values from the arguments.
 *
 *  INPUTS
 *      argc   - The number of command line arguments
 *      argv   - The list of command line argument strings
 *
 *  RESULTS
 *      c      - The control structure the receives the argument information
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *      In case of command line errors, the usage routine is called.
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 01-08-1996
 *
 ******************************************************************************/
{
/*
 *  Default file name is calling name with .conf extension
 */
    int		i, j, k;

    strcpy(c->dir_path, "");
    strcpy(c->outfile, "");
    strcpy(c->prefix, "");
    strcpy(c->postfix, "");

    c->quiet = 1;
    c->listdriven = FALSE;
    if (argc == 1) usage(argv[0]);
    init_options(0, my_opts);
    for (i=1; i<argc; i++) {
      if (argv[i][0] == '-') {
         switch(argv[i][1]) {
         case 'c': strcpy(control.conffile,argv[++i]);
                   break;
         case 'p': strcpy(c->dir_path,argv[++i]);
                   break;
         case 'f': strcpy(c->prefix,argv[++i]);
                   break;
         case 'e': strcpy(c->postfix,argv[++i]);
                   break;
         case 'o': strcpy(c->outfile,argv[++i]);
                   break;
         case 'l': strcpy(c->listfile,argv[++i]);
                   c->listdriven = TRUE;
                   break;
         case 'q': c->quiet = 0;
                   break;
         default:  parse_options(control.conffile);
                   j = i++; k = i;
                   set_command_option(argv[j], argv[k]);
                   break;
         }
      } else {
         syserr(WARNING,"init","Command line syntax error\n");
         usage(argv[0]);
      }
   }
   if (c->dir_path[0] == '\0' || c->outfile[0] == '\0') {
      syserr(WARNING,"init","directory path or output file missing\n");
      usage(argv[0]);
   }
   finish_options(control.conffile);
   printf("PREFIX=%s, POSTFIX=%s\n",control.prefix, control.postfix);
}

string_list load_files_list(char *name)
{
    FILE	*fd;
    char	line[5120], *s;
    string_list f_list;

    ED_FOPEN(fd, "load_files_list", name, "rt");
    nfiles = 0;
    ED_CALLOC(f_list.list, "load_files_list", char *, (nfiles+1));
    while (fgets(line, 5120, fd)) {
       if (line[0] != '#') {
          if (nfiles != 0)
             ED_REALLOC(f_list.list, "load_files_list", char *, (nfiles+1));
          ED_CALLOC(f_list.list[nfiles], "load_files_list", char, MAXCHAR);
          s = strtok(line," 	\n\t");
          strcpy(f_list.list[nfiles], s);
          nfiles++;
       }
    }
    ED_FCLOSE(fd, "load_files_list");
    f_list.n = nfiles;
    return (f_list);
}

int main(int argc, char *argv[])
{
/****** make_files_cat ***************************************************
 *
 *  NAME	
 *      make_files_cat - Create a catalog with filename and content info
 *
 *  SYNOPSIS
 *      make_files_cat -p root_night_path 
 *                     -o outcat 
 *                     [-q] 
 *                     [-f fits_prefix]
 *
 *  FUNCTION
 *      This program examines the directory tree and interpretes all fits 
 *      files in that tree. The result is stored in an output files catalog.
 *      The information stored in the catalog file are:
 *      - fits filename
 *      - night number
 *      - running number
 *      - observation type (CALIB or STRIP)
 *      - channel name
 *      - object name
 *      - running sequence number (photo frame)
 *      - running field number (strip frame)
 *
 *  INPUTS
 *      -p root_night_path  - The directory in which the nights fitsfiles
 *                            are stored
 *      -o outcat           - The name of the catalog file (full path optional)
 *      -f fits_prefix      - Fits filename prefix for the %s*.fits search
 *                            operation
 *      -q                  - Make program verbose
 *
 *  RESULTS
 *      The output catalog is filled with the above information.
 *
 *  RETURNS
 *      None
 *
 *  MODIFIED GLOBALS
 *
 *  NOTES
 *      This program needs only to be run once for a night of observations.
 *      It can be run after successful completion of the tape to disk
 *      reading program.
 *
 *  BUGS
 *
 *  AUTHOR
 *      E.R. Deul - dd 05-09-1996
 *
 ******************************************************************************/
    char	header[MAXCHAR];
    string_list file_list;

    init(argc, argv, &control);

    sprintf(header,"\n   %s Version: %s (%s)\n",NAME,VERS,DATE);
    VPRINTF(NORMAL,header);

    strcpy(prev_object,"");
    strcpy(prev_chan,"");
    seq_nr=1;
    prev_number = 0;

    if (control.listdriven)
       file_list = load_files_list(control.listfile);
    create_files_tab();
    if (control.listdriven)
       list_files(file_list);
    else
       list_dir(control.dir_path);
    save_files_tab(control.outfile, "make_files_cat");

    return 0;
}

