/*
 				bp2.h

*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*
*	Part of:	neural V2.0 routines
*
*	Author:		E.BERTIN, IAP/DeNIS/LDAC.
*
*	Contents:	type definitions for ANN (Perceptron) routines.
*
*	Last modify:	09/11/96
*
*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/


/*------------------------------- definitions -------------------------------*/
#define	SIGMOID(u)	((u)<15.0?((u)>-15.0?1/(1+exp(-(u))):0.0):1.0)
				/* In-line activation function */

#define	SHFT(a,b,c,d)	{(a)=(b);(b)=(c);(c)=(d);}	/* For line-search */
#define	SIGN(a,b)	((b)>0.0? fabs(a) : -fabs(a))	/* For line-search */

#define	GOLD		1.6180340	/* Golden section for line-search */
#define	CGOLD		0.3819660	/* Complement to the golden section */
#define	TINY		1e-20		/* Almost nothing */
#define	GLIMIT		100.0		/* Max. magnification in line-search */
#define	ITMAX		100		/* Max. nb of iter. in line-search */
#define	TOL		1e-2		/* Fract. tolerance in line-search */

/*---------------------------------- types ----------------------------------*/
typedef	float	NFLOAT;		/* Floating point units for neural data */

/*------------------------------- structures --------------------------------*/
typedef	struct structbpann
	{
	int	nlayers;		/* Number of "active" layers */
	int	*nn;			/* Nb of neurons per "active" layer */
/*------ The ANN itself */
	NFLOAT	**neuron;		/* Neuron array (layer,pos in layer) */
	NFLOAT	**weight;		/* Weight array (layer,pos in layer) */
	NFLOAT	**dweight;		/* Weight increment array */
	NFLOAT	**graderr;		/* Back-propagated error gradient */
	NFLOAT	**dgraderr;		/* Error gradient from previous iter */
	int	initseed;		/* Random seed for init (0=random!) */
/*------ Learn-stacks */
	int	nstack, nstackmax;	/* Current and max. nb of patterns */
	NFLOAT	*instack, *outstack;	/* Input and output pattern arrays */
	NFLOAT	*wstack;		/* Pattern weight array */
/*------ Inputs and output scaling */
	NFLOAT	*inbias, *inscale;	/* Input scaling */
	NFLOAT	*outbias, *outscale;	/* Output scaling */
/*----- Learning parameters */
	NFLOAT	learnc[10];		/* Learning coefficients */
	NFLOAT	err;			/* Estimate of the cost function */
	int	ntrain;			/* Total nb of trainings */
	int	nsweep;			/* Nb of passes through each block */
	int	linearoutflag;		/* Flag: 0 if outputs are non-linear */
	}	bpannstruct;


/*------------------------------ Prototypes ---------------------------------*/

bpannstruct	*load_bpann(char *filename, int nstackmax),
		*new_bpann(int *nn, int nl, int nstackmax, int linearflag);

NFLOAT		bp_error(bpannstruct *bpann, NFLOAT x),
		find_eta(bpannstruct *bpann),
		train_bpann(bpannstruct *bpann, int restart);

int		feed_bpann(bpannstruct *bpann, NFLOAT *invec, NFLOAT *outvec,
			float w);

void		free_bpann(bpannstruct *bpann),
		play_bpann(bpannstruct *bpann, NFLOAT *invec, NFLOAT *outvec),
		save_bpann(bpannstruct *bpann, char *filename);

