/*
 				check.c

*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*
*	Part of:	EYE
*
*	Author:		E.BERTIN, IAP & Leiden observatory
*
*	Contents:	handling of "check-images".
*
*	Last modify:	10/05/97
*
*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

#include	"define.h"
#include	"globals.h"
#include	"fitscat.h"
#include	"check.h"

/******************************** initcheck **********************************/
/*
initialize check-image.
*/
void	initcheck(picstruct *field, char *filename)

  {
   char		*buf;
   int		i, ival;
   size_t	padsize;
   double	dval;
   USHORT	*ptri;
   int		*ptri2;
   size_t	*ptri3;
   PIXTYPE	*ptrf;

  strcpy(check.filename, filename);
  if (!(check.file = fopen(check.filename, "wb")))
    error(EXIT_FAILURE, "*Error*: Cannot open for output ", check.filename);

/* Inherit the field FITS header */
  check.fitsheadsize = field->fitsheadsize;
  QMALLOC(check.fitshead, char, check.fitsheadsize);
  memcpy(check.fitshead, field->fitshead, check.fitsheadsize);
  check.y = 0;
/* Neutralize possible scaling factors */
  dval = 1.0;fitswrite(check.fitshead, "BSCALE  ", &dval, H_FLOAT, T_DOUBLE);
  dval = 0.0;fitswrite(check.fitshead, "BZERO   ", &dval, H_FLOAT, T_DOUBLE);
  ival = 1;fitswrite(check.fitshead, "BITSGN  ", &ival, H_INT, T_LONG);
  if (field->compress_type != COMPRESS_NONE)
    fitswrite(check.fitshead, "IMAGECOD", "NONE", H_STRING, T_STRING);
  fitswrite(check.fitshead, "ORIGIN  ", BANNER, H_STRING, T_STRING);

  switch(prefs.check_type)
    {
/*
    case CHECK_IDENTICAL:
    case CHECK_BACKGROUND:
    case CHECK_FILTERED:
    case CHECK_SUBTRACTED:
    case CHECK_SUBOBJECTS:
      ival = -32;
      fitswrite(check.fitshead, "BITPIX  ", &ival, H_INT, T_LONG);
      check.width = field->width;
      check.height = field->height;
      check.npix = field->npix;
      QMALLOC(ptrf, PIXTYPE, check.width);
      check.pix = (void *)ptrf;
      QFWRITE(check.fitshead, check.fitsheadsize, check.file, check.filename);
      free(check.fitshead);
      break;

    case CHECK_OBJECTS:
    case CHECK_APERTURES:
      ival = -32;
      fitswrite(check.fitshead, "BITPIX  ", &ival, H_INT, T_LONG);
      check.width = field->width;
      check.height = field->height;
      check.npix = field->npix;
      check.overlay = 30*field->backsig;
      QCALLOC(ptrf, PIXTYPE, check.npix);
      check.pix = (void *)ptrf;
      QFWRITE(check.fitshead, check.fitsheadsize, check.file, check.filename);
      free(check.fitshead);
      break;

    case CHECK_SEGMENTATION:
      ival = 16;
      fitswrite(check.fitshead, "BITPIX  ", &ival, H_INT, T_LONG);
      check.width = field->width;
      check.height = field->height;
      check.npix = field->npix;
      QCALLOC(ptri, USHORT, check.npix);
      check.pix = (void *)ptri;
      QFWRITE(check.fitshead, check.fitsheadsize, check.file, check.filename);
      free(check.fitshead);
      break;

    case CHECK_ASSOC:
      ival = -32;
      fitswrite(check.fitshead, "BITPIX  ", &ival, H_INT, T_LONG);
      check.width = field->width;
      check.height = field->height;
      check.npix = field->npix;
      QMALLOC(ptrf, PIXTYPE, check.npix);
      check.pix = (void *)ptrf;
      memset(ptrf, 0xFF, check.npix*sizeof(LONG));
      QFWRITE(check.fitshead, check.fitsheadsize, check.file, check.filename);
      free(check.fitshead);
      break;
*/
    case CHECK_HISTOGRAM:
      ival = 32;
      fitswrite(check.fitshead, "BITPIX  ", &ival, H_INT, T_LONG);
      fitswrite(check.fitshead, "NAXIS1  ", &check.width, H_INT, T_LONG);
      fitswrite(check.fitshead, "NAXIS2  ", &check.height, H_INT, T_LONG);
      check.npix = check.width*check.height;
      QMALLOC(ptri2, int, check.npix);
      ptri3 = (size_t *)check.pix;
      check.pix = (void *)ptri2;
      for (i=check.npix; i--;)
        *(ptri2++) = *(ptri3++);
      QFWRITE(check.fitshead, check.fitsheadsize, check.file, check.filename);
      free(check.fitshead);
#     ifdef BSWAP
        swapbytes(check.pix, sizeof(int), (int)check.npix);
#     endif
      QFWRITE(check.pix, check.npix*sizeof(int), check.file, check.filename);
/*---- Put the buffer back to its original state */
#     ifdef BSWAP
        swapbytes(check.pix, sizeof(int), (int)check.npix);
#     endif
      free(check.pix);
      QCALLOC(buf, char, FBSIZE);
      padsize = (FBSIZE -((check.npix*sizeof(PIXTYPE))%FBSIZE))% FBSIZE;
      if (padsize)
        QFWRITE (buf, padsize, check.file, check.filename);
      fclose(check.file);
      free(buf);
      break;

    default:
      error(EXIT_FAILURE, "*Internal Error* in ", "initcheck()!");
    }


  return;
  }


/******************************** writecheck *********************************/
/*
Write ONE line of npix pixels of a check-image.
*/
void	writecheck(PIXTYPE *data, int w)

  {
/*
  if (prefs.check_type == CHECK_APERTURES)
    memcpy((PIXTYPE *)check.pix + w*(check.y++), data, w*sizeof(PIXTYPE));
  else
    {
*/
#   ifdef BSWAP
    swapbytes(data, sizeof(PIXTYPE), w);
#   endif
    QFWRITE(data, w*sizeof(PIXTYPE), check.file, check.filename);
#   ifdef BSWAP
/*-- Put the buffer back to its original state */
    swapbytes(data, sizeof(PIXTYPE), w);
#   endif
/*
    }
*/
  return;
  }


/********************************* endcheck **********************************/
/*
close check-image.
*/
void	endcheck(picstruct *field)
  {
   char		*buf;
   size_t	padsize;

  switch(prefs.check_type)
    {
    case CHECK_HISTOGRAM:
      return;
/*
    case CHECK_IDENTICAL:
    case CHECK_BACKGROUND:
    case CHECK_FILTERED:
    case CHECK_SUBTRACTED:
      free(check.pix);
      padsize = (FBSIZE -((check.npix*sizeof(PIXTYPE))%FBSIZE))% FBSIZE;
      break;

    case CHECK_OBJECTS:
    case CHECK_APERTURES:
    case CHECK_ASSOC:
#     ifdef BSWAP
      swapbytes(check.pix, sizeof(PIXTYPE), (int)check.npix);
#     endif
      QFWRITE(check.pix,check.npix*sizeof(PIXTYPE),check.file,check.filename);
      free(check.pix);
      padsize = (FBSIZE -((check.npix*sizeof(PIXTYPE))%FBSIZE))% FBSIZE;
      break;

    case CHECK_SEGMENTATION:
#     ifdef BSWAP
      swapbytes(check.pix, sizeof(USHORT), (int)check.npix);
#     endif
      QFWRITE(check.pix,check.npix*sizeof(USHORT),check.file,check.filename);
      free(check.pix);
      padsize = (FBSIZE -((check.npix*sizeof(USHORT))%FBSIZE))% FBSIZE;
      break;

    case CHECK_SUBOBJECTS:
      {
       int	y;

      for (y=field->ymin; y<field->ymax; y++)
        writecheck(&PIX(0,y), field->width);
      free(check.pix);
      padsize = (FBSIZE -((check.npix*sizeof(PIXTYPE))%FBSIZE))% FBSIZE;
      break;
      }
*/
    default:
      error(EXIT_FAILURE, "*Internal Error* in ", "endcheck()!");
    }

  QCALLOC(buf, char, FBSIZE);
  if (padsize)
    QFWRITE (buf, padsize, check.file, check.filename);
  fclose(check.file);
  free(buf);

  return;
  }

