 /*
 				histo.c

*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*
*	Part of:	EYE
*
*	Author:		E.BERTIN, IAP & Leiden observatory
*
*	Contents:	Handling of field structures.
*
*	Last modify:	10/05/97
*
*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

#include	<math.h>
#include	<stdio.h>
#include	<stdlib.h>

#include	"define.h"
#include	"globals.h"
#include	"check.h"

#define		DYNAMIC_RANGE	16	/* > ln(1e6) : enough for floats */
					/* must be > 0 */
#define		BINS_PER_SIGMA	2	/* number of bin per background RMS */
					/* must be > 2.0/DYNAMIC_RANGE */

/******************************* make_histo **********************************/
/*
Build a multi-dimensional histogram for weighting image-image learning.
*/
void	make_histo(picstruct **field, int nfield)

  {
   picstruct	**fieldt;
   PIXTYPE	**scan, **scant,
		pix;
   float	binoff;
   size_t	*histo, *step,*stept,
		index, nhisto;
   int		i,x,y, w,h, histow;

  w = (*field)->width;
  h = (*field)->height;

/* Init scan counters */
  histow = 2*DYNAMIC_RANGE*BINS_PER_SIGMA;	/* twice: pos + neg. values */
  binoff = ((histow-1)/2)+0.5;
  QMALLOC(step, size_t, nfield);
  nhisto = 1;
  stept = step;
  fieldt = field;
  for (i=nfield; i--; fieldt++)
    {
    (*fieldt)->y = (*fieldt)->stripy = 0;
    (*fieldt)->ymin = (*fieldt)->stripylim = 0;
    (*fieldt)->stripysclim = 0;
    *(stept++) = nhisto;
    nhisto *= histow;
    }

/* Allocate memory space for the multi-dimensional histogram */
  QCALLOC(histo, size_t, nhisto);
  QMALLOC(scan, PIXTYPE *, nfield*w);

/* Go line-per-line */
  for (y=0; y<h; y++)
    {
    fieldt = field;
    scant = scan;
    for (i=nfield; i--; fieldt++)
      {
      (*fieldt)->stripy = ((*fieldt)->y=y)%(*fieldt)->stripheight;
      *(scant++) = ((*fieldt)->stripy==(*fieldt)->stripysclim)?
		loadstrip((*fieldt))
		: &(*fieldt)->strip[(*fieldt)->stripy*(*fieldt)->width];
      }
    for (x=w; x--;)
      {
      index = 0;
      scant = scan;
      stept = step;
      fieldt = field;
      for (i=nfield; i--; fieldt++)
        {
        pix = *((*(scant++))++)/(*fieldt)->backsig;
        index += *(stept++)
		*(int)(BINS_PER_SIGMA*(pix>0.0?log(1+pix):-log(1-pix))+binoff);
        }
      if (index>0 && index<nhisto)
        histo[index]++;
      }
    }

  free(scan);
  free(step);

  check.pix = histo;
  check.width = histow;
  check.height = nhisto/histow;

  return;
  }


