 /*
 				prefs.c

*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*
*	Part of:	EYE
*
*	Author:		E.BERTIN, IAP & Leiden observatory
*
*	Contents:	Functions to handle the configuration file.
*
*	Last modify:	26/05/97
*
*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

#include	<ctype.h>
#include	<math.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

#include	"define.h"
#include	"globals.h"
#include	"key.h"
#include	"prefs.h"


/********************************* readprefs ********************************/
/*
Read a configuration file in ``standard'' format (see the SExtractor
documentation)
*/
void    readprefs(char *filename, char **argkey, char **argval, int narg)

  {
  FILE          *infile;
  char          *cp, *sstr, str[MAXCHAR], *keyword, *value;
  int           i, ival, nkey, warn, argi, flag;
  double        dval;


  if ((infile = fopen(filename,"r")) == NULL)
    error(EXIT_FAILURE,"*ERROR*: can't read ", filename);

/*Build the keyword-list from pkeystruct-array */

  for (i=0; key[i].name[0]; i++)
    strcpy(keylist[i], key[i].name);
  keylist[i][0] = '\0';


/*Scan the configuration file*/

  argi=0;
  flag = 1;
  for (warn=0;;)
    {
    if (flag>0)
      {
      if (!fgets(str, MAXCHAR, infile))
        flag=0;
      }

    if (flag==0)
      {
      if (argi<narg)
        {
        sprintf(str, "%s %s", argkey[argi], argval[argi]);
        argi++;
        }
      else
        break;
      }

    keyword = strtok(str," \t=,;\n\r");
    if (keyword && keyword[0]!=0 && keyword[0]!=(char)'#')
      {
      if (warn>=10)
        error(EXIT_FAILURE, "*Error*: No valid keyword found in ", filename);
      nkey = findkeys(keyword, keylist, FIND_STRICT);
      if (nkey!=RETURN_ERROR)
        {
        if (!(value = strtok((char *)NULL," \t=,;\n\r")))
          error(EXIT_FAILURE, keyword," keyword has incorrect type");
        switch(key[nkey].type)
          {
          case P_FLOAT:  dval = atof(value);
                         if (dval>=key[nkey].dmin && dval<=key[nkey].dmax)
                           *(double *)(key[nkey].ptr) = dval;
                         else
                           error(EXIT_FAILURE, keyword," keyword out of range");
                         break;

          case P_INT:    ival = atoi(value);
                         if (ival>=key[nkey].imin && ival<=key[nkey].imax)
                           *(int *)(key[nkey].ptr) = ival;
                         else
                           error(EXIT_FAILURE, keyword," keyword out of range");
                       break;

          case P_STRING: if (value[0])
                           strcpy((char *)key[nkey].ptr, value);
                         else
                           error(EXIT_FAILURE, keyword, " keyword empty");
                       break;

          case P_BOOL:   if (cp = strchr("yYnN", (int)value[0]))
                           *(int *)(key[nkey].ptr)
                                = (tolower((int)*cp)=='y')?1:0;
                         else
                           error(EXIT_FAILURE, keyword," value must be Y or N");
                       break;

          case P_KEY:    if ((ival = findkeys(value, key[nkey].keylist,
				FIND_STRICT)) != RETURN_ERROR)
                         *(int *)(key[nkey].ptr) = ival;
                       else
                         error(EXIT_FAILURE, keyword,
				" set to an unknown keyword");
                       break;

          case P_INTLIST:for (i=0; i<MAXLIST && value && value[0]!=(char)'#'; i++)
                           {
                           ival = atoi(value);
                           if (i>=key[nkey].nlistmax)
                             error(EXIT_FAILURE, keyword,
					" has too many members");
                           if (ival>=key[nkey].imin && ival<=key[nkey].imax)
                             ((int *)key[nkey].ptr)[i] = ival;
                           else
                             error(EXIT_FAILURE, keyword,
					" keyword out of range");
                           value = strtok((char *)NULL," \t=,;\n\r");
                           }
                         if (i<key[nkey].nlistmin)
                           error(EXIT_FAILURE, keyword,
				" list has not enough members");

                         *(key[nkey].nlistptr) = i;
                       break;

          case P_FLOATLIST:for (i=0; i<MAXLIST && value && value[0]!=(char)'#'; i++)
                           {
                           dval = atof(value);
                           if (i>=key[nkey].nlistmax)
                             error(EXIT_FAILURE, keyword,
					" has too many members");
                           if (dval>=key[nkey].dmin && dval<=key[nkey].dmax)
                             ((double *)key[nkey].ptr)[i] = dval;
                           else
                             error(EXIT_FAILURE, keyword,
				" keyword out of range");
                           value = strtok((char *)NULL," \t=,;\n\r");
                           }
                         if (i<key[nkey].nlistmin)
                           error(EXIT_FAILURE, keyword,
				" list has not enough members");
                         *(key[nkey].nlistptr) = i;
                       break;

          default:     error(EXIT_FAILURE, "*Internal ERROR*: Type Unknown",
                                        " in readprefs()");
                       break;
          }
        key[nkey].flag = 1;
        }
      else
        {
        warning(keyword, " keyword unknown");
        warn++;
        }
      }
    }

  for (i=0; key[i].name[0]; i++)
    if (!key[i].flag)
      error(EXIT_FAILURE, key[i].name,
        " config. keyword missing");
  fclose(infile);

  return;
  }


/******************************* cistrcmp ***********************************/
/*
case-insensitive strcmp.
*/
int	cistrcmp(char *cs, char *ct, int mode)

  {
   int  diff;

  if (mode)
    {
    while (*cs && *ct)
      if (diff=tolower((int)*(cs++))-tolower((int)*(ct++)))
        return diff;
    }
  else
    {
    while (*cs || *ct)
      if (diff=tolower((int)*(cs++))-tolower((int)*(ct++)))
        return diff;
    }

  return 0;
  }


/********************************** findkeys **********************************/
/*
find an item within a list of keywords, SExtractor version.
*/
int	findkeys(char *str, char keyw[][16], int mode)

  {
  int i;

  for (i=0; keyw[i][0]; i++)
    if (!cistrcmp(str, keyw[i], mode))
      return i;

  return RETURN_ERROR;
  }


/********************************* useprefs **********************************/
/*
Update various structures according to the prefs.
*/
void	useprefs()

  {
   int		i;
   char		*str, wstr[MAXCHAR];


/* Retina size */
  if (prefs.nretina_size<2)
    prefs.retina_size[1] = prefs.retina_size[0];

/* Limits not set are defaulted to -1 */
  for (i=prefs.nlim; i<4; i++)
    prefs.lim[i] = -1;

/* Get the input layer count of neurones from the retina size */
  prefs.nn_size[0] = prefs.retina_size[0]*prefs.retina_size[1];

/* The last layer is given size 1 and the number of layers is updated */
  prefs.nn_size[prefs.nnn_size++] = 1;

/* Set the default max. learning rate for RPROP */
  if (prefs.nlearn_rate<2)
    prefs.learn_rate[1] = MAX_LEARNRATE;
/* If learning is RESUMEd, ignore learning rates in preferences */
  if (prefs.learn_type == LEARN_RESUME)
    prefs.learn_rate[1] = prefs.learn_rate[0] = 0.0;

/* Background */
  if (prefs.nbacksize<2)
    prefs.backsize[1] = prefs.backsize[0];
  if (prefs.nbackfsize<2)
    prefs.backfsize[1] = prefs.backfsize[0];

  return;
  }


