/*
 				types.h

*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*
*	Part of:	EYE
*
*	Author:		E.BERTIN, IAP & Leiden Observatory
*
*	Contents:	global type definitions.
*
*	Last modify:	20/08/97
*
*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

typedef	float		PIXTYPE;	/* pixel type */
typedef	unsigned short	USHORT;		/* 0 to 65535 integers */

/*------------------------------- structures --------------------------------*/
typedef struct
  {
  char		filename[MAXCHAR];	/* pointer to the image filename */
  char		*rfilename;		/* pointer to the reduced image name */
  char		ident[80];		/* field identifier (read from FITS)*/
  FILE		*file;			/* pointer the image file structure */
  char		*fitshead;		/* pointer to the FITS header */
  int		fitsheadsize;		/* FITS header size */
/* ---- main image parameters */
  int		bitpix, bytepix;	/* nb of bits and bytes per pixel */
  int		bitsgn;			/* non-zero if signed integer data */
  int		width, height;		/* x,y size of the field */
  size_t	npix;			/* total number of pixels */
  double	bscale, bzero;		/* FITS scale and offset */
  double	ngamma;			/* normalized photo gamma */
  int		nlevels;		/* nb of quantification levels */
  float		pixmin, pixmax;		/* min and max values in frame */
  int		y;			/* y current position in field */
  int		ymin;			/* y limit (lowest accessible) */
  int		ymax;			/* y limit (highest accessible+1) */
  PIXTYPE	*strip;			/* pointer to the image buffer */
  int		stripheight;		/* height  of a strip (in lines) */
  int		stripmargin;		/* number of lines in margin */
  int		stripstep;		/* number of lines at each read */
  int		stripy;			/* y position in buffer */
  int		stripylim;		/* y limit in buffer */
  int		stripysclim;		/* y scroll limit in buffer */
/* ---- image (de-)compression */
  enum {COMPRESS_NONE, COMPRESS_BASEBYTE, COMPRESS_PREVPIX}
		compress_type;		/* image compression type */
  char		*compress_buf;		/* de-compression buffer */
  char		*compress_bufptr;	/* present pixel in buffer */
  int		compress_curval;	/* current pixel or checksum value */
  int		compress_checkval;	/* foreseen pixel or checksum value */
  int		compress_npix;		/* remaining pixels in buffer */
/* ---- background parameters */
  float		*back;			/* ptr to the background map in mem */
  float		*dback;			/* ptr to the background deriv. map */
  int		backw, backh;		/* x,y size of a bkgnd mesh */
  int		nbackp;			/* total nb of pixels per bkgnd mesh */
  int		nbackx, nbacky;		/* x,y number of bkgnd meshes */
  int		nback;			/* total number of bkgnd meshes */
  int		nbackfx, nbackfy;	/* x,y size of bkgnd filtering mask */
  float		backmean;		/* median bkgnd value in image */
  float		backsig;		/* median bkgnd rms in image */
  PIXTYPE	*backline;		/* current interpolated bkgnd line */
/*----- customize */
#ifdef MEMORY_READ
/* These parameters allow one to read an image directly in memory */
/* The image must be a PIXTYPE array */
  PIXTYPE	*memlink_ima;				/* ptr to an image */
  size_t	memlink_imaoffset;			/* offset in ima */
#endif /* ifdef MEMORY_READ */
  }	picstruct;


/*------------------------------- preferences -------------------------------*/
typedef struct
  {
  char		prefs_name[MAXCHAR];			/* prefs filename*/
  char		retina_name[MAXCHAR];			/* retina filename*/
  int		retina_size[2];				/* retina size */
  int		nretina_size;				/* nb of params */
  int		nn_size[4];				/* neural net size */
  int		nnn_size;				/* nb of params */
  enum	{OUT_RELATIVE, OUT_ABSOLUTE, OUT_BINARY}
					out_type;	/* output type */
  enum	{LEARN_NONE, LEARN_NEW, LEARN_RESUME, LEARN_RESTART}
					learn_type;	/* learning type */
  double	learn_rate[2];				/* learning rate */
  int		nlearn_rate;				/* nb of params */
  int		niter;					/* nb of passes */
  int		nsamp_max;				/* nb of patterns */
/*----- background */
  int		backsize[2];				/* bkgnd mesh size */
  int		nbacksize;				/* nb of params */
  int		backfsize[2];				/* bkgnd filt. size */
  int		nbackfsize;				/* nb of params */
  int		lim[4];					/* frame limits */
  int		nlim;					/* nb of params */
  enum	{CHECK_NONE, CHECK_FILTERED, CHECK_RESIDUALS, CHECK_HISTOGRAM}
					check_type;	/* check-image type */
  char		check_name[MAXCHAR];			/* check-image name */
/*----- miscellaneous */
  enum	{QUIET, NORM, FULL}		verbose_type;	/* display type */

/*----- customize */
#ifdef MEMORY_READ
  char		*memlink_head;				/* ptr to header */
#endif /* ifdef MEMORY_READ */
  }	prefstruct;

