#!/bin/bash

# testscript for the 'replacekey' command

# $1: path to executable of replacekey

if [ $# -ne 1 ]; then
  echo "$0 <path to replacekey command>"
  exit 1;
fi

REPLACEKEY=$1

echo "Testing program ${REPLACEKEY}"

# script to test functionality of replacekey
FAIL=0

ic -c 512 512 '0' > test_$$.fits

# Note that the following tests destroy the integroty
# of the FITS file. But this is unimportant for this exercise:
echo "Testing single keyword replacement"
${REPLACEKEY} test_$$.fits "NAXIS1  =                  513" NAXIS1

NAXIS1=`dfits test_$$.fits | fitsort -d NAXIS1 | awk '{print $2}'`

if [ ${NAXIS1} -ne 513 ]; then
  echo "Single keyword change test failed!"
  FAIL=$(( ${FAIL} + 1 ))
fi

echo "Testing multiple keyword replacement"
${REPLACEKEY} test_$$.fits "NAXIS2  =                  513" NAXIS2 \
                        "NAXIS   =                    3" NAXIS

NAXIS=`dfits test_$$.fits | fitsort -d NAXIS | awk '{print $2}'`
NAXIS2=`dfits test_$$.fits | fitsort -d NAXIS2 | awk '{print $2}'`

if [ ${NAXIS2} -ne 513 ] || [ ${NAXIS} -ne 3 ]; then
  echo "Multiple keyword change test failed!"
  FAIL=$(( ${FAIL} + 1 ))
fi

echo "${FAIL} tests failed for replacekey!"

rm test_$$.fits

exit ${FAIL}
